/*
 * $Id: ModuleFR.java,v 1.3 2009/04/12 11:48:48 sirocic Exp $
 */
package flexray;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.KeyboardFocusManager;
import java.awt.Point;
import java.awt.Rectangle;

import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.InvocationTargetException;
import java.net.URL;
import java.util.HashMap;
import java.util.List;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JRootPane;
import javax.swing.JSlider;
import javax.swing.JSplitPane;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import learningVisualizations.LeviModule;
import learningVisualizations.LeviResourceManager;
import learningVisualizations.gui.LeviFrame;
import learningVisualizations.gui.LeviMenu;
import learningVisualizations.gui.LeviPopupListener;
import learningVisualizations.gui.LeviToolbar;
import learningVisualizations.gui.actions.LeviActionForwardSimulation;

import org.w3c.dom.Document;


import flexray.configuration.ChannelName;
import flexray.gui.AlphaBlendingComponent;
import flexray.gui.FlexRayStatusBar;
import flexray.gui.ScenarioPanel;
import flexray.gui.Property;
import flexray.gui.TurningKnob;
import flexray.gui.actions.CloseCurrentScenarioAction;
import flexray.gui.actions.SetActiveScenarioAction;
import flexray.gui.controller.EditCommandType;
import flexray.levi.gui.FrameFR;
import flexray.levi.help.FRHelp;
import flexray.levi.simulator.FRleviSimulator;
import flexray.levi.visualizer.FRVisualizer;
import flexray.scenario.FlexRayScenario;
import flexray.util.DebugUtil;
import flexray.util.TextUtil;
import flexray.xml.XMLTags;

/**
 * Component class for the FlexRay simulation.
 *
 *
 * @author Dirk Frsterling
 *
 */
public class ModuleFR extends LeviModule
                      implements XMLTags,
                                 ConfigKeys {

    // BS 15.11.06 Versionsnummer im Titel anzuzeigen ...
    public static final String 	VERSION = "Version 1.0";
    public static final String TYPE = "FlexRay";

    /**
     * Reference to one (the) ComponentFR instance
     */
    private static ModuleFR mySelf;// = null;

    // storage for the window parameters during switched views.
    private LeviPopupListener popupListener = null;
    private Dimension savedFrameSize = null;
    private Point savedFrameLocation = null;
    private boolean savedFrameUnDecorated = false;
    private int savedFrameWindowDecorationStyle = 0;
    protected final FlexRayStatusBar statusBar;
    private LeviToolbar toolBar = null;
    private TurningKnob simSpeedTurningKnob;
    private CloseCurrentScenarioAction closeCurrentScenarioAction = null;
    private HashMap<Integer, JMenuItem> menuViewScenarioItems =
        new HashMap<Integer, JMenuItem>();
    private HashMap<Integer, JMenuItem> popupViewScenarioItems =
        new HashMap<Integer, JMenuItem>();

    private boolean separator = false;

    private boolean useTurningKnob = true;

    /*
     *
     * ------------------------- construction ---------------------------
     *
     */

    /**
     * Construct the module
     *
     * @param cname Name of the module
     */
    public ModuleFR(final String mname) {
        super( mname, "flexray.ConfigurationFR", "leviFRPConfig.xml" , VERSION);

        // keep a static reference to prevent introducing parameters
        mySelf = this;

        /*
         *                simulator
         */
        setSimulator( new FRleviSimulator( this ) );
        getSimulator().start();


        setVisualizer(new FRVisualizer(this));

        // create the own frame and keep a reference
        FrameFR f = new FrameFR( mySelf );
        this.setFrame(f);
        setHelp( new FRHelp(this) );
        f.composeFrame();


        // show JFrame decoration -> looks the same on any platform.
        if( ModuleFR.isMetal() ) {
            f.getRootPane().setWindowDecorationStyle(JRootPane.FRAME);
            f.setUndecorated( true );
        }

        Object o = null;
        o = getConfiguration().getSettingByKey(CONFIG_WINDOWRECT);
        if((o != null) && (o instanceof Rectangle)) {
            Rectangle r = (Rectangle)o;
            f.setSize(r.width,r.height);
            f.setLocation(r.x, r.y);
        } else {
            f.setSize( 1000,700 );
            double fx = f.getFullscreenSize().getWidth() - f.getWidth();
            double fy = f.getFullscreenSize().getHeight() - f.getHeight();
            f.setLocation( (int)(fx / 2), (int)(fy / 2));
        }


        savedFrameSize = f.getNormalSize();

        /*
         * Toolbar
         */
        toolBar = new LeviToolbar( this );
        toolBar.setBorderPainted(true);
        getConfiguration().addPropertyChangeListener(toolBar);
        
        toolBar.enablePlayButton();
        toolBar.enablePauseButton();
        toolBar.enableStopButton();
        toolBar.enableForwardButton();
        toolBar.enableBackwardButton();

        Color bbg = (Color)getConfiguration().getSettingByKey("ColorButtonBackground");
        Color bfg = (Color)getConfiguration().getSettingByKey("ColorButtonForeground");
        Color bhi;
        if ((Color)getConfiguration().getSettingByKey("ColorComponentHighlight") != null)
        	bhi = (Color)getConfiguration().getSettingByKey("ColorComponentHighlight");
        else
        	bhi = (Color)getConfiguration().getSettingByKey("ColorModuleHighlight");

        if( useTurningKnob ) {
            simSpeedTurningKnob = new TurningKnob("button.tacho", this, 38);
            simSpeedTurningKnob.setBackground(bbg);
            simSpeedTurningKnob.setForeground(bfg);
            TurningKnob.setHighlightColor(bhi);
            simSpeedTurningKnob.addPropertyChangeListener(Property.ANGLE, getSimulator());
            simSpeedTurningKnob.setAngle((int)(270-Math.sqrt((getSimulator().getSleepFactor()-1)*40)));
            simSpeedTurningKnob.setText(""+getSimulator().getSleepFactor());
            toolBar.add(simSpeedTurningKnob);
        } else {

            class PanLis extends JPanel implements ChangeListener {
                private static final long serialVersionUID = 215856870260971973L;
                private JLabel label = null;
                public void setLabel(JLabel l) {
                    label = l;
                }
                public void stateChanged(ChangeEvent e) {
                    if( e.getSource() instanceof JSlider ) {
                        getSimulator().stateChanged(e);
                        label.setText("" + getSimulator().getSleepFactor());
                    }
                }
            }
            PanLis miniPanel = new PanLis();

            miniPanel.setLayout(new BorderLayout());

            JSlider slider = new JSlider(0,270,(int)(270-Math.sqrt(getSimulator().getSleepFactor()*40)));
            Dimension sliderDim = new Dimension(36,18);
            slider.setMaximumSize(sliderDim);
            slider.setPreferredSize(sliderDim);
            slider.setSize(sliderDim);
            slider.setBackground(bbg);
            slider.setForeground(bfg);
            slider.setAlignmentX(Component.LEFT_ALIGNMENT);
            slider.setAlignmentY(Component.TOP_ALIGNMENT);
            slider.setBorder(BorderFactory.createEmptyBorder());
            slider.addChangeListener( miniPanel );

            JLabel label = new JLabel("x");// + (int)(270-Math.sqrt(getSimulator().getSleepFactor()*40)));
            miniPanel.setLabel(label);
            label.setBackground(bbg);
            label.setForeground(bfg);
            label.setAlignmentX(Component.CENTER_ALIGNMENT);

            JPanel labelPanel = new JPanel();
            labelPanel.setLayout(new BoxLayout(labelPanel, BoxLayout.PAGE_AXIS));
            labelPanel.setPreferredSize(sliderDim);
            labelPanel.setMaximumSize(sliderDim);
            labelPanel.setBackground(bbg);
            labelPanel.setForeground(bfg);
            labelPanel.add(label);

            Dimension panelDim = new Dimension(38,38);
            miniPanel.setMaximumSize(panelDim);
            miniPanel.setPreferredSize(panelDim);
            miniPanel.setBorder(BorderFactory.createEtchedBorder());
            miniPanel.setBackground(bbg);
            miniPanel.setForeground(bfg);
            miniPanel.add(slider, BorderLayout.NORTH);
            miniPanel.add(labelPanel, BorderLayout.SOUTH);
            miniPanel.setAlignmentX(Component.LEFT_ALIGNMENT);

            toolBar.add(miniPanel);
        }

        // Sichtwechsel-Button in die Toolbar einfgen
        toolBar.enableViewButton();
        
        f.add(toolBar,BorderLayout.EAST);

        statusBar = new FlexRayStatusBar( this );
        statusBar.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.RAISED));
        statusBar.setPreferredSize(new Dimension(4000,22));

        f.add(statusBar, BorderLayout.SOUTH);

        LeviMenu m = f.getMenubar();
        closeCurrentScenarioAction = new CloseCurrentScenarioAction();
        getFrame().changeItemAction(m.getMenuScenario(), "key.close", closeCurrentScenarioAction);
        closeCurrentScenarioAction.setEnabled(false);
        LeviResourceManager res = getRessources();
        res.addPropertyChangeListener(LeviResourceManager.PROPERTY_LOCALE, statusBar);
        res.addPropertyChangeListener(LeviResourceManager.PROPERTY_LOCALE, f);
        res.addPropertyChangeListener(LeviResourceManager.PROPERTY_LOCALE, closeCurrentScenarioAction );

        boolean bod = false;
        if( getConfiguration().keyExist(CONFIG_BLENDONDRAG)) {
            bod = ((Boolean)getConfiguration().getSettingByKey(CONFIG_BLENDONDRAG)).booleanValue();
        } else {
            getConfiguration().putSettingByKey(CONFIG_BLENDONDRAG, Boolean.TRUE);
        }
        AlphaBlendingComponent.setDefaultBlendOnDrag( bod );

        boolean aa = true;
        if( getConfiguration().keyExist(CONFIG_ANTIALIAS)) {
            aa = ((Boolean)getConfiguration().getSettingByKey(CONFIG_ANTIALIAS)).booleanValue();
        } else {
            getConfiguration().putSettingByKey(CONFIG_ANTIALIAS, Boolean.TRUE);
        }
        getConfiguration().saveConfiguration();
        ScenarioPanel.setAntiAlias(aa);

        if( hasDefaultScenario() ) {
            openScenario(getXmlScenarioFile());
        }

		if (getXmlScenarioDoc() == null) {
			// Es wurde kein Defaultscenario angegeben oder das scenario konnte nicht geffnet werden.
			setXmlScenarioFile(null);
			addScenario(new FlexRayScenario(this));
			getCurrentOrNewScenario().setChanged(false);
		}
		
        if (getConfiguration().keyExist("Help") || getConfiguration().keyExist("Avatar")) {
            Boolean tmp2;
            if (getConfiguration().keyExist("Help"))
            	tmp2= (Boolean) getConfiguration().getSettingByKey("Help");
            else
            	tmp2= (Boolean) getConfiguration().getSettingByKey("Avatar");
            setHelpVisible(tmp2.booleanValue());
            f.composeFrame();
        }

        getHelp().addPropertyChangeListener(statusBar);

    } // Constructor( name )

    /*
     * Methode:		disableCommands
     *
     * Description:	Abschalten von Kommandos (GUI-Elementen), die z.B. whrend der Simulation nicht
     * 				erlaubt sind. Hier sollten die fr alle Lernmodule gltigen Kommandos disabled werden.
     * 				In den lernmodule-spezifischen Klassen sollte dann die lernmodul-spezifischen
     * 				Kommandos disabled werden.
     *
     * Date: 		    27.02.2007
     * @author        sirocic
     */
    public void disableCommands() {}

    /*
     * Methode:		enableCommands
     *
     * Description:	Anschalten von Kommandos (GUI-Elementen), die z.B. whrend der Simulation nicht
     * 				erlaubt sind. Hier sollten die fr alle Lernmodule gltigen Kommandos enabled werden.
     * 				In den lernmodule-spezifischen Klassen sollte dann die lernmodul-spezifischen
     * 				Kommandos enabled werden.
     *
     * Date: 		    27.02.2007
     * @author        sirocic
     */
    public void enableCommands()  {}

    /**
     * Override to get the own Simulator type without casting.
     * (possible since java 5)
     * @return this component's FRleviSimulator frame
     * @see learningVisualizations.LeviModule#getSimulator()
     */
    @Override
    public FRleviSimulator getSimulator() {
        return (FRleviSimulator) super.getSimulator();
    }

    /**
     * Override to get the own Help type without casting.
     * (possible since java 5)
     * @return this component's FRHelp frame
     * @see learningVisualizations.LeviModule#getHelp()
     */
    @Override
    public FRHelp getHelp() {
        return (FRHelp)super.getHelp();
    }

    /**
     *
     * @return a reference to this component
     */
    public static ModuleFR getModule() {
        return mySelf;
    }

    /**
     * Convenience method for reaching the refresh more simple.
     *
     */
    public void refresh() {
        getFrame().refresh();
    }




    /*
     *
     * ------------------------- scenario ---------------------------
     *
     */

    private String lastLoadedScenarioName = "";
    @Override
    public Document loadScenarioFromFile(File scenarioFile) {
    	Document ret = super.loadScenarioFromFile(scenarioFile);
        String filename = scenarioFile.getName();
        lastLoadedScenarioName = filename.substring(0,filename.lastIndexOf(".xml"));
        return ret;
    }

    /**
     * "Open" XML scenario. This is called after XML file got loaded  into XMLDoc.
     */
    @Override
    public void openScenario(File scenarioFile) {
    	Document xmlDoc;
    	
        if(getScenarios().size() > 11 ) return;

        if (scenarioFile != null) {
        	xmlDoc = loadScenarioFromFile(scenarioFile);
        	
        	if (xmlDoc != null) {
	        	// scenariofile konnte erfolgreich eingelesen werden. XML-Parser ok.
	        	// Dann kann jetzt ein scenario-Document erzeugt werden und ein zugehriges scenario-Objekt angelegt 
	        	// und angezeigt werden.
	        
		        FlexRayScenario scenario = new FlexRayScenario(this);
		
		        scenario.restoreFromXML(xmlDoc.getDocumentElement());
		
		        if( lastLoadedScenarioName.equals("") ) {
		            String filename = scenarioFile.getName();
		            lastLoadedScenarioName = filename.substring(0,filename.lastIndexOf(".xml"));
		        }
		        // set scenario name (meta information)
		        scenario.setName(lastLoadedScenarioName);
	   	
		    	// neugeladenes scenario in Komponente bernehmen
		    	setXmlScenarioFile(scenarioFile);
		    	setXmlScenarioDoc(xmlDoc);
		    	scenario.setName(scenarioFile.getName());
		        addScenario(scenario);
		        // a newly loaded scenario is never changed
		        scenario.setChanged(false);
		        
        	}
        }

    } // openScenario

    /*
     * Methode:		loadSchemaFile
     *
     * Description:	Ladend des komponentenspezifischen Schema-Files z.b. RealTimeScheduling.xsd aus den Ressourcen.
     *
     * Date: 		17.02.2007
     * @author      sirocic
     */
    public URL loadSchemaFile() {
    	return LeviModule.class.getResource("/resources/flexray/FlexRay.xsd");
    }
    
    /**
     * Add a scenario to the list of scenarios and make it the current scenario
     * @param scenario The scenario to add.
     */
    private void addScenario(FlexRayScenario scenario) {

        if(getScenarios().size() > 11 ) return;

        getScenarios().add(scenario);
        FlexRayScenario previousActiveScenario = null;

        if( getCurrentScenarioIndex() > -1) {
            previousActiveScenario = (FlexRayScenario) getScenarios().get(getCurrentScenarioIndex());
            previousActiveScenario.setOpen(false);
        }

        setCurrentScenarioIndex(getScenarios().indexOf(scenario));
        scenario.setOpen(true);

        // put the scenario's panel in the visible area
        FrameFR f = (FrameFR) getFrame();
        int dividerLocation = f.getSelfstudyDividerLocation();
        f.getSelfstudyPanel().setLeftComponent( ((FlexRayScenario)getScenario()).getScenarioComponent() );
        f.setSelfstudyDividerLocation(dividerLocation);

        // the following method depends on the "current scenario"
        JMenu menuView = f.getMenubar().getMenuView();
        SetActiveScenarioAction setActiveScenarioAction = null;
        if( menuView != null ) {
            if( (getScenarios().size() == 1) && !separator ) {
                menuView.addSeparator();
                // separator will be set below
            }
            JMenuItem menuSetActiveItem = new JMenuItem();
            setActiveScenarioAction = new SetActiveScenarioAction(scenario,
                                                            menuSetActiveItem,
                                                            menuView);
            menuSetActiveItem.setAction(setActiveScenarioAction);
            getRessources().addPropertyChangeListener(Property.LOCALE,setActiveScenarioAction);
            menuView.add(menuSetActiveItem);
            menuViewScenarioItems.put(Integer.valueOf(getScenarios().indexOf(scenario)), menuSetActiveItem);
        } else {
            System.err.println( "Menu 'view' does not exist?");
        }

        /*
         * Have to create another item for the popup
         */
        JMenu popupView = f.getPopupMenu().getMenuView();
        if( menuView != null ) {
            if( getScenarios().size() == 1 && !separator) {
                popupView.addSeparator();
                separator = true;
            }
            JMenuItem menuSetActiveItem = new JMenuItem();
            setActiveScenarioAction = new SetActiveScenarioAction(scenario,menuSetActiveItem,popupView);
            getRessources().addPropertyChangeListener(Property.LOCALE,setActiveScenarioAction);
            menuSetActiveItem.setAction(setActiveScenarioAction);
            popupView.add(menuSetActiveItem);
            popupViewScenarioItems.put(Integer.valueOf(getScenarios().indexOf(scenario)), menuSetActiveItem);
        } else {
            System.err.println( "Menu 'view' does not exist?");
        }

        /*
         * The added scenario should listen to property changes.
         */
        getRessources().addPropertyChangeListener(scenario);
        getConfiguration().addPropertyChangeListener(scenario);

        setActiveScenario(scenario);
        if(previousActiveScenario != null) {
            previousActiveScenario.fireStateChanged();
        }
        closeCurrentScenarioAction.setEnabled(true);
    }


    /**
     * Save the current scenario.
     *
     * @see learningVisualizations.LeviModule#saveScenario()
     */
    @Override
    public void saveScenario() {

        File scenarioFile = getXmlScenarioFile();

        String filename = scenarioFile.getName();
        System.err.println( "name : " + filename );
        if( !filename.endsWith(".xml") ) {
            filename = filename+".xml";
            scenarioFile = new File(filename);
            setXmlScenarioFile(scenarioFile);
        }

        FlexRayScenario scenario = (FlexRayScenario)getScenario(); // the current scenario

        scenario.setName(filename.substring(0,filename.lastIndexOf(".xml")));

        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        DocumentBuilder builder = null;
        try {
            builder = factory.newDocumentBuilder();
        } catch (ParserConfigurationException pce) {
            System.err.println("Could not create DocumentBuilder");
            System.err.println(TextUtil.e2s(pce));
        }
        if( builder == null ) {
            System.err.println("errorDialog");
            return;
        }
        Document xmlDoc = builder.newDocument();

        // The XML root structure
        xmlDoc.appendChild(scenario.toXML(xmlDoc));
        setXmlScenarioDoc(xmlDoc);

        try {
            /*
             * special treatment for outputting XML. See also
             * http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=6296446
             */
            Source source = new DOMSource(xmlDoc);
            FileOutputStream fos = new FileOutputStream(scenarioFile);
            // wrap the outputstream with a writer to work around
            // a "buggy" behaviour in the xml handling code
            Result result = new StreamResult( new OutputStreamWriter(fos,"utf-8"));
            // get the TransformerFactory (so we can change parameters)
            TransformerFactory xformfactory = TransformerFactory.newInstance();
            // tell the factory how much indentation we want to have.
            xformfactory.setAttribute("indent-number", Integer.valueOf(4));
            Transformer xformer = xformfactory.newTransformer();
            xformer.setOutputProperty(OutputKeys.INDENT, "yes");
            xformer.setOutputProperty(OutputKeys.STANDALONE, "no");
            // the following is OK when the indentation bug REALLY gets fixed.
            //xformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "4");
            xformer.transform(source, result);
            fos.close();
            scenario.setChanged(true);
            statusBar.stateChanged( new ChangeEvent(scenario) );
        } catch (TransformerConfigurationException tce) {
            System.err.println("ComponentFR exception: " + TextUtil.e2s(tce));
        } catch (TransformerException te) {
            System.err.println("ComponentFR exception: " + TextUtil.e2s(te));
        } catch (FileNotFoundException fnfe) {
            System.err.println("ComponentFR exception: " + TextUtil.e2s(fnfe));
        } catch (UnsupportedEncodingException uee) {
            System.err.println("ComponentFR exception: " + TextUtil.e2s(uee));
        } catch (IOException ioe ) {
            System.err.println("ComponentFR exception: " + TextUtil.e2s(ioe));
        }

        // freshly saved scenarios are never modified
        scenario.setChanged(false);
        scenario.fireStateChanged();
    } // saveScenario()

    /**
     *
     * @see learningVisualizations.LeviModule#newScenario()
     */
    @Override
    public void newScenario() {
        // create the scenario
        FlexRayScenario scenario = new FlexRayScenario(this);
        // if there already was a scenario, remove the statusBar from listeners
        if( getScenarios().size() > 0 ) {
        	((FlexRayScenario)getScenarios().get(getCurrentScenarioIndex())).removeChangeListener(statusBar);
        }
        scenario.setChanged(true);

        addScenario(scenario);
    } // newScenario()

    /**
     * Close the currently active scenario
     *
     */
    @Override
    public void closeScenario() {
        FlexRayScenario scenario = (FlexRayScenario)getScenario();
        JComponent scenarioComponent = scenario.getScenarioComponent();
        FrameFR frame = (FrameFR) getFrame();

        if( scenario.hasChanged() ) {
            askUserToSave();
        }
        // FIXME: how to find out if the user cancelled the
        // save file dialog?

        scenario.setLastEdit(EditCommandType.SCENARIO_CLOSED);
        scenario.fireStateChanged();
        scenario.setOpen(false);

        int index = getScenarios().indexOf(scenario);
        getScenarios().remove(scenario);

        JMenuItem m = menuViewScenarioItems.get(Integer.valueOf(index));
        if( m != null ) {
            getRessources().removePropertyChangeListener((PropertyChangeListener)m.getAction());
            frame.getMenubar().getMenuView().remove(m);
            menuViewScenarioItems.remove(Integer.valueOf(index));
        }
        m = popupViewScenarioItems.get(Integer.valueOf(index));
        if( m != null ) {
            getRessources().removePropertyChangeListener((PropertyChangeListener)m.getAction());
            frame.getPopupMenu().getMenuView().remove(m);
            popupViewScenarioItems.remove(Integer.valueOf(index));
        }

        if(index > (getScenarios().size()-1)) index--;
        if( index >= 0 ) {
        	setCurrentScenarioIndex(-1); // does no longer exist
            setActiveScenario((FlexRayScenario)getScenarios().get(index));
            getScenario().setOpen(true);
        } else {
            int dividerLocation = frame.getSelfstudyDividerLocation();
            frame.getSelfstudyPanel().remove(scenarioComponent);
            frame.getSelfstudyPanel().setLeftComponent((JComponent)getEditor());
            frame.setSelfstudyDividerLocation(dividerLocation);
            closeCurrentScenarioAction.setEnabled(false);
            ((FrameFR)getFrame()).getToggleTwoChannelAction().setEnabled(false);
            setCurrentScenarioIndex(-1);;
        }

        getRessources().removePropertyChangeListener(scenario);
        getConfiguration().removePropertyChangeListener(scenario);
        ChangeEvent ce = new ChangeEvent( this );
        frame.stateChanged( ce );
        statusBar.stateChanged( ce );

    } // closeScenario()

    /**
     * Return the current scenario.
     * If no scenario is currently available, create a new scenario and make it
     * the current scenario before returning it.
     *
     * @return the current FlexRayScenario
     */
    public FlexRayScenario getCurrentOrNewScenario() {
        if( getCurrentScenarioIndex() < 0 ) newScenario();
        return (FlexRayScenario) getScenarios().get( getCurrentScenarioIndex() );
    }

    /**
     * Change the current scenario
     * @param scenario
     */
    public void setActiveScenario(final FlexRayScenario scenario) {
        if(!getSimulator().isStop()) getSimulator().stopSimulation();
        int scenarioIndex = getScenarios().indexOf( scenario );
        final FrameFR frame = (FrameFR) getFrame(); // the frame never changes!
        FlexRayScenario prevMod = null;
        if( scenarioIndex > -1) {
            if( getCurrentScenarioIndex() != -1 ) {
                prevMod = (FlexRayScenario) getScenarios().get(getCurrentScenarioIndex());
                prevMod.removeChangeListener(statusBar);
                prevMod.removeChangeListener(frame);
                prevMod.setOpen(false);
                if( getView() == LECTUREVIEW )
                    prevMod.getScenarioPanel().removeMouseListener(popupListener);
            }
            setCurrentScenarioIndex(scenarioIndex); 
            int dividerLocation = frame.getSelfstudyDividerLocation();
            // remove the left before adding it again,
            frame.getSelfstudyPanel().remove(frame.getSelfstudyPanel().getLeftComponent());
            frame.getSelfstudyPanel().setLeftComponent( ((FlexRayScenario)getScenario()).getScenarioComponent() );
            frame.setSelfstudyDividerLocation(dividerLocation);
            if( getView() == LECTUREVIEW )
            	scenario.getScenarioPanel().addMouseListener(popupListener);
        }
        // FIXME: popup Listener!
        scenario.addChangeListener(statusBar);
        scenario.addChangeListener(frame);
        scenario.addChangeListener(getHelp());
        scenario.setOpen(true);
        scenario.fireStateChanged();

        frame.getToggleTwoChannelAction().setEnabled(true);
        frame.getToggleTwoChannelAction().set(scenario.getCluster().getClusterConfiguration().gChannels().equals(ChannelName.AB));

        if( prevMod != null)
            prevMod.fireStateChanged(); // it is no longer active
    } // switchScenario()

    /*
     *
     * ------------------------- ??????? ---------------------------
     *
     */

    /**
     * Override setView() to implement the following for just the
     * FlexRay application:
     * <ul>
     * <li>respect window configuration (decorations)</li>
     * <li>restore the original window size after returning from full screen</li>
     * <li>make full screen independent from window position</li>
     * </ul><br>
     * TODO: Make this method not assume that the first view is <code>SELFSTUDYVIEW</code>
     *
     * @param newView the number of the view to set
     */
    @Override
    public void setView( int newView ) {

        // no frame yet? Nothing to modify. Go on with the super
        // method which constructs everything
        if( getFrame() == null ) {
            super.setView( newView );
        }

        /*
         * Switching the view without a scenario makes no sense.
         */
        if( getCurrentScenarioIndex() < 0 ) {
            return;
        }

        // don't do anything if nothing will change.
        if( view == newView ) return;

        // get the frame
        FrameFR frame = (FrameFR) getFrame();

        // dispose of the display resources
        frame.dispose();

        switch ( newView ) {
        case LeviModule.SELFSTUDYVIEW:
            // re-set window parameters
            frame.setSize( savedFrameSize );
            frame.setLocation( savedFrameLocation );
            frame.setUndecorated( savedFrameUnDecorated );
            frame.getRootPane().setWindowDecorationStyle( savedFrameWindowDecorationStyle );

            // re-add menu bar
            frame.setJMenuBar( frame.getMenubar() );

            frame.add( statusBar, BorderLayout.SOUTH );

            // remove mouselistener for popup
            ((FlexRayScenario)getScenario()).getScenarioPanel().removeMouseListener( popupListener );
            break;
        case LeviModule.LECTUREVIEW:
            // store window parameters
            savedFrameWindowDecorationStyle = frame.getRootPane().getWindowDecorationStyle();
            frame.getRootPane().setWindowDecorationStyle(JRootPane.NONE);

            savedFrameUnDecorated = frame.isUndecorated();
            frame.setUndecorated( true );

            savedFrameSize = frame.getSize();
            frame.setSize( frame.getFullscreenSize() );

            savedFrameLocation = frame.getLocation();
            frame.setLocation( 0, 0 );

            // reset menu bar
            frame.setJMenuBar( null );

            frame.remove( statusBar );

            // add mouse listener for leviPopup
            popupListener = new LeviPopupListener(frame.getPopupMenu());
            ((FlexRayScenario)getScenario()).getScenarioPanel().addMouseListener( popupListener );

            // Hilfe verstecken
            setHelpVisible(false);
            
            break;
        default:
            break;
        }
        // set the frame visible and validate it, so components are redrawn correctly.
        // if called from an event, we are in the EDT, so no invokeLater is necessary.
        frame.setVisible( true );
        // remove any keyboard focus
        frame.validate();
        KeyboardFocusManager.getCurrentKeyboardFocusManager().clearGlobalFocusOwner();
        view = newView;
    } // setView()


    /**
     *
     * @param val
     * @see learningVisualizations.LeviModule#setHelpVisible(boolean)
     */
    @Override
    public void setHelpVisible(boolean val) {
    	super.setHelpVisible(val);
        if( getFrame() == null ) return;
        FrameFR f = (FrameFR) getFrame();
        JSplitPane s = f.getSelfstudyPanel();


        if( val ) {
            s.setRightComponent( getHelp() );
            f.setHelpPanelVisible(val);
            
            correctDividerLocation(f);
        } else {
            softOutCallBack();
        }
    }

    public void correctDividerLocation(FrameFR f) {
        int dl  = f.getSelfstudyPanel().getWidth() -
            f.getSelfstudyPanel().getDividerSize() -
            getHelp().getWorkingWidth();
        f.setSelfstudyDividerLocation(dl);
    }

    public void softOutCallBack() {
        super.setHelpVisible(false);
        getFrame().setHelpPanelVisible(false);
    }

    /*
     * @see learningVisualizations.leviModule#getMode()
     */
    @Override
    public int getMode() {
        int mode = super.getMode();
        String caller = DebugUtil.getCaller(2, true);
        // FIXME: mode info?
//        DebugUtil.mark( caller + " requesting mode info.");
        if( caller.equals(LeviActionForwardSimulation.class.getCanonicalName())){
            return LeviModule.SIMULATIONMODE;
        }

        return mode;
    }


    @Override
    public void exit() {
        statusBar.cleanup(); // because of the memoryBar
        super.exit();
    }


    public void updateStatusBar() {
        EventQueue.invokeLater( new Runnable() {
            public void run() {
                statusBar.updateMacroTick((FlexRayScenario)getScenario());
            }
        });
    }
    
    /**
     * Methode:		updateDialogsColorLanguageFonts
     *
     * Description:	Aktualisiert bei den LeVi-Dialogen die Farben und Textbausteine gem der aktuellen
     * 				Konfiguration bzw. Spracheinstellung (Ressourcen).
     * 
     * Date: 	    24.04.2007
     * @author        sirocic
     * @param frame the frame
     */
      public void updateDialogsColorLanguageFonts(LeviFrame frame) {

          // Standard-levi-dialoge an aktuelle Einstellungen anpassen.
          super.updateDialogsColorLanguageFonts();
          
      }

    public static boolean isMetal() {
        return false;
    }

    /*
     *
     * ------------------------- Application invocation ---------------------------
     *
     */

    static ModuleFR dummyModule = null;
    static FrameFR dummyFrame;

    /**
     * main method to run the application
     *
     * @param args command line arguments
     */
    public static void main(final String[] args) {
    	
    	 System.out.println("Starting FRP learning simulation ...");
    	 
        // Set the RepaintManager to a thread checking repaint manager for
        // making EDT problems visible
        if( isMetal() ) {
            JFrame.setDefaultLookAndFeelDecorated( true );
            JDialog.setDefaultLookAndFeelDecorated( true );
        }
        // for now, disable auto blending on dragging.
        /*try {
            EventQueue.invokeAndWait( new Runnable() {
                public void run() {
                    dummyModule = new ModuleFR( "FlexRay");
                    dummyFrame = (FrameFR) dummyModule.getFrame();
                    dummyFrame.setVisible(true);
                    dummyModule.correctDividerLocation(dummyFrame);
                }
            });
        } catch( InvocationTargetException ite ) {
            System.err.println( "Could not create GUI: " + ite.getMessage() );
            System.err.println( TextUtil.e2s(ite));
            System.exit(1);
        } catch( InterruptedException ie ) {
            System.err.println( "GUI creation interrupted. GUI is " + dummyModule );
        }*/
        
        dummyModule = new ModuleFR( "FlexRay");
        dummyFrame = (FrameFR) dummyModule.getFrame();
        dummyFrame.setVisible(true);
        dummyModule.correctDividerLocation(dummyFrame);

    }


    public TurningKnob getSimSpeedTurningKnob() {
        return simSpeedTurningKnob;
    }


}
