/*
 * $Id: Channel.java,v 1.1 2009/04/01 13:57:09 sirocic Exp $
 *
 * Created on 25.04.2006 14:40:20
 * by Dirk Frsterling <r@zorbla.de>
 */
package flexray.cluster;

import java.util.HashSet;
import java.util.Observable;

import flexray.communication.FlexRayFrame;
import flexray.configuration.ChannelName;

/**
 * The channel representative.
 *
 * @author Dirk Frsterling <r@zorbla.de>
 */
abstract class Channel extends Observable {

    private Cluster cluster;

    private ChannelName channelName = null;

    private FlexRayFrame messageInTransit = null;
    private long messageReceivedTime = 0;
    private long messageDeleteTime = 0; // when the message must be deleted

    private HashSet<Node> nodes = new HashSet<Node>();

    public Channel( Cluster cluster, ChannelName channelName ) {
        this.channelName = channelName;
        this.cluster = cluster;
    }

    /**
     * "Connect" a node to this channel
     * @param n
     */
    public void connect( Node n ) {
        if( !nodes.contains(n)) {
            nodes.add(n);
        }
        this.setChanged();
        notifyObservers();
    }

    /**
     * "Disconnect" a node from this channel
     * @param n
     */
    public void disconnect( Node n ) {
        nodes.remove(n);
        this.setChanged();
        notifyObservers();
    }

    public ChannelName getChannelName() {
        return this.channelName;
    }

    /**
     * allow users of the channel to put a message onto it.
     *
     * @param message
     */
    public void put( FlexRayFrame message ) {
        if( messageInTransit == null ) {
            messageInTransit = message;
            messageReceivedTime = cluster.getCurrentMacroTick();
            messageDeleteTime = messageReceivedTime + message.getLength();
        } else {
            long remaining = messageDeleteTime - cluster.getCurrentMacroTick();
            // create a "garbled" grame.
            long max = Math.max(remaining, message.getLength());
          //  messageInTransit = new FlexRayFrame(0, 0, 0, 0);
            messageDeleteTime = cluster.getCurrentMacroTick() + max;
        }
        this.setChanged();
        notifyObservers();
    }

    /**
     * Do the work of one macrotick...
     *
     */
    public void doMacroTick() {
        // if there is a message in transit
        if( cluster.getCurrentMacroTick() == messageDeleteTime ) {
            messageInTransit = null;
            messageReceivedTime = 0;
            messageDeleteTime = 0;
            this.setChanged();
            notifyObservers();
        }
    }

}
