/*
 * $Id: FlexRayFrame.java,v 1.1 2009/04/01 13:57:11 sirocic Exp $
 *
 * Created on 25.04.2006 16:07:32
 * by Dirk Frsterling <r@zorbla.de>
 */
package flexray.communication;

import java.util.List;
import java.util.Vector;

/**
 * An object storing the information relevant for transporting
 * some data.
 *
 * @author Dirk Frsterling <r@zorbla.de>
 */
public class FlexRayFrame {

    public static final int EMPTY_FRAME_LENGTH = 1;

    private int frameID = 0; // frame identifier = slot number
    private int messageID;
    private int  fragment = 1;
    private int tickInFrame = 0;
    private int tickInFrameMax = 0;
    private long length; // length measured in MacroTicks here.
    private int tickInMiniSlot;
    private int cycle; // cycle of frame creation
    private boolean sent = false;
    private boolean emptyFrame = true;
    private Vector<Integer> fragments = new Vector<Integer>();

    /**
     * Create a FlexRayFrame representation that "contains" a message fragment.
     * Therefore, the number of the fragment has to be given.
     *
     * @param cycle the communication cycle in which this frame was created
     * @param slot the slot for this frame
     * @param messageID a message id.
     * @param length length of the frame in minislots
     */
    public FlexRayFrame( int cycle, int slot, int messageID, long length, boolean emptyFrame, int tickInFrameMax) {
        this.cycle = cycle;
        this.frameID = slot;
        this.messageID = messageID;
        this.length = length;
        this.emptyFrame = emptyFrame;
        this.tickInFrameMax = tickInFrameMax;
        this.tickInMiniSlot = tickInFrameMax / (int)length;
    }

    public int getCycle() {
        return cycle;
    }

    public int getFrameID() {
        return frameID;
    }

    public int getFragment() {
        return fragment;
    }

    public void incrementFragment() {
        fragments.add(Integer.valueOf(tickInFrame));
        fragment++;
    }

    public List<Integer> getFragments() {
        return fragments;
    }

    public long getLength() {
        return length;
    }
    public boolean isEmptyFrame() {
        return emptyFrame;
    }
    public int getCurrentFragmentLength() {
    	int fragmentLength;
    	
        if (getFragments().size() > 0) {
        	if (tickInFrame == getFragments().get(getFragment()-2))
        		fragmentLength = tickInFrame/tickInMiniSlot;
        	else
        		fragmentLength = (tickInFrameMax-getFragments().get(getFragment()-2))/tickInMiniSlot;
        }
        else
        	fragmentLength = (int)length;
        
        return fragmentLength;
    }
    public long getMessageID() {
        return messageID;
    }


    public int getTickInFrame() {
        return tickInFrame;
    }

    public int getTickInFrameMax() {
        return tickInFrameMax;
    }
    
    /**
     * increment the tick within the frame.
     * FIXME: Assumes that the external tickInFrame
     * housekeeping is correct.
     *
     */
    public void incrementTickInFrame() {
        tickInFrame++;
    }

    public void resetTickInFrame() {
        tickInFrame = 0;
    }

    public boolean isSent() {
        return sent;
    }

    public void setSent( boolean b ) {
        sent = b;
    }

    /*
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        StringBuffer text = new StringBuffer();
        text.append("cycle " + cycle);
        text.append(", id " + frameID);
        text.append(", tick " + tickInFrame);
        text.append(", frag: " + fragment);
        text.append("/" + fragments.size());
        text.append(" (" + (sent?"":"un") + "sent)" );
        return text.toString();
    }


    public synchronized FlexRayFrame copy() {
        //DebugUtil.println("Copying " + this);
        FlexRayFrame frame = new FlexRayFrame(this.cycle,this.frameID,this.messageID,this.length, this.emptyFrame, this.tickInFrameMax);
        frame.fragment = this.fragment;
        for( Integer iFrag : fragments) frame.fragments.add(iFrag);
        frame.sent = this.sent;
        frame.tickInFrame = this.tickInFrame;
        return frame;
    }
}
