/* ChannelConfigPanelFR.java
 * 
 * Created on 21.12.2007
 * @author sirocic
 */
package flexray.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.font.FontRenderContext;
import java.awt.geom.Rectangle2D;
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JComboBox;
import javax.swing.SpinnerNumberModel;
import javax.swing.Box;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import learningVisualizations.LeviModule;
import learningVisualizations.gui.LeviFrame;


import flexray.cluster.Node;
import flexray.configuration.ChannelName;
import flexray.gui.dialogs.NodeConfigDialog.DynConfigListener;

/**
 * @author sirocic
 *
 * Class: ChannelConfigPanelFR
 *
 * Description: Diese Klasse stellte Eingabemglichkeiten fr das Verhalten eines Knotens auf
 * 				einem Kanal (A oder B) zur Verfgung. Es knnent Nachrichten in Lnge und Zeit
 * 				sowohl fr das statische Segment als auch das dynamische Segment spezifiziert werden.
 *
 * Date:	21.12.2007
 */
public class ChannelConfigPanelFR extends JPanel {

    private LeviModule module;
    private ChannelName myChannel;
    private ChannelConfigPanelFR otherChannelConfigPanel;
    private Node node;
    private int dynamicSlotNode;
    private int staticSlotNode;

    private Box contentHB, contentVB, staticVB, statFrequenzHB, dynamicVB, dynSlotLengthHB, dynFrequenzHB, requestHB;
    private Box staticSlotHB, dynamicSlotHB;
    private DynConfigListener dynConfigListener;
    
    private JPanel staticP, dynamicP;
    
    JLabel dynLengthText;
    boolean initialDynSend;
    JSpinner fromDynLengthSpinner, toDynLengthSpinner;
    JLabel toDynLengthText, mslotLengthText;

    JLabel everyStatCycleText;
    JSpinner fromStatCycleSpinner, toStatCycleSpinner;
    JLabel toStatCycleText, statCycleText;

    JLabel everyDynCycleText;
    JSpinner fromDynCycleSpinner, toDynCycleSpinner;
    JLabel toDynCycleText, dynCycleText;

    JLabel dynRequestButtonText;
    JSpinner fromDynRequestSpinner, toDynRequestSpinner;
    JLabel toDynRequestText, mslotRequestText;
    private static ImageIcon actionIcon = null;
    static {
        if( actionIcon == null ) actionIcon = loadIcon("action-node-16x16.gif");
    }

    private JLabel dynamicTitle, staticTitle;
    private JLabel staticSlotL, dynamicSlotL;
    private JComboBox staticSlotCB, dynamicSlotCB;
    private SlotCBListener slotSelectListener;
    private String[] statSlotStrings, dynSlotStrings;
	

    /*
     * Methode:		Contructors
     * 
     * Description:	
     * Date: 		 21.12.2007
     * @author        sirocic
     */
    public ChannelConfigPanelFR(LeviModule module, ChannelName channel,  Node node) {
    	super();
    	
    	int i;
    	
    	this.module=module;
    	this.myChannel= channel;
    	this.node = node;
     
        setLayout(new BorderLayout());
        setAlignmentX(Component.LEFT_ALIGNMENT);
        
        contentVB = Box.createVerticalBox();
        contentVB.setAlignmentX(Component.LEFT_ALIGNMENT);
        contentHB = Box.createHorizontalBox();
        contentHB.setAlignmentX(Component.LEFT_ALIGNMENT);
        statFrequenzHB = Box.createHorizontalBox();
        statFrequenzHB.setAlignmentX(Component.RIGHT_ALIGNMENT);
        staticVB = Box.createVerticalBox();
        staticVB.setAlignmentX(Component.LEFT_ALIGNMENT);
        dynamicVB = Box.createVerticalBox();
        dynamicVB.setAlignmentX(Component.LEFT_ALIGNMENT);
        dynSlotLengthHB = Box.createHorizontalBox();
        dynSlotLengthHB.setAlignmentX(Component.RIGHT_ALIGNMENT);
        staticSlotHB = Box.createHorizontalBox();
        staticSlotHB.setAlignmentX(Component.RIGHT_ALIGNMENT);
        dynamicSlotHB = Box.createHorizontalBox();
        dynamicSlotHB.setAlignmentX(Component.RIGHT_ALIGNMENT);
        dynFrequenzHB = Box.createHorizontalBox();
        dynFrequenzHB.setAlignmentX(Component.RIGHT_ALIGNMENT);
        requestHB = Box.createHorizontalBox();
        requestHB.setAlignmentX(Component.RIGHT_ALIGNMENT);
        
        // initialisieren der Auswahllisten fr die Slotszuordung bei den Knoten
        statSlotStrings = new String[node.getCluster().getClusterConfiguration().gNumberOfStaticSlots()+1]; 
        statSlotStrings[0] = "Kein Slot";
        for (i=1; i <= node.getCluster().getClusterConfiguration().gNumberOfStaticSlots(); i++)
    		statSlotStrings[i] = "Slot "+i;
    	dynSlotStrings = new String[node.getCluster().getClusterConfiguration().gNumberOfMinislots()+1]; 
    	dynSlotStrings[0] = "Kein Slot";
    	for (i=1; i <= node.getCluster().getClusterConfiguration().gNumberOfMinislots(); i++)
    		dynSlotStrings[i] = "Slot "+(node.getCluster().getClusterConfiguration().gNumberOfStaticSlots()+i);
    	
    	
    	slotSelectListener = new SlotCBListener();
    	
    	staticTitle = new JLabel();
    	staticTitle.setAlignmentX(Component.LEFT_ALIGNMENT);
    	
        createStaticPanel();
        
        dynamicTitle = new JLabel();
    	dynamicTitle.setAlignmentX(Component.LEFT_ALIGNMENT);
    	
        createDynamicPanel();
		
        // 	 FARBWECHSEL ...
 	    setPanelColors();

 	    // FONTWECHSEL
 	    setPanelFont();
 	  
 	    // SPRACHWECHSEL ...
 	    setPanelTexte();
        
    }
    
    /*
     * Methode:		showPanel
     * 
     * Description:	Anzeigen die aktuellen Werte.
     *    
     * Parameters:    
     * 
     * Date: 		 28.12.2007
     * @author        sirocic
     */	
    public void showPanel(boolean visible) {
       int i;
       
  	  // TRACE OUTPUT
  	  if (module.isTraceActive()) System.out.println("CALL ChannelConfigPanelFR.showPanel:");
  	  
  	  // alle Knotenparametereingaben zurcksetzen.
  	  cleanMe();
    	  
  	  if (visible) {
  		  // Panel wird sichtbar gemacht.
  		  
  		  // Gui-Elemente auf die aktuellen Kanaleigenschaften setzen ...
  		  
  		  //initialisieren der Auswahllisten fr die Slotszuordung bei den Knoten
  		  staticSlotCB.removeAllItems();
          statSlotStrings = new String[node.getCluster().getClusterConfiguration().gNumberOfStaticSlots()+1]; //get month names
          statSlotStrings[0] = "Kein Slot";
          staticSlotCB.addItem(statSlotStrings[0]);
          for (i=1; i <= node.getCluster().getClusterConfiguration().gNumberOfStaticSlots(); i++) {
      		statSlotStrings[i] = "Slot "+i;
      		staticSlotCB.addItem(statSlotStrings[i]);
          }
          dynamicSlotCB.removeAllItems();
      	  dynSlotStrings = new String[node.getCluster().getClusterConfiguration().gNumberOfMinislots()+1]; //get month names
      	  dynSlotStrings[0] = "Kein Slot";
      	  dynamicSlotCB.addItem(dynSlotStrings[0]);
      	  for (i=1; i <= node.getCluster().getClusterConfiguration().gNumberOfMinislots(); i++) {
      		dynSlotStrings[i] = "Slot "+(node.getCluster().getClusterConfiguration().gNumberOfStaticSlots()+i);
      		dynamicSlotCB.addItem(dynSlotStrings[i]);
      	  }
      	  
      	  // BERNEHMEN DER AKTUELLEN KNOTENPARAMETER
      	  // fr das statische Segment ...
      	  staticSlotNode = -1;
      	  for (i=0; i < node.getCluster().getClusterConfiguration().gNumberOfStaticSlots(); i++) {
      		if (node.isTransmittingInSlot(i)) {
      			staticSlotNode = i;
      			staticSlotCB.setSelectedIndex(i+1);
      		}
          }
      	  if (staticSlotCB.getSelectedIndex() != 0) {
      		  fromStatCycleSpinner.setEnabled(true);
      		  toStatCycleSpinner.setEnabled(true);
      		  everyStatCycleText.setEnabled(true);
      		  toStatCycleText.setEnabled(true);
      		  statCycleText.setEnabled(true);
      		  fromStatCycleSpinner.setValue(node.getSendStatCycleMin(myChannel));
      		  toStatCycleSpinner.setValue(node.getSendStatCycleMax(myChannel));
      	  }
      	  else {
      		  fromStatCycleSpinner.setEnabled(false);
    		  toStatCycleSpinner.setEnabled(false);
    		  everyStatCycleText.setEnabled(false);
      		  toStatCycleText.setEnabled(false);
      		  statCycleText.setEnabled(false);
      		  fromStatCycleSpinner.setValue(0);
    		  toStatCycleSpinner.setValue(0);
      	  }
      	  
      	  // fr das dynamische Segment ...
      	  dynamicSlotNode = -1;
      	  for (i= 0; i <= node.getCluster().getClusterConfiguration().gNumberOfMinislots(); i++) {
      		if (node.isTransmittingInSlot(node.getCluster().getClusterConfiguration().gNumberOfStaticSlots()+i)) {
      			dynamicSlotNode = node.getCluster().getClusterConfiguration().gNumberOfStaticSlots()+i;
      			dynamicSlotCB.setSelectedIndex(i+1);
      		}
           }
      	  if (dynamicSlotCB.getSelectedIndex() != 0) {
	          if( (node.getSendLengthMin(myChannel) == 1) && (node.getSendLengthMax(myChannel) == 1)) {
	              initialDynSend = false;
	          } else {
	              initialDynSend = node.isDynamicSend(myChannel);
	          }
	          fromDynLengthSpinner.setEnabled(true);
	          toDynLengthSpinner.setEnabled(true);
	          fromDynCycleSpinner.setEnabled(true);
	          toDynCycleSpinner.setEnabled(true);
	          fromDynRequestSpinner.setEnabled(true);
	          toDynRequestSpinner.setEnabled(true);
	          dynLengthText.setEnabled(true);
	          toDynLengthText.setEnabled(true);
	          mslotLengthText.setEnabled(true);
	          everyDynCycleText.setEnabled(true);
	          toDynCycleText.setEnabled(true);
	          dynCycleText.setEnabled(true);
	          dynRequestButtonText.setEnabled(true);
	          toDynRequestText.setEnabled(true);
	          mslotRequestText.setEnabled(true);
	          fromDynLengthSpinner.setValue(node.getSendLengthMin(myChannel));
	          toDynLengthSpinner.setValue(node.getSendLengthMax(myChannel));
	          fromDynCycleSpinner.setValue(node.getSendCycleMin(myChannel));
	          toDynCycleSpinner.setValue(node.getSendCycleMax(myChannel));
	          fromDynRequestSpinner.setValue(node.getDynRequestMin(myChannel));
	          toDynRequestSpinner.setValue(node.getDynRequestMax(myChannel));
      	  }
      	  else {
      		  fromDynLengthSpinner.setEnabled(false);
	          toDynLengthSpinner.setEnabled(false);
	          fromDynCycleSpinner.setEnabled(false);
	          toDynCycleSpinner.setEnabled(false);
	          fromDynRequestSpinner.setEnabled(false);
	          toDynRequestSpinner.setEnabled(false);
	          dynLengthText.setEnabled(false);
	          toDynLengthText.setEnabled(false);
	          mslotLengthText.setEnabled(false);
	          everyDynCycleText.setEnabled(false);
	          toDynCycleText.setEnabled(false);
	          dynCycleText.setEnabled(false);
	          dynRequestButtonText.setEnabled(false);
	          toDynRequestText.setEnabled(false);
	          mslotRequestText.setEnabled(false);
	          fromDynLengthSpinner.setValue(1);
	          toDynLengthSpinner.setValue(1);
	          fromDynCycleSpinner.setValue(0);
	          toDynCycleSpinner.setValue(0);
	          fromDynRequestSpinner.setValue(1);
	          toDynRequestSpinner.setValue(1);
      	  }
        
  		   contentVB.add(staticTitle);
  		   contentVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
  		   contentVB.add(staticP);
  		   contentVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
  		   contentVB.add(dynamicTitle);
  	       contentVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
 	       contentVB.add(dynamicP);
 	    
 	       contentHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
 	       contentHB.add(contentVB);
 	       contentHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
 	     
  		   add(contentHB, BorderLayout.NORTH);
  	  }

   	  // TRACE OUTPUT
   	  if (module.isTraceActive()) System.out.println("END ChannelConfigPanelFR.showPanel");

    }
    
    /*
     * Methode:		fitPanel
     * 
     * Description:	Bestimmen der bentigten Panelgre fr das Anzeigen des vollstndigen Panels. 
     *    
     * Parameters:    
     * 
     * Date: 		 28.12.2007
     * @author        sirocic
     */	
    public void fitPanel() {
    	Graphics2D g2;
	  	  int maxWidth, tmpWidth, maxHeight, maxPanelWidth, staticHeight, dynamicHeight;
	  	  int textHeight, spinnerHeight;
	  	  Rectangle2D boundsLabel;
		  FontRenderContext context;
	  	  
	  	  // TRACE OUTPUT
	  	  if (LeviModule.isTraceActive()) System.out.println("CALL ChannelConfigPanelFR.fitPanel:");
	  	
	  	  // init
	  	  g2 = (Graphics2D) getGraphics();
	  	  maxWidth = 0;
		  maxHeight =0;
		  
		  if (g2 != null){
			  // Bentigte Zeilenbreite und -hhe bestimmen anhand der Textausmasse bei aktuelle
		      // eingestellten Font ...
			  g2.setFont(getFont());
		      context = g2.getFontRenderContext();

		      //  PANEL-ELEMENTE AN AKTUELLE FONTSIZE USW ANPASSEN ...
		      validate();
		      // JSpinner wenn notwendig an Fontsize anpassen... 
		      boundsLabel = getFont().getStringBounds(toStatCycleText.getText(), context);
		      textHeight = (int) boundsLabel.getHeight();
		      if (textHeight > LeviFrame.TFHEIGHT) 
		    	  spinnerHeight = textHeight;
		      else 
		    	  spinnerHeight = LeviFrame.TFHEIGHT;
	    	  fromStatCycleSpinner.setSize(LeviFrame.NUMBERTFWIDTH, spinnerHeight);
		      toStatCycleSpinner.setSize(LeviFrame.NUMBERTFWIDTH,spinnerHeight);
		      fromDynLengthSpinner.setSize(LeviFrame.NUMBERTFWIDTH,spinnerHeight);
		      toDynLengthSpinner.setSize(LeviFrame.NUMBERTFWIDTH,spinnerHeight);
		      fromDynCycleSpinner.setSize(LeviFrame.NUMBERTFWIDTH,spinnerHeight);
		      toDynCycleSpinner.setSize(LeviFrame.NUMBERTFWIDTH,spinnerHeight);
		      fromDynRequestSpinner.setSize(LeviFrame.NUMBERTFWIDTH,spinnerHeight);
		      toDynRequestSpinner.setSize(LeviFrame.NUMBERTFWIDTH,spinnerHeight);
		      fromStatCycleSpinner.setPreferredSize(fromStatCycleSpinner.getSize());
		      fromStatCycleSpinner.setMaximumSize(fromStatCycleSpinner.getSize());
		      toStatCycleSpinner.setPreferredSize(toStatCycleSpinner.getSize());
		      toStatCycleSpinner.setMaximumSize(toStatCycleSpinner.getSize());
		      fromDynLengthSpinner.setPreferredSize(fromDynLengthSpinner.getSize());
		      fromDynLengthSpinner.setMaximumSize(fromDynLengthSpinner.getSize());
		      toDynLengthSpinner.setPreferredSize(toDynLengthSpinner.getSize());
		      toDynLengthSpinner.setMaximumSize(toDynLengthSpinner.getSize());
		      fromDynCycleSpinner.setPreferredSize(fromDynCycleSpinner.getSize());
		      fromDynCycleSpinner.setMaximumSize(fromDynCycleSpinner.getSize());
		      toDynCycleSpinner.setPreferredSize(toDynCycleSpinner.getSize());
		      toDynCycleSpinner.setMaximumSize(toDynCycleSpinner.getSize());
		      fromDynRequestSpinner.setPreferredSize(fromDynRequestSpinner.getSize());
		      fromDynRequestSpinner.setMaximumSize(fromDynRequestSpinner.getSize());
		      toDynRequestSpinner.setPreferredSize(toDynRequestSpinner.getSize());
		      toDynRequestSpinner.setMaximumSize(toDynRequestSpinner.getSize());
		      
		      boundsLabel = getFont().getStringBounds(statSlotStrings[0], context);
		      staticSlotCB.setSize((int)boundsLabel.getWidth()+LeviFrame.COMBOBOXOFFSET,spinnerHeight);
		      dynamicSlotCB.setSize((int)boundsLabel.getWidth()+LeviFrame.COMBOBOXOFFSET,spinnerHeight);
		      staticSlotCB.setPreferredSize(staticSlotCB.getSize());
		      staticSlotCB.setMaximumSize(staticSlotCB.getSize());
		      dynamicSlotCB.setPreferredSize(dynamicSlotCB.getSize());
		      dynamicSlotCB.setMaximumSize(dynamicSlotCB.getSize());
		      
		      // Breiteste Zeile der Unterpanels bestimmen  ...
		      boundsLabel = getFont().getStringBounds(staticSlotL.getText(), context);
		      maxPanelWidth =  (int) boundsLabel.getWidth(); 
		  	  maxPanelWidth =  maxPanelWidth + staticSlotCB.getWidth(); 
		  	  maxPanelWidth = 3* LeviFrame.ATTRIBUTOFFSET + maxPanelWidth;
		  	  
		      boundsLabel = getFont().getStringBounds(everyStatCycleText.getText(), context);
		      tmpWidth =  (int) boundsLabel.getWidth(); 
		  	  boundsLabel = getFont().getStringBounds(toStatCycleText.getText(), context);
		  	  tmpWidth =  tmpWidth + (int) boundsLabel.getWidth(); 
		  	  boundsLabel = getFont().getStringBounds(statCycleText.getText(), context);
		  	  tmpWidth =  tmpWidth + (int) boundsLabel.getWidth(); 
		  	  tmpWidth =  tmpWidth + fromStatCycleSpinner.getWidth() + toStatCycleSpinner.getWidth(); 
		  	  tmpWidth = 6* LeviFrame.ATTRIBUTOFFSET + tmpWidth;
		  	  if (tmpWidth > maxPanelWidth)
		  		maxPanelWidth = tmpWidth;
		  	  
		  	  boundsLabel = getFont().getStringBounds(dynamicSlotL.getText(), context);
		  	  tmpWidth =  (int) boundsLabel.getWidth(); 
		  	  tmpWidth =  tmpWidth + staticSlotCB.getWidth(); 
		  	  tmpWidth = 3* LeviFrame.ATTRIBUTOFFSET + tmpWidth;
		  	  if (tmpWidth > maxPanelWidth)
			  		maxPanelWidth = tmpWidth;
		  	  
		  	  boundsLabel = getFont().getStringBounds(toDynLengthText.getText(), context);
		      tmpWidth =  (int) boundsLabel.getWidth(); 
		  	  boundsLabel = getFont().getStringBounds(mslotLengthText.getText(), context);
		  	  tmpWidth =  tmpWidth + (int) boundsLabel.getWidth(); 
		  	  tmpWidth =  tmpWidth + dynLengthText.getWidth() + fromDynLengthSpinner.getWidth() + toDynLengthSpinner.getWidth(); 
		  	  tmpWidth = 8* LeviFrame.ATTRIBUTOFFSET + tmpWidth;
		  	  if (tmpWidth > maxPanelWidth)
		  		maxPanelWidth = tmpWidth;
		  	  
		  	  boundsLabel = getFont().getStringBounds(everyDynCycleText.getText(), context);
		      tmpWidth =  (int) boundsLabel.getWidth(); 
		  	  boundsLabel = getFont().getStringBounds(toDynCycleText.getText(), context);
		  	  tmpWidth =  tmpWidth + (int) boundsLabel.getWidth(); 
		  	  boundsLabel = getFont().getStringBounds(dynCycleText.getText(), context);
		  	  tmpWidth =  tmpWidth + (int) boundsLabel.getWidth(); 
		  	  tmpWidth =  tmpWidth + fromDynCycleSpinner.getWidth() + toDynCycleSpinner.getWidth(); 
		  	  tmpWidth = 6* LeviFrame.ATTRIBUTOFFSET + tmpWidth;
		  	  if (tmpWidth > maxPanelWidth)
		  		maxPanelWidth = tmpWidth;

		  	  boundsLabel = getFont().getStringBounds(dynRequestButtonText.getText(), context);
		      tmpWidth =  (int) boundsLabel.getWidth(); 
		  	  boundsLabel = getFont().getStringBounds(toDynRequestText.getText(), context);
		  	  tmpWidth =  tmpWidth + (int) boundsLabel.getWidth(); 
		  	  boundsLabel = getFont().getStringBounds(mslotRequestText.getText(), context);
		  	  tmpWidth =  tmpWidth + (int) boundsLabel.getWidth(); 
		  	  tmpWidth =  tmpWidth + fromDynRequestSpinner.getWidth() + toDynRequestSpinner.getWidth(); 
		  	  tmpWidth = 6* LeviFrame.ATTRIBUTOFFSET + tmpWidth;
		  	  if (tmpWidth > maxPanelWidth)
		  		maxPanelWidth = tmpWidth;
	        
		  	  // Hhe der Unterpanels bestimmen  ...
		  	  staticHeight = 2*spinnerHeight + 3 * LeviFrame.LINEOFFSET; 
		  	  dynamicHeight = 4 * spinnerHeight+ 6 * LeviFrame.LINEOFFSET; 

		  	  staticP.setSize(maxPanelWidth, staticHeight);
		  	  staticP.setMaximumSize(new Dimension(maxPanelWidth,staticHeight));
		  	  staticP.setPreferredSize(new Dimension(maxPanelWidth,staticHeight));
		  	  dynamicP.setSize(maxPanelWidth, dynamicHeight);
		  	  dynamicP.setMaximumSize(new Dimension(maxPanelWidth,dynamicHeight));
		  	  dynamicP.setPreferredSize(new Dimension(maxPanelWidth,dynamicHeight));
		  	  staticVB.setSize(maxPanelWidth, staticHeight);
		  	  staticVB.setMaximumSize(new Dimension(maxPanelWidth,staticHeight));
		  	  staticVB.setPreferredSize(new Dimension(maxPanelWidth,staticHeight));
		  	  dynamicVB.setSize(maxPanelWidth, dynamicHeight);
		  	  dynamicVB.setMaximumSize(new Dimension(maxPanelWidth,dynamicHeight));
		  	  dynamicVB.setPreferredSize(new Dimension(maxPanelWidth,dynamicHeight));
		  	
		  	  // BREITE DES PANELS BESTIMMEN ...
		      maxWidth =  maxPanelWidth + 2* LeviFrame.ATTRIBUTOFFSET; 
		      
		      // HHE DES PANELS BESTIMMEN ...
		      maxHeight = getInsets().top +  getInsets().bottom + textHeight;
		  	  maxHeight =  maxHeight + 2* textHeight + 2 * LeviFrame.LINEOFFSET + staticHeight + dynamicHeight; 
		  	  
			  // Panelgre aktualisieren ...
		  	  setSize(maxWidth,maxHeight);
		  	  setMaximumSize(new Dimension(maxWidth,maxHeight));
		  	  setPreferredSize(new Dimension(maxWidth,maxHeight));

 	  }

   	  // TRACE OUTPUT
   	  if (LeviModule.isTraceActive()) System.out.println("END ChannelConfigPanelFR.fitPanel");
   	
    }
    
    /*
     * Methode:		setDialogFont
     * 
     * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
     * 				Werte. 
     *    
     * Parameters:   Aktuelle Font fr Labels und Textfelder.
     * 
     * Date: 		 23.04.2007
     * @author        sirocic
     */	
    private void setPanelFont() {
 	   Font menuFont;
 	   
 	  menuFont = (Font)module.getConfiguration().getSettingByKey("FontMenu");
 	   
 	   setFont(menuFont);
 	  
 	   dynamicP.setFont(menuFont);
 	   staticP.setFont(menuFont);
 	   
 	   dynamicTitle.setFont(menuFont);
 	   staticTitle.setFont(menuFont);
 	   staticSlotL.setFont(menuFont);
 	   dynamicSlotL.setFont(menuFont);
 	   everyStatCycleText.setFont(menuFont);
       toStatCycleText.setFont(menuFont);
       statCycleText.setFont(menuFont);
       dynLengthText.setFont(menuFont);
       toDynLengthText.setFont(menuFont);
       mslotLengthText.setFont(menuFont);
       toDynCycleText.setFont(menuFont);
       dynCycleText.setFont(menuFont);
       everyDynCycleText.setFont(menuFont);
       dynRequestButtonText.setFont(menuFont);
       toDynRequestText.setFont(menuFont);
       mslotRequestText.setFont(menuFont);
       
       fromStatCycleSpinner.setFont(menuFont);
       toStatCycleSpinner.setFont(menuFont);
       fromDynLengthSpinner.setFont(menuFont);
       toDynLengthSpinner.setFont(menuFont);
       fromDynCycleSpinner.setFont(menuFont);
       toDynCycleSpinner.setFont(menuFont);
       fromDynRequestSpinner.setFont(menuFont);
       toDynRequestSpinner.setFont(menuFont);
       staticSlotCB.setFont(menuFont);
       dynamicSlotCB.setFont(menuFont);
 	   	  
    }
    
    /*
     * Methode:		setDialogTexte
     * 
     * Description:	Setz alle Text-Bausteine des Dialoges neu. Wichtig fr Sprachwechsel.
     *    
     * Parameters:   
     * 
     * Date: 		 23.04.2007
     * @author        sirocic
     */	
    private void setPanelTexte() {
    
        staticTitle.setText(module.getRessources().getResourceByKey("dlg.staticSegment"));
        dynamicTitle.setText(module.getRessources().getResourceByKey("dlg.dynamicSegment"));
        
        staticSlotL.setText("" + module.getRessources().getResourceByKey("dlg.nodeconfig.statSlotID"));
        dynamicSlotL.setText("" + module.getRessources().getResourceByKey("dlg.nodeconfig.dynSlotID"));
        everyStatCycleText.setText("" + module.getRessources().getResourceByKey("dlg.nodeconfig.every"));
        toStatCycleText.setText("" + module.getRessources().getResourceByKey("dlg.nodeconfig.dynsendto"));
        statCycleText.setText("" + module.getRessources().getResourceByKey("dlg.nodeconfig.cycles"));
        dynLengthText.setText(" " + module.getRessources().getResourceByKey("dlg.nodeconfig.dynsendframesfrom"));
        toDynLengthText.setText("" + module.getRessources().getResourceByKey("dlg.nodeconfig.dynsendto"));
        mslotLengthText.setText("" + module.getRessources().getResourceByKey("dlg.commcycle.minislots"));
        everyDynCycleText.setText("" + module.getRessources().getResourceByKey("dlg.nodeconfig.every"));
        toDynCycleText.setText("" + module.getRessources().getResourceByKey("dlg.nodeconfig.dynsendto"));
        dynCycleText.setText("" + module.getRessources().getResourceByKey("dlg.nodeconfig.cycles"));
        dynRequestButtonText.setText("" + module.getRessources().getResourceByKey("dlg.nodeconfig.requestlength"));
        toDynRequestText.setText("" + module.getRessources().getResourceByKey("dlg.nodeconfig.requestto"));
        mslotRequestText.setText("" + module.getRessources().getResourceByKey("dlg.commcycle.minislots"));
    }
    
    /*
     * Methode:		setDialogColors
     * 
     * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
     * 				Werte. 
     *    
     * Parameters:   Aktuelle Farbwerte fr Hintergrund und Vordergrund.
     * 
     * Date: 		 23.04.2007
     * @author        sirocic
     */	
    private void setPanelColors() {
 	   Color buttonColor;
 	   Color backgroundColor,  foregroundColor;
		 
	   if (module.getConfiguration().getSettingByKey("ColorComponentBackground")!=null)
	   {
		   backgroundColor = (Color)module.getConfiguration().getSettingByKey("ColorComponentBackground");
		   foregroundColor = (Color)module.getConfiguration().getSettingByKey("ColorComponentForeground");
	   }
	   else {
		   backgroundColor = (Color)module.getConfiguration().getSettingByKey("ColorModuleBackground");
		   foregroundColor = (Color)module.getConfiguration().getSettingByKey("ColorModuleForeground");
	   }
 	   buttonColor = (Color)module.getConfiguration().getSettingByKey("ColorButtonBackground");
 	  
 	   // 	 Hintergrund und Zeichenfarbe auf aktuelle Werte setzen
       setBackground(backgroundColor);
 	   setForeground(foregroundColor);
 	   
 	   dynLengthText.setForeground(foregroundColor);
 	   dynLengthText.setBackground(backgroundColor);
 	  
 	   everyStatCycleText.setForeground(foregroundColor);
       toStatCycleText.setForeground(foregroundColor);
       statCycleText.setForeground(foregroundColor);
       dynLengthText.setForeground(foregroundColor);
       toDynLengthText.setForeground(foregroundColor);
       mslotLengthText.setForeground(foregroundColor);
       everyDynCycleText.setForeground(foregroundColor);
       toDynCycleText.setForeground(foregroundColor);
       dynCycleText.setForeground(foregroundColor);
       dynRequestButtonText.setForeground(foregroundColor);
       toDynRequestText.setForeground(foregroundColor);
       mslotRequestText.setForeground(foregroundColor);
       staticTitle.setForeground(foregroundColor);
       dynamicTitle.setForeground(foregroundColor);
       staticSlotL.setForeground(foregroundColor);
       dynamicSlotL.setForeground(foregroundColor);
      
       fromStatCycleSpinner.setForeground(foregroundColor);
       toStatCycleSpinner.setForeground(foregroundColor);
       fromDynLengthSpinner.setForeground(foregroundColor);
       toDynLengthSpinner.setForeground(foregroundColor);
       fromDynCycleSpinner.setForeground(foregroundColor);
       toDynCycleSpinner.setForeground(foregroundColor);
       fromDynRequestSpinner.setForeground(foregroundColor);
       toDynRequestSpinner.setForeground(foregroundColor);
       staticSlotCB.setForeground(foregroundColor);
       dynamicSlotCB.setForeground(foregroundColor);
       
       fromStatCycleSpinner.setBackground(backgroundColor);
       toStatCycleSpinner.setBackground(backgroundColor);
       fromDynLengthSpinner.setBackground(backgroundColor);
       toDynLengthSpinner.setBackground(backgroundColor);
       fromDynCycleSpinner.setBackground(backgroundColor);
       toDynCycleSpinner.setBackground(backgroundColor);
       fromDynRequestSpinner.setBackground(backgroundColor);
       toDynRequestSpinner.setBackground(backgroundColor);
       staticSlotCB.setBackground(backgroundColor);
       dynamicSlotCB.setBackground(backgroundColor);
       
       staticP.setForeground(foregroundColor);
       dynamicP.setForeground(foregroundColor);
       staticP.setBackground(buttonColor);
       dynamicP.setBackground(buttonColor);
    }
    
    /*
     * Methode:		updateColorLanguageFonts
     * 
     * Description:	Setzt die aktuellen Farben aus der Konfigurationbzw. und Texte aus den Ressourcen.
     * 				Wird aufgerufen in erster Linie nach einem dynamischen Sprach- oder Farbwechsel.
     *    
     * Date: 		16.03.2007
     * @author      sirocic
     */	
    public void updateColorLanguageFonts() {
     
        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("CALL ChannelConfigPanelFR.updateColorLanguageFonts:");

        // SUBKOMPONENTE UPDATEN...

        // 	 FARBWECHSEL ...
 	   setPanelColors();

 	   // FONTWECHSEL
 	   setPanelFont();
 	  
 	   // SPRACHWECHSEL ...
 	   setPanelTexte();
 	   
        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("END ChannelConfigPanelFR.updateColorLanguageFonts:");

    }
    
   
   /*
    * Methode:		cleanMe
    * 
    * Description:	Setzt alle Eingabe- und Auswahlfelder zurck auf Defaultwerte.
    *    
    * Parameters:   
    * 
    * Date: 		 28.12.2007
    * @author        sirocic
    */	
   private void cleanMe() {

	   removeAll();
	   contentVB.removeAll();
	   
   }
   
    /**
     * create a configuration pane for the static segment behaviour
     * @param channel
     * @return
     */
    private void createStaticPanel() {
    	
        /*
         * configuration of pause between messages
         */
    	staticP = new JPanel();
    	staticP.setAlignmentX(Component.LEFT_ALIGNMENT);
    	staticP.setLayout(new BorderLayout());
    	staticSlotL = new JLabel();
    	staticSlotL.setAlignmentX(Component.LEFT_ALIGNMENT);
    	staticSlotCB = new JComboBox(statSlotStrings);
    	staticSlotCB.setAlignmentX(Component.LEFT_ALIGNMENT);
    	staticSlotCB.addActionListener(slotSelectListener);
        everyStatCycleText = new JLabel();
        everyStatCycleText.setAlignmentX(Component.LEFT_ALIGNMENT);
        fromStatCycleSpinner = new JSpinner(new SpinnerNumberModel(0,0,20,1));
        fromStatCycleSpinner.addChangeListener(slotSelectListener);
        fromStatCycleSpinner.setAlignmentX(Component.LEFT_ALIGNMENT);
        toStatCycleText = new JLabel();
        toStatCycleText.setAlignmentX(Component.LEFT_ALIGNMENT);
        toStatCycleSpinner = new JSpinner(new SpinnerNumberModel(0,0,20,1));
        toStatCycleSpinner.addChangeListener(slotSelectListener);
        toStatCycleSpinner.setAlignmentX(Component.LEFT_ALIGNMENT);
        statCycleText = new JLabel();
        statCycleText.setAlignmentX(Component.LEFT_ALIGNMENT);

        staticSlotHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        staticSlotHB.add(staticSlotL);
        staticSlotHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        staticSlotHB.add(staticSlotCB);
        staticSlotHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        
        statFrequenzHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        statFrequenzHB.add(everyStatCycleText);
        statFrequenzHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        statFrequenzHB.add(fromStatCycleSpinner);
        statFrequenzHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        statFrequenzHB.add(toStatCycleText);
        statFrequenzHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        statFrequenzHB.add(toStatCycleSpinner);
        statFrequenzHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        statFrequenzHB.add(statCycleText);
        statFrequenzHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
	    
        //staticVB.add(staticTitle);
        staticVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
        staticVB.add(staticSlotHB);
        staticVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
        staticVB.add(statFrequenzHB);
        
        staticP.add(staticVB, BorderLayout.NORTH);

    } // createStatBehaviourPane

    /**
     * Create the panel containing the configuration controls for the dynamic
     * segment sending behaviour
     * one channel.
     * @param channel
     * @return JPanel with the dynamic segment behaviour configuration
     */
    private void createDynamicPanel() {
 
        /*
         * configuration of the message length
         */
    	dynamicP = new JPanel();
    	dynamicP.setAlignmentX(Component.LEFT_ALIGNMENT);
    	dynamicP.setLayout(new BorderLayout());
    	dynamicSlotL = new JLabel();
    	dynamicSlotL.setAlignmentX(Component.LEFT_ALIGNMENT);
    	dynamicSlotCB = new JComboBox(dynSlotStrings);
    	dynamicSlotCB.setAlignmentX(Component.LEFT_ALIGNMENT);
    	dynamicSlotCB.addActionListener(slotSelectListener);
        dynLengthText = new JLabel();
        dynLengthText.setAlignmentX(Component.LEFT_ALIGNMENT);

        fromDynLengthSpinner = new JSpinner(new SpinnerNumberModel(1,0,20,1));
        fromDynLengthSpinner.addChangeListener(slotSelectListener);
        fromDynLengthSpinner.setAlignmentX(Component.LEFT_ALIGNMENT);
        toDynLengthText = new JLabel();
        toDynLengthText.setAlignmentX(Component.LEFT_ALIGNMENT);
        toDynLengthSpinner = new JSpinner(new SpinnerNumberModel(1,0,20,1));
        toDynLengthSpinner.addChangeListener(slotSelectListener);
        toDynLengthSpinner.setAlignmentX(Component.LEFT_ALIGNMENT);
        mslotLengthText = new JLabel();
        mslotLengthText.setAlignmentX(Component.LEFT_ALIGNMENT);

        dynamicSlotHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        dynamicSlotHB.add(dynamicSlotL);
        dynamicSlotHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        dynamicSlotHB.add(dynamicSlotCB);
        dynamicSlotHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        
        dynSlotLengthHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        dynSlotLengthHB.add(dynLengthText);
        dynSlotLengthHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        dynSlotLengthHB.add(fromDynLengthSpinner);
        dynSlotLengthHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        dynSlotLengthHB.add(toDynLengthText);
        dynSlotLengthHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        dynSlotLengthHB.add(toDynLengthSpinner);
        dynSlotLengthHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        dynSlotLengthHB.add(mslotLengthText);
        dynSlotLengthHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));

        /*
         * configuration of pause between messages
         */
        everyDynCycleText = new JLabel();
        everyDynCycleText.setAlignmentX(Component.LEFT_ALIGNMENT);
        fromDynCycleSpinner = new JSpinner(new SpinnerNumberModel(0,0,20,1));
        fromDynCycleSpinner.addChangeListener(slotSelectListener);
        fromDynCycleSpinner.setAlignmentX(Component.LEFT_ALIGNMENT);
        toDynCycleText = new JLabel();
        toDynCycleText.setAlignmentX(Component.LEFT_ALIGNMENT);
        toDynCycleSpinner = new JSpinner(new SpinnerNumberModel(0,0,20,1));
        toDynCycleSpinner.addChangeListener(slotSelectListener);
        toDynCycleSpinner.setAlignmentX(Component.LEFT_ALIGNMENT);
        dynCycleText = new JLabel();
        dynCycleText.setAlignmentX(Component.LEFT_ALIGNMENT);

        dynFrequenzHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        dynFrequenzHB.add(everyDynCycleText);
        dynFrequenzHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        dynFrequenzHB.add(fromDynCycleSpinner);
        dynFrequenzHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        dynFrequenzHB.add(toDynCycleText);
        dynFrequenzHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        dynFrequenzHB.add(toDynCycleSpinner);
        dynFrequenzHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        dynFrequenzHB.add(dynCycleText);
        dynFrequenzHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));

        /*
         * configuration of the request message button
         */

        dynRequestButtonText = new JLabel(actionIcon);
        dynRequestButtonText.setAlignmentX(Component.LEFT_ALIGNMENT);
        fromDynRequestSpinner = new JSpinner(new SpinnerNumberModel(1,0,20,1));
        fromDynRequestSpinner.addChangeListener(slotSelectListener);
        fromDynRequestSpinner.setAlignmentX(Component.LEFT_ALIGNMENT);
        toDynRequestText= new JLabel();
        toDynRequestText.setAlignmentX(Component.LEFT_ALIGNMENT);
        toDynRequestSpinner = new JSpinner(new SpinnerNumberModel(1,0,20,1));
        toDynRequestSpinner.addChangeListener(slotSelectListener);
        toDynRequestSpinner.setAlignmentX(Component.LEFT_ALIGNMENT);
        mslotRequestText = new JLabel();
        mslotRequestText.setAlignmentX(Component.LEFT_ALIGNMENT);

        requestHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        requestHB.add(dynRequestButtonText);
        requestHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        requestHB.add(fromDynRequestSpinner);
        requestHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        requestHB.add(toDynRequestText);
        requestHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        requestHB.add(toDynRequestSpinner);
        requestHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        requestHB.add(mslotRequestText);
        requestHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));

        dynamicVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
        dynamicVB.add(dynamicSlotHB);
        dynamicVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
        dynamicVB.add(dynFrequenzHB);
        dynamicVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
        dynamicVB.add(dynSlotLengthHB);
        dynamicVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
        dynamicVB.add(requestHB);
        dynamicVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));

        dynamicP.add(dynamicVB, BorderLayout.NORTH);
        
    } // createDynBehaviourPane

    /**
     * Add a configuration listener for the dynamic segment configuration
     * @param channel
     */
    private void addDynConfigListener(DynConfigListener dynConfigListener ) {
        
        fromDynLengthSpinner.addChangeListener(dynConfigListener);
        toDynLengthSpinner.addChangeListener(dynConfigListener);
        fromDynCycleSpinner.addChangeListener(dynConfigListener);
        toDynCycleSpinner.addChangeListener(dynConfigListener);
        fromStatCycleSpinner.addChangeListener(dynConfigListener);
        toStatCycleSpinner.addChangeListener(dynConfigListener);
        fromDynRequestSpinner.addChangeListener(dynConfigListener);
        toDynRequestSpinner.addChangeListener(dynConfigListener);
    } // addDynConfigListener

    public void setDynConfigListener(DynConfigListener dynConfigListener) {
        this.dynConfigListener = dynConfigListener;
        addDynConfigListener(this.dynConfigListener);
    }
 
    public void setValues(ActionEvent e) {
	    /*
	     * Spinners for cycle pause
	     */
	
	    int fromCycleValue = getValueFromDynCycleSpinner();
	    int toCycleValue = getValueToDynCycleSpinner();
	    
	    if( e.getSource() == fromDynCycleSpinner ) {
	        if(fromCycleValue > toCycleValue) {
	            toDynCycleSpinner.setValue(Integer.valueOf(fromCycleValue));
	            return;
	        }
	    } // source = fromDynCycleSpinner
	
	    if( e.getSource() == toDynCycleSpinner ) {
	        if(toCycleValue < fromCycleValue) {
	            fromDynCycleSpinner.setValue(Integer.valueOf(toCycleValue));
	            return;
	        }
	    } // source = toDynCycleSpinner

	    // same in static
	
	    int fromStatCycleValue = getValueFromStatCycleSpinner();
	    int toStatCycleValue = getValueToStatCycleSpinner();
	
	    if( e.getSource() == fromStatCycleSpinner ) {
	        if(fromStatCycleValue > toStatCycleValue) {
	            toStatCycleSpinner.setValue(Integer.valueOf(fromStatCycleValue));
	            return;
	        }
	    } // source = fromStatCycleSpinner
	
	    if( e.getSource() == toStatCycleSpinner ) {
	        if(toStatCycleValue < fromStatCycleValue) {
	            fromStatCycleSpinner.setValue(Integer.valueOf(toStatCycleValue));
	            return;
	        }
	    } // source = toStatCycleSpinner
	
	    /*
	     * Spinners for request button
	     */
	
	    int fromRequestValue = getValueFromDynRequestSpinner();
	    int toRequestValue = getValueToDynRequestSpinner();
	
	    if( e.getSource() == fromDynRequestSpinner ) {
	        if(fromRequestValue > toRequestValue) {
	            toDynRequestSpinner.setValue(Integer.valueOf(fromRequestValue));
	            return;
	        }
	    } // source = fromRequestSpinner
	
	    if( e.getSource() == toDynRequestSpinner ) {
	        if(toRequestValue < fromRequestValue) {
	            fromDynRequestSpinner.setValue(Integer.valueOf(toRequestValue));
	            return;
	        }
	    } // source = toRequestSpinner
	
	
	    /*
	     * Spinners for message length and the dynamic send active checkbox
	     */
	    int fromLengthValue = getValueFromDynLengthSpinner();
	    int toLengthValue = getValueToDynLengthSpinner();
	
	    if( e.getSource() == fromDynLengthSpinner ) {
	        if(fromLengthValue > toLengthValue) {
	            toDynLengthSpinner.setValue(Integer.valueOf(fromLengthValue));
	            toLengthValue = fromLengthValue; // for the "if"
	        }
	    } // source = fromDynLengthSpinner
	
	    if( e.getSource() == toDynLengthSpinner ) {
	        if(toLengthValue < fromLengthValue) {
	            fromDynLengthSpinner.setValue(Integer.valueOf(toLengthValue));
	            fromLengthValue = toLengthValue; // for the "if"
	        }
	    } // source = toDynLengthSpinner
	
    }
    
    // Methoden zum Auslesen der Parametereinstellungen fr den Kanal
    public boolean isSelectedDynSend() {
    	return (dynamicSlotCB.getSelectedIndex()!=0);
    }
    public int getValueFromDynLengthSpinner() {
    	return ((Integer)fromDynLengthSpinner.getValue()).intValue();
    }
    public int getValueToDynLengthSpinner() {
    	return ((Integer)toDynLengthSpinner.getValue()).intValue();
    }
    public int getValueFromDynCycleSpinner() {
    	return ((Integer)fromDynCycleSpinner.getValue()).intValue();
    }
    public void setValueFromDynCycleSpinner(int index) {
    	fromDynCycleSpinner.setValue(index);
    }
    public int getValueToDynCycleSpinner() {
    	return ((Integer)toDynCycleSpinner.getValue()).intValue();
    }
    public void setValueToDynCycleSpinner(int index) {
    	toDynCycleSpinner.setValue(index);
    }
    public void setValueFromStatCycleSpinner(int index) {
    	fromStatCycleSpinner.setValue(index);
    }
    public void setValueToStatCycleSpinner(int index) {
    	toStatCycleSpinner.setValue(index);
    }
    
    public int getValueFromStatCycleSpinner() {
    	return ((Integer)fromStatCycleSpinner.getValue()).intValue();
    }
    public int getValueToStatCycleSpinner() {
    	return ((Integer)toStatCycleSpinner.getValue()).intValue();
    }
    public int getStaticSlotNode() {
    	return staticSlotNode;
    }
    public int getDynamicSlotNode() {
    	return dynamicSlotNode;
    }
    public int getValueSelectedStaticSlot() {
    	return staticSlotCB.getSelectedIndex()-1;
    }
    public int getValueSelectedDynamicSlot() {
    	if (dynamicSlotCB.getSelectedIndex()!=0)
    		return node.getCluster().getClusterConfiguration().gNumberOfStaticSlots()+dynamicSlotCB.getSelectedIndex()-1;
  
    	return dynamicSlotCB.getSelectedIndex()-1;
    }
    public void setValueSelectedDynamicSlot(int index) {
    	dynamicSlotCB.setSelectedIndex(index);
    }
    
    
	public void setValueSelectedStaticSlot(int index) {
    	staticSlotCB.setSelectedIndex(index);
    }
	public void setOtherChannelConfigPanel(ChannelConfigPanelFR otherChannelConfigPanel) {
		this.otherChannelConfigPanel = otherChannelConfigPanel;
	}
    public int getValueFromDynRequestSpinner() {
    	return ((Integer)fromDynRequestSpinner.getValue()).intValue();
    }
    public int getValueToDynRequestSpinner() {
    	return ((Integer)toDynRequestSpinner.getValue()).intValue();
    }
    
    public JSpinner getFromDynCycleSpinner() {
    	return fromDynCycleSpinner;
    }
    public JSpinner getToCycleSpinner() {
    	return toDynCycleSpinner;
    }
    /*
     * load an icon
     */
    private static ImageIcon loadIcon( String name) {
        ImageIcon icon = null;
        URL u = NodeComponent.class.getResource("/flexray/gui/icons/" + name);
        if( u != null ) icon = new ImageIcon(u);
        else icon = new ImageIcon(new byte[1],"image not found");
        return icon;
    }
    
    /**
     * Beobachtete die CheckBoxen fr die Slotauswahl. 
     *
     * @author Birgit Sirocic
     */
    public class SlotCBListener implements ActionListener, ChangeListener  {

        public SlotCBListener() {
		}
        
        /**
         * changeListener part. Dispatch this to actionPerformed()
         * @param e
         * @see javax.swing.event.ChangeListener#stateChanged(javax.swing.event.ChangeEvent)
         */
        public void stateChanged( ChangeEvent e ) {
            ActionEvent ae = new ActionEvent(e.getSource(),0,"stateChange");
            actionPerformed(ae);
        }

        /**
         * Setzt den neu ausgewhlten Slot als Kommunikationsslot fr den Knoten
         */
        public void actionPerformed(ActionEvent e) {
            if( e.getSource() == staticSlotCB) {
                if (staticSlotCB.getSelectedIndex() == 0) {
                	fromStatCycleSpinner.setEnabled(false);
          		  	toStatCycleSpinner.setEnabled(false);
          		  	everyStatCycleText.setEnabled(false);
            		toStatCycleText.setEnabled(false);
            		statCycleText.setEnabled(false);
                }
                else {
                	fromStatCycleSpinner.setEnabled(true);
          		  	toStatCycleSpinner.setEnabled(true);
          		  	everyStatCycleText.setEnabled(true);
            		toStatCycleText.setEnabled(true);
            		statCycleText.setEnabled(true);
                }
                // Auswahlbox fr den jeweils anderen Kanal muss im auch gesetzt werden
                otherChannelConfigPanel.setValueSelectedStaticSlot(staticSlotCB.getSelectedIndex());
            }
           
            if( e.getSource() == fromStatCycleSpinner) {
                
                // fromStatCycleSpinner fr den jeweils anderen Kanal muss im auch gesetzt werden
                otherChannelConfigPanel.setValueFromStatCycleSpinner(((Integer)fromStatCycleSpinner.getValue()).intValue());
            }
            
            if( e.getSource() == toStatCycleSpinner) {
                
                // fromStatCycleSpinner fr den jeweils anderen Kanal muss im auch gesetzt werden
            	otherChannelConfigPanel.setValueToStatCycleSpinner(((Integer)toStatCycleSpinner.getValue()).intValue());
            }
            
            if( e.getSource() == dynamicSlotCB) {
            	if (dynamicSlotCB.getSelectedIndex() == 0) {
            	  fromDynLengthSpinner.setEnabled(false);
      	          toDynLengthSpinner.setEnabled(false);
      	          fromDynCycleSpinner.setEnabled(false);
      	          toDynCycleSpinner.setEnabled(false);
      	          fromDynRequestSpinner.setEnabled(false);
      	          toDynRequestSpinner.setEnabled(false);
      	          dynLengthText.setEnabled(false);
      	          toDynLengthText.setEnabled(false);
      	          mslotLengthText.setEnabled(false);
      	          everyDynCycleText.setEnabled(false);
      	          toDynCycleText.setEnabled(false);
      	          dynCycleText.setEnabled(false);
      	          dynRequestButtonText.setEnabled(false);
      	          toDynRequestText.setEnabled(false);
      	          mslotRequestText.setEnabled(false);
                }
                else {
                  fromDynLengthSpinner.setEnabled(true);
      	          toDynLengthSpinner.setEnabled(true);
      	          fromDynCycleSpinner.setEnabled(true);
      	          toDynCycleSpinner.setEnabled(true);
      	          fromDynRequestSpinner.setEnabled(true);
      	          toDynRequestSpinner.setEnabled(true);
      	          dynLengthText.setEnabled(true);
      	          toDynLengthText.setEnabled(true);
      	          mslotLengthText.setEnabled(true);
      	          everyDynCycleText.setEnabled(true);
      	          toDynCycleText.setEnabled(true);
      	          dynCycleText.setEnabled(true);
      	          dynRequestButtonText.setEnabled(true);
      	          toDynRequestText.setEnabled(true);
      	          mslotRequestText.setEnabled(true);
                }
            	// Auswahlbox fr den jeweils anderen Kanal muss im auch gesetzt werden
                otherChannelConfigPanel.setValueSelectedDynamicSlot(dynamicSlotCB.getSelectedIndex());
            }
            
        } // actionPerformed
        
    } // SlotCBListener
}
