/*
 * $Id: FlexRayStatusBar.java,v 1.1 2009/04/01 13:57:09 sirocic Exp $
 *
 * Created on 10.05.2006 10:28:47
 * by Dirk Frsterling <r@zorbla.de>
 */
package flexray.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Locale;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.BevelBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import learningVisualizations.LeviResourceManager;

import flexray.ModuleFR;
import flexray.cluster.Cluster;
import flexray.configuration.ChannelName;
import flexray.levi.help.FRHelp;
import flexray.scenario.FlexRayScenario;
import flexray.util.DebugUtil;

/**
 * This status bar should display useful information to the user like
 * - number of nodes in current scenario
 * - modification status of the current scenario
 *
 *
 * @author Dirk Frsterling <r@zorbla.de>
 *
 */
public class FlexRayStatusBar extends JComponent
                           implements MouseListener,
                                      PropertyChangeListener,
                                      ChangeListener {

    private final ModuleFR learningModule;
    private final LeviResourceManager res;
    private final JPanel mainPanel;
    private final JLabel currentMT;
    private final JLabel currentCC;
    private final JLabel currentSlots;
    private final JLabel saveStatusLabel;
    private final JLabel nodeLabel;
    private final JLabel tutorialLabel;
    private final MemoryBar memoryBar;


    /**
     * Spacer - A component to display 3D lines between
     * components of the Statusbar
     *
     * @author Dirk Frsterling <r@zorbla.de>
     */
    private class Spacer extends JComponent {
        private static final long serialVersionUID = 6999704870757006252L;
        public Spacer() {
            Dimension dimension = new Dimension(6,18);
            setPreferredSize(dimension);
            setMaximumSize(dimension);
            setMinimumSize(dimension);
        }
        @Override
        protected void paintComponent(Graphics g) {
            super.paintComponent(g);
            g.setColor(getBackground());
            g.fill3DRect(2, 1, 2, 17, false);
        }
    } // class spacer

    /**
     * Create a status bar for the given FlexRay module.
     *
     * @param pModule
     */
    public FlexRayStatusBar(final ModuleFR pModule) {
        super();
        learningModule = pModule;

        res = learningModule.getRessources();
        FlexRayScenario scenario = (FlexRayScenario)learningModule.getScenario();

        mainPanel = new JPanel( new GridBagLayout() );

        GridBagConstraints gbc = new GridBagConstraints();
        gbc.fill = GridBagConstraints.NONE;
        gbc.anchor = GridBagConstraints.LINE_START;

        // This is unsafe.
        Font statusBarFont = mainPanel.getFont().deriveFont(Font.PLAIN).deriveFont(10f);

        /*
         * Macrotick label
         */
        currentMT = new JLabel();
        currentMT.setFont( statusBarFont );
        updateMacroTickLabel(scenario);
        currentMT.addMouseListener( this );
        gbc.weightx = 0;
        gbc.gridx = 0;
        mainPanel.add(currentMT,gbc);


        gbc.weightx = 0;
        gbc.gridx = 1;
        mainPanel.add(new Spacer(),gbc);

        /*
         * Cycle label
         */
        currentCC = new JLabel();
        currentCC.setFont( statusBarFont );
        updateCycleCounterLabel(scenario);
        currentCC.addMouseListener( this );
        gbc.weightx = 0;
        gbc.gridx = 2;
        mainPanel.add(currentCC,gbc);


        gbc.weightx = 0;
        gbc.gridx = 3;
        mainPanel.add(new Spacer(),gbc);

        /*
         * Slots label
         */
        currentSlots = new JLabel();
        currentSlots.setFont( statusBarFont );
        updateSlotsLabel(scenario);
        currentSlots.addMouseListener( this );
        gbc.weightx = 0;
        gbc.gridx = 4;
        mainPanel.add(currentSlots,gbc);


        gbc.weightx = 1;
        gbc.gridx = 5;
        mainPanel.add(new Spacer(),gbc);

        gbc.weightx = 0;
        gbc.gridx = 6;
        mainPanel.add(new Spacer(),gbc);

        /*
         * Node Label
         */
        nodeLabel = new JLabel();
        nodeLabel.setFont(statusBarFont);
        updateNodeLabel(scenario);
        gbc.weightx = 0;
        gbc.gridx = 7;
        gbc.anchor = GridBagConstraints.LINE_END;
        mainPanel.add(nodeLabel,gbc);

        gbc.weightx = 0;
        gbc.gridx = 8;
        mainPanel.add(new Spacer(),gbc);

        /*
         * Tutorial indicator
         */
        tutorialLabel = new JLabel();
        tutorialLabel.setFont(statusBarFont);
        updateTutorialLabel();
        gbc.weightx = 0;
        gbc.gridx = 9;
        gbc.anchor = GridBagConstraints.LINE_END;
        mainPanel.add(tutorialLabel,gbc);

        gbc.weightx = 0;
        gbc.gridx = 10;
        mainPanel.add(new Spacer(),gbc);



        /*
         * "Save" status
         */
        JPanel saveStatusPanel = new JPanel(new BorderLayout());
        saveStatusLabel = new JLabel();
        saveStatusLabel.setFont(statusBarFont);
        updateSaveStatus(scenario);
        saveStatusPanel.add(saveStatusLabel,BorderLayout.CENTER);
        gbc.weightx = 0;
        gbc.gridx = 11;
        mainPanel.add(saveStatusPanel,gbc);

        /*
         * spacer
         */
        gbc.weightx = 0;
        gbc.gridx = 12;
        mainPanel.add(new Spacer(),gbc);

        /*
         * A memory status bar
         */
        JPanel memoryPanel = new JPanel( new BorderLayout() );
        JLabel j = new JLabel(" Heap: ");
        j.setFont(statusBarFont);
        memoryPanel.add(j,BorderLayout.WEST);
        memoryBar = new MemoryBar();
        memoryBar.setAutonomous(true);
        memoryBar.setWidth(100);
        memoryBar.setMaximumSize(new Dimension(100,16));
        memoryBar.setMinimumSize(new Dimension(100,16));
        memoryPanel.add(memoryBar,BorderLayout.CENTER);

        gbc.weightx = 0;
        gbc.gridx = 13;
        mainPanel.add(memoryPanel, gbc);

        /*
         * Add a vertical spacing element
         */
        JLabel space = new JLabel("");
        space.setSize(space.getPreferredSize());
        space.setPreferredSize( new Dimension(space.getWidth(),20) );
        space.setSize(space.getPreferredSize());
        gbc.weightx = 0;
        gbc.gridx = 14;
        gbc.anchor = GridBagConstraints.EAST;
        mainPanel.add(space,gbc);

        setLayout( new BorderLayout() );
        setBorder( BorderFactory.createBevelBorder(BevelBorder.LOWERED) );
        add(mainPanel, BorderLayout.CENTER);

    }

    /*
     * --------------------- Updates ---------------------------
     */


    /**
     * Update the node label
     * @param scenario
     */
    private void updateNodeLabel(final FlexRayScenario scenario) {
        if( scenario != null ) {
            nodeLabel.setText(res.getResourceByKey("flexray.nodes") + ": " +
            		scenario.getCluster().getNumberOfNodes() + " ");
        } else {
            nodeLabel.setText(res.getResourceByKey("flexray.nodes") + ": - ");
        }
    }

    /**
     * Update simulation information
     * @param scenario
     */
    public void updateMacroTick(final FlexRayScenario scenario) {
        if( scenario == null) return;
        updateMacroTickLabel(scenario);
        updateCycleCounterLabel(scenario);
        updateSlotsLabel(scenario);
    }

    private void updateMacroTickLabel(final FlexRayScenario scenario ) {
        if(scenario == null) return;
        Cluster mcluster = scenario.getCluster();
        final StringBuffer macroTicks = new StringBuffer();
        macroTicks.append(res.getResourceByKey("flexray.macrotick") + ": ");
        if( mcluster != null ) {
            macroTicks.append( String.format(" %5d ", Long.valueOf(mcluster.getCurrentMacroTick() % (mcluster.getClusterConfiguration().gMacroPerCycle())) ));
        } else {
            macroTicks.append(String.format(" %5d ", Integer.valueOf(0)));
        }
        currentMT.setText(macroTicks.toString());
    }

    private void updateCycleCounterLabel(final FlexRayScenario scenario ) {
        if(scenario == null) return;
        Cluster mcluster = scenario.getCluster();
        final StringBuffer cycle = new StringBuffer();
        cycle.append(res.getResourceByKey("flexray.commcycle") + ": ");
        if( mcluster != null ) {
            cycle.append( String.format(" %5d ", Long.valueOf(mcluster.getCycleCounter() % (mcluster.getClusterConfiguration().cCycleCountMax)+1) ));
        } else {
            cycle.append(String.format(" %5d ", Integer.valueOf(0)));
        }
        currentCC.setText(cycle.toString());
    }

    private void updateSlotsLabel(final FlexRayScenario scenario) {
        if(scenario == null) return;
        Cluster mcluster = scenario.getCluster();
        final StringBuffer slots = new StringBuffer();
        slots.append(res.getResourceByKey("flexray.slots") + ": ");
        if( mcluster != null ) {
            slots.append(" A:" + mcluster.getSlotCounter(ChannelName.A) +
                         " / B:");
            if( mcluster.getClusterConfiguration().gChannels().contains(ChannelName.B))
                slots.append( "" + mcluster.getSlotCounter(ChannelName.B));
            else
                slots.append( "--");
        } else {
            slots.append(" A:? / B:?");
        }
        currentSlots.setText(slots.toString());
    }


    /**
     * Update the label showing if the tutorial is active
     *
     */
    private void updateTutorialLabel() {
        FRHelp help = learningModule.getHelp();
        boolean helpVisible = learningModule.isHelpVisible();
        if( helpVisible ) {
            boolean tutorialMode = help.isTutorialMode();
            if( tutorialMode )
                tutorialLabel.setText("Tutorial");
            else
                tutorialLabel.setText(" Normal ");
        } else {
            tutorialLabel.setText(" ----- ");
        }
    }

    private static final Color myGREEN = new Color(0,236,0);
    /**
     * Update the save status display
     * @param scenario
     */
    private void updateSaveStatus(final FlexRayScenario scenario) {
        if( scenario != null ) {
            if( scenario.hasChanged() ) {
                saveStatusLabel.setText(" M ");
                saveStatusLabel.setForeground(Color.RED);
            } else {
                saveStatusLabel.setText(" S ");
                saveStatusLabel.setForeground(myGREEN);
            }
        } else {
            saveStatusLabel.setText(" ? ");
            saveStatusLabel.setForeground(Color.BLACK);
        }
    }

    /*
     * --------------------- ?????????? ---------------------------
     */
    @Override
    protected void paintComponent(Graphics g) {
        super.paintComponent(g);
    }

    /*
     * --------------------- MouseListener ---------------------------
     */

    public void mouseClicked(MouseEvent e) { /**/ }
    public void mouseEntered(MouseEvent e) { /**/ }
    public void mouseExited(MouseEvent e) { /**/ }
    public void mousePressed(MouseEvent e) { /**/ }
    public void mouseReleased(MouseEvent e) { /**/ }

    /**
     * Set the Locale
     * @param l
     * @see java.awt.Component#setLocale(java.util.Locale)
     */
    @Override
    public void setLocale(Locale l) {
        super.setLocale(l);
        // channel display
        FlexRayScenario scenario = (FlexRayScenario)ModuleFR.getModule().getScenario();
        updateSaveStatus(scenario);
        updateNodeLabel(scenario);
        updateMacroTick(scenario);
    }

    /**
     * React to property changes.
     * @param evt
     * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
     */
    public void propertyChange(PropertyChangeEvent evt) {
        if( evt.getPropertyName().equals(LeviResourceManager.PROPERTY_LOCALE)) {
            setLocale(Locale.getDefault());
        }
        if( evt.getPropertyName().equals(Property.TUTORIALMODE)) updateTutorialLabel();
    }

    /*
     * react to state changes.
     * @param e
     * @see javax.swing.event.ChangeListener#stateChanged(javax.swing.event.ChangeEvent)
     */
    public void stateChanged(ChangeEvent e) {
        if( e.getSource() instanceof FlexRayScenario ) {
            FlexRayScenario scenario = (FlexRayScenario)e.getSource();
            updateSaveStatus(scenario);
            updateNodeLabel(scenario);
            updateTutorialLabel();
        }
    }


    public void cleanup() {
        memoryBar.setAutonomous(false);
    }

}
