/*
 * $Id: NodeComponent.java,v 1.1 2009/04/01 13:57:09 sirocic Exp $
 *
 * Created on 25.01.2006 21:56:37
 * by Dirk Frsterling <r@zorbla.de>
 */
package flexray.gui;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.MediaTracker;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.Stroke;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.geom.Rectangle2D;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;

import learningVisualizations.LeviConfiguration;
import learningVisualizations.LeviResourceManager;
import learningVisualizations.simulator.LeviSimulator;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import flexray.ModuleFR;
import flexray.ConfigKeys;
import flexray.cluster.Node;
import flexray.communication.FlexRayFrame;
import flexray.configuration.ChannelName;
import flexray.gui.actions.ConfigureNodeAction;
import flexray.gui.actions.DeleteSelectedNodeAction;
import flexray.levi.gui.FrameFR;
import flexray.scenario.FlexRayScenario;
import flexray.util.DebugUtil;
import flexray.xml.XMLTags;


/**
 * Prototype of a graphical module displaying a FlexRay node.
 * Currently, this Component contains two clock symbols, but
 * there's no means to update their content yet.
 *
 * @author Dirk Frsterling <r@zorbla.de>
 *
 */
@SuppressWarnings("serial")
public class NodeComponent extends AlphaBlendingComponent
                        implements XMLTags,
                                   DragZone,
                                   FocusListener,
                                   ActionListener,
                                   PropertyChangeListener,
                                   MouseListener,
                                   MouseMotionListener {

    private static Rectangle zeroBounds = new Rectangle(0,0,72,90);

    private ModuleFR learningModul;
    private int width = zeroBounds.width;
    private int height = zeroBounds.height;

    private JLabel nodeNameLabel = null;

    private FlexRayScenario scenario;
    private LeviResourceManager res;

    private static Color highlight = null;
    private static Color moduleForeground = null;
    private static Color moduleBackground = null;

    private Rectangle2D titleRect = new Rectangle2D.Double( 2, 2, width-4-1, 12);

    private static NodeComponent selected = null;

    protected static ImageIcon configIcon = null;
    private static ImageIcon deleteIcon = null;
    private static ImageIcon actionIcon = null;

    static {
        if( configIcon == null ) configIcon = loadIcon("config-node-16x16.gif");
        if( deleteIcon == null ) deleteIcon = loadIcon("delete-node-16x16.gif");
        if( actionIcon == null ) actionIcon = loadIcon("action-node-16x16.gif");
    }

    private JButton configButton;
    private static final String CMD_CONFIG = "config";
    private JButton deleteButton;
    private static final String CMD_DELETE = "delete";
    private JButton actionButton;
    private static final String CMD_ACTION = "action";


    /**
     * the Node ("scenario")
     */
    protected Node node;

    /**
     * Just for the GUI: Node colors.
     */
    private static HashMap<Integer,Color> nodeColors = new HashMap<Integer,Color>();

    private Color nodeColor = Color.WHITE;

    private Color defaultNodeColors[] = {
            Color.RED,
            Color.GREEN,
            Color.BLUE,
            Color.CYAN,
            Color.DARK_GRAY,
            Color.MAGENTA,
            Color.WHITE,
            Color.YELLOW,
            Color.BLACK,
            Color.GRAY,
    };


    /**
     * construct the module with two clock symbols
     * and an empty  button (so far)
     *
     * @param node The Node this Component is representing
     * @param scenario the scenario this Component belongs to
     */
    public NodeComponent(  ModuleFR learningModul, Node node, FlexRayScenario scenario ) {
        super();

        this.learningModul = learningModul;
        this.scenario = scenario;

        if( highlight == null ) 
            highlight = (Color)scenario.getLearningModule().getConfiguration().getSettingByKey(LeviConfiguration.COLOR_MODULE_HIGHLIGHT);
        if( highlight == null ) 
            highlight = (Color)scenario.getLearningModule().getConfiguration().getSettingByKey(LeviConfiguration.COLOR_COMPONENT_HIGHLIGHT);
        if( highlight == null ) // still nothing
            highlight = Color.RED;

        if( moduleForeground == null )
            moduleForeground = (Color)scenario.getLearningModule().getConfiguration().getSettingByKey(LeviConfiguration.COLOR_MODULE_FOREGROUND);
        if( moduleForeground == null )
            moduleForeground = (Color)scenario.getLearningModule().getConfiguration().getSettingByKey(LeviConfiguration.COLOR_COMPONENT_FOREGROUND);
        if( moduleForeground == null ) // still nothing
            moduleForeground = Color.BLACK;

        if( moduleBackground == null )
            moduleBackground = (Color)scenario.getLearningModule().getConfiguration().getSettingByKey(LeviConfiguration.COLOR_MODULE_BACKGROUND);
        if( moduleBackground == null )
            moduleBackground = (Color)scenario.getLearningModule().getConfiguration().getSettingByKey(LeviConfiguration.COLOR_COMPONENT_BACKGROUND);
        if( moduleBackground == null ) // still nothing
            moduleBackground = Color.WHITE;

        this.node = node;
        this.res = scenario.getLearningModule().getRessources();
        this.node.addPropertyChangeListener(this);

        nodeNameLabel = new JLabel();
        nodeNameLabel.setFont(nodeNameLabel.getFont().deriveFont(Font.PLAIN).deriveFont(10f));
        nodeNameLabel.setLocation(3,3);
        nodeNameLabel.setOpaque(false);
        nodeNameLabel.setBackground(moduleBackground);
        nodeNameLabel.setForeground(moduleForeground);
        updateNodeNameDisplay();
        this.add(nodeNameLabel);

        final int iconPad = 6;
        final int locationPad = 3;

        configButton = new JButton();
        configButton.setIcon(configIcon);
        configButton.setActionCommand(CMD_CONFIG);
        configButton.setFocusPainted(false);
        configButton.setFocusable(false);
        configButton.setSize(configIcon.getIconWidth()+iconPad,configIcon.getIconHeight()+iconPad);
        configButton.setLocation(new Point(locationPad,90-configButton.getHeight()-locationPad));
        add(configButton);
        configButton.addActionListener(this);

        deleteButton = new JButton();
        deleteButton.setIcon(deleteIcon);
        deleteButton.setActionCommand(CMD_DELETE);
        deleteButton.setFocusPainted(false);
        deleteButton.setFocusable(false);
        deleteButton.setSize(deleteIcon.getIconWidth()+iconPad,deleteIcon.getIconHeight()+iconPad);
        deleteButton.setLocation(new Point(locationPad+22,90-deleteButton.getHeight()-locationPad));
        add(deleteButton);
        deleteButton.addActionListener(this);

        actionButton = new JButton();
        actionButton.setIcon(actionIcon);
        actionButton.setActionCommand(CMD_ACTION);
        actionButton.setFocusPainted(false);
        actionButton.setFocusable(false);
        actionButton.setSize(actionIcon.getIconWidth()+iconPad,actionIcon.getIconHeight()+iconPad);
        actionButton.setLocation(new Point(locationPad+22+22,90-actionButton.getHeight()-locationPad));
        add(actionButton);
        actionButton.addActionListener(this);
        actionButton.setEnabled(false);

        // nodeID starts with #1, arrays start with #0
        int colorNumber = (node.getID()-1) % defaultNodeColors.length;
        nodeColor = defaultNodeColors[colorNumber];
        NodeComponent.nodeColors.put( Integer.valueOf(node.getID()), nodeColor );
        setFocusable(true);
        addMouseMotionListener(this);
        addFocusListener(this);
        setColorsFromConfig();
    }

    public Node getNode() {
        return node;
    }

    /**
     * overwritten reSize which repositions the module's contents
     * on a resize operation
     *
     * @param width
     * @param height
     * @see java.awt.Component#setSize(int, int)
     */
    @Override
    public void setSize(int width, int height) {
        super.setSize(width, height);
        this.width = width;
        this.height = height;
        titleRect = new Rectangle2D.Double( 2, 2, width-4-1, 12);
        repaint();
    }


    /*
     * @param g
     * @see flexray.gui.AlphaBlendingComponent#paintComponent(java.awt.Graphics)
     */
    @Override
    public void paintComponent( Graphics g ) {
        super.paintComponent( g );

        Graphics2D g2d = (Graphics2D)g;

        // enable antialiasing
        if( scenario.getLearningModule().getConfiguration().getSettingByKey(ConfigKeys.CONFIG_ANTIALIAS).equals(Boolean.TRUE))
            g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);

        // node body
        g2d.setColor( nodeColor );
        Color c = g2d.getColor();
        g2d.fillRect( 2, 2, width-3-1, height-3-1);


        LeviSimulator sim = scenario.getLearningModule().getSimulator();
        // color for node border
        if( ((getMousePosition()!=null) || (selected == this)) && (sim.isStop()) ) {
            g2d.setColor(highlight);
        } else {
            g2d.setColor( c.darker() );
        }

        if( !sim.isStop() ) {
            configButton.setEnabled(false);
            deleteButton.setEnabled(false);
            if( node.isRequestAllowed() )
                actionButton.setEnabled(true);
        } else {
            configButton.setEnabled(true);
            deleteButton.setEnabled(true);
            actionButton.setEnabled(false);
        }

        // node border
        Stroke s = g2d.getStroke();
        g2d.setStroke( new BasicStroke(4));
        g2d.drawRect( 0, 0, width-1, height-1 );
        g2d.setStroke(s);

        // name/move label
        g2d.setColor(moduleBackground);

        int slotA = node.getCluster().getSlotCounter(ChannelName.A);
        int slotB = node.getCluster().getSlotCounter(ChannelName.B);
        ChannelName clusterChannel = node.getCluster().getClusterConfiguration().gChannels();
        if ((node.isTransmittingInSlot(slotA) &&
             node.getNodeConfiguration().pChannels().contains(ChannelName.A) &&
             clusterChannel.contains(ChannelName.A)
            ) ||
            (node.isTransmittingInSlot(slotB) &&
             node.getNodeConfiguration().pChannels().contains(ChannelName.B) &&
             clusterChannel.contains(ChannelName.B)
            )
           ) {
            g2d.setColor(highlight);
        }


        g2d.fillRect( 3, 3, width-6, 11);
        g2d.setColor(moduleForeground);

        g2d.draw(titleRect);
        int nodePosX = nodeNameLabel.getLocation().x;
        int nnlWidth = (int)nodeNameLabel.getSize().getWidth();

        if( (nodePosX-2) > 4 ) {
            g2d.drawLine(4, 5,nodePosX-2, 5);
            g2d.drawLine(4, 8,nodePosX-2, 8);
            g2d.drawLine(4,11,nodePosX-2,11);
        }

        if( (nodePosX+nnlWidth+2) < (width-5) ) {
            g2d.drawLine(nodePosX+nnlWidth+2, 5,width-5, 5);
            g2d.drawLine(nodePosX+nnlWidth+2, 8,width-5, 8);
            g2d.drawLine(nodePosX+nnlWidth+2,11,width-5,11);
        }

        if( node.getCluster().getClusterConfiguration().gNumberOfMinislots() > 0) {
            paintFrameIndicator(g2d, ChannelName.A);
            paintFrameIndicator(g2d, ChannelName.B);
        }

        if (learningModul.isTooltipsVisible()) {
	        configButton.setToolTipText(res.getResourceByKey("node.config.tip"));
	        deleteButton.setToolTipText(res.getResourceByKey("node.delete.tip"));
	        actionButton.setToolTipText(res.getResourceByKey("node.action.tip"));
        }
        else {
        	configButton.setToolTipText(null);
        	deleteButton.setToolTipText(null);
        	actionButton.setToolTipText(null);
        }
        
    } // paintComponent()

    private void paintFrameIndicator(Graphics2D g2d, ChannelName channel) {

        FlexRayFrame frFrame = node.getCurrentFrame(channel);
        int sx = 12;
        int space = width-sx-4-1;
        int gdMinislot = node.getCluster().getClusterConfiguration().gdMinislot();
        int iheight = 12;
        int ypsilon = 20 + channel.ordinal()*(iheight+2);

        Font font = g2d.getFont();

        g2d.setColor(Color.WHITE);
//        g2d.fillRect(sx-9, ypsilon-1, 7, 7);
//        g2d.drawImage(MicroFont.createStringImage(channel.toString()), sx-8, ypsilon, null);
        g2d.fillRect(sx-9, ypsilon-1, 8, iheight+2);
        g2d.setFont(font.deriveFont(10.0f));
        FontMetrics fm = g2d.getFontMetrics();
        int ascdesc = fm.getAscent()-fm.getDescent();
        Rectangle2D stringbounds = fm.getStringBounds(channel.toString(), g2d);
        g2d.setColor(Color.BLACK);
        g2d.drawString(channel.toString(),sx-9,ypsilon+ascdesc+(int)((stringbounds.getHeight()-ascdesc)/2));

        if( node.getNodeConfiguration().pChannels().contains(channel)) {

            g2d.setColor(Color.WHITE);

            g2d.drawRect(sx-1, ypsilon-1, space+1, iheight+1);
            g2d.setColor(Color.BLACK);
            g2d.fillRect(sx, ypsilon, space, iheight);

            if( (frFrame != null) && (frFrame.getLength() > FlexRayFrame.EMPTY_FRAME_LENGTH) ) {

                long frameLength = frFrame.getLength() * gdMinislot;
                double scale = (double)(space-2) / frameLength;

                g2d.setColor(Color.WHITE);

                int advance = frFrame.getTickInFrame();
                g2d.fillRect(sx+1, ypsilon+1, (int)(advance*scale), iheight-2);

                /*
                 * Draw fragment markers
                 */
                List<Integer> fragments = frFrame.getFragments();
                synchronized(fragments) {
                    if( fragments.size() > 0 ) {
                        for(Integer fragment : fragments) {
                            g2d.setColor(new Color(255,32,32));
                            Stroke s = g2d.getStroke();
                            g2d.setStroke(new BasicStroke(2.0f));
                            g2d.drawLine(sx+(int)(fragment.intValue()*scale),
                                    ypsilon+1,
                                    sx+(int)(fragment.intValue()*scale),
                                    ypsilon+iheight-2);
                            g2d.setStroke(s);
                        }
                    } // fragments
                }
            } else { // frame != null
                g2d.setFont(font.deriveFont(Font.BOLD, 11.0f));
                g2d.setColor(Color.WHITE);
                String countStr = "" + node.getDynMessageCount(channel);
                stringbounds = fm.getStringBounds(countStr, g2d);
                ascdesc = fm.getAscent()-fm.getDescent();
                g2d.drawString(countStr, sx+(int)(space/2 - stringbounds.getWidth()/2), ypsilon+ascdesc+(int)((stringbounds.getHeight()-ascdesc))/2);
                g2d.setFont(font);
            }
        } // if nodeConfig contains channel
    } // paintFrameIndicator

    public Color getColor() {
        return nodeColor;
    }

    /**
     *
     * @return the preferred size
     * @see javax.swing.JComponent#getPreferredSize()
     */
    @Override
    public Dimension getPreferredSize() {
        return new Dimension( width, height );
    }

    /**
     *
     * @param l
     * @see java.awt.Component#setLocale(java.util.Locale)
     */
    @Override
    public void setLocale(Locale l) {
        super.setLocale(l);
        updateNodeNameDisplay();
        if (learningModul.isTooltipsVisible()) {
	        configButton.setToolTipText(res.getResourceByKey("node.config.tip"));
	        deleteButton.setToolTipText(res.getResourceByKey("node.delete.tip"));
	        actionButton.setToolTipText(res.getResourceByKey("node.action.tip"));
        }
        else {
        	configButton.setToolTipText(null);
        	deleteButton.setToolTipText(null);
        	actionButton.setToolTipText(null);
        }
	    repaint();
    }

    public void updateNodeNameDisplay() {
        if(node.getName() == null) {
            nodeNameLabel.setText( ModuleFR.getModule().getRessources().getResourceByKey("flexray.node") + " #" + node.getID());
            nodeNameLabel.setSize((int)nodeNameLabel.getPreferredSize().getWidth(),(int)titleRect.getHeight()-2);
        } else {
            String nodeName = node.getName();
            if( nodeName.length() > 10 ) nodeName = nodeName.substring(0,8) + "...";
            nodeNameLabel.setText( nodeName );
            int tWidth = Math.min((int)nodeNameLabel.getPreferredSize().getWidth(), (int)titleRect.getWidth()-8);
            nodeNameLabel.setSize(tWidth,(int)titleRect.getHeight()-2);
        }
        // can't center easily
        if( nodeNameLabel.getSize().getWidth() < (titleRect.getWidth()-8)) {
            int x = (int)(titleRect.getWidth() - nodeNameLabel.getSize().getWidth())/2+1;
            nodeNameLabel.setLocation(x, nodeNameLabel.getLocation().y);
            nodeNameLabel.setBackground(Color.WHITE);
        }
    }


    /*
     * non-javadoc
     *
     * @see java.awt.Component#setLocation(int, int)
     */
    @Override
    public void setLocation(int x, int y) {
        int oldx = this.getX();
        int oldy = this.getY();
        if( (oldx != x) || (oldy != y)) {
            super.setLocation(x,y);
            firePropertyChange(Property.LOCATION, new Point(oldx,oldy), new Point(x,y));
        }
    }


    /**
     * access the mapping of the node colors.
     *
     * @param nodeID
     * @return the Color object set for the node nodeID
     */
    public static Color getDefaultNodeColor( int nodeID ) {
        Color c = nodeColors.get( Integer.valueOf(nodeID) );
        if( c != null ) return c;
        return Color.WHITE;
    }

    public Color getNodeColor() {
        return nodeColor;
    }

    public void setNodeColor(Color color) {
        nodeColor = color;
        repaint();
    }


    public void setNode( Node n ) {
        this.node = n;
    }

    /*
     *  ----------- flexray.gui.Resizable methods --------------
     *
     *  FIXME: hard coded values?
     */
    public int getSensitiveX() {
        return 15;
    }

    public int getSensitiveY() {
        return 15;
    }

    public int getMinimumHeight() {
        return 50;
    }

    public int getMinimumWidth() {
        return 70;
    }


    /**
     * Create an XML representation of this NodeComponent's parameters
     * @param xmlDoc
     * @return the XML Element containing this NodeComponent's representation
     */
    public Element toXML(final Document xmlDoc) {
        Element myXML = xmlDoc.createElement(XT_NODEVIEW);
        myXML.setAttribute(XT_NODEVIEW_ID, Integer.valueOf(node.getID()).toString());
        Element screenpos = xmlDoc.createElement(XT_SCREENPOS);
        screenpos.setAttribute(XT_SCREENPOS_X, Integer.valueOf(getX()).toString());
        screenpos.setAttribute(XT_SCREENPOS_Y, Integer.valueOf(getY()).toString());
        screenpos.setAttribute(XT_SCREENPOS_Z,
                Integer.valueOf(getParent().getComponentZOrder(this)).toString());
        myXML.appendChild(screenpos);
        Element color = xmlDoc.createElement(XT_COLOR_RGB);
        color.setAttribute(XT_COLOR_RGB_B, Integer.valueOf(nodeColor.getBlue()).toString());
        color.setAttribute(XT_COLOR_RGB_G, Integer.valueOf(nodeColor.getGreen()).toString());
        color.setAttribute(XT_COLOR_RGB_R, Integer.valueOf(nodeColor.getRed()).toString());
        myXML.appendChild(color);
        return myXML;
    }

    /**
     * Restore this node module's parameters from an XML element
     * @param element
     * @return the stored z-order number of this module
     */
    public int restoreFromXML(final Element element) {
        NodeList nodeList = element.getChildNodes();
        int z = 0;
        for( int i = 0 ; i < nodeList.getLength() ; i++ ) {
        	if (nodeList.item(i) instanceof Element) {
	            Element item = (Element)nodeList.item(i);
	            if( item.getNodeName().equals(XT_SCREENPOS)) {
	                int x = Integer.parseInt(item.getAttribute(XT_SCREENPOS_X));
	                int y = Integer.parseInt(item.getAttribute(XT_SCREENPOS_Y));
	                z = Integer.parseInt(item.getAttribute(XT_SCREENPOS_Z));
	                setLocation(x, y);
	            } // SCREENPOS
	            if( item.getNodeName().equals(XT_COLOR_RGB)) {
	                int r = Integer.parseInt(item.getAttribute(XT_COLOR_RGB_R));
	                int g = Integer.parseInt(item.getAttribute(XT_COLOR_RGB_G));
	                int b = Integer.parseInt(item.getAttribute(XT_COLOR_RGB_B));
	                nodeColor = new Color(r,g,b);
	            }
        	}
        } // viewnode_children
        return z;
    } // restoreFromXML

    public static void setHighlight(Color nodeHighlight) {
        NodeComponent.highlight = nodeHighlight;
    }

    public static void setComponentBackground(Color moduleBackground) {
        NodeComponent.moduleBackground = moduleBackground;
    }

    public static void setComponentForeground(Color moduleForeground) {
        NodeComponent.moduleForeground = moduleForeground;
    }

    public Rectangle getDragZone() {
        return titleRect.getBounds();
    }


    // -------------------- mouse motion listener ------------------------

    public void mouseDragged(MouseEvent e){
//        DebugUtil.println("dragged");
    }

    private boolean inMoveZone = false;
    public void mouseMoved(MouseEvent e) {
    	String tooltip;
    	
        if( (!inMoveZone) && titleRect.contains(e.getPoint())) {
            setCursor(Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR));
            inMoveZone = true;
        }
        if( inMoveZone && (!titleRect.contains(e.getPoint()))) {
            setCursor(Cursor.getDefaultCursor());
            inMoveZone = false;
        }
        
        // Tooltip fr den Knoten setzen
        tooltip = "<html><br>";
        tooltip=tooltip +res.getResourceByKey("flexray.node.tip")+"<br><br>";
    	tooltip=tooltip+"</html>";
    	setToolTipText(tooltip);
    	
    }

    // ----------------------- mouse listener ---------------------------

    public void mouseClicked(MouseEvent e) {/**/}

    public void mouseEntered(MouseEvent e) {/**/}

    public void mouseExited(MouseEvent e) {/**/}

    public void mousePressed(MouseEvent e) {/**/}

    public void mouseReleased(MouseEvent e) {/**/}

    //   ------------------- focusListener ----------------------
    public void focusGained(FocusEvent e) {
        NodeComponent prev = selected;
        selected = (NodeComponent)e.getSource();
        selected.repaint();
        if( prev != null ) prev.repaint();
    }

    public void focusLost(FocusEvent e) { repaint(); }

    public static NodeComponent getSelectedNodeComponent() {
        return selected;
    }

    /*
     * load an icon
     */
    private static ImageIcon loadIcon( String name) {
        ImageIcon icon = null;
        URL u = NodeComponent.class.getResource("/flexray/gui/icons/" + name);
        if( u != null ) icon = new ImageIcon(u);
        else icon = new ImageIcon(new byte[1],"image not found");
        return icon;
    }

    public void actionPerformed(ActionEvent e) {
        requestFocus();
        NodeComponent prev = selected;
        selected = this;
        repaint();
        if( prev != null ) prev.repaint();
        if( e.getActionCommand().equals(CMD_DELETE) ) {
            DeleteSelectedNodeAction dsna = new DeleteSelectedNodeAction(scenario.getLearningModule());
            dsna.actionPerformed(e);
        } else  if( e.getActionCommand().equals(CMD_CONFIG)) {
            new ConfigureNodeAction(scenario).actionPerformed(new ActionEvent( e, ActionEvent.ACTION_PERFORMED, "NCright" ));
        } else if(e.getActionCommand().equals(CMD_ACTION)) {
            actionButton.setEnabled(false);

            node.requestMessage(ChannelName.A);
            node.requestMessage(ChannelName.B);
        }

    }

    private void setColorsFromConfig() {
    	Object o;
    	
        LeviConfiguration config = scenario.getLearningModule().getConfiguration();
        
        if (config.getSettingByKey(LeviConfiguration.COLOR_COMPONENT_BACKGROUND)!= null)
        	o = config.getSettingByKey(LeviConfiguration.COLOR_COMPONENT_BACKGROUND);
        else
            o = config.getSettingByKey(LeviConfiguration.COLOR_MODULE_BACKGROUND);
        if( o instanceof Color ) {
            setComponentBackground((Color)o);
            nodeNameLabel.setBackground((Color)o);
        }

        if (config.getSettingByKey(LeviConfiguration.COLOR_COMPONENT_FOREGROUND)!= null)
        	o = config.getSettingByKey(LeviConfiguration.COLOR_COMPONENT_FOREGROUND);
        else
        o = config.getSettingByKey(LeviConfiguration.COLOR_MODULE_FOREGROUND);
        if( o instanceof Color ) {
            setComponentForeground((Color)o);
            nodeNameLabel.setForeground((Color)o);
        }

        if (config.getSettingByKey(LeviConfiguration.COLOR_COMPONENT_HIGHLIGHT)!= null)
        	o = config.getSettingByKey(LeviConfiguration.COLOR_COMPONENT_HIGHLIGHT);
        else
        o = config.getSettingByKey(LeviConfiguration.COLOR_MODULE_HIGHLIGHT);
        if( o instanceof Color ) {
            setHighlight((Color)o);
        }


    } // setColorsFromConfig

    @Override
    public void propertyChange(PropertyChangeEvent evt) {
        super.propertyChange(evt);
        if( evt.getPropertyName().startsWith("Color")) {
            setColorsFromConfig();
        }
    }
}
