/*
 * $Id: Resizable.java,v 1.1 2009/04/01 13:57:10 sirocic Exp $
 *
 * Created on 25.01.2006 23:24:36
 * by Dirk Frsterling <r@zorbla.de>
 */
package flexray.gui;

/**
 * Provisional interface for resizable components.
 *
 * @author Dirk Frsterling <r@zorbla.de>
 *
 */
public interface Resizable {

	/**
	 * get the (component relative) x coordinate of the sensitive spot
	 * of the component, where it may be clicked and dragged to be resized.
	 *
	 * @return the x coordinate of the sensitive spot
	 */
	public int getSensitiveX();

	/**
	 * get the (component relative) y coordinate of the sensitive spot
	 * of the component, where it may be clicked and dragged to be resized.
	 *
	 * @return the y coordinate of the sensitive spot
	 */
	public int getSensitiveY();

	/**
	 * get the minimum height that this component needs. Stop vertical
	 * resizing below that point
	 *
	 * @return the component's minimum height
	 */
	public int getMinimumHeight();

	/**
	 * get the minimum width that this component needs. Stop horizontal
	 * resizing below that point
	 *
	 * @return the component's minimum width
	 */
	public int getMinimumWidth();
}
