/*
 * $Id: SetActiveScenarioAction.java,v 1.1 2009/04/01 13:57:10 sirocic Exp $
 *
 */
package flexray.gui.actions;

import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import learningVisualizations.LeviResourceManager;

import flexray.ModuleFR;
import flexray.scenario.FlexRayScenario;
import flexray.util.DebugUtil;

/**
 * An Action to set the currently displayed/edited scenario.
 *
 * @author Dirk Frsterling <r@zorbla.de>
 *
 */
public class SetActiveScenarioAction extends AbstractAction
                               implements ChangeListener,
                                          PropertyChangeListener {

    private static final long serialVersionUID = 7659880378505430625L;
    FlexRayScenario scenario = null;
    ModuleFR moduleFR = null;
    LeviResourceManager res = null;
    JMenu parent = null;
    JMenuItem menuItem = null;

    public SetActiveScenarioAction( FlexRayScenario scenario,
                                 JMenuItem menuItem,
                                 JMenu menuParent) {
        this.scenario = scenario;
        this.scenario.addChangeListener(this);
        this.parent = menuParent;
        this.menuItem = menuItem;
        moduleFR = this.scenario.getLearningModule();
        res = moduleFR.getRessources();
        setValues();
    }

    /**
     * Update the text for this action
     *
     */
    private void updateDisplayText() {
        int number = -1;
        int base = -1;
        if( parent == null ) {
            DebugUtil.mark("parent is null");
            return;
        }
        /*
         * Find out where the menu entry of this action is and set the numbering
         * accordingly
         */
        for( int index = 0; index < parent.getMenuComponentCount(); index++) {
            if( parent.getMenuComponent(index) instanceof JMenuItem) {
                JMenuItem item = ((JMenuItem)parent.getMenuComponent(index));
                if( (item.getAction() instanceof SetActiveScenarioAction) && (base==-1)) {
                    base = index - 1;
                }
                if( item.getAction().equals(this) && (number<0)) {
                    number = index - base;
                    break;
                }
            }
        }
        // the action has to be unset and re-set to make the changes visible
        menuItem.setAction(null);
        this.putValue(Action.MNEMONIC_KEY, Integer.valueOf(KeyEvent.VK_0 + number));
        this.putValue(Action.ACCELERATOR_KEY,
                KeyStroke.getKeyStroke(KeyEvent.VK_F1-1+number, InputEvent.SHIFT_DOWN_MASK));
        this.putValue(Action.NAME,
                 Integer.valueOf(number).toString() + " " +
                 (scenario.hasChanged()?" * ":"") +
                 scenario.getName()
                );

        menuItem.setAction(this);
    }

    /**
     * set the action values according to the current Locale.
     *
     */
    public void setValues() {
        this.putValue(Action.SHORT_DESCRIPTION, res.getResourceByKey("viewmode.switchscenario.tip"));
        updateDisplayText();
    }

    /**
     * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     */
    public void actionPerformed(ActionEvent e) {
        moduleFR.setActiveScenario(scenario);
    }

    /**
     * React to state changes (of the scenario)
     * FIXME: wenn bereits aktiv => return
     * @param e
     * @see javax.swing.event.ChangeListener#stateChanged(javax.swing.event.ChangeEvent)
     */
    public void stateChanged(ChangeEvent e) {
        updateDisplayText();
        if( moduleFR.getScenario().equals(scenario)) {
            setEnabled(false);
        } else {
            setEnabled(true);
        }
    }

    /**
     *
     * @param evt
     * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
     */
    public void propertyChange(PropertyChangeEvent evt) {
        if( evt.getPropertyName().equals(LeviResourceManager.PROPERTY_LOCALE)) {
            setValues();
        }
    }

}
