/*
 * $Id: ToggleAntiAliasingAction.java,v 1.1 2009/04/01 13:57:10 sirocic Exp $
 *
 * Created on 27.05.2006 08:47:11
 * by Dirk Frsterling <r@zorbla.de>
 */
package flexray.gui.actions;

import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.AbstractAction;
import javax.swing.Action;

import learningVisualizations.LeviConfiguration;
import learningVisualizations.LeviResourceManager;

import flexray.ModuleFR;
import flexray.ConfigKeys;
import flexray.gui.ScenarioPanel;
import flexray.gui.Property;
import flexray.util.DebugUtil;

/**
 * @author Dirk Frsterling <r@zorbla.de>
 *
 */
public class ToggleAntiAliasingAction extends AbstractAction
                                   implements PropertyChangeListener,
                                              ConfigKeys {

    private static final long serialVersionUID = 3517531841560468028L;
    private ModuleFR learningModule;
    private LeviConfiguration conf;
    private LeviResourceManager res;

    public ToggleAntiAliasingAction(final ModuleFR learningModule) {
        this.learningModule = learningModule;
        this.res = this.learningModule.getRessources();
        res.addPropertyChangeListener(Property.LOCALE, this);
        this.conf = this.learningModule.getConfiguration();
        putValue(Action.NAME, res.getResourceByKey("antialiasing"));
        putValue(Action.SHORT_DESCRIPTION, res.getResourceByKey("antialiasing.tip"));
        putValue(Action.MNEMONIC_KEY, Integer.valueOf( KeyEvent.VK_T ));
    }

    /**
     * @param evt
     * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
     */
    public void propertyChange(PropertyChangeEvent evt) {
        if( evt.getPropertyName().equals(LeviResourceManager.PROPERTY_LOCALE)) {
            putValue(Action.NAME, res.getResourceByKey("antialiasing"));
            // starts to work in Java 6
            // http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=4626632
            putValue(Action.SHORT_DESCRIPTION, res.getResourceByKey("antialiasing.tip"));
        } else {
            DebugUtil.println("All your base are belong to us!");
        }
    } // propertyChange


    /**
     * @param e
     * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     */
    public void actionPerformed(ActionEvent e) {
        Object setting = conf.getSettingByKey(CONFIG_ANTIALIAS);
        if( setting instanceof Boolean ) {
            boolean newSetting = !((Boolean)setting).booleanValue();
            conf.putSettingByKey(CONFIG_ANTIALIAS, Boolean.valueOf(newSetting));
            conf.saveConfiguration();
            ScenarioPanel.setAntiAlias(newSetting);
        }
    } // actionPerformed

}
