/*
 * $Id: ToggleTopologyAction.java,v 1.1 2009/04/01 13:57:10 sirocic Exp $
 *
 * Created on 27.05.2006 08:47:11
 * by Dirk Frsterling <r@zorbla.de>
 */
package flexray.gui.actions;

import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JCheckBoxMenuItem;

import learningVisualizations.LeviConfiguration;
import learningVisualizations.LeviResourceManager;

import flexray.ModuleFR;
import flexray.ConfigKeys;
import flexray.cluster.Cluster;
import flexray.configuration.ChannelName;
import flexray.configuration.ChannelTopology;
import flexray.gui.Property;
import flexray.scenario.FlexRayScenario;
import flexray.util.DebugUtil;

/**
 * @author Dirk Frsterling <r@zorbla.de>
 *
 */
public class ToggleTopologyAction extends AbstractAction
                                   implements PropertyChangeListener,
                                              ConfigKeys {

    private static final long serialVersionUID = 9144092321033006868L;
    private ModuleFR learningModule;
    private LeviConfiguration conf;
    private LeviResourceManager res;
    private JCheckBoxMenuItem myMenuItem;
    ChannelName channel;
    ChannelTopology topology;

    public ToggleTopologyAction(final ModuleFR learningModule, JCheckBoxMenuItem jcbmi, ChannelName pChannel, ChannelTopology pTopo) {
        this.learningModule = learningModule;
        this.res = this.learningModule.getRessources();
        this.channel = pChannel;
        this.topology = pTopo;
        res.addPropertyChangeListener(Property.LOCALE, this);
        this.conf = this.learningModule.getConfiguration();
        putValue(Action.NAME, res.getResourceByKey("menu.topology." + topology));
        putValue(Action.MNEMONIC_KEY, Integer.valueOf( KeyEvent.VK_A ));
        myMenuItem = jcbmi;
    }

    /**
     * @param evt
     * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
     */
    public void propertyChange(PropertyChangeEvent evt) {
        if( evt.getPropertyName().equals(LeviResourceManager.PROPERTY_LOCALE)) {
            putValue(Action.NAME, res.getResourceByKey("twochannels"));
            putValue(Action.SHORT_DESCRIPTION, res.getResourceByKey("twochannels.tip"));
        } else {
            DebugUtil.println("All your base are belong to us!");
        }
    } // propertyChange


    /**
     * @param e
     * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     */
    public void actionPerformed(ActionEvent e) {
        learningModule.getSimulator().stopSimulation();
        FlexRayScenario scenario = (FlexRayScenario)learningModule.getScenario();
        if(topology != scenario.getTopology(channel)) {
        	scenario.setTopology(channel, topology);
        	scenario.fireStateChanged();
        }
    } // actionPerformed

    public void set( boolean b ) {
        myMenuItem.setSelected( b );
    }

}
