/*
 * $Id: CommCycleConfigDialog.java,v 1.1 2009/04/01 13:57:06 sirocic Exp $
 *
 */
package flexray.gui.dialogs;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.font.FontRenderContext;
import java.awt.geom.Rectangle2D;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JSpinner;
import javax.swing.Box;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import learningVisualizations.LeviModule;
import learningVisualizations.gui.LeviFrame;


import flexray.configuration.ClusterConfiguration;
import flexray.gui.actions.CancelAction;
import flexray.gui.actions.OKAction;
import flexray.gui.controller.EditCommandType;
import flexray.scenario.FlexRayScenario;
import flexray.util.DebugUtil;

/**
 * @author Dirk Frsterling <r@zorbla.de>
 *
 */
public class CommCycleConfigDialog extends AbstractFlexRayDialog {


	public final static int SPINNERWIDTH = 70;
	public final static int SPINNERHEIGHT = 20;

    protected SpinnerNumberModel numberOfStaticSlotsValueModel;
    protected SpinnerNumberModel durationOfStaticSlotValueModel;
    protected SpinnerNumberModel numberOfMinislotsValueModel;
    protected SpinnerNumberModel durationOfMinislotValueModel;
    protected SpinnerNumberModel nitDurationValueModel;
    private ClusterConfiguration previousConfig;
    private JPanel staticSegmentConfigPanel;
    private JLabel numberOfStaticSlotsL;
    private JLabel durationOfStaticSlotL;
    private JPanel dynamicSegmentConfigPanel;
    private JLabel numberOfMinislotsL;
    private JLabel durationOfMinislotL, nitDurationL;
    private JPanel nitPanel;
    private JLabel macrotickStaticL, macrotickDynamicL, macrotickNITL;
    private JSpinner numberOfStaticSlotsSpinner;
    private JSpinner durationOfStaticSlotSpinner;
    private JSpinner numberOfMinislotsSpinner;
    private JSpinner durationOfMinislotSpinner;
    private JSpinner nitDurationSpinner;
    private CommCycleSpinnerListener ccsl;
    private Box staticSegmentHB, staticSegmentVB, dynamicSegmentHB, dynamicSegmentVB;
    private Box nitSegmentHB, nitSegmentVB;

    /**
     * Special JLabel receiving change events from this dialog's slider
     * @author Dirk Frsterling <r@zorbla.de>
     *
     */
    public class CommCycleSpinnerListener implements ChangeListener {
        public void stateChanged(final ChangeEvent e) {
            int value;
            if( e.getSource() == numberOfStaticSlotsValueModel ) {
                value = ((Integer)numberOfStaticSlotsValueModel.getValue()).intValue();
                cluster.getClusterConfiguration().gNumberOfStaticSlots(value);
                scenario.setLastEdit(EditCommandType.TEMPORARY_EDIT);
                scenario.fireStateChanged();
            } else if (e.getSource() == durationOfStaticSlotValueModel ) {
                value = ((Integer)durationOfStaticSlotValueModel.getValue()).intValue();
                cluster.getClusterConfiguration().gdStaticSlot(value);
                scenario.setLastEdit(EditCommandType.TEMPORARY_EDIT);
                scenario.fireStateChanged();
            } else if( e.getSource() == numberOfMinislotsValueModel ) {
                value = ((Integer)numberOfMinislotsValueModel.getValue()).intValue();
                cluster.getClusterConfiguration().gNumberOfMinislots(value);
                scenario.setLastEdit(EditCommandType.TEMPORARY_EDIT);
                scenario.fireStateChanged();
            } else if (e.getSource() == durationOfMinislotValueModel ) {
                value = ((Integer)durationOfMinislotValueModel.getValue()).intValue();
                cluster.getClusterConfiguration().gdMinislot(value);
                scenario.setLastEdit(EditCommandType.TEMPORARY_EDIT);
                scenario.fireStateChanged();
            } else if ( e.getSource() == nitDurationValueModel ) {
                value = ((Integer)nitDurationValueModel.getValue()).intValue();
                cluster.getClusterConfiguration().gdNIT(value);
                scenario.setLastEdit(EditCommandType.TEMPORARY_EDIT);
                scenario.fireStateChanged();
            }

        }
    } // ActionLabel

    public CommCycleConfigDialog(FlexRayScenario pScenario) {
        super(pScenario, "dlg.commcycle.title");
            
        ccsl = new CommCycleSpinnerListener();

        // Boxen erzeugen ...
 	    contentVB.setAlignmentX(Component.LEFT_ALIGNMENT);
 	    buttonHB.setAlignmentX(Component.LEFT_ALIGNMENT);
        staticSegmentHB = Box.createHorizontalBox();
        staticSegmentHB.setAlignmentX(Component.LEFT_ALIGNMENT);
        staticSegmentVB = Box.createVerticalBox();
        staticSegmentVB.setAlignmentX(Component.LEFT_ALIGNMENT);
        dynamicSegmentHB = Box.createHorizontalBox();
        dynamicSegmentHB.setAlignmentX(Component.LEFT_ALIGNMENT);
        dynamicSegmentVB = Box.createVerticalBox();
        dynamicSegmentVB.setAlignmentX(Component.LEFT_ALIGNMENT);
        nitSegmentHB = Box.createHorizontalBox();
        nitSegmentHB.setAlignmentX(Component.LEFT_ALIGNMENT);
        nitSegmentVB = Box.createVerticalBox();
        nitSegmentVB.setAlignmentX(Component.LEFT_ALIGNMENT);
    
        /*
         * static slot configuration
         */
        staticSegmentConfigPanel = new JPanel();
        staticSegmentConfigPanel.setLayout(new BorderLayout());
        staticSegmentConfigPanel.setAlignmentX(Component.LEFT_ALIGNMENT);

        // Number fo static slots
        numberOfStaticSlotsL = new JLabel();
        macrotickStaticL = new JLabel(); 
        numberOfStaticSlotsSpinner = new JSpinner();
        numberOfStaticSlotsSpinner.setSize(SPINNERWIDTH, SPINNERHEIGHT);
        numberOfStaticSlotsSpinner.setPreferredSize(new Dimension(SPINNERWIDTH, SPINNERHEIGHT));
        numberOfStaticSlotsSpinner.setMaximumSize(new Dimension(SPINNERWIDTH, SPINNERHEIGHT));
        staticSegmentHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        staticSegmentHB.add(numberOfStaticSlotsL);
        staticSegmentHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        staticSegmentHB.add(numberOfStaticSlotsSpinner);
        staticSegmentHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));

        // duration of a static slot
        durationOfStaticSlotL = new JLabel();
        durationOfStaticSlotSpinner = new JSpinner();
        durationOfStaticSlotSpinner.setSize(SPINNERWIDTH, SPINNERHEIGHT);
        durationOfStaticSlotSpinner.setPreferredSize(durationOfStaticSlotSpinner.getSize());
        durationOfStaticSlotSpinner.setMaximumSize(durationOfStaticSlotSpinner.getSize());
        staticSegmentHB.add(durationOfStaticSlotL);
        staticSegmentHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        staticSegmentHB.add(durationOfStaticSlotSpinner);
        staticSegmentHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        staticSegmentHB.add(macrotickStaticL);
        staticSegmentHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        
        staticSegmentVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
        staticSegmentVB.add(staticSegmentHB);
        staticSegmentVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
        staticSegmentConfigPanel.add(staticSegmentVB);
        
        /*
         * Minislot configuration
         */
        dynamicSegmentConfigPanel = new JPanel();
        dynamicSegmentConfigPanel.setLayout(new BorderLayout());
        dynamicSegmentConfigPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
        // Number of minislots
        numberOfMinislotsL = new JLabel();
        macrotickDynamicL = new JLabel(); 
        numberOfMinislotsSpinner = new JSpinner();
        numberOfMinislotsSpinner.setSize(SPINNERWIDTH, SPINNERHEIGHT);
        numberOfMinislotsSpinner.setPreferredSize(numberOfMinislotsSpinner.getSize());
        numberOfMinislotsSpinner.setMaximumSize(numberOfMinislotsSpinner.getSize());
        dynamicSegmentHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        dynamicSegmentHB.add(numberOfMinislotsL);
        dynamicSegmentHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        dynamicSegmentHB.add(numberOfMinislotsSpinner);
        dynamicSegmentHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));

        // duration of a minislot
        durationOfMinislotL = new JLabel();
        durationOfMinislotSpinner = new JSpinner();
        durationOfMinislotSpinner.setSize(SPINNERWIDTH, SPINNERHEIGHT);
        durationOfMinislotSpinner.setPreferredSize(durationOfMinislotSpinner.getSize());
        durationOfMinislotSpinner.setMaximumSize(durationOfMinislotSpinner.getSize());
        dynamicSegmentHB.add(durationOfMinislotL);
        dynamicSegmentHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        dynamicSegmentHB.add(durationOfMinislotSpinner);
        dynamicSegmentHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        dynamicSegmentHB.add(macrotickDynamicL);
        dynamicSegmentHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        
        dynamicSegmentVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
        dynamicSegmentVB.add(dynamicSegmentHB);
        dynamicSegmentVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
        dynamicSegmentConfigPanel.add(dynamicSegmentVB);
        
        // NIT
        nitPanel = new JPanel();
        nitPanel.setLayout(new BorderLayout());
        nitPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
        nitDurationL = new JLabel();
        macrotickNITL = new JLabel();
        nitDurationSpinner = new JSpinner();
        nitDurationSpinner.setSize(SPINNERWIDTH, SPINNERHEIGHT);
        nitDurationSpinner.setPreferredSize(nitDurationSpinner.getSize());
        nitDurationSpinner.setMaximumSize(nitDurationSpinner.getSize());
        nitSegmentHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        nitSegmentHB.add(nitDurationL);
        nitSegmentHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        nitSegmentHB.add(nitDurationSpinner);
        nitSegmentHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        nitSegmentHB.add(macrotickNITL);
        nitSegmentHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        
        nitSegmentVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
        nitSegmentVB.add(nitSegmentHB);
        nitSegmentVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
        nitPanel.add(nitSegmentVB);
        
        //      Abbruch und Ok-Button einfgen
        cancelButton.addActionListener(this);
        cancelButton.setActionCommand(CancelAction.COMMAND);
		okButton.addActionListener(this);
		okButton.setActionCommand(OKAction.COMMAND);
	       
        //      add it to the veritcal box
 	    contentVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
 	    contentVB.add(staticSegmentConfigPanel);
	    contentVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
 	    contentVB.add(dynamicSegmentConfigPanel);
 	    contentVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
 	    contentVB.add(nitPanel);
	    contentVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
 	    contentVB.add(buttonHB);
 	    contentVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
 	    
 	    contentHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
 	    contentHB.add(contentVB);
 	    contentHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
 	    // Defaultmig ist die Box contentVB alles enthaltende Box im Panel. 
 	    setContentBox(contentHB);

		// FARBEN, FONTS UND TEXTE SETZEN ...
	 	updateLanguageColorFonts();
        
    }
    
    /*
     * Methode:		showDialog
     * 
     * Description:	Setzt die aktuellen Farben aus der Konfiguration bevor der Dialog angezeigt bzw.
     * 				wieder versteckt wird.
     *    
     * Parameters:    
     * 
     * Date: 		 11.12.2007
     * @author        sirocic
     */	
    public void showDialog(boolean visible) {
       
  	  // TRACE OUTPUT
  	  if (myFrame.getLearningModule().isTraceActive()) System.out.println("CALL ArrowEditorDialogPN.showDialog:");
  	  
  	  // alle Taskparametereingaben zurcksetzen.
  	  cleanMe();
    	  
  	  if (visible) {
  		  // Dialog wird sichtbar gemacht.
  		  
  		  // Werte der aktuellen Zykluskonfiguration anzeigen ???
  		  
  		  //  Get the current cluster Configuration
          ClusterConfiguration clusterConfiguration = cluster.getClusterConfiguration();

          // clone the config for cancel
          try {
              previousConfig = clusterConfiguration.clone();
          } catch (CloneNotSupportedException cnse) {
              DebugUtil.mark("clone not supported?");
          }

          numberOfStaticSlotsValueModel = new SpinnerNumberModel(
                  clusterConfiguration.gNumberOfStaticSlots(),
                  ClusterConfiguration.gNumberOfStaticSlots_min,
                  ClusterConfiguration.gNumberOfStaticSlots_max + 1,
                  1);
          numberOfStaticSlotsValueModel.addChangeListener(ccsl);
          numberOfStaticSlotsSpinner.setModel(numberOfStaticSlotsValueModel);
          durationOfStaticSlotValueModel = new SpinnerNumberModel (
                  clusterConfiguration.gdStaticSlot(),
                  ClusterConfiguration.gdStaticSlot_min,
                  ClusterConfiguration.gdStaticSlot_max + 1,
                  1);
          durationOfStaticSlotValueModel.addChangeListener(ccsl);
          durationOfStaticSlotSpinner.setModel(durationOfStaticSlotValueModel);
          numberOfMinislotsValueModel = new SpinnerNumberModel(
                  clusterConfiguration.gNumberOfMinislots(),
                  ClusterConfiguration.gNumberOfMinislots_min,
                  ClusterConfiguration.gNumberOfMinislots_max + 1,
                  1);
          numberOfMinislotsValueModel.addChangeListener(ccsl);
          numberOfMinislotsSpinner.setModel(numberOfMinislotsValueModel);
          durationOfMinislotValueModel = new SpinnerNumberModel (
                  clusterConfiguration.gdMinislot(),
                  ClusterConfiguration.gdMinislot_min,
                  ClusterConfiguration.gdMinislot_max + 1,
                  1);
          durationOfMinislotValueModel.addChangeListener(ccsl);
          durationOfMinislotSpinner.setModel(durationOfMinislotValueModel);
          nitDurationValueModel = new SpinnerNumberModel (
                  clusterConfiguration.gdNIT(),
                  ClusterConfiguration.gdNIT_min,
                  ClusterConfiguration.gdNIT_max + 1,
                  1);
          nitDurationValueModel.addChangeListener(ccsl);
          nitDurationSpinner.setModel(nitDurationValueModel);
          
  	  }

  	  // ... so jetzt den Dialog anzeigen bzw. verstecken.
  	  super.showDialog(visible);
    	  
   	  // TRACE OUTPUT
   	  if (myFrame.getLearningModule().isTraceActive()) System.out.println("END ArrowEditorDialogPN.showDialog");

    }
    
	   /*
	    * Methode:		updateDialogsColorLanguageFonts
	    * 
	    * Description:	Setzt die aktuellen Farben aus der Konfigurationbzw. und Texte aus den Ressourcen.
	    * 				Wird aufgerufen in erster Linie nach einem dynamischen Sprach- oder Farbwechsel.
	    *    
	    * Date: 		 16.03.2007
	    * @author        sirocic
	    */	
	   public void updateDialogsColorLanguageFonts() {
	      
	 	  // TRACE OUTPUT
	 	  if (myFrame.getLearningModule().isTraceActive()) System.out.println("CALL BufferEditorDialogPN.updateDialogsColorLanguageFonts:");

	 	  // SUBKOMPONENTE UPDATEN...

	 	 super.updateLanguageColorFonts();
	   	  
	  	  // TRACE OUTPUT
	  	  if (myFrame.getLearningModule().isTraceActive()) System.out.println("END BufferEditorDialogPN.updateDialogsColorLanguageFonts");

	   }
	   
	   /*
	    * Methode:		setDialogFont
	    * 
	    * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
	    * 				Werte. 
	    *    
	    * Parameters:   Aktuelle Font fr Labels und Textfelder.
	    * 
	    * Date: 		 17.01.2007
	    * @author        sirocic
	    */	
	   protected void setDialogFont() {
		   Font menuFont;
		   
		   menuFont = (Font)myFrame.getLearningModule().getConfiguration().getSettingByKey("FontMenu");
		   
		   super.setDialogFont("FontMenu");
  
		   numberOfStaticSlotsL.setFont(menuFont);	  
		   durationOfStaticSlotL.setFont(menuFont); 
		   durationOfMinislotL.setFont(menuFont);
		   numberOfMinislotsL.setFont(menuFont);
		   nitDurationL.setFont(menuFont);
		   
		   numberOfStaticSlotsSpinner.setFont(menuFont);	  
		   durationOfStaticSlotSpinner.setFont(menuFont); 
		   durationOfMinislotSpinner.setFont(menuFont);
		   numberOfMinislotsSpinner.setFont(menuFont);
		   nitDurationSpinner.setFont(menuFont);
		   macrotickStaticL.setFont(menuFont);
		   macrotickDynamicL.setFont(menuFont);
		   macrotickNITL.setFont(menuFont);
		   
	   }
	   
	   /*
	    * Methode:		setDialogTexte
	    * 
	    * Description:	Setz alle Text-Bausteine des Dialoges neu. Wichtig fr Sprachwechsel.
	    *    
	    * Parameters:   
	    * 
	    * Date: 		 26.01.2007
	    * @author        sirocic
	    */	
	   protected void setDialogTexte() {
		   String labelStr;
		   
		   super.setDialogTexte();

		   setTitle(res.getResourceByKey("dlg.commcycle.title"));
		   
		   staticSegmentConfigPanel.setBorder(BorderFactory.createTitledBorder(
	                res.getResourceByKey("dlg.staticSegment")));
		   nitPanel.setBorder(BorderFactory.createTitledBorder(
	                res.getResourceByKey("dlg.commcycle.nit")));
		   dynamicSegmentConfigPanel.setBorder(BorderFactory.createTitledBorder(
	                res.getResourceByKey("dlg.dynamicSegment")));
		  
		   if (myFrame.getLearningModule().isTooltipsVisible()) {
			   staticSegmentConfigPanel.setToolTipText(res.getResourceByKey("dlg.commcycle.staticSegmentTip"));
			   dynamicSegmentConfigPanel.setToolTipText(res.getResourceByKey("dlg.commcycle.dynamicSegmentTip"));
			   nitPanel.setToolTipText(res.getResourceByKey("dlg.commcycle.nitTip"));
			   
			   numberOfStaticSlotsSpinner.setToolTipText(res.getResourceByKey("dlg.commcycle.numberStaticSlotsTip"));
			   durationOfStaticSlotSpinner.setToolTipText(res.getResourceByKey("dlg.commcycle.durationStaticSlotsTip"));
			   numberOfMinislotsSpinner.setToolTipText(res.getResourceByKey("dlg.commcycle.numberMiniSlotsTip"));
			   durationOfMinislotSpinner.setToolTipText(res.getResourceByKey("dlg.commcycle.durationMiniSlotsTip"));
			   nitDurationSpinner.setToolTipText(res.getResourceByKey("dlg.commcycle.durationNITTip"));
		   }
		   else {
			   staticSegmentConfigPanel.setToolTipText(null);
			   dynamicSegmentConfigPanel.setToolTipText(null);
			   nitPanel.setToolTipText(null);
			   
			   numberOfStaticSlotsSpinner.setToolTipText(null);
			   durationOfStaticSlotSpinner.setToolTipText(null);
			   numberOfMinislotsSpinner.setToolTipText(null);
			   durationOfMinislotSpinner.setToolTipText(null);
			   nitDurationSpinner.setToolTipText(null);
		   }
		   
		   macrotickStaticL.setText(res.getResourceByKey("dlg.commcycle.macroticks"));
		   macrotickDynamicL.setText(res.getResourceByKey("dlg.commcycle.macroticks"));
		   macrotickNITL.setText(res.getResourceByKey("dlg.commcycle.macroticks"));
		   numberOfStaticSlotsL.setText(res.getResourceByKey("dlg.commcycle.numberOfStaticSlots"));
		   durationOfStaticSlotL.setText(res.getResourceByKey("dlg.commcycle.durationOfStaticSlots"));
		   numberOfMinislotsL.setText(res.getResourceByKey("dlg.commcycle.numberOfMiniSlots"));
		   durationOfMinislotL.setText(res.getResourceByKey("dlg.commcycle.durationOfMiniSlots"));
		   nitDurationL.setText(res.getResourceByKey("dlg.commcycle.durationOfNIT"));

	   }
	   
	   /*
	    * Methode:		setDialogColors
	    * 
	    * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
	    * 				Werte. 
	    *    
	    * Parameters:   Aktuelle Farbwerte fr Hintergrund und Vordergrund.
	    * 
	    * Date: 		 17.01.2007
	    * @author        sirocic
	    */	
	   protected void setDialogColors() {
		   Color backgroundColor,  foregroundColor;
		 
		   if (myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentBackground")!=null)
		   {
			   backgroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentBackground");
			   foregroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentForeground");
		   }
		   else {
			   backgroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorModuleBackground");
			   foregroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorModuleForeground");
		   }
		   
		   super.setDialogColors();

		   staticSegmentConfigPanel.setBackground(backgroundColor);
		   staticSegmentConfigPanel.setForeground(foregroundColor); 
		   dynamicSegmentConfigPanel.setBackground(backgroundColor);
		   dynamicSegmentConfigPanel.setForeground(foregroundColor); 	
		   nitPanel.setBackground(backgroundColor);
		   nitPanel.setForeground(foregroundColor); 	
		   
		   numberOfStaticSlotsL.setBackground(backgroundColor);
		   numberOfStaticSlotsL.setForeground(foregroundColor); 
		   durationOfStaticSlotL.setBackground(backgroundColor);
		   durationOfStaticSlotL.setForeground(foregroundColor); 	 
		   durationOfMinislotL.setBackground(backgroundColor);
		   durationOfMinislotL.setForeground(foregroundColor); 	
		   numberOfMinislotsL.setBackground(backgroundColor);
		   numberOfMinislotsL.setForeground(foregroundColor); 	
		   nitDurationL.setBackground(backgroundColor);
		   nitDurationL.setForeground(foregroundColor); 
		   macrotickStaticL.setBackground(backgroundColor);
		   macrotickDynamicL.setBackground(backgroundColor);
		   macrotickNITL.setBackground(backgroundColor);
		   macrotickStaticL.setForeground(foregroundColor); 
		   macrotickDynamicL.setForeground(foregroundColor); 
		   macrotickNITL.setForeground(foregroundColor); 
		   
	   }  
	   
	   /*
	    * Methode:		cleanTextfields
	    * 
	    * Description:	Setzt alle Eingabefelder zurck auf Defaultwerte.
	    *    
	    * Parameters:   
	    * 
	    * Date: 		 16.08.2007
	    * @author        sirocic
	    */	
	   private void cleanMe() {

	   }
	   
    
    /*
     * Methode:		fitDialog
     * 
     * Description:	Bestimmen der bentigten Panelgre fr das Anzeigen des vollstndigen Dialogs. 
     *    
     * Parameters:    
     * 
     * Date: 		 16.01.2007
     * @author        sirocic
     */	
    public void fitDialog() {
    	Graphics2D g2;
	  	  int maxWidth, maxHeight, maxPanelWidth, nitWidth, staticWidth, dynamicWidth ;
	  	  int nitHeight, staticHeight, dynamicHeight;
	  	  Rectangle2D boundsLabel;
		  FontRenderContext context;
	  	  
	  	  // TRACE OUTPUT
	  	  if (LeviModule.isTraceActive()) System.out.println("CALL BufferEditorDialogPN.fitDialog:");
	  	
	  	  // init
	  	  g2 = (Graphics2D) getGraphics();
	  	  maxWidth = 0;
		  maxHeight =0;
		  
		  if (g2 != null){
			  // Bentigte Zeilenbreite und -hhe bestimmen anhand der Textausmasse bei aktuelle
		      // eingestellten Font ...
			  g2.setFont(getFont());
		      context = g2.getFontRenderContext();

		      //  GUI-ELEMENTE AN AKTUELLE FONTSIZE UND INHALTE ANPASSEN ...
		      validate();
		      boundsLabel = getFont().getStringBounds(numberOfStaticSlotsSpinner.getValue().toString(), context);
		      if ((int)boundsLabel.getHeight() > SPINNERHEIGHT) {
		    	  // Spinner anpassen ...
		    	  numberOfStaticSlotsSpinner.setSize(SPINNERWIDTH, (int)boundsLabel.getHeight());
		          numberOfStaticSlotsSpinner.setPreferredSize(numberOfStaticSlotsSpinner.getSize());
		          numberOfStaticSlotsSpinner.setMaximumSize(numberOfStaticSlotsSpinner.getSize());
		          durationOfStaticSlotSpinner.setSize(SPINNERWIDTH, (int)boundsLabel.getHeight());
		          durationOfStaticSlotSpinner.setPreferredSize(durationOfStaticSlotSpinner.getSize());
		          durationOfStaticSlotSpinner.setMaximumSize(durationOfStaticSlotSpinner.getSize());
		          numberOfMinislotsSpinner.setSize(SPINNERWIDTH, (int)boundsLabel.getHeight());
		          numberOfMinislotsSpinner.setPreferredSize(numberOfMinislotsSpinner.getSize());
		          numberOfMinislotsSpinner.setMaximumSize(numberOfMinislotsSpinner.getSize());
		          durationOfMinislotSpinner.setSize(SPINNERWIDTH, (int)boundsLabel.getHeight());
		          durationOfMinislotSpinner.setPreferredSize(durationOfMinislotSpinner.getSize());
		          durationOfMinislotSpinner.setMaximumSize(durationOfMinislotSpinner.getSize());
		          nitDurationSpinner.setSize(SPINNERWIDTH, (int)boundsLabel.getHeight());
		          nitDurationSpinner.setPreferredSize(nitDurationSpinner.getSize());
		          nitDurationSpinner.setMaximumSize(nitDurationSpinner.getSize());
		      }
		      // statische Segment Panel anpassen !!
		      boundsLabel = getFont().getStringBounds(numberOfStaticSlotsL.getText(), context);
		      staticWidth =staticSegmentConfigPanel.getInsets().left +(int)boundsLabel.getWidth();
		      boundsLabel = getFont().getStringBounds(durationOfStaticSlotL.getText(), context);
		      staticWidth = staticWidth + (int)boundsLabel.getWidth();
		      boundsLabel = getFont().getStringBounds(macrotickStaticL.getText(), context);
		      staticWidth = staticWidth + (int)boundsLabel.getWidth();
		      staticWidth =staticWidth + numberOfStaticSlotsSpinner.getWidth() + durationOfStaticSlotSpinner.getWidth() 
		      					 + 6 * LeviFrame.ATTRIBUTOFFSET + staticSegmentConfigPanel.getInsets().right;
		      staticHeight = staticSegmentConfigPanel.getInsets().top; 
		      if (numberOfStaticSlotsSpinner.getHeight() > (int)boundsLabel.getHeight())
		    	  staticHeight = staticHeight + numberOfStaticSlotsSpinner.getHeight();
		      else
		    	  staticHeight = staticHeight + (int)boundsLabel.getHeight();
		      staticHeight = staticHeight + 3 * LeviFrame.LINEOFFSET + staticSegmentConfigPanel.getInsets().bottom;
		    	  
		      // dynamische Segment Panel anpassen !!
		      boundsLabel = getFont().getStringBounds(numberOfMinislotsL.getText(), context);
		      dynamicWidth = dynamicSegmentConfigPanel.getInsets().left+ (int)boundsLabel.getWidth();
		      boundsLabel = getFont().getStringBounds(durationOfMinislotL.getText(), context);
		      dynamicWidth =dynamicWidth + (int)boundsLabel.getWidth();
		      boundsLabel = getFont().getStringBounds(macrotickDynamicL.getText(), context);
		      dynamicWidth = dynamicWidth + (int)boundsLabel.getWidth();
		      dynamicWidth =dynamicWidth + numberOfMinislotsSpinner.getWidth() + durationOfMinislotSpinner.getWidth() 
		      					 + 6 * LeviFrame.ATTRIBUTOFFSET + dynamicSegmentConfigPanel.getInsets().right;
		      dynamicHeight = dynamicSegmentConfigPanel.getInsets().top;
		      if (numberOfMinislotsSpinner.getHeight() > (int)boundsLabel.getHeight())
		    	  dynamicHeight = dynamicHeight + numberOfMinislotsSpinner.getHeight();
		      else
		    	  dynamicHeight = dynamicHeight + (int)boundsLabel.getHeight();
		      dynamicHeight = dynamicHeight + 3 * LeviFrame.LINEOFFSET + dynamicSegmentConfigPanel.getInsets().bottom;
		      
		      // NIT Panel anpassen !!
		      boundsLabel = getFont().getStringBounds(nitDurationL.getText(), context);
		      nitWidth = nitPanel.getInsets().left + (int)boundsLabel.getWidth();
		      boundsLabel = getFont().getStringBounds(macrotickNITL.getText(), context);
		      nitWidth = nitWidth + (int)boundsLabel.getWidth();
		      nitWidth =nitWidth + nitDurationSpinner.getWidth() 
		      					 + 4 * LeviFrame.ATTRIBUTOFFSET + nitPanel.getInsets().right;
		      nitHeight = nitPanel.getInsets().top;
		      if (nitDurationSpinner.getHeight() > (int)boundsLabel.getHeight())
		    	  nitHeight = nitHeight + nitDurationSpinner.getHeight();
		      else
		    	  nitHeight = nitHeight + (int)boundsLabel.getHeight();
		      nitHeight = nitHeight + 3 * LeviFrame.LINEOFFSET + nitPanel.getInsets().bottom;
		      
		  	  // breiteste Unterpanel bestimmen 
		  	  maxPanelWidth = staticWidth;
		  	  if (dynamicWidth > maxPanelWidth)
		  		maxPanelWidth = dynamicWidth;
		  	  if (nitWidth > maxPanelWidth)
		  		maxPanelWidth = nitWidth;

		  	  // Panels anpassen
		  	  staticSegmentConfigPanel.setSize(maxPanelWidth, staticHeight);
		      staticSegmentConfigPanel.setPreferredSize(staticSegmentConfigPanel.getSize());
		      staticSegmentConfigPanel.setMaximumSize(staticSegmentConfigPanel.getSize());
		      dynamicSegmentConfigPanel.setSize(maxPanelWidth, dynamicHeight);
		      dynamicSegmentConfigPanel.setPreferredSize(dynamicSegmentConfigPanel.getSize());
		      dynamicSegmentConfigPanel.setMaximumSize(dynamicSegmentConfigPanel.getSize());
		      nitPanel.setSize(maxPanelWidth, nitHeight);
		      nitPanel.setPreferredSize(nitPanel.getSize());
		      nitPanel.setMaximumSize(nitPanel.getSize());
		      
		      // BENTIGTE Dialog-BREITE BESTIMMEN ....
		      maxWidth = maxPanelWidth + 2 * LeviFrame.ATTRIBUTOFFSET;
		      
		  	  // BENTIGTE DIALOG-HHE BESTIMMEN ....
		  	  maxHeight =  3* LeviFrame.LINEOFFSET; 
		  	  maxHeight = maxHeight + nitPanel.getHeight() + staticSegmentConfigPanel.getHeight() 
		  	  					    + dynamicSegmentConfigPanel.getHeight();
		  	
			  // Panelgre aktualisieren ...
		  	  getRootPane().setSize(maxWidth,maxHeight);
		  	  getRootPane().setMaximumSize(new Dimension(maxWidth,maxHeight));
		  	  getRootPane().setPreferredSize(new Dimension(maxWidth,maxHeight));
 	  }
		  
	  super.fitDialog();
 	  
   	  // TRACE OUTPUT
   	  if (LeviModule.isTraceActive()) System.out.println("END ResourceEditorRTS.fitPanel");
   	
    }


    /**
     *
     * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     */
    public void actionPerformed(ActionEvent e) {
        if( e.getActionCommand().equals(OKAction.COMMAND)) {
            scenario.setLastEdit(EditCommandType.CONFIGURE_COMMCYCLE);
            scenario.fireStateChanged();
        }

        if( e.getActionCommand().equals(CancelAction.COMMAND)) {
            cluster.setClusterConfiguration(previousConfig);
            scenario.setLastEdit(EditCommandType.CONFIGURE_COMMCYCLE);
            scenario.fireStateChanged();
        }
        dispose();
    }

}
