/*
 * $Id: NodeConfigDialog.java,v 1.2 2009/04/28 04:48:55 sirocic Exp $
 *
 * Created on 08.05.2006 15:45:26
 * by Dirk Frsterling <r@zorbla.de>
 */
package flexray.gui.dialogs;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.font.FontRenderContext;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.SortedSet;
import java.util.TreeSet;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JCheckBox;
import javax.swing.JColorChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import learningVisualizations.LeviModule;
import learningVisualizations.gui.LeviChooseButton;
import learningVisualizations.gui.LeviColorDialog;
import learningVisualizations.gui.LeviFrame;

import flexray.ModuleFR;
import flexray.cluster.Node;
import flexray.configuration.ChannelName;
import flexray.configuration.ClusterConfiguration;
import flexray.configuration.NodeConfiguration;
import flexray.gui.ChannelConfigPanelFR;
import flexray.gui.NodeComponent;
import flexray.gui.actions.CancelAction;
import flexray.gui.actions.OKAction;
import flexray.gui.controller.EditCommandType;
import flexray.scenario.FlexRayScenario;

/**
 * @author Dirk Frsterling <r@zorbla.de>
 *
 */
public class NodeConfigDialog extends AbstractFlexRayDialog
                           implements ActionListener {

	public final static int TFNAMEWIDTH = 70;
	
    private NodeComponent nodeComponent;
    private Node node;

    private NodeConfiguration nodeConfiguration = null;
    private ClusterConfiguration clusterConfiguration = null;

    /*
     * Dialog elements
     */
    JPanel colorPreview;
    JLabel channelL, nameL;
    JTextField nameTF;
    LeviChooseButton colorButton;
    JCheckBox[] channelsCB= new JCheckBox[2];
    ChannelConfigPanelFR[] channelConfigPanels = new ChannelConfigPanelFR[2];
    Box channelHB, nodeHB, channelBehaviorHB, channelBehaviorVB;
    DynConfigListener dynConfigAListener, dynConfigBListener;
    
    /**
     * Dialog for reviewing and changing the configuration data
     * relevant to one selected FlexRay node.
     *
     * @param frame the owner of this dialog
     * @param theNodeComponent the node component that was clicked
     */
    public NodeConfigDialog(NodeComponent theNodeComponent ) {
        super((FlexRayScenario)ModuleFR.getModule().getScenario(), "dlg.nodeconfig.title");

        // retreive configuration data
        nodeComponent = theNodeComponent;
        node = nodeComponent.getNode();
        nodeConfiguration = node.getNodeConfiguration();
        clusterConfiguration = node.getCluster().getClusterConfiguration();
        
        // Boxen erzeugen ...
 	    contentVB.setAlignmentX(Component.LEFT_ALIGNMENT);
 	    buttonHB.setAlignmentX(Component.LEFT_ALIGNMENT);
        channelHB = Box.createHorizontalBox();
        channelHB.setAlignmentX(Component.LEFT_ALIGNMENT);
        nodeHB = Box.createHorizontalBox();
        nodeHB.setAlignmentX(Component.LEFT_ALIGNMENT);
        channelBehaviorHB = Box.createHorizontalBox();
        channelBehaviorHB.setAlignmentX(Component.LEFT_ALIGNMENT);
        channelBehaviorVB = Box.createVerticalBox();
        channelBehaviorVB.setAlignmentX(Component.LEFT_ALIGNMENT);
 
        /*
         *  create the configuration pane for channel connectivity
         */
        // Eingabe Knotenname und -farbe
        nameL = new JLabel();
        nameL.setAlignmentX(Component.LEFT_ALIGNMENT);
        nameTF = new JTextField(10);
        nameTF.setAlignmentX(Component.LEFT_ALIGNMENT);
        nodeHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        nodeHB.add(nameL);
        nodeHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        nodeHB.add(nameTF);
        nodeHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        colorPreview = new JPanel();
        colorPreview.setAlignmentX(Component.LEFT_ALIGNMENT);
        Dimension size = new Dimension(24,24);
        colorPreview.setMinimumSize(size);
        colorPreview.setBackground(nodeComponent.getNodeColor());
        colorPreview.setBorder(BorderFactory.createLineBorder(getForeground(), 1));
        nodeHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        nodeHB.add(colorPreview);
        nodeHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        colorButton =
            // BS 16.11 Icons aktualisiert !!!
            new LeviChooseButton("button.color", myFrame.getLearningModule(), colorPreview, null);

        	colorButton.addActionListener(new ActionListener() {
                            public void actionPerformed(ActionEvent e) {
                                LeviChooseButton selectedButton;
                                Color color;

                                // get the selected button
                                selectedButton = (LeviChooseButton)e.getSource();
                                // show the color choose dialog
                                color=JColorChooser.showDialog(myself,
                                        res.getResourceByKey("dlg.color.choose.title"),
                                        selectedButton.getPreview().getBackground());
                                if (color!=null)
                                    selectedButton.getPreview().setBackground(color);
                            }
                        });
        colorButton.setAlignmentX(Component.LEFT_ALIGNMENT);
        nodeHB.add(colorButton);
        nodeHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));

        // Eingabe der Verbindung des Knotens mit den Kanlen
        channelL = new JLabel();
        channelL.setAlignmentX(Component.LEFT_ALIGNMENT);
        channelsCB[ChannelName.A.ordinal()] = createChannelCheckbox(ChannelName.A);
        channelsCB[ChannelName.A.ordinal()].setAlignmentX(Component.LEFT_ALIGNMENT);
        channelsCB[ChannelName.B.ordinal()] = createChannelCheckbox(ChannelName.B);
        channelsCB[ChannelName.B.ordinal()].setAlignmentX(Component.LEFT_ALIGNMENT);
        
        channelHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        channelHB.add(channelL);
        channelHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        channelHB.add(channelsCB[ChannelName.A.ordinal()]);
        channelHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        channelHB.add(channelsCB[ChannelName.B.ordinal()]);
        channelHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
         
        // Eingabe Verhalten auf Kanal A 
        channelConfigPanels[ChannelName.A.ordinal()] = new ChannelConfigPanelFR(myFrame.getLearningModule(), ChannelName.A, node);
        // ... und Kanal B
        channelConfigPanels[ChannelName.B.ordinal()] = new ChannelConfigPanelFR(myFrame.getLearningModule(), ChannelName.B, node);
        channelConfigPanels[ChannelName.A.ordinal()].setOtherChannelConfigPanel(channelConfigPanels[ChannelName.B.ordinal()]);
        channelConfigPanels[ChannelName.B.ordinal()].setOtherChannelConfigPanel(channelConfigPanels[ChannelName.A.ordinal()]);
        
        dynConfigAListener = new DynConfigListener(ChannelName.A);
        channelConfigPanels[ChannelName.A.ordinal()].setDynConfigListener(dynConfigAListener);
        channelsCB[ChannelName.A.ordinal()].addActionListener(dynConfigAListener);
        dynConfigBListener = new DynConfigListener(ChannelName.B);
        channelConfigPanels[ChannelName.B.ordinal()].setDynConfigListener(dynConfigBListener);
        channelsCB[ChannelName.B.ordinal()].addActionListener(dynConfigBListener);

        //      Abbruch und Ok-Button einfgen
        cancelButton.addActionListener(this);
        cancelButton.setActionCommand(CancelAction.COMMAND);
		okButton.addActionListener(this);
		okButton.setActionCommand(OKAction.COMMAND);
		
		// FARBEN, FONTS UND TEXTE SETZEN ...
	 	updateLanguageColorFonts();

    }
 
    /**
	 * Set the config values for one channel.
	 * FIXME: the slot configuration has to be updated.
	 * @param channel
	 */
	private void setValuesForChannel(ChannelName channel) {
	    int channelOrd = channel.ordinal();
	    ArrayList nodesForSlot;
	    Node nodeForSlot;
	    
	    if( channelsCB[channelOrd].isSelected()) {
	        node.addChannel(channel);
	        // channel was added, now look who is configured to transmit there in
	        // the same slot
	        for(Integer slot : node.getTransmitSlots()) {
	            Node otherNode = node.getCluster().getNodeForSlot(channel, slot.intValue());
	            if( (otherNode != null) && (!node.equals(otherNode))) {
	                otherNode.removeTransmitSlot(slot.intValue());
	            }
	        }
	    }
	    
	    node.removeTransmitSlot(channelConfigPanels[channelOrd].getStaticSlotNode());
	    if ((channelConfigPanels[channelOrd].getValueSelectedStaticSlot() >= 0) && 
	    		(node.getNodeConfiguration().pChannels().contains(channel))) {
	    	// Wenn der Slot schon besetzt war, dann muss bei diesen Knoten der Slot zurckgesetzt werden.
	    	nodesForSlot = cluster.getNodesForSlot(channel, channelConfigPanels[channelOrd].getValueSelectedStaticSlot());
	    	if (nodesForSlot != null) {
	    		for (int i=0; i<nodesForSlot.size();i++) {
	    			nodeForSlot = (Node)nodesForSlot.get(i);
	    			nodeForSlot.removeTransmitSlot(channelConfigPanels[channelOrd].getValueSelectedStaticSlot());
	    		}
	    	}
	    	// jetzt alle neuen Werte bernehmen
	    	node.addTransmitSlot(channelConfigPanels[channelOrd].getValueSelectedStaticSlot());
	        node.setSendStatCycleMin(channel, channelConfigPanels[channelOrd].getValueFromStatCycleSpinner());
	        node.setSendStatCycleMax(channel, channelConfigPanels[channelOrd].getValueToStatCycleSpinner());
	    }
	    else if (channelConfigPanels[0].getStaticSlotNode() == channelConfigPanels[1].getStaticSlotNode())
	    	node.addTransmitSlot(channelConfigPanels[channelOrd].getStaticSlotNode());
	    
	    node.removeTransmitSlot(channelConfigPanels[channelOrd].getDynamicSlotNode());
	    if ((channelConfigPanels[channelOrd].getValueSelectedDynamicSlot() >= 0) && 
			(node.getNodeConfiguration().pChannels().contains(channel))) {
	    	//  Wenn der Slot schon besetzt war, dann muss bei diesen Knoten der Slot zurckgesetzt werden.
	    	nodesForSlot = cluster.getNodesForSlot(channel, channelConfigPanels[channelOrd].getValueSelectedDynamicSlot());
	    	if (nodesForSlot != null) {
	    		for (int i=0; i<nodesForSlot.size();i++) {
	    			nodeForSlot = (Node)nodesForSlot.get(i);
	    			nodeForSlot.removeTransmitSlot(channelConfigPanels[channelOrd].getValueSelectedDynamicSlot());
	    		}
	    	}
	    	// jetzt alle neuen Werte bernehmen
	    	node.addTransmitSlot(channelConfigPanels[channelOrd].getValueSelectedDynamicSlot());
	        node.setDynamicSend(channel, channelConfigPanels[channelOrd].isSelectedDynSend());
	        node.setSendLengthMin(channel, channelConfigPanels[channelOrd].getValueFromDynLengthSpinner());
	        node.setSendLengthMax(channel, channelConfigPanels[channelOrd].getValueToDynLengthSpinner());  
	        node.setSendDynCycleMin(channel, channelConfigPanels[channelOrd].getValueFromDynCycleSpinner());
	        node.setSendDynCycleMax(channel, channelConfigPanels[channelOrd].getValueToDynCycleSpinner());
	        node.setDynRequestMin(channel, channelConfigPanels[channelOrd].getValueFromDynRequestSpinner());
	        node.setDynRequestMax(channel, channelConfigPanels[channelOrd].getValueToDynRequestSpinner());
	    }
	    else if (channelConfigPanels[0].getDynamicSlotNode() == channelConfigPanels[1].getDynamicSlotNode())
	    	node.addTransmitSlot(channelConfigPanels[channelOrd].getDynamicSlotNode());
	    
	    if( !nameTF.getText().equals("") ) {
	        node.setName(nameTF.getText());
	    } else {
	        node.setName(null);
	    }
	    nodeComponent.updateNodeNameDisplay();
	    
	} // setValuesForChannel

	/*
     * Methode:		showDialog
     * 
     * Description:	Setzt die aktuellen Farben aus der Konfiguration bevor der Dialog angezeigt bzw.
     * 				wieder versteckt wird.
     *    
     * Parameters:    
     * 
     * Date: 		 11.12.2007
     * @author        sirocic
     */	
    public void showDialog(boolean visible) {
       
  	  // TRACE OUTPUT
  	  if (myFrame.getLearningModule().isTraceActive()) System.out.println("CALL ArrowEditorDialogPN.showDialog:");
  	  
  	  // alle Taskparametereingaben zurcksetzen.
  	  cleanMe();
    	  
  	  if (visible) {
  		  // Dialog wird sichtbar gemacht.
  		  
  		  // Werte des aktuellen Knotens anzeigen ???
  		  nameTF.setText(node.getName());
		   
  		  // Verbindung mit den Kanlen setzen ...
  		 if( nodeConfiguration.pChannels().contains(ChannelName.A) ) 
  			channelsCB[ChannelName.A.ordinal()].setSelected(true);
         if( nodeConfiguration.pChannels().contains(ChannelName.B) ) 
        	 channelsCB[ChannelName.B.ordinal()].setSelected(true);
         
         channelBehaviorHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
         channelConfigPanels[ChannelName.A.ordinal()].showPanel(true);
         if( channelsCB[ChannelName.A.ordinal()].isSelected() ) {
  			 channelBehaviorHB.add(channelConfigPanels[ChannelName.A.ordinal()]);
  	  	     channelBehaviorHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
  		 }
         channelConfigPanels[ChannelName.B.ordinal()].showPanel(true);
         if( channelsCB[ChannelName.B.ordinal()].isSelected() ) 
        	 channelBehaviorHB.add(channelConfigPanels[ChannelName.B.ordinal()]);
         if( channelsCB[ChannelName.A.ordinal()].isSelected() || channelsCB[ChannelName.B.ordinal()].isSelected()) {
        	 channelBehaviorVB.add(channelBehaviorHB);
  	     }
         
         //  Vertikale Box des Dialogs fllen ...
         contentVB.removeAll();
         contentVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
  	     contentVB.add(nodeHB);
  	     contentVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
 	     contentVB.add(channelHB);
 	     if( channelsCB[ChannelName.A.ordinal()].isSelected() || channelsCB[ChannelName.B.ordinal()].isSelected()) {
 	 	    contentVB.add(channelBehaviorVB);
 	     }
         contentVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
   	     contentVB.add(buttonHB);
   	     contentVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
        
  	  }

  	  // ... so jetzt den Dialog anzeigen bzw. verstecken.
  	  super.showDialog(visible);
    	  
   	  // TRACE OUTPUT
   	  if (myFrame.getLearningModule().isTraceActive()) System.out.println("END ArrowEditorDialogPN.showDialog");

    }
    
	   /*
	    * Methode:		updateDialogsColorLanguageFonts
	    * 
	    * Description:	Setzt die aktuellen Farben aus der Konfigurationbzw. und Texte aus den Ressourcen.
	    * 				Wird aufgerufen in erster Linie nach einem dynamischen Sprach- oder Farbwechsel.
	    *    
	    * Date: 		 16.03.2007
	    * @author        sirocic
	    */	
	   public void updateDialogsColorLanguageFonts() {
	      
	 	  // TRACE OUTPUT
	 	  if (myFrame.getLearningModule().isTraceActive()) System.out.println("CALL BufferEditorDialogPN.updateDialogsColorLanguageFonts:");

	 	  // SUBKOMPONENTE UPDATEN...
	 	  channelConfigPanels[ChannelName.A.ordinal()].updateColorLanguageFonts();
	 	  channelConfigPanels[ChannelName.B.ordinal()].updateColorLanguageFonts();
	 	 
	 	  super.updateLanguageColorFonts();
	   	  
	  	  // TRACE OUTPUT
	  	  if (myFrame.getLearningModule().isTraceActive()) System.out.println("END BufferEditorDialogPN.updateDialogsColorLanguageFonts");

	   }
	   
	   /*
	    * Methode:		setDialogFont
	    * 
	    * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
	    * 				Werte. 
	    *    
	    * Parameters:   Aktuelle Font fr Labels und Textfelder.
	    * 
	    * Date: 		 17.01.2007
	    * @author        sirocic
	    */	
	   protected void setDialogFont() {
		   Font menuFont;
		   
		   menuFont = (Font)myFrame.getLearningModule().getConfiguration().getSettingByKey("FontMenu");
		   
		   super.setDialogFont("FontMenu");
  
		   nameTF.setFont(menuFont);
		   nameL.setFont(menuFont);
		   channelL.setFont(menuFont);
		   channelsCB[ChannelName.A.ordinal()].setFont(menuFont);
	       channelsCB[ChannelName.B.ordinal()].setFont(menuFont);
	        
		  
	   }
	   
	   /*
	    * Methode:		setDialogTexte
	    * 
	    * Description:	Setz alle Text-Bausteine des Dialoges neu. Wichtig fr Sprachwechsel.
	    *    
	    * Parameters:   
	    * 
	    * Date: 		 26.01.2007
	    * @author        sirocic
	    */	
	   protected void setDialogTexte() {
		   String title, borderTitle;
		   TitledBorder channelBorder, colorBorder, nodeNameBorder;
		   
		   super.setDialogTexte();

		   title = myFrame.getLearningModule().getRessources().getResourceByKey("dlg.nodeconfig.title");
	       title = title.replace("%s", Integer.valueOf(nodeComponent.getNode().getID()).toString());
	       setTitle( title );
		   
	       channelL.setText(res.getResourceByKey("dlg.nodeconfig.connections"));
	       nameL.setText(res.getResourceByKey("dlg.nodeconfig.nodename"));
	       borderTitle = res.getResourceByKey("dlg.nodeconfig.color");
	       channelsCB[ChannelName.A.ordinal()].setText(" " + res.getResourceByKey("flexray.channel") + " " + ChannelName.A);
	       channelsCB[ChannelName.B.ordinal()].setText(" " + res.getResourceByKey("flexray.channel") + " " + ChannelName.B);
	       
	       channelConfigPanels[ChannelName.A.ordinal()].setBorder(new TitledBorder( res.getResourceByKey("dlg.nodeconfig.channelABehaviour")));
	       channelConfigPanels[ChannelName.B.ordinal()].setBorder(new TitledBorder(res.getResourceByKey("dlg.nodeconfig.channelBBehaviour")));
	       
	   }
	   
	   /*
	    * Methode:		setDialogColors
	    * 
	    * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
	    * 				Werte. 
	    *    
	    * Parameters:   Aktuelle Farbwerte fr Hintergrund und Vordergrund.
	    * 
	    * Date: 		 17.01.2007
	    * @author        sirocic
	    */	
	   protected void setDialogColors() {
		   Color backgroundColor,  foregroundColor;
			 
		   if (myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentBackground")!=null)
		   {
			   backgroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentBackground");
			   foregroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentForeground");
		   }
		   else {
			   backgroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorModuleBackground");
			   foregroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorModuleForeground");
		   }
		   
		   super.setDialogColors();
		   
		   channelsCB[ChannelName.A.ordinal()].setBackground(backgroundColor);
		   channelsCB[ChannelName.A.ordinal()].setForeground(foregroundColor);
		   channelsCB[ChannelName.B.ordinal()].setBackground(backgroundColor);
		   channelsCB[ChannelName.B.ordinal()].setForeground(foregroundColor);
		  
	   }  
	   
	   /*
	    * Methode:		cleanMe
	    * 
	    * Description:	Setzt alle Eingabe- und Auswahlfelder usw. zurck auf Defaultwerte.
	    *    
	    * Parameters:   
	    * 
	    * Date: 		 28.12.2007
	    * @author        sirocic
	    */	
	   private void cleanMe() {

		   channelsCB[ChannelName.A.ordinal()].setSelected(false);
		   channelsCB[ChannelName.B.ordinal()].setSelected(false);
		   
	   }
	   
    
    /*
     * Methode:		fitDialog
     * 
     * Description:	Bestimmen der bentigten Panelgre fr das Anzeigen des vollstndigen Dialogs. 
     *    
     * Parameters:    
     * 
     * Date: 		 16.01.2007
     * @author        sirocic
     */	
    public void fitDialog() {
    	Graphics2D g2;
	  	  int maxWidth, maxHeight, maxPanelWidth, tmpWidth;
	  	  int labelHeight;
	  	  Rectangle2D boundsLabel;
		  FontRenderContext context;
	  	  
	  	  // TRACE OUTPUT
	  	  if (LeviModule.isTraceActive()) System.out.println("CALL BufferEditorDialogPN.fitDialog:");
	  	
	  	  // init
	  	  g2 = (Graphics2D) getGraphics();
	  	  maxWidth = 0;
		  maxHeight =0;
		  
		  if (g2 != null){
			  // Bentigte Zeilenbreite und -hhe bestimmen anhand der Textausmasse bei aktuelle
		      // eingestellten Font ...
			  g2.setFont(getFont());
		      context = g2.getFontRenderContext();

		      //  GUI-ELEMENTE AN AKTUELLE FONTSIZE UND INHALTE ANPASSEN ...
		      validate();
		      // Nameneingabefeld ...
		      boundsLabel = getFont().getStringBounds(nameTF.getText(), context);
		      if ((nameTF.getText()!=null) && ((int)boundsLabel.getWidth() > LeviFrame.TEXTTFWIDTH)) {
		    	  nameTF.setSize((int)boundsLabel.getWidth()+LeviFrame.TFOFFSET, (int)boundsLabel.getHeight()+LeviFrame.TFOFFSET);
		    	  nameTF.setMaximumSize(nameTF.getSize());
		    	  nameTF.setPreferredSize(nameTF.getSize());
		      }
		      else {
		    	  nameTF.setSize(LeviFrame.TEXTTFWIDTH, LeviFrame.TFHEIGHT);
		    	  nameTF.setMaximumSize(nameTF.getSize());
		    	  nameTF.setPreferredSize(nameTF.getSize());
		      }
		      // ColorPreview-Anzeige...
		      colorPreview.setSize(LeviColorDialog.PREVIEWSIZE, LeviColorDialog.PREVIEWSIZE);
		      colorPreview.setMaximumSize(colorPreview.getSize());
		      colorPreview.setPreferredSize(colorPreview.getSize());
	    	  
		      if( channelsCB[ChannelName.A.ordinal()].isSelected() ) 
		    	  // Panel Kanal A ... 
			      channelConfigPanels[ChannelName.A.ordinal()].fitPanel();
	          if( channelsCB[ChannelName.B.ordinal()].isSelected() )
	        	 // Panel Kanal B ... 
			      channelConfigPanels[ChannelName.B.ordinal()].fitPanel();
		      
		  	  // BREITE DES DIALOG-CONTENTPANEL BESTIMMEN ...
		      boundsLabel = getFont().getStringBounds(nameL.getText(), context);
		      maxPanelWidth = (int) boundsLabel.getWidth() + colorButton.getWidth() + nameTF.getWidth()
		      								+ colorPreview.getWidth() + 7* LeviFrame.ATTRIBUTOFFSET;
		      boundsLabel = getFont().getStringBounds(channelL.getText(), context);
		      tmpWidth = (int) boundsLabel.getWidth() + channelsCB[ChannelName.A.ordinal()].getWidth() 
		      								+ channelsCB[ChannelName.B.ordinal()].getWidth()+ 5* LeviFrame.ATTRIBUTOFFSET;
		      if (tmpWidth > maxPanelWidth)
		  		maxPanelWidth = tmpWidth;
		      
		      tmpWidth= 3*LeviFrame.ATTRIBUTOFFSET;
		      if( channelsCB[ChannelName.A.ordinal()].isSelected() )
		    		  tmpWidth = tmpWidth+channelConfigPanels[ChannelName.A.ordinal()].getWidth();
	          if( channelsCB[ChannelName.B.ordinal()].isSelected() ) 
	            	  tmpWidth = tmpWidth+channelConfigPanels[ChannelName.B.ordinal()].getWidth();
	          if (tmpWidth > maxPanelWidth)
	        	  maxWidth = tmpWidth;
	          else
	        	  maxWidth = maxPanelWidth;
		      
		      // HHE DES DIALOG-CONTENTPANEL BESTIMMEN ...
		      boundsLabel = getFont().getStringBounds(nameL.getText(), context);
		      labelHeight = (int) boundsLabel.getHeight();
		  	  if (labelHeight > nameTF.getHeight())
		  		  maxHeight = labelHeight;
		  	  else
		  		maxHeight = nameTF.getHeight();
		  	  if ( colorButton.getHeight() > maxHeight)
		  		maxHeight = colorButton.getHeight();
		  	  maxHeight = maxHeight + channelsCB[ChannelName.A.ordinal()].getHeight()
		  	  					    + LeviFrame.LINEOFFSET;
		  	  if( channelsCB[ChannelName.A.ordinal()].isSelected() || channelsCB[ChannelName.B.ordinal()].isSelected()) {
		    	// Hhe Panel Kanal A ... 
		  		maxHeight = maxHeight + labelHeight+ LeviFrame.LINEOFFSET;
		  		if( channelsCB[ChannelName.A.ordinal()].isSelected()) 
		  			maxHeight = maxHeight + channelConfigPanels[ChannelName.A.ordinal()].getHeight();
		  		else
		  			maxHeight = maxHeight + channelConfigPanels[ChannelName.B.ordinal()].getHeight();
				    
		  	  }
		  	  else maxHeight = maxHeight + LeviFrame.LINEOFFSET;

			  // Panelgre aktualisieren ...
		  	  getRootPane().setSize(maxWidth,maxHeight);
		  	  getRootPane().setMaximumSize(new Dimension(maxWidth,maxHeight));
		  	  getRootPane().setPreferredSize(new Dimension(maxWidth,maxHeight));

 	  }
	
	  super.fitDialog();
 	  
   	  // TRACE OUTPUT
   	  if (LeviModule.isTraceActive()) System.out.println("END ResourceEditorRTS.fitPanel");
   	
    }



    /**
     * Create a checkbox to select / unselect one channel.
     * @param channel
     * @return
     */
    private JCheckBox createChannelCheckbox(ChannelName channel) {
        int channelOrd = channel.ordinal();
        JCheckBox jcbChannelActive;
        
        jcbChannelActive = new JCheckBox();

        ChannelName gChannels = clusterConfiguration.gChannels();

        if( !gChannels.contains(channel))
            jcbChannelActive.setEnabled(false);

        if( nodeConfiguration.pChannels().contains(channel))
            jcbChannelActive.setSelected(true);

        return jcbChannelActive;
    } // createChannelCheckbox


    /**
     * actionPerformed for the listener part
     *
     * @param e
     * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     */
    public void actionPerformed(ActionEvent e) {
        // cancel -> do nothing
        if(e.getActionCommand().equals(CancelAction.COMMAND)) dispose();

        // OK -> get data
        if(e.getActionCommand().equals(OKAction.COMMAND)) {
            NodeConfiguration config = node.getNodeConfiguration();
            // FIXME: This has to work better.
            config.pChannels(ChannelName.NONE);
            setValuesForChannel(ChannelName.A);
            setValuesForChannel(ChannelName.B);
            nodeComponent.setNodeColor(colorPreview.getBackground());

            ((FlexRayScenario)myFrame.getLearningModule().getScenario()).setLastEdit(EditCommandType.CONFIGURE_NODE);
            ((FlexRayScenario)myFrame.getLearningModule().getScenario()).fireStateChanged();
        }

        dispose();
    }  // actionPerformed

    /**
     * Watch the spinners and correct their values to keep
     * meaningful settings.
     *
     * @author Dirk Frsterling <r@zorbla.de>
     */
    public class DynConfigListener implements ActionListener, ChangeListener {

        private ChannelName channel;
        private int channelOrd;

        public DynConfigListener(ChannelName channel) {
            this.channel = channel;
            this.channelOrd = this.channel.ordinal();
        }

        /**
         * changeListener part. Dispatch this to actionPerformed()
         * @param e
         * @see javax.swing.event.ChangeListener#stateChanged(javax.swing.event.ChangeEvent)
         */
        public void stateChanged( ChangeEvent e ) {
            ActionEvent ae = new ActionEvent(e.getSource(),0,"stateChange");
            actionPerformed(ae);
        }

        /**
         * watch over the meaningfulness of the values.
         * @param e
         * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
         */
        public void actionPerformed(ActionEvent e) {
            if( e.getSource() == channelsCB[channelOrd]) {
                boolean active = channelsCB[channelOrd].isSelected() &&
                channelsCB[channelOrd].isEnabled();

                // Weiss nicht wozu das gebraucht wird??? lass es aber mal stehen ... BS
                if( active ) {
                    // No better idea for now than to hard wire this.
                    StringBuffer text = new StringBuffer();
                    text.append(String.format(res.getResourceByKey("dlg.nodeconfig.slotwarning.text"),channel));
                    boolean conflict = false;
                    Vector<Integer> affectedNodes = new Vector<Integer>();
                    SortedSet<Integer> sortAffectedNodes = new TreeSet<Integer>();
                    for(Integer slot : node.getTransmitSlots()) {
                        Node otherNode = node.getCluster().getNodeForSlot(channel, slot.intValue());
                        if( (otherNode != null) && (!node.equals(otherNode))) {
                            conflict = true;
                            sortAffectedNodes.add(Integer.valueOf(otherNode.getID()));
                        }
                    }
                    affectedNodes.addAll(sortAffectedNodes);

                    if( conflict ) {
                        int end = affectedNodes.size()-1;
                        int and = end-1;

                        for( int i = 0 ; i <= end ; i++) {
                            text.append(" " + affectedNodes.elementAt(i));
                            if( (i >= 0) && (i<and) ) text.append(",");
                            if( (i>=and) && (i<end) ) text.append(" " + res.getResourceByKey("and"));
                        }

                        int result = JOptionPane.showConfirmDialog(myself,
                                text.toString(),
                                res.getResourceByKey("dlg.nodeconfig.slotwarning.title"),
                                JOptionPane.OK_CANCEL_OPTION);
                        if( result == JOptionPane.CANCEL_OPTION ) {
                            channelsCB[channelOrd].setSelected(false);
                            return;
                        }
                    }
                    ((FlexRayScenario)myFrame.getLearningModule().getScenario()).setLastEdit(EditCommandType.NODE_CHANNEL_SELECTED);
                    ((FlexRayScenario)myFrame.getLearningModule().getScenario()).fireStateChanged();
                }
                
            } // source = ChannelCB

            //  Vertikale Box des Dialogs aktualisieren ...
            channelBehaviorHB.removeAll();
            channelBehaviorVB.removeAll();
            channelBehaviorHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
            if( channelsCB[ChannelName.A.ordinal()].isSelected() ) {
     			 channelBehaviorHB.add(channelConfigPanels[ChannelName.A.ordinal()]);
     	  	     channelBehaviorHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
     		 }
            if( channelsCB[ChannelName.B.ordinal()].isSelected() ) {
           	  	channelBehaviorHB.add(channelConfigPanels[ChannelName.B.ordinal()]);
            }
            if( channelsCB[ChannelName.A.ordinal()].isSelected() || channelsCB[ChannelName.B.ordinal()].isSelected()) {
           	    channelBehaviorVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
           	    channelBehaviorVB.add(channelBehaviorHB);
     	     
           	   //  Vertikale Box des Dialogs fllen ...
           	   contentVB.removeAll();
               contentVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
         	   contentVB.add(nodeHB);
         	   contentVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
        	   contentVB.add(channelHB);
        	   contentVB.add(channelBehaviorVB);
               contentVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
          	   contentVB.add(buttonHB);
          	   contentVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
               
            }
            
      	    fitDialog();
      	    
            channelConfigPanels[channelOrd].setValues(e);
            
        } // actionPerformed
        
    } // DynConfigListener
}
