/*
 * $Id: SlotDialog.java,v 1.2 2009/05/02 18:36:31 sirocic Exp $
 *
 * Created on 24.05.2006 07:12:14
 * by Dirk Frsterling <r@zorbla.de>
 */
package flexray.gui.dialogs;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.font.FontRenderContext;
import java.awt.geom.Rectangle2D;
import java.util.Enumeration;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.ButtonGroup;
import javax.swing.ButtonModel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.border.TitledBorder;

import learningVisualizations.LeviModule;
import learningVisualizations.gui.LeviFrame;


import flexray.cluster.Node;
import flexray.configuration.ChannelName;
import flexray.gui.actions.CancelAction;
import flexray.gui.actions.OKAction;
import flexray.gui.controller.EditCommandType;
import flexray.scenario.FlexRayScenario;
import flexray.util.DebugUtil;

/**
 * @author Dirk Frsterling <r@zorbla.de>
 *
 */
public class SlotDialog extends AbstractFlexRayDialog {

    private int slot = -1;
    private ChannelName channel = ChannelName.NONE;
    private ButtonGroup nodeButtonGroup = new ButtonGroup();
    private int nodeIDForSlot = 0;

    private JPanel nodeButtonPanel;
    private JRadioButton unusedNodeButton, nodeButton;
    Box nodesVB, nodesHB;
    
    /**
     * Create a selection dialog for the node/slot-Configuration based
     * on the channel+Slot information ("select a node")
     *
     * @param pScenario
     * @param channel
     * @param slot
     */
    public SlotDialog(FlexRayScenario pScenario, ChannelName channel, int slot) {
    	
        super(pScenario, "dlg.slotconfig.title");
        
        String nodeName;
        
        this.slot = slot;
        this.channel = channel;
        
        // Boxen erzeugen ...
 	    contentVB.setAlignmentX(Component.LEFT_ALIGNMENT);
 	    buttonHB.setAlignmentX(Component.LEFT_ALIGNMENT);
 	    nodesVB = Box.createVerticalBox();
 	    nodesVB.setAlignmentX(Component.LEFT_ALIGNMENT);
 	    nodesHB = Box.createHorizontalBox();
 	    nodesHB.setAlignmentX(Component.LEFT_ALIGNMENT);
        
        /*
         * Create the radiobuttons for all nodes.
         */
        nodeButtonPanel = new JPanel();
        nodeButtonPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
        nodeButtonPanel.setLayout(new BorderLayout());
 
        // "unused" button
        unusedNodeButton = new JRadioButton();
        unusedNodeButton.setAlignmentX(Component.LEFT_ALIGNMENT);
        unusedNodeButton.setActionCommand("0");
        unusedNodeButton.setFocusPainted(false);
        unusedNodeButton.setBorderPainted(true);
        unusedNodeButton.setBorder(BorderFactory.createEtchedBorder());
        
        nodesVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
        nodesVB.add(unusedNodeButton);
        nodeButtonGroup.add(unusedNodeButton);
        // node buttons
        for( Node node : cluster.getNodes() ) {
            if( node.getNodeConfiguration().pChannels().contains(channel)) {
                nodeButton = new JRadioButton();
                nodeButton.setActionCommand(Integer.valueOf(node.getID()).toString() );
                nodeButton.setFocusPainted(false);
                nodeButton.setBorderPainted(true);
                nodeButton.setBorder(BorderFactory.createLineBorder(scenario.getNodeComponent(node).getNodeColor()));
                if(node.isTransmittingInSlot(slot) ) {
                    nodeIDForSlot = node.getID();
                    nodeButton.setSelected(true);
                }
                nodesVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
                nodesVB.add(nodeButton);
                nodeButtonGroup.add(nodeButton);
            }
        }
        if( nodeIDForSlot == 0 ) unusedNodeButton.setSelected(true);

        nodesHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        nodesHB.add(nodesVB);
        nodesHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        
        nodeButtonPanel.add(nodesHB, BorderLayout.NORTH);

        // Abbruch und Ok-Button einfgen
        cancelButton.addActionListener(this);
        cancelButton.setActionCommand(CancelAction.COMMAND);
		okButton.addActionListener(this);
		okButton.setActionCommand(OKAction.COMMAND);
		
		//	  Vertikale Box des Dialogs fllen ...
 	    contentVB.add(nodeButtonPanel); 
        contentVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
  	    contentVB.add(buttonHB);
  	    contentVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
  	     
		// FARBEN, FONTS UND TEXTE SETZEN ...
	 	updateLanguageColorFonts();

    }

    /*
     * Methode:		showDialog
     * 
     * Description:	Setzt die aktuellen Farben aus der Konfiguration bevor der Dialog angezeigt bzw.
     * 				wieder versteckt wird.
     *    
     * Parameters:    
     * 
     * Date: 		 11.12.2007
     * @author        sirocic
     */	
    public void showDialog(boolean visible) {
       
  	  // TRACE OUTPUT
  	  if (myFrame.getLearningModule().isTraceActive()) System.out.println("CALL ArrowEditorDialogPN.showDialog:");
  	  
  	  // alle Taskparametereingaben zurcksetzen.
  	  cleanMe();
    	  
  	  if (visible) {
  		  // Dialog wird sichtbar gemacht.
  		  
  		  // Werte des aktuellen Slots anzeigen ???
        
  	  }

  	  // ... so jetzt den Dialog anzeigen bzw. verstecken.
  	  super.showDialog(visible);
    	  
   	  // TRACE OUTPUT
   	  if (myFrame.getLearningModule().isTraceActive()) System.out.println("END ArrowEditorDialogPN.showDialog");

    }
    
	   /*
	    * Methode:		updateDialogsColorLanguageFonts
	    * 
	    * Description:	Setzt die aktuellen Farben aus der Konfigurationbzw. und Texte aus den Ressourcen.
	    * 				Wird aufgerufen in erster Linie nach einem dynamischen Sprach- oder Farbwechsel.
	    *    
	    * Date: 		 16.03.2007
	    * @author        sirocic
	    */	
	   public void updateDialogsColorLanguageFonts() {
	      
	 	  // TRACE OUTPUT
	 	  if (myFrame.getLearningModule().isTraceActive()) System.out.println("CALL BufferEditorDialogPN.updateDialogsColorLanguageFonts:");

	 	  // SUBKOMPONENTE UPDATEN...

	 	  super.updateLanguageColorFonts();
	   	  
	  	  // TRACE OUTPUT
	  	  if (myFrame.getLearningModule().isTraceActive()) System.out.println("END BufferEditorDialogPN.updateDialogsColorLanguageFonts");

	   }
	   
	   /*
	    * Methode:		setDialogFont
	    * 
	    * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
	    * 				Werte. 
	    *    
	    * Parameters:   Aktuelle Font fr Labels und Textfelder.
	    * 
	    * Date: 		 17.01.2007
	    * @author        sirocic
	    */	
	   protected void setDialogFont() {
		   Font menuFont;
		   JRadioButton nodeRB;
		   Enumeration nodeButtons;
		   
		   menuFont = (Font)myFrame.getLearningModule().getConfiguration().getSettingByKey("FontMenu");
		   
		   super.setDialogFont("FontMenu");
		   
		   nodeButtons = nodeButtonGroup.getElements();
		   while (nodeButtons.hasMoreElements()) {
			   nodeRB = (JRadioButton) nodeButtons.nextElement();
			   nodeRB.setFont(menuFont);
		   }
		   
	   }
	   
	   /*
	    * Methode:		setDialogTexte
	    * 
	    * Description:	Setz alle Text-Bausteine des Dialoges neu. Wichtig fr Sprachwechsel.
	    *    
	    * Parameters:   
	    * 
	    * Date: 		 26.01.2007
	    * @author        sirocic
	    */	
	   protected void setDialogTexte() {
		   String title, borderTitle, buttonText;
		   TitledBorder channelBorder, colorBorder, nodeNameBorder;
		   JRadioButton nodeRB;
		   int i, s;
		   boolean notFound;
		   Enumeration nodeButtons;
		   
		   super.setDialogTexte();

		   title = myFrame.getLearningModule().getRessources().getResourceByKey("dlg.slotconfig.title");
	       s = slot+1;
		   title = title.replace("%s", channel + ", Slot " + s);
	       setTitle( title );
	       
	       unusedNodeButton.setText(res.getResourceByKey("unused"));
	       for( Node node : cluster.getNodes() ) {
	            if( node.getNodeConfiguration().pChannels().contains(channel)) {
	            	buttonText = node.getName();
	                if( buttonText == null ) buttonText = res.getResourceByKey("flexray.node") + " " + node.getID();
	                if(buttonText.length() > 10) buttonText = buttonText.substring(0,8) + "...";
	                i=0;
	                nodeButtons = nodeButtonGroup.getElements();
	                notFound=true;
	     		    while (nodeButtons.hasMoreElements() && notFound) {
	     			   nodeRB = (JRadioButton) nodeButtons.nextElement();
	     			   if (nodeRB.getActionCommand().equals(Integer.valueOf(node.getID()).toString() )) {
	     				   nodeRB.setText(buttonText);
	     				   notFound = false;
	     			   }
	     		    }
	                
	             }
	        } // End for-Schleife
	       
	   }
	   
	   /*
	    * Methode:		setDialogColors
	    * 
	    * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
	    * 				Werte. 
	    *    
	    * Parameters:   Aktuelle Farbwerte fr Hintergrund und Vordergrund.
	    * 
	    * Date: 		 17.01.2007
	    * @author        sirocic
	    */	
	   protected void setDialogColors() {
		   Color backgroundColor,  foregroundColor;
		   JRadioButton nodeRB;
		   Enumeration nodeButtons;
			 
		   if (myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentBackground")!=null)
		   {
			   backgroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentBackground");
			   foregroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentForeground");
		   }
		   else {
			   backgroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorModuleBackground");
			   foregroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorModuleForeground");
		   }
		   
	        nodeButtonPanel.setBackground(backgroundColor);
	        nodeButtonPanel.setForeground(foregroundColor);
	        unusedNodeButton.setBackground(backgroundColor);
	        unusedNodeButton.setForeground(foregroundColor);
            nodeButton.setBackground(backgroundColor);
            nodeButton.setForeground(foregroundColor);
            nodeButtons = nodeButtonGroup.getElements();
 		    while (nodeButtons.hasMoreElements()) {
 			   nodeRB = (JRadioButton) nodeButtons.nextElement();
 			   nodeRB.setBackground(backgroundColor);
 			   nodeRB.setForeground(foregroundColor);
 		    }

		   super.setDialogColors();
		   
	   }  
	   
	   /*
	    * Methode:		cleanMe
	    * 
	    * Description:	Setzt alle Eingabe- und Auswahlfelder usw. zurck auf Defaultwerte.
	    *    
	    * Parameters:   
	    * 
	    * Date: 		 28.12.2007
	    * @author        sirocic
	    */	
	   private void cleanMe() {

	   }
	   
    
    /*
     * Methode:		fitDialog
     * 
     * Description:	Bestimmen der bentigten Panelgre fr das Anzeigen des vollstndigen Dialogs. 
     *    
     * Parameters:    
     * 
     * Date: 		 16.01.2007
     * @author        sirocic
     */	
    public void fitDialog() {
    	Graphics2D g2;
	  	  int maxWidth, maxHeight, panelWidth, panelHeight;
	  	  int labelHeight;
	  	  Rectangle2D boundsLabel;
		  FontRenderContext context;
		  JRadioButton nodeRB;
	  	  
	  	  // TRACE OUTPUT
	  	  if (LeviModule.isTraceActive()) System.out.println("CALL BufferEditorDialogPN.fitDialog:");
	  	
	  	  // init
	  	  g2 = (Graphics2D) getGraphics();
	  	  maxWidth = 0;
		  maxHeight =0;
		  
		  if (g2 != null){
			  // Bentigte Zeilenbreite und -hhe bestimmen anhand der Textausmasse bei aktuelle
		      // eingestellten Font ...
			  g2.setFont(getFont());
		      context = g2.getFontRenderContext();

		      //  GUI-ELEMENTE AN AKTUELLE FONTSIZE UND INHALTE ANPASSEN ...
		      validate();
		      panelWidth = unusedNodeButton.getWidth();
		      for (int i=0; i < nodeButtonGroup.getButtonCount(); i++) {
		    	  nodeRB = (JRadioButton) nodeButtonGroup.getElements().nextElement();
		    	  if (nodeRB.getWidth() > panelWidth)
		    		  panelWidth = nodeRB.getWidth();
		      }
		      panelWidth = panelWidth + nodeButtonPanel.getInsets().left 
		      						  + 3 * LeviFrame.ATTRIBUTOFFSET
		      						  + nodeButtonPanel.getInsets().right;
		      
		      // HHE DES DIALOG-CONTENTPANEL BESTIMMEN ...
		      panelHeight = nodeButtonGroup.getButtonCount() * nodeButtonGroup.getElements().nextElement().getHeight()
		    	  				+ nodeButtonGroup.getButtonCount() * LeviFrame.LINEOFFSET;
		      panelHeight = panelHeight + nodeButtonPanel.getInsets().top + nodeButtonPanel.getInsets().bottom;
		      
		      nodeButtonPanel.setSize(panelWidth,panelHeight);
		      nodeButtonPanel.setMaximumSize(new Dimension(panelWidth,panelHeight));
		      nodeButtonPanel.setPreferredSize(new Dimension(panelWidth,panelHeight));
		      
		  	  // BREITE DES DIALOG-CONTENTPANEL BESTIMMEN ...
		      boundsLabel = getFont().getStringBounds(getTitle(), context);
		      if (panelWidth < (int) boundsLabel.getWidth() + OFFSET)
		    	  maxWidth = (int) boundsLabel.getWidth()+ OFFSET;
		      else
		    	  maxWidth = panelWidth;
		      
		      // HHE DES DIALOG-CONTENTPANEL BESTIMMEN ...
		      maxHeight = panelHeight;
		      
			  // Panelgre aktualisieren ...
		  	  getRootPane().setSize(maxWidth,maxHeight);
		  	  getRootPane().setMaximumSize(new Dimension(maxWidth,maxHeight));
		  	  getRootPane().setPreferredSize(new Dimension(maxWidth,maxHeight));

 	  }
	
	  super.fitDialog();
 	  
   	  // TRACE OUTPUT
   	  if (LeviModule.isTraceActive()) System.out.println("END ResourceEditorRTS.fitPanel");
   	
    }


    /*
     * @param e
     * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     */
    public void actionPerformed(ActionEvent e) {
        if( e.getActionCommand().equals(OKAction.COMMAND)) {
            ButtonModel buttonModel = nodeButtonGroup.getSelection();
            int newNodeIDForSlot = Integer.parseInt(buttonModel.getActionCommand());
            Node nodeForSlot = cluster.getNodeForSlot(channel, slot);
            Node nodeForSlotOtherChannel = cluster.getNodeForSlot(channel.other(),slot);
            if( nodeForSlot == null ) {
                nodeIDForSlot = 0;
            } else {
                nodeIDForSlot = nodeForSlot.getID();
            }
            if( newNodeIDForSlot != nodeIDForSlot ) {
//                DebugUtil.println("Slot #" + slot + " old=" + nodeIDForSlot + ", new=" + newNodeIDForSlot);
                if ( nodeForSlot != null ) {
                    DebugUtil.println("removing slot " + slot + " from node " + nodeIDForSlot);
                    nodeForSlot.removeTransmitSlot(slot);
                }
                if( newNodeIDForSlot != 0 ) {
                    Node node = cluster.getNode(newNodeIDForSlot);
                    node.addTransmitSlot(slot);
                    if( (nodeForSlotOtherChannel != null) &&
                         node.getNodeConfiguration().pChannels().contains(channel.other())) {
                        nodeForSlotOtherChannel.removeTransmitSlot(slot);
                    }
                    // add channel ?
                    if( !node.getNodeConfiguration().pChannels().contains(channel)) {
                        String message = res.getResourceByKey("channel.added");
                        message = message.replace("%c", channel.toString());
                        if(node.getName() != null) {
                            message = message.replace("%n", node.getName());
                        } else {
                            message = message.replace("%n", res.getResourceByKey("flexray.node") + " #" + node.getID());
                        }
                        JOptionPane.showMessageDialog(this, message);
                        node.addChannel(channel);
                    }
                }
            }
            dispose();
            scenario.setLastEdit(EditCommandType.CONFIGURE_SLOT);
            scenario.fireStateChanged();
        }

        // discard changes: Just remove the dialog
        if( e.getActionCommand().equals(CancelAction.COMMAND))
            dispose();
    }


}
