/*
 * $Id: FrameFR.java,v 1.1 2009/04/01 13:57:11 sirocic Exp $
 */
package flexray.levi.gui;


import java.awt.Color;
import java.awt.EventQueue;
import java.awt.KeyboardFocusManager;
import java.awt.Rectangle;
import java.awt.event.InputEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Locale;

import javax.swing.BorderFactory;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComponent;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JRootPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import learningVisualizations.LeviConfiguration;
import learningVisualizations.LeviModule;
import learningVisualizations.LeviResourceManager;
import learningVisualizations.gui.LeviFrame;
import learningVisualizations.gui.LeviMenu;
import learningVisualizations.gui.LeviSubmenu;

import flexray.ModuleFR;
import flexray.ConfigKeys;
import flexray.gui.AlphaBlendingComponent;
import flexray.gui.Property;
import flexray.gui.actions.ConfigureCommCycleAction;
import flexray.gui.actions.CreateNodeAction;
import flexray.gui.actions.DeleteSelectedNodeAction;
import flexray.gui.actions.ToggleAntiAliasingAction;
import flexray.gui.actions.ToggleTransparencyAction;
import flexray.gui.actions.ToggleTwoChannelAction;
import flexray.levi.gui.actions.*;
import flexray.scenario.FlexRayScenario;

/**
 * This Frame adds a confirmation for "close" other than from the
 * file menu.
 *
 * @author Dirk Frsterling
 */
public class FrameFR extends LeviFrame
                  implements PropertyChangeListener,
                             ChangeListener,
                             ConfigKeys {

    /**
     * The UID for serialized objects of this class.
     */
    private static final long serialVersionUID = 7968469795280371432L;

    private String documentTitle = "";

    @SuppressWarnings("hiding")
    ModuleFR learningModule = null; // I want ModuleFR not leviModule


    private ToggleTwoChannelAction toggleTwoChannelAction;

    /**
     * Own constructor for a leviFrame.
     *
     * @author Dirk Frsterling
     * @param c the learningModule
     */
    public FrameFR(ModuleFR c) {
        super(c);
//        SelfstudyPanel = new FSplitPane(JSplitPane.HORIZONTAL_SPLIT);
        learningModule = c;

        LeviConfiguration config = learningModule.getConfiguration();
        assert( config != null);

        /*
         * Farben
         */
        Color foreground = getForeground();
        Color background = getBackground();
        Object o;
        if (config.getSettingByKey(LeviConfiguration.COLOR_COMPONENT_BACKGROUND)!= null)
        	o = config.getSettingByKey(LeviConfiguration.COLOR_COMPONENT_BACKGROUND);
        else
        	o = config.getSettingByKey(LeviConfiguration.COLOR_MODULE_BACKGROUND);
        if( o instanceof Color ) {
            background = (Color)o;
        }

        if (config.getSettingByKey(LeviConfiguration.COLOR_COMPONENT_FOREGROUND)!=null)
        	o = config.getSettingByKey(LeviConfiguration.COLOR_COMPONENT_FOREGROUND);
        else
        	o = config.getSettingByKey(LeviConfiguration.COLOR_MODULE_FOREGROUND);
        if( o instanceof Color ) {
            foreground = (Color)o;
        }
        setBackground(background);
        getSelfstudyPanel().setBackground(background);

        setForeground(foreground);
        getSelfstudyPanel().setForeground(foreground);

        getSelfstudyPanel().setBorder(BorderFactory.createEmptyBorder());

        // show JFrame decoration -> looks the same on any platform.
        if( ModuleFR.isMetal() ) {
            getRootPane().setWindowDecorationStyle(JRootPane.FRAME);
            setUndecorated( true );
        }

        getMenubar().addMenuEdit(c.getRessources().getResourceByKey("menu.edit"), 1);
        getMenubar().addToMenuEdit(CreateNodeAction.class.getCanonicalName(), "edit.new", "key.edit.new", null, null);
        getMenubar().addToMenuEdit(DeleteSelectedNodeAction.class.getCanonicalName(), "edit.delete", "key.edit.delete",  null, null);

        JMenuItem editCommCycleItem = new JMenuItem();
        editCommCycleItem.setAction(new ConfigureCommCycleAction(getLearningModule()));
        getMenubar().getMenuEdit().add(editCommCycleItem);


        JCheckBoxMenuItem twoChannelItem = new JCheckBoxMenuItem();
        toggleTwoChannelAction =
            new ToggleTwoChannelAction(learningModule, twoChannelItem);
        learningModule.getRessources().addPropertyChangeListener(Property.LOCALE,toggleTwoChannelAction);
        twoChannelItem.setAction(toggleTwoChannelAction);
        twoChannelItem.setForeground(foreground);
        twoChannelItem.setBackground(background);
        toggleTwoChannelAction.setEnabled(false);
        getMenubar().getMenuEdit().add(twoChannelItem);

        /*
         * Transparency -> Menu view
         */
        JMenu viewMenu = getMenubar().getMenuView();
        JCheckBoxMenuItem item = new JCheckBoxMenuItem();
        ToggleTransparencyAction toggleTransparencyAction =
            new ToggleTransparencyAction(learningModule);
        learningModule.getRessources().addPropertyChangeListener(Property.LOCALE,toggleTransparencyAction);
        item.setAction(toggleTransparencyAction);
        item.setForeground(foreground);
        item.setBackground(background);
        boolean blendOnDrag = false;
        Object setting = config.getSettingByKey(CONFIG_BLENDONDRAG);
        if( setting instanceof Boolean ) {
            blendOnDrag = ((Boolean)setting).booleanValue();
        } else {
            blendOnDrag = AlphaBlendingComponent.isDefaultBlendOnDrag();
        }
        item.setState(blendOnDrag);
        // BS 16.11 Item verschoben. Sollte oberhalb des Hilfeeintrags sein.
        viewMenu.add(item, 1);

        /*
         * AntiAliasing -> Menu view
         */
        item = new JCheckBoxMenuItem();
        ToggleAntiAliasingAction toggleAntialiasingAction =
            new ToggleAntiAliasingAction(learningModule);
        learningModule.getRessources().addPropertyChangeListener(Property.LOCALE,toggleAntialiasingAction);
        item.setAction(toggleAntialiasingAction);
        item.setForeground(foreground);
        item.setBackground(background);
        boolean antialiasing = true;
        setting = config.getSettingByKey(CONFIG_ANTIALIAS);
        if( setting instanceof Boolean ) {
            antialiasing = ((Boolean)setting).booleanValue();
        }
        item.setState(antialiasing);
        // BS 16.11 Item verschoben. Sollte oberhalb des Hilfeeintrags sein.
        viewMenu.add(item, 2);
        
        // add a similar item to the popup.
        JMenu help = getMenubar().getMenuHelp();
        LeviSubmenu submenuHelp;
        submenuHelp = new LeviSubmenu(this, "menu.help");
        submenuHelp.addToMenu(FRActionWelcome.class.getName(), "help.welcome", "key.welcome", null, null);
        submenuHelp.addToMenu(FRActionHelp.class.getName(), "help.manual", "key.manual", null, null);
        submenuHelp.addToMenu(FRActionTutorial.class.getName(), "help.tutorial", "key.tutorial",null, null);
        
        getPopupMenu().add(submenuHelp);
        getMenubar().remove(help);
        getMenubar().setMenuHelp(submenuHelp);
        getMenubar().addMenuHelp(5);

        /*
         * Add a window listener _before_ the current one.
         */
        WindowListener windowListener = null;
        for(WindowListener w : getWindowListeners()) {
            if(w.getClass().getName().contains("leviFrame")) {
                windowListener = w;
                removeWindowListener(w);
            }
        }

        addWindowListener( new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent e) {
                Rectangle r = new Rectangle(getX(),getY(),getWidth(),getHeight());
                learningModule.getConfiguration().putSettingByKey(CONFIG_WINDOWRECT, r);
                learningModule.getConfiguration().saveConfiguration();
            }
        });

        if( windowListener != null ) {
            addWindowListener(windowListener);
        }

        /*
         * Validate display and clear focus
         */
        EventQueue.invokeLater( new Runnable() {
            public void run() {
                validate();
                KeyboardFocusManager.getCurrentKeyboardFocusManager().clearGlobalFocusOwner();
            }
        } );
        
        //  Farbe- und Spracheinstellung fr die neuen und erweitereten Eintrge setzen
        getMenubar().updateLanguageColorFonts();
        getPopupMenu().updateLanguageColorFonts();
    } // FrameFR( leviModule )

    /**
     * Convenience: I want ModuleFR
     * @return the lesrning module as ModuleFR type
     * @see learningVisualizations.gui.LeviFrame#getLearningModule()
     */
    @Override
    public ModuleFR getLearningModule() {
        return (ModuleFR) super.getLearningModule();
    }

    /**
     * Override refresh() to prevent rebuilding of the frame.
     *
     * @see LeviFrame#refresh()
     */
    @Override
    public void refresh() {
        // this is called from events, thus we're in the EDT
        repaint();
    }

    // BS 29.01.06 Verschoben aus leviModule in leviFrame
    @Override
    public void changeView() {
        refresh();
    }

    /*
     * (non-javadoc)
     *
     * @see learningVisualizations.gui.leviFrame#changeMode()
     */
    @Override
    public void changeMode() {
//        DebugUtil.mark("changeMode called from " + DebugUtil.getCaller(2));
        refresh();
    }


    private boolean composed = false;

    /*
     *
     * @see learningVisualizations.gui.leviFrame#composeFrame()
     */
    @Override
    public void composeFrame() {
        if( !composed ) {
            super.composeFrame();
            composed = true;
        }
        
        switch (learningModule.getView()) {
	        case (LeviModule.SELFSTUDYVIEW): {
		        //      CONSIDER THE ATTRIBUTE VALUES
		        if (learningModule.getHelp()!=null) {
		           if (!learningModule.isHelpVisible())
		               // makes the Help invisible
		               setHelpPanelVisible(false);
		           else
		               setHelpPanelVisible(true); 
		        }
	
	            break;
	        }
	        case (LeviModule.LECTUREVIEW): {
	            // COMPOSE THE CONTENT PARTS OF THE LECTURE VIEW.
	            
	            break;
	        }
        }

        refresh();

    }

    /**
     * Update the locale setting of this frame.
     */
    public void updateLocale() {
        setLocale(Locale.getDefault());
        updateTitle(); // calls updateDocumentTitle
        // FIXME: this is a hack that should go to leviResourceManager
        JComponent.setDefaultLocale(Locale.getDefault());
    }

    /**
     * Update the frame title. This includes the document title of the current
     * document (scenario)
     * The document title will contain the following:
     * <ul>
     *   <li>Scenario Name</li>
     *   <li>modification status</li>
     * </ul>
     */
    public void updateTitle() {
        FlexRayScenario scenario = (FlexRayScenario)learningModule.getScenario();
        if( scenario == null ) {
            documentTitle = "";
        } else {
            StringBuffer dt = new StringBuffer();
            dt.append(scenario.getName());
            if( scenario.hasChanged() ) {
                String mod = learningModule.getRessources().getResourceByKey("modified");
                dt.append(" (").append(mod).append(")");
            }
            documentTitle = dt.toString();
        }
        StringBuffer title = new StringBuffer();
        title.append(learningModule.getRessources().getResourceByKey("frame.title")+ " " + learningModule.getVersion());
        if( !documentTitle.equals("")) {
            title.append(" - ").append(documentTitle);
        }
        this.setTitle(title.toString());
        refresh();
    }
    
    /*
     * Methode:		updateLanguageColorFonts
     *
     * Description:	Draws the total frame new.
     *
     * Date: 		17.07.2005
     * @author sirocic
     */
    public void updateLanguageColorFonts()
    {
  	 
    	// auch die levi- und Standarddialoge usw. aktualisieren
    	super.updateLanguageColorFonts();
    	
    	// alles neuzeichnen
    	super.refresh();

    }


    /**
     * React to propertyChanges (language in this case);
     * @param evt
     * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
     */
    public void propertyChange(PropertyChangeEvent evt) {
        if( evt.getPropertyName().equals(LeviResourceManager.PROPERTY_LOCALE)) {
            updateLocale();
        }
    }

    /**
     * React to state changes.
     * usually, this frame only listens to changes of the current scenario.
     * @param e
     * @see javax.swing.event.ChangeListener#stateChanged(javax.swing.event.ChangeEvent)
     */
    public void stateChanged(ChangeEvent e) {
        updateTitle();
    }

    public ToggleTwoChannelAction getToggleTwoChannelAction() {
        return toggleTwoChannelAction;
    }

} // FRFrame
