/*
 * $Id: FRHelp.java,v 1.1 2009/04/01 13:57:11 sirocic Exp $
 *
 * Created on 07.07.2006 12:36:36
 * by Dirk Frsterling <r@zorbla.de>
 */
package flexray.levi.help;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Enumeration;
import java.util.EventObject;

import javax.sound.midi.MidiUnavailableException;
import javax.swing.BorderFactory;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.event.HyperlinkEvent.EventType;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.Element;
import javax.swing.text.AbstractDocument.LeafElement;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLDocument.RunElement;

import learningVisualizations.LeviConfiguration;
import learningVisualizations.gui.LeviFrame;
import learningVisualizations.help.LeviHelp;

import flexray.ModuleFR;
import flexray.cluster.Cluster;
import flexray.cluster.Node;
import flexray.configuration.ChannelName;
import flexray.configuration.ClusterConfiguration;
import flexray.configuration.NodeConfiguration;
import flexray.gui.Property;
import flexray.gui.controller.EditCommandType;
import flexray.levi.gui.FrameFR;
import flexray.scenario.FlexRayScenario;
import flexray.util.DebugUtil;

/**
 * Please note: leviHelp is-a JPanel and so is FRHelp
 *
 * @author Dirk Frsterling <r@zorbla.de>
 */
public class FRHelp extends LeviHelp
                   implements ChangeListener,
                              ComponentListener,
                              HyperlinkListener,
                              PropertyChangeListener {

    private static FRHelp self;
    public JTextPane textPane = new JTextPane();
    private JScrollPane innerBorderPanel;

    public static int MY_AVATARWIDTH = 400;

    private String base;
    private String language = "";
    private boolean contentFromErnie = false;

    private String currentPage = "";
    private URL waitForLoad;
    private boolean contentLoaded = false;

    private boolean firstLoad = true;

    private boolean tutorialMode = false;

    private int tutStep = 0;

    /**
     *
     */
    public FRHelp() {
    }

    /**
     * @param learningModule
     */
    public FRHelp(ModuleFR learningModule) {
        super(learningModule);

        // String base = "/resources/flexray/html/";

//        base = "http://ernie/~dirk/flexray/";
//
//        URL test = null;
//        try {
//            test = new URL(base + "fromErnie");
//        } catch (MalformedURLException e) {
//            e.printStackTrace();
//        }
//        if( test != null ) {
//            InputStream stream;
//            int oneByte = -1;
//            contentFromErnie = false;
//            try {
//                stream = test.openStream();
//                oneByte = stream.read();
//                if( oneByte == 116 ) {
//                    contentFromErnie = true;
//                }
//            } catch (IOException e) {
//                e.printStackTrace();
//            }
//        }
//
//        if( !contentFromErnie )
            base = "/resources/flexray/html/";


        language = learningModule.getLanguage();

        setLayout(new BorderLayout());

        setMinimumSize(new Dimension(HELPWIDTH,10));
        setPreferredSize(new Dimension(MY_AVATARWIDTH,10));


        innerBorderPanel = new JScrollPane(textPane);
        innerBorderPanel.setBorder(BorderFactory.createEmptyBorder(5, 2, 5, 2));
        innerBorderPanel.setBackground(getBackground());
        add(innerBorderPanel,BorderLayout.CENTER);

        textPane.setBackground(getBackground());
        textPane.setEditable(false);
        textPane.addPropertyChangeListener(this);
        textPane.addHyperlinkListener(this);

        setNewContent("welcome.html");



        self = this;

        learningModule.getConfiguration().addPropertyChangeListener(this);
        addComponentListener(this);
    } // constructor

    /**
     * Change the content of the help JTextComponent based on the local
     * URLs and language setting.
     * @param url
     */
    public void setNewContent(String url) {
        if(!contentFromErnie ) {
            // setting an anchor from a resource is pain
            int i = url.indexOf("#");
            if( i < 0 ) {
                waitForLoad = getClass().getResource(base + language + "/" + url);
            } else {
                waitForLoad = getClass().getResource(base + language + "/" + url.substring(0,i));
                try {
                    waitForLoad = new URL(waitForLoad.toString() + url.substring(i));
                } catch (MalformedURLException mue) {
                    DebugUtil.println("could not append anchor " + url.substring(i));
                }
            }
        } else {
            try {
                waitForLoad = new URL(base + language + "/" + url);
            } catch (MalformedURLException e) {
                e.printStackTrace();
            }
        }
        contentLoaded = false;
        EventQueue.invokeLater(new Runnable() {
            public void run() {
                try {
                    textPane.setPage(waitForLoad);
                } catch (IOException ioe ) {
                    DebugUtil.println("error: " + ioe);
                }
            }
        });
        currentPage = url;
        // the following does not work, because loading
        // takes place in the EDT and we are already
        // there :/
        // waitForPageLoaded();
    } // setNewConetne


    /**
     * A try to wait for the end of setPage()
     * It does not work, because loading
     * takes place in the EDT and we are already
     * there :/
     *
     */
    public void waitForPageLoaded() {
        while(!contentLoaded) {
            synchronized(waitForLoad) {
                try {
                    DebugUtil.println("waiting");
                    waitForLoad.wait(1000);
                } catch (InterruptedException ie ) {
                    //
                }
            } // synchronized
        }
    } // waitForPageLoaded



    /**
     * Return a width to satisfy display needs of tha help
     * @return
     */
    public int getWorkingWidth() {
        int w = getPreferredSize().width;
        return Math.min(w, MY_AVATARWIDTH);
    }


    /**
     * Reacto to scenario changes (edits). This is essential for the tutorial.
     *
     * @param e
     * @see javax.swing.event.ChangeListener#stateChanged(javax.swing.event.ChangeEvent)
     */
    public void stateChanged(ChangeEvent e) {
        if( !tutorialMode ) return;
        if(contentFromErnie)
            DebugUtil.println("source: " + e.getSource().getClass().getSimpleName());
        if( !(e.getSource() instanceof FlexRayScenario) ) return;
        FlexRayScenario scenario = (FlexRayScenario)e.getSource();
        if( scenario.equals(learningModule.getScenario() )) {
            if(contentFromErnie)
                DebugUtil.println(scenario.getName() + ": " + scenario.getLastEdit());
            EditCommandType edit = scenario.getLastEdit();
            switch(edit) {
                case CREATE_CLUSTER:
                    setNewContent("createnode.html");
                    break;
                case CREATE_NODEVIEW:
                    if( scenario.getCluster().getNumberOfNodes() == 1) {
                        setNewContent("createanothernode.html");
                    } else {
                        setNewContent("connectnodes.html");
                    }
                    break;
                case SCENARIO_CLOSED:
                    setNewContent("welcome.html");
                    break;
                case CONFIGURE_NODE:
                    if( nodesCreated(scenario) && nodesConfigured(scenario)) setNewContent("configurecommcycle.html");
                    break;
                case NODE_CHANNEL_SELECTED:
                    if( currentPage.equals("connectnodes.html") )
                        setNewContent("connectnodes.html#config");
                    break;
                case CONFIGURE_COMMCYCLE:
                    if( nodesCreated(scenario) && nodesConfigured(scenario) && slotsCreated(scenario) )
                        setNewContent("assignnodes.html");
                    break;
                case CONFIGURE_SLOT:
                    if( nodesCreated(scenario) && nodesConfigured(scenario) && slotsCreated(scenario) && nodesAssigned(scenario) )
                        setNewContent("simstart.html");
                    break;
                case SIM_STEP_FORWARD:
                    if( nodesCreated(scenario) && nodesConfigured(scenario) && slotsCreated(scenario) && nodesAssigned(scenario) ) {
                        Cluster cluster = scenario.getCluster();
                        long tick  = cluster.getCurrentMacroTick();
                        switch( (int)tick ) {
                        case 0:
                            setNewContent("simstep01.html");
                            break;
                        case 8:
                            setNewContent("simstep08.html");
                            break;
                        case 11:
                            setNewContent("simstep11.html");
                            break;
                        case 13:
                            setNewContent("simstep13.html");
                            break;
                        case 14:
                            setNewContent("simstep14.html");
                            break;
                        case 28:
                            setNewContent("simstep28.html");
                            break;
                        default:
                            if( (tick > 28) ) {
                                if(learningModule.getSimulator().isPause()) {
                                    setNewContent("simrun.html");
                                } else {
                                    setNewContent("goodbye.html");
                                    setTutorialMode(false);
                                }
                            }
                            break;

                        }
                    }
                    break;
            }
        }
    } // state changed

    private boolean nodesCreated(FlexRayScenario scenario) {
        return scenario.getCluster().getNumberOfNodes() >= 2;
     }

    private boolean nodesConfigured(FlexRayScenario scenario) {
        if(scenario.getCluster().getNumberOfNodes() < 2) return false;
        int correctConfig = 0;
        for( Node node : scenario.getCluster().getNodes()) {
            NodeConfiguration config = node.getNodeConfiguration();
            if( config.pChannels().equals(ChannelName.A) &&
                    (node.getSendCycleMin(ChannelName.A) == 0) &&
                    (node.getSendCycleMax(ChannelName.A) == 0) &&
                    (node.getSendLengthMin(ChannelName.A) == 2) &&
                    (node.getSendLengthMax(ChannelName.A) == 2) )
                correctConfig++;
        }
        if( correctConfig >= 2) return true;
        return false;
    } // nodesConfigured

    private boolean slotsCreated(FlexRayScenario scenario) {
        ClusterConfiguration cc = scenario.getCluster().getClusterConfiguration();
        if( (cc.gNumberOfMinislots() == 3) &&
                cc.gChannels().equals(ChannelName.A))
            return true;
        return false;
    }

    private boolean nodesAssigned(FlexRayScenario scenario) {
        Node slot0 = scenario.getCluster().getNodeForSlot(ChannelName.A, 0);
        Node slot1 = scenario.getCluster().getNodeForSlot(ChannelName.A, 1);
        Node slot2 = scenario.getCluster().getNodeForSlot(ChannelName.A, 2);
        Node slot3 = scenario.getCluster().getNodeForSlot(ChannelName.A, 3);
        if( (slot0 != null) && (slot1 != null) && (slot2 != null) && (slot3 != null) ) {
            if( (slot0.getID()== 1) && (slot1.getID() == 2) && (slot2.getID()== 1) && (slot3.getID() == 2) )
                return true;
        }
        return false;
    }

    /**
     * React to property changes (language, page loading, color)
     * @param evt
     * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
     */
    public void propertyChange(PropertyChangeEvent evt) {
        if( evt.getPropertyName().equals("page") ) {
            if( firstLoad ) {
                getLearningModule().correctDividerLocation((FrameFR) getLearningModule().getFrame());
                firstLoad = false;
            }
        }
        if( evt.getPropertyName().startsWith("Color")) {
            setColorsFromConfig();
        }

        if( evt.getPropertyName().equals("Language")) {
            language = evt.getNewValue().toString();
            setNewContent(currentPage);
        }

    } // propertyChange

    /**
     * React to clicks on the links. This is essential for the tutorial.
     *
     * @param e
     * @see javax.swing.event.HyperlinkListener#hyperlinkUpdate(javax.swing.event.HyperlinkEvent)
     */
    public void hyperlinkUpdate(HyperlinkEvent e) {
        if(e.getEventType().equals(EventType.ACTIVATED)) {
            URL eventURL = e.getURL();
            Element source = e.getSourceElement();
            Element sourceparent = source.getParentElement();
            Document spdoc = sourceparent.getDocument();
            String tagtest = "";
            try {
                tagtest = spdoc.getText(sourceparent.getStartOffset(),3);
            } catch(BadLocationException ble) {
                //
            }
            String file = eventURL.getFile();
            file =  file.substring(file.lastIndexOf("/")+1);
            setNewContent(file);
            Element element = e.getSourceElement();
            Document doc = element.getDocument();
            int start = element.getStartOffset();
            String linkText;
            try {
                linkText = doc.getText(start, element.getEndOffset()-start);
            } catch (BadLocationException ble) {
                linkText = "";
            }
            boolean inTutorial = false;
            if( file.equals("tutorial.html")) {
                try {
                    linkText = doc.getText(start, element.getEndOffset()-start);
                } catch (BadLocationException ble) {
                    linkText = "";
                }
                if( linkText.toLowerCase().startsWith(">") || linkText.toLowerCase().startsWith("t")) {
                    inTutorial = true;
                }
            }
            // alternative "within tutorial" check
            int linknumber = -1;
            try {
                linknumber = Integer.parseInt(linkText.trim());
                if( (linknumber > 0) && (linknumber < 15) ) {
                    tagtest="Tut";
                }
            } catch (NumberFormatException nfe) {
                // no number
            }
            if( isTutorialMode() && tagtest.toLowerCase().startsWith("tut")) {
                return;
            }
            setTutorialMode(inTutorial);
            FlexRayScenario scenario = (FlexRayScenario)getLearningModule().getScenario();
            stateChanged( new ChangeEvent((scenario==null?this:scenario)) );
        }
    } // hyperlinkUpdate

    public boolean isTutorialMode() {
        return tutorialMode;
    }

    public void setTutorialMode( boolean b ) {
        boolean old = tutorialMode;
        if ( old == b ) return;

        tutorialMode = b;
        firePropertyChange(Property.TUTORIALMODE, old, b);
    }

    public void showWelcome() {
        setTutorialMode( false );
        setNewContent("help.html");
    }

    /*
     * -------------- modulelistener ----------------
     */

    public void componentHidden(ComponentEvent e) {
        firePropertyChange(Property.TUTORIALMODE, !tutorialMode, tutorialMode);
    }

    public void componentMoved(ComponentEvent e) { /**/ }
    public void componentResized(ComponentEvent e) { /**/ }

    public void componentShown(ComponentEvent e) {
        firePropertyChange(Property.TUTORIALMODE, !tutorialMode, tutorialMode);
    }



    @Override
    public ModuleFR getLearningModule() {
        return (ModuleFR)super.getLearningModule();
    }



    /*
     * Set the Help colors from the configuration
     */
    private void setColorsFromConfig() {
    	Object o ;
        LeviConfiguration config = learningModule.getConfiguration();
        
        if (config.getSettingByKey(LeviConfiguration.COLOR_COMPONENT_BACKGROUND)!= null)
        	o = config.getSettingByKey(LeviConfiguration.COLOR_COMPONENT_BACKGROUND);
        else
           o = config.getSettingByKey(LeviConfiguration.COLOR_MODULE_BACKGROUND);
        if( o instanceof Color ) {
            setBackground((Color)o);
            textPane.setBackground((Color)o);
            innerBorderPanel.setBackground((Color)o);
        }

        if (config.getSettingByKey(LeviConfiguration.COLOR_COMPONENT_FOREGROUND)!= null)
        	o = config.getSettingByKey(LeviConfiguration.COLOR_COMPONENT_FOREGROUND);
        else
        	o = config.getSettingByKey(LeviConfiguration.COLOR_MODULE_FOREGROUND);
        if( o instanceof Color ) {
            setForeground((Color)o);
            textPane.setForeground((Color)o);
        }

    }




}
