/**
 *
 */
package flexray.levi.simulator;


import java.awt.EventQueue;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import learningVisualizations.LeviModule;
import learningVisualizations.simulator.LeviSimulator;

import flexray.ModuleFR;
import flexray.cluster.Cluster;
import flexray.gui.TurningKnob;
import flexray.gui.controller.EditCommandType;
import flexray.scenario.FlexRayScenario;
import flexray.util.DebugUtil;


/**
 * Simulator for FlexRay based on leviSimulator.
 * This simulator receives property changes from a turning knob to adjust
 * the simulation delay
 *
 * @author dirk
 */
public class FRleviSimulator extends LeviSimulator
                          implements PropertyChangeListener,
                                     ChangeListener {

    /*
     * Define own values for speed, because we use a turning knob
     * and a scaled value
     */
    static {
        SLEEPUNIT=1;
        MINSLEEPFACTOR=1;
    }

    /**
     * Constructor that sets a good sleepFactor when there is no sleepFactor
     * configured.
     * @param c
     */
    public FRleviSimulator( LeviModule c ) {
        super( c );
        if( !c.getConfiguration().keyExist("SleepFactor")) {
            sleepFactor = 1000;
        }
    }

    /**
     *
     * @see learningVisualizations.simulator.LeviSimulator#executeStep()
     */
   @Override
   public synchronized boolean executeStep() {
       FlexRayScenario scenario = (FlexRayScenario)getLearningModule().getScenario();
       scenario.getCluster().receiveMacroTick();
       scenario.setLastEdit(EditCommandType.SIM_STEP_FORWARD);
       scenario.fireStateChanged();
       
       return true;
   }


   /**
    *
    * @see learningVisualizations.simulator.LeviSimulator#executeStepBack()
    */
   @Override
   public synchronized void executeStepBack() {
       Cluster cluster = ((FlexRayScenario)getLearningModule().getScenario()).getCluster();
       cluster.undoMacroTick();
   }


   /*
    * @see learningVisualizations.Simulator.leviSimulator#stepForwardSimulation()
    */
   @Override
   public synchronized void stepForwardSimulation()
   {
       if (getTimer().getTime()==0) {
          // init simulator ...
           init();
       }

       setNextStopTime(getTimer().getTime() + STEPWIDTH);
       // start simulation step ...
       setForeward(true);
       setBackward(false);
       setPlay(false);
       setPause(true);
       setStop(false);
       executeStep();
       getTimer().nextTime();
       refreshVisual();
   }

   /*
    * @see learningVisualizations.Simulator.leviSimulator#stepBackwardSimulation()
    */
   @Override
   public synchronized void stepBackwardSimulation() {
       if (getTimer().getTime()>1) { // it's "1" here!
           setNextStopTime(getTimer().getTime() - STEPWIDTH);
           //set the status
           setForeward(false);
           setBackward(true);
           setPlay(false);
           setPause(true);
           setStop(false);
           executeStepBack();
           refreshVisual();
           getTimer().previousTime();
       }
   }

   private void refreshVisual() {
       EventQueue.invokeLater( new Runnable() {
           public void run() {
               getLearningModule().getVisualizer().repaintMe();
           }
       });
   }

   @Override
   public void stopSimulation() {
       super.stopSimulation();
       ((FlexRayScenario)getLearningModule().getScenario()).getCluster().reset();
   }

   /**
    * Dirty hack :(
    *
    * @see learningVisualizations.simulator.LeviSimulator#slowdownSimulation()
    */
   @Override
   public void slowdownSimulation() {
       TurningKnob simSpeedTurningKnob = getLearningModule().getSimSpeedTurningKnob();
       int angle = simSpeedTurningKnob.getAngle();
       if( angle <= 0 ) return;
       angle--;
       int speedVal = 270-angle;
       sleepFactor = ((speedVal*speedVal)/40) + 1;
       simSpeedTurningKnob.setAngle(angle);
       simSpeedTurningKnob.setText(Integer.valueOf(sleepFactor).toString());
   }

   /**
    * Dirty hack :(
    *
    * @see learningVisualizations.simulator.LeviSimulator#speedupSimulation()
    */
   @Override
   public void speedupSimulation() {
       TurningKnob simSpeedTurningKnob = getLearningModule().getSimSpeedTurningKnob();
       int angle = simSpeedTurningKnob.getAngle();
       if( angle >= 270 ) return;
       angle++;
       int speedVal = 270-angle;
       sleepFactor = ((speedVal*speedVal)/40) + 1;
       simSpeedTurningKnob.setAngle(angle);
       simSpeedTurningKnob.setText(Integer.valueOf(sleepFactor).toString());
   }

   /*
    * Override for convenience
    * @see learningVisualizations.Simulator.leviSimulator#getLearningModule()
    */
   @Override
   public ModuleFR getLearningModule() {
       return (ModuleFR)super.getLearningModule();
   }


   /**
    * React to property changes, meaning: Set the sleepFactor scaled
    * from the angle of the turning knob.
    *
    * @param evt
    * @see java.beans.PropertyChangeListener#propertyChange(PropertyChangeEvent)
    */
    public void propertyChange(PropertyChangeEvent evt) {
        if( evt.getSource() instanceof TurningKnob ) {
            TurningKnob knob = (TurningKnob)evt.getSource();
            int speedVal = 270-knob.getAngle();
            sleepFactor = ((speedVal*speedVal)/40) + 1;
            knob.setText(""+sleepFactor);
        }
    } // propertyChange

    public void stateChanged(ChangeEvent e) {
        if( e.getSource() instanceof JSlider ) {
            JSlider s = (JSlider)e.getSource();
            int speedVal = 270-s.getValue();
            sleepFactor = ((speedVal*speedVal)/40) + 1;
//            knob.setText(""+sleepFactor);
        }
    }
}
