/*
 * $Id: FlexRayScenario.java,v 1.1 2009/04/01 13:57:10 sirocic Exp $
 *
 */
package flexray.scenario;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.EventListener;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Locale;
import java.util.SortedMap;
import java.util.TreeMap;
import java.util.Map.Entry;
import java.util.concurrent.atomic.AtomicInteger;

import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComponent;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JViewport;
import javax.swing.SpinnerModel;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.EventListenerList;
import javax.xml.parsers.ParserConfigurationException;

import learningVisualizations.LeviConfiguration;
import learningVisualizations.LeviModule;
import learningVisualizations.LeviResourceManager;
import learningVisualizations.gui.LeviMenu;
import learningVisualizations.gui.LeviPopup;
import learningVisualizations.gui.LeviPopupListener;
import learningVisualizations.scenario.LeviScenario;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import com.sun.org.apache.bcel.internal.generic.GETSTATIC;

import flexray.ModuleFR;
import flexray.cluster.Cluster;
import flexray.cluster.Node;
import flexray.configuration.ChannelName;
import flexray.configuration.ChannelTopology;
import flexray.gui.ActiveStarCouplerComponent;
import flexray.gui.CommCycleComponent;
import flexray.gui.ComponentDragger;
import flexray.gui.ScenarioPanel;
import flexray.gui.NodeComponent;
import flexray.gui.Property;
import flexray.gui.TurningKnob;
import flexray.gui.actions.ToggleTopologyAction;
import flexray.gui.controller.EditCommandType;
import flexray.gui.controller.NodeComponentListener;
import flexray.gui.dialogs.CommCycleConfigDialog;
import flexray.levi.gui.FrameFR;
import flexray.util.DebugUtil;
import flexray.xml.XMLTags;

/**
 * "scenario" is the root of the GUI and simulation Object trees
 *
 * The PropertyChangeListener part is for receiving the Change of
 * the Language, which basically affects all GUI components.
 *
 * @author Dirk Frsterling <r@zorbla.de>
 *
 */
public class FlexRayScenario extends LeviScenario
                       implements PropertyChangeListener,
                                  ChangeListener,
                                  XMLTags {

    /*
     *  ---- "scenario" ----
     */
    private Cluster cluster = null; // needed for the Nodes and ClusterConfig
    private AtomicInteger nodeNumber = new AtomicInteger(0);
//    private AtomicInteger numberOfNodes = new AtomicInteger(0);
    private int number = 0;

    /*
     *  ---- "View" ----
     */
    private final FrameFR appFrame;
    private final ScenarioPanel scenarioPanel;
    private final JScrollPane scenarioScroller;
    private static AtomicInteger scenarioNumber = new AtomicInteger(1);

    /**
     * collection of the node components keyed by nodeID
     */
    private HashMap<Integer, NodeComponent> nodeComponents = new HashMap<Integer, NodeComponent>();
    /**
     * any "active star couplers"
     */
    private HashMap<ChannelName,ActiveStarCouplerComponent> activeStarCouplers = new HashMap<ChannelName, ActiveStarCouplerComponent>();
    private ChannelTopology[] topology = {ChannelTopology.STAR, ChannelTopology.STAR};

    /**
     * The communication cycle visualization
     */
    private final CommCycleComponent commCycleVisualizer;
    private Dimension scenarioDimension = new Dimension(0,0);

    /*
     *  ---- "Controller" ----
     */
    private ComponentDragger componentDragger = new ComponentDragger();
    private NodeComponentListener nodeControlMouseListener = new NodeComponentListener( this );
    private boolean editable = true;
    EditCommandType lastEdit;

//    /*
//     * ----- mixed MVC ------
//     */
//
//    /*
//     * previous X and Y position of new nodes.
//     */
//    private AtomicInteger lastNodeX = new AtomicInteger(10);
//    private AtomicInteger lastNodeY = new AtomicInteger(10);
//    private Node previousNode = null;


    /*
     * ---------- scenario handling --------------
     */
    private boolean changed = false;

    /*
     * ---------- Framework --------------
     */
    LeviConfiguration config;
    LeviResourceManager res;

    /**
     * Create a new FlexRayScenario
     * @param name The Name of the Scenario
     * @param pLearningComponent the learning moduleForgroundColor this scenario
     *                           will be used with
     */
    public FlexRayScenario(final String name, final ModuleFR  pLearningComponent) {
        super(pLearningComponent);
        config = pLearningComponent.getConfiguration();
        res = pLearningComponent.getRessources();
        appFrame = (FrameFR) pLearningComponent.getFrame();
        cluster = new Cluster();

        // panel without any layout
        scenarioPanel = new ScenarioPanel(this);
        setName(name);
        number = scenarioNumber.get();
        scenarioNumber.incrementAndGet();
        commCycleVisualizer = new CommCycleComponent(appFrame.getLearningModule(),this, componentDragger);
//        commCycleVisualizer.setBorder(BorderFactory.createLineBorder(Color.RED, 14));
        scenarioPanel.add(commCycleVisualizer);
        commCycleVisualizer.adjustLocation();
        commCycleVisualizer.setSize(commCycleVisualizer.getPreferredSize());
        commCycleVisualizer.addPropertyChangeListener(Property.LOCATION, this);
        addChangeListener(commCycleVisualizer);

        scenarioScroller = new JScrollPane(scenarioPanel);
        scenarioScroller.setBorder(BorderFactory.createEmptyBorder());

        // add the ccv as componentlistener to both, the panel and
        // the scroller (for sroll and resize events)
        scenarioScroller.addComponentListener(commCycleVisualizer);
        scenarioPanel.addComponentListener(commCycleVisualizer);

        setColorsFromConfig();

        createAndAddActiveStarCouplerView(ChannelName.A);
        createAndAddActiveStarCouplerView(ChannelName.B);

        setLastEdit(EditCommandType.CREATE_CLUSTER);

    } // Constructor

    /**
     * Constructor with automatic naming.
     * @param pLearningComponent the learning module this scenario
     *                           will be used with
     */
    public FlexRayScenario( ModuleFR pLearningComponent) {
        this( "FlexRay #" + scenarioNumber.get(), pLearningComponent );
    }

    private void setColorsFromConfig() {
    	Object o;
    	
    	if (config.getSettingByKey(LeviConfiguration.COLOR_COMPONENT_BACKGROUND)!= null)
        	o = config.getSettingByKey(LeviConfiguration.COLOR_COMPONENT_BACKGROUND);
        else
            o = config.getSettingByKey(LeviConfiguration.COLOR_MODULE_BACKGROUND);
        if( o instanceof Color ) {
            scenarioPanel.setBackground((Color)o);
        }

        if (config.getSettingByKey(LeviConfiguration.COLOR_COMPONENT_FOREGROUND)!= null)
        	o = config.getSettingByKey(LeviConfiguration.COLOR_COMPONENT_FOREGROUND);
        else
        	o = config.getSettingByKey(LeviConfiguration.COLOR_MODULE_FOREGROUND);
        if( o instanceof Color ) {
            scenarioPanel.setForeground((Color)o);
        }

        if (config.getSettingByKey(LeviConfiguration.COLOR_COMPONENT_HIGHLIGHT)!= null)
        	o = config.getSettingByKey(LeviConfiguration.COLOR_COMPONENT_HIGHLIGHT);
        else
        	o = config.getSettingByKey(LeviConfiguration.COLOR_MODULE_HIGHLIGHT);
        if( o instanceof Color ) {
            CommCycleComponent.setSlotHighlightInnerColor((Color)o);
            TurningKnob.setHighlightColor((Color)o);
        }

    } // setColorsFromConfig

    public FrameFR getFrame() {
        return appFrame;
    }

    public void setLastEdit(EditCommandType edit) {
        lastEdit = edit;
    }

    public EditCommandType getLastEdit() {
        return lastEdit;
    }

    /**
     * Return the number (serial) of the scenario.
     * @return int containing this scenario's number.
     */
    public int getScenarioNumber() {
        return number;
    }

    /**
     * Return the root Component of the scenario containinig the
     * panel containing the representation of the cluster
     *
     * @return The Component displaying the scenario
     */
    public JComponent getScenarioComponent() {
        return scenarioScroller;
    }

    /**
     * Return the Panel of the scenario containinig a graphical
     * representation of the cluster
     *
     * @return The Component displaying the scenario
     */
    public JPanel getScenarioPanel() {
        return scenarioPanel;
    }


    /**
     * Return the Cluster belonging to this scenario
     *
     * @return A Cluster
     */
    public Cluster getCluster() {
        return cluster;
    }


    /**
     * Create a node with an unused Node ID.
     *
     * @return the ID of the new node.
     */
    public int createNode() {
        int nodeId = 1;
        // get the lowest available nodeId (some node could have been deleted)
        while ( (nodeId <= nodeNumber.get()) &&
                (cluster.getNode(nodeId) != null)) {
            nodeId++;
        }
        if( cluster.getNode(nodeId) != null )
            nodeId = nodeNumber.incrementAndGet();
        // make sure we don't have this id.
        while( cluster.getNode(nodeId) != null ) {
            nodeId=nodeNumber.incrementAndGet();
        }
        createNode(nodeId);
        return nodeId;
    } // createNode()

    /**
     * Create a new Node in scenario and view.
     *
     * @param id the id for the new node
     */
    public void createNode(final int id) {
        this.changed = true;
        Node node = new Node(cluster, id);
        cluster.addNode(node);
        createAndAddNodeView(id);
    } // createNode(id)

    /**
     * Create a biew for a node and add it to the visualization
     * @param id
     * @return the created NodeComponent (-View)
     */
    public NodeComponent createAndAddNodeView( int id ) {
        // No cluster?
        assert( cluster != null );

        this.changed = true;

        // scenario construction
        Node node = cluster.getNode(id);
//        numberOfNodes.incrementAndGet();
        // GUI construction
        NodeComponent nc = new NodeComponent( appFrame.getLearningModule(), node, this );
        nodeComponents.put(Integer.valueOf(node.getID()), nc);
        nc.setSize( nc.getPreferredSize() );

        int x = 10;
        int y = 10;
        Component c = scenarioPanel.getComponentAt(x,y);
        while( (c != null) && (c instanceof NodeComponent) && (c.getX() == x) && (c.getY() == y) ) {
            x+=10;
            y+=10;
            c = scenarioPanel.getComponentAt(x, y);
        }
        nc.setLocation(x,y);

        scenarioPanel.add(nc);
        nc.addMouseListener(componentDragger);
        nc.addMouseMotionListener(componentDragger);
        nc.addMouseListener(nodeControlMouseListener);
        nc.addKeyListener(nodeControlMouseListener);
        config.addPropertyChangeListener(nc);
        nc.addPropertyChangeListener(this);
        nc.requestFocus();
        scenarioPanel.setComponentZOrder(nc, 0);
        scenarioPanel.validate();
        scenarioPanel.repaint();

        updateScenarioDimension( );

        setLastEdit(EditCommandType.CREATE_NODEVIEW);
        fireStateChanged(); // a new Node has been added
        return nc;
    } // createNodeView

    /**
     *
     * @param node
     */
    public void deleteNode(final Node node) {
        node.getCluster().removeNode(node.getID());
        int nodeID = node.getID();
        NodeComponent nodeComponent = nodeComponents.get(Integer.valueOf(nodeID));
        nodeComponents.remove(Integer.valueOf(nodeID));
        scenarioPanel.remove(nodeComponent);
        if( nodeID == nodeNumber.get() ) {
            nodeNumber.decrementAndGet();
        }
//        numberOfNodes.decrementAndGet();
        nodeID--;
        nodeComponent = nodeComponents.get(Integer.valueOf(nodeID));
        if( nodeComponent != null ) {
            nodeComponent.requestFocus();
        }
        updateScenarioDimension();
        setLastEdit(EditCommandType.DELETE_NODE);
        fireStateChanged(); // a Node has been removed
    } // deleteNode(node)


    public void createStarCoupler(ChannelName channel) {
        createAndAddActiveStarCouplerView(channel);
    }

    public ChannelTopology getTopology( ChannelName channel ) {
        return topology[channel.ordinal()];
    }

    public void setTopology( ChannelName pChannel, ChannelTopology pTopology ) {
        topology[pChannel.ordinal()] = pTopology;
        this.changed = true;
    }

    /**
     *
     * @param channel
     * @return the created StarCouplerComponent
     */
    public ActiveStarCouplerComponent createAndAddActiveStarCouplerView(ChannelName channel) {
        ActiveStarCouplerComponent asc = new ActiveStarCouplerComponent(channel, this);
        getLearningModule().getConfiguration().addPropertyChangeListener(asc);
        this.changed = true;

        scenarioPanel.add(asc);

        asc.setSize(asc.getPreferredSize());
        switch( channel ) {
        case A:
            asc.setLocation(100,200);
            break;
        case B:
            asc.setLocation(200,200);
            break;
        default:
            break;
        }
        asc.addMouseListener(componentDragger);
        asc.addMouseListener(new MouseAdapter() {
            @Override
            public void mouseClicked(MouseEvent e) {
                if( e.getButton() == MouseEvent.BUTTON2 ) { // middle click
                    assert( e.getSource() instanceof ActiveStarCouplerComponent );
                    ActiveStarCouplerComponent source =
                        (ActiveStarCouplerComponent) e.getSource();
                    ChannelName channel = source.getChannel();
                    if( topology[channel.ordinal()] == ChannelTopology.BUS ) {
                        topology[channel.ordinal()] = ChannelTopology.STAR;
                    } else {
                        topology[channel.ordinal()] = ChannelTopology.BUS;
                    }
                    scenarioPanel.repaint();
                } // middle click

                if( e.getButton() == MouseEvent.BUTTON3 ) { // right click
                    assert( e.getSource() instanceof ActiveStarCouplerComponent );
                    ActiveStarCouplerComponent source =
                        (ActiveStarCouplerComponent) e.getSource();
                    // cannot use leviPopup here, because it is no base class
                    // but a complete menu.
                    JPopupMenu popUp = new JPopupMenu();
                    ChannelName channel = source.getChannel();
                    ChannelTopology channelTopology = topology[channel.ordinal()];
                    boolean isBus = (channelTopology == ChannelTopology.BUS);

                    JCheckBoxMenuItem star = new JCheckBoxMenuItem(res.getResourceByKey("menu.topology.star") /*, starIcon */);
                    JCheckBoxMenuItem bus = new JCheckBoxMenuItem(res.getResourceByKey("menu.topology.bus") /*, busIcon */);

                    if (config.getSettingByKey(LeviConfiguration.COLOR_COMPONENT_BACKGROUND)!= null)
                    	star.setBackground((Color)config.getSettingByKey(LeviConfiguration.COLOR_COMPONENT_BACKGROUND));
                    else
                    	star.setBackground((Color)config.getSettingByKey(LeviConfiguration.COLOR_MODULE_BACKGROUND));
                    if (config.getSettingByKey(LeviConfiguration.COLOR_COMPONENT_FOREGROUND)!= null)
                    	star.setForeground((Color)config.getSettingByKey(LeviConfiguration.COLOR_COMPONENT_FOREGROUND));
                    else
                    	star.setForeground((Color)config.getSettingByKey(LeviConfiguration.COLOR_MODULE_FOREGROUND));
                    star.setSelected(!isBus);
                    ToggleTopologyAction starAction = new ToggleTopologyAction(getLearningModule(),star,channel,ChannelTopology.STAR);
                    star.setAction(starAction);
                    popUp.add(star);
                    if (config.getSettingByKey(LeviConfiguration.COLOR_COMPONENT_BACKGROUND)!= null)
                    	bus.setBackground((Color)config.getSettingByKey(LeviConfiguration.COLOR_COMPONENT_BACKGROUND));
                    else
                    	bus.setBackground((Color)config.getSettingByKey(LeviConfiguration.COLOR_MODULE_BACKGROUND));
                    if (config.getSettingByKey(LeviConfiguration.COLOR_COMPONENT_FOREGROUND)!= null)
                    	bus.setForeground((Color)config.getSettingByKey(LeviConfiguration.COLOR_COMPONENT_FOREGROUND));
                    else
                    	bus.setForeground((Color)config.getSettingByKey(LeviConfiguration.COLOR_MODULE_FOREGROUND));
                    bus.setSelected(isBus);
                    ToggleTopologyAction busAction = new ToggleTopologyAction(getLearningModule(),star,channel,ChannelTopology.BUS);
                    bus.setAction(busAction);
                    popUp.add(bus);
                    popUp.show(source, e.getX()-(int)popUp.getPreferredSize().getWidth()/2, e.getY()-(int)popUp.getPreferredSize().getHeight()/2);
                }
            }
        });
        asc.addMouseMotionListener(componentDragger);
        //asc.addPropertyChangeListener(this);
        this.addChangeListener(asc);
        asc.setVisible(false);
        activeStarCouplers.put(channel, asc);

        setLastEdit(EditCommandType.CREATE_CHANNELVIEW);
        fireStateChanged();

        return asc;
    } // createAndAddActiveStarCouplerView

    /**
     * Update the scenario dimensions (to keep the scrollers nice ;-))
     *
     */
    public void updateScenarioDimension() {
        int previousWidth = scenarioDimension.width;
        int previousHeight = scenarioDimension.height;
        /*
         * Prevent conflicting parallel updates
         */
        synchronized (scenarioDimension) {
            int maxWidth = 0;
            int maxHeight = 0;
            for( Component c : this.scenarioPanel.getComponents() ) {
                /*
                 * take the CommCycleComponent only into account
                 * if it is not automoving. (Would make no sense
                 * since the component moves while scrolling)
                 */
                boolean change = true;
                if( c instanceof CommCycleComponent ) {
                    change=!((CommCycleComponent)c).isAutoMove();
                }
                if( change ) {
                    int moduleWidth = c.getX() + c.getWidth();
                    int moduleHeight = c.getY() + c.getHeight();
                    maxWidth = Math.max(maxWidth,moduleWidth);
                    maxHeight = Math.max(maxHeight,moduleHeight);
                }
            }
            // if something has changed, apply the changes
            if( (previousWidth != maxWidth) || (previousHeight != maxHeight) ) {
                scenarioDimension.setSize(maxWidth,maxHeight);
                scenarioPanel.setPreferredSize(scenarioDimension);
            }

            // set the current size if the scenario view changed.
            // Do not set the size if it is smaller than the viewport.
            JViewport viewPort = scenarioScroller.getViewport();
            int viewportWidth = viewPort.getWidth();
            int viewportHeight = viewPort.getHeight();
            int panelWidth = scenarioPanel.getWidth();
            int panelHeight = scenarioPanel.getHeight();
            if( (maxWidth < panelWidth) || (maxHeight < panelHeight) ) {
                scenarioPanel.setSize( Math.max(viewportWidth,maxWidth), Math.max(viewportHeight,maxHeight));
            }
        }
    } // updateScenarioDimension


    /**
     * Get the View (NodeComponent) of a node.
     * @param n A node in the cluster
     * @return The NodeComponent that displays n
     */
    public NodeComponent getNodeComponent(Node n) {
        return nodeComponents.get(Integer.valueOf(n.getID()));
    }



    public void updateView() {
        scenarioPanel.repaint();
        commCycleVisualizer.repaint();
    }

    /**
     * Entry point for recursively setting Locales
     * on all components in the scenarioPanel/viewer panel.
     *
     */
    public void updateLocales() {
        scenarioPanel.setLocale(Locale.getDefault());
        for(Component c : scenarioPanel.getComponents()) {
            c.setLocale(Locale.getDefault());
        }
    }



    /*
     * non-javadoc
     *
     * @see leviScenario#buildDOMElements(org.w3c.dom.Element, org.w3c.dom.Document)
     */
    @Override
    public Element buildDOMElements(Element scenario, Document xmlDoc) throws ParserConfigurationException {
        throw new RuntimeException("check the design");
    } // buildDOMElements


    /*
     * non-javadoc
     *
     * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
     */
    public void propertyChange(PropertyChangeEvent evt) {
        // locale changed
        if( evt.getPropertyName().equals(LeviResourceManager.PROPERTY_LOCALE)) updateLocales();

        // some location changed. This may change the modification status ("changed")
        if( evt.getPropertyName().equals(Property.LOCATION)) {
            updateScenarioDimension();
            boolean isCommCycle = evt.getSource() instanceof CommCycleComponent;
            if( !(isCommCycle && ((CommCycleComponent)evt.getSource()).isAutoMove()) ) {
                setChanged(true);
            }
        } else

        // changing the automove status changes the "scenario" (the "view", to be precise)
        if( evt.getPropertyName().equals(Property.AUTOMOVE)) {
            setChanged(true);
        } else

        if( evt.getPropertyName().startsWith("Color")) {
            setColorsFromConfig();
        }
    }


    /*
     * -------------- ChangeListener part -----------------
     *            (from AbstractSpinnerModel)
     */


    /**
     * Only one ChangeEvent is needed per scenario instance since the
     * event's only (read-only) state is the source property.  The source
     * of events generated here is always "this".
     */
    private transient ChangeEvent changeEvent = null;


    /**
     * The list of ChangeListeners for this scenario.  Subclasses may
     * store their own listeners here.
     */
    protected EventListenerList listenerList = new EventListenerList();


    /**
     * Adds a ChangeListener to the scenario's listener list.  The
     * ChangeListeners must be notified when the scenarios value changes.
     *
     * @param l the ChangeListener to add
     * @see #removeChangeListener
     * @see SpinnerModel#addChangeListener
     */
    public void addChangeListener(ChangeListener l) {
//        DebugUtil.hint("adding " + l.getClass().getName() + " to " + getName() );
        listenerList.add(ChangeListener.class, l);
    }


    /**
     * Removes a ChangeListener from the scenario's listener list.
     *
     * @param l the ChangeListener to remove
     * @see #addChangeListener
     * @see SpinnerModel#removeChangeListener
     */
    public void removeChangeListener(ChangeListener l) {
//        DebugUtil.hint("removing " + l.getClass().getName() + " from " + getName() );
        listenerList.remove(ChangeListener.class, l);
    }


    /**
     * Returns an array of all the <code>ChangeListener</code>s added
     * to this AbstractSpinnerModel with addChangeListener().
     *
     * @return all of the <code>ChangeListener</code>s added or an empty
     *         array if no listeners have been added
     */
    public ChangeListener[] getChangeListeners() {
        return listenerList.getListeners(ChangeListener.class);
    }


    /**
     * Run each ChangeListeners stateChanged() method.
     * made public to allow ComponentFR to fire changed status
     * @see EventListenerList
     */
    public void fireStateChanged() {
        Object[] listeners = listenerList.getListenerList();
        for (int i = listeners.length - 2; i >= 0; i -=2 ) {
            if (listeners[i] == ChangeListener.class) {
                if (changeEvent == null) {
                    changeEvent = new ChangeEvent(this);
                }
                ((ChangeListener)listeners[i+1]).stateChanged(changeEvent);
            } else {
                DebugUtil.mark("Not sending to " + listeners[i]);
            }
        }
    } // fireStateChanged()


    /**
     * Return an array of all the listeners of the given type that
     * were added to this scenario.  For example to find all of the
     * ChangeListeners added to this scenario:
     * <pre>
     * myAbstractSpinnerModel.getListeners(ChangeListener.class);
     * </pre>
     *
     * @param <T> the generic type parameter for listenerType
     * @param listenerType the type of listeners to return, e.g. ChangeListener.class
     * @return all of the objects receiving <em>listenerType</em> notifications
     *         from this scenario
     */
    public <T extends EventListener> T[] getListeners(Class<T> listenerType) {
        return listenerList.getListeners(listenerType);
    }

    /**
     * If this scenario has changed since the last save operation, this
     * will return false;
     *
     * FIXME: I think this should be delegated to Cluster
     *
     * @return the saved
     */
    @Override
    public boolean hasChanged() {
        return changed;
    }

    @Override
    public void setName(String name) {
        super.setName(name);
        setLastEdit(EditCommandType.CHANGE_NAME);
        fireStateChanged();
    }

    /**
     * Preventing typecasts by using Java 5 covariant methods to return
     * the proper type learningModule.
     * @return the learning module
     * @see learningVisualizations.scenario.LeviScenario#getLearningModule()
     */
    @Override
    public ModuleFR getLearningModule() {
        return (ModuleFR)super.getLearningModule();
    }


    @Override
    public boolean isOk() {
        // FIXME: alway true?
//        DebugUtil.mark("returning always true!");
        return true;
    }

    /**
     * Set the "changed" status.
     * A runtime exception is thrown if this method is called from
     * forbidden places.
     *
     * @param changed
     */
    public void setChanged(boolean changed) {
//        DebugUtil.hint("setting changed!");
        StackTraceElement ste = null;
        try {
            throw new Exception();
        } catch (Exception e ) {
            ste = e.getStackTrace()[1];
        }
        if(ste.getClassName().startsWith("flexray.")) {
            if( this.changed != changed ) {
                this.changed = changed;
//                DebugUtil.hint("state change");
                fireStateChanged(); // just in case
            }
        } else {
            throw new RuntimeException("setChanged() call not allowed from " + ste.toString());
        }
    } // setChanged



    /**
     * Write this scenario into an XML structure
     *
     * @param xmlDoc
     * @return the XML Element containing the scenario's XML description
     */
    public Element toXML(final Document xmlDoc) {
        Element element = xmlDoc.createElement(XT_FLEXRAYSCENARIO);
        element.appendChild(cluster.toXML(xmlDoc));
        element.appendChild(basicViewToXML(xmlDoc));
        return element;
    }

    /**
     * Construct the XML representing this scenario's standard visualization
     *
     * @param xmlDoc
     * @return the XML representation
     */
    public Element basicViewToXML(final Document xmlDoc) {
        Element viewTree = xmlDoc.createElement(XT_BASICVIEW);

        Element topologyElement = xmlDoc.createElement(XT_CHANNEL_TOPOLOGY);
        topologyElement.setAttribute(XT_CHANNEL_TOPOLOGY_A, topology[ChannelName.A.ordinal()].name());
        topologyElement.setAttribute(XT_CHANNEL_TOPOLOGY_B, topology[ChannelName.B.ordinal()].name());
        viewTree.appendChild(topologyElement);

        for(Entry<Integer,NodeComponent> nce : nodeComponents.entrySet()){
            viewTree.appendChild(nce.getValue().toXML(xmlDoc));
        }
        for(Entry<ChannelName,ActiveStarCouplerComponent> asc : activeStarCouplers.entrySet()){
            viewTree.appendChild(asc.getValue().toXML(xmlDoc));
        }
        viewTree.appendChild(commCycleVisualizer.toXML(xmlDoc));
        return viewTree;
    }

    /**
     * restore scenario from XML
     * @param element
     * @return true if the restore was successfull, false otherwise
     */
    public boolean restoreFromXML(final Element element) {
        
        if ( element.getTagName().equals("flexrayscenario") || element.getTagName().equals("flexraymodel")) {
	        NodeList nodeList = element.getChildNodes();
	        for( int i = 0 ; i < nodeList.getLength() ; i++ ) {
	        	if (nodeList.item(i) instanceof Element) {
	        		Element item = (Element)nodeList.item(i);
		            if( item.getNodeName().equals(XT_CLUSTER)) {
		                cluster.restoreFromXML(item);
		           }
		            if( item.getNodeName().equals(XT_BASICVIEW)) {
		                restoreBasicViewFromXML(item);
		           }
	            }
	
	        }
        }
        
        return true;
    } // restoreFromXML

    /**
     * restore the view parameters from an XML tree node
     * @param treeNode
     * @return true on success, false otherwise
     */
    public boolean restoreBasicViewFromXML(final org.w3c.dom.Node treeNode) {
        if( treeNode == null ) return false;
        NodeList viewTreeNodes = treeNode.getChildNodes();

        // Z-Orders have to be stored, sorted and applied later, because
        // they have to be set in correct order.
        //       <Z-Order, Node-ID>
        SortedMap<Integer, Integer> zOrders = new TreeMap<Integer,Integer>();

        for(int index = 0 ; index < viewTreeNodes.getLength(); index++) {
        	if (viewTreeNodes.item(index) instanceof Element) {
	            Element element = (Element)viewTreeNodes.item(index);
	            String nodeName = element.getNodeName();
	
	            /*
	             *
	             */
	
	            if( nodeName.equals(XT_CHANNEL_TOPOLOGY)) {
	                topology[ChannelName.A.ordinal()] =
	                    ChannelTopology.valueOf(element.getAttribute(XT_CHANNEL_TOPOLOGY_A));
	                topology[ChannelName.B.ordinal()] =
	                    ChannelTopology.valueOf(element.getAttribute(XT_CHANNEL_TOPOLOGY_B));
	            } // XT_CHANNEL_TOPOLOGY
	
	            /*
	             * element is a node view
	             */
	            if( nodeName.equals(XT_NODEVIEW)) {
	                Integer id = Integer.valueOf(element.getAttribute(XT_NODEVIEW_ID));
	                NodeComponent nodeComponent = createAndAddNodeView(id.intValue());
	                //= nodeComponents.get(id);
	                int z = nodeComponent.restoreFromXML(element);
	                zOrders.put(Integer.valueOf(z), id);
	            } // XT_NODEVIEW
	
	            if( nodeName.equals(XT_STARCOUPLERVIEW)) {
	                ChannelName channel = ChannelName.valueOf(element.getAttribute(XT_STARCOUPLERVIEW_CHANNEL));
	                ActiveStarCouplerComponent asc = activeStarCouplers.get(channel);
	//                NodeComponent nodeComponent = createAndAddNodeView(id.intValue());
	                int z = asc.restoreFromXML(element);
	                zOrders.put(Integer.valueOf(z), Integer.valueOf(-2-channel.ordinal()));
	            } // XT_VIEWNODE
	
	            /*
	             * element is a comm cycle view
	             */
	            if(nodeName.equals(XT_COMMCYCLEVIEW)) {
	                int z = commCycleVisualizer.fromXML(element);
	                zOrders.put(Integer.valueOf(z), Integer.valueOf(-1));
	            } // XT_COMMCYCLEVIEW
        	}
        } // ALL Nodes


        /*
         * Set the Z-Orders
         */
        for( Entry<Integer,Integer> zEntry : zOrders.entrySet() ) {
            int id = zEntry.getValue().intValue();
            int z = zEntry.getKey().intValue();
            switch( id ) {
            case -1:
                scenarioPanel.setComponentZOrder(commCycleVisualizer, z);
                break;
            case -2: // Star Coupler channel A
                scenarioPanel.setComponentZOrder(activeStarCouplers.get(ChannelName.A), z);
                break;
            case -3: // Star Coupler channel B
                scenarioPanel.setComponentZOrder(activeStarCouplers.get(ChannelName.B), z);
                break;
            default:
                Node n = cluster.getNode(id);
                scenarioPanel.setComponentZOrder(getNodeComponent(n), z);
                break;
            }
        }
        return true;
    } // restoreBasicViewFromXML


    /**
     * @return the editable
     */
    public boolean isEditable() {
        if(!getLearningModule().getSimulator().isStop() ) return false;
        return editable;
    }

    /**
     * @param editable the editable to set
     */
    public void setEditable(boolean editable) {
        this.editable = editable;
    }

    /*
     * react to state changes
     * @param e
     * @see javax.swing.event.ChangeListener#stateChanged(javax.swing.event.ChangeEvent)
     */
    public void stateChanged(ChangeEvent e) {
        Object source = e.getSource();
        if( source instanceof CommCycleConfigDialog.CommCycleSpinnerListener ||
            source instanceof CommCycleConfigDialog) {
            commCycleVisualizer.repaint();
            updateScenarioDimension();
        } else {
            DebugUtil.mark("state change from " + source.getClass().getName());
        }
    }


    public int getEditState() {
        return 0;
    }

    /*
     * Methode:		checkScenario
     * 
     * Description:	brerprfen, ob das Szenario korrekt d.h. lauffhig ist.
     * 
     * Parameters:  
     * 
     * Date: 		30.01.2008
     * @author      sirocic
     */
	public boolean checkScenario() {
		     
		// TRACE OUTPUT
		if (LeviModule.isTraceActive()) System.out.println("CALL FlexRayScenario.checkScenario:");
		  
		// Szenario berprfen ...
		   
		// TRACE OUTPUT
		if (LeviModule.isTraceActive()) System.out.println("END FlexRayScenario.checkScenario !!!!");
		
		return true;
		
	}
	
  /*
    * Methode:		init
    * 
    * Description:	Initialisiert das Scenario d.h. alle Objekte des Scenarios, die einen Zustand
    * 				whrend der Simulation annehmen.
    * 
    * Date: 		 30.01.2008
    * @author        sirocic
    */	
   public void init() {
	   Iterator d;
	   
	   // TRACE OUTPUT
	   if (LeviModule.isTraceActive()) System.out.println("CALL FlexRayScenario.init:");
	  
	   // TRACE OUTPUT
	   if (LeviModule.isTraceActive()) System.out.println("END FlexRayScenario.init !!!!");
   }
	   
   
   /*
    * Methode:		createObjectsFromXML
    * 
    * Description:	Erzeugt zu den im XML-file spezifizierten Objekten die entsprechenden Szenario-objekte.
    * 
    * Parameters:   xmlObjects = xml element of the objects
    * 
    * Date: 		30.01.2008
    * @author       sirocic
    */
    public void createObjectsFromXML(Element xmlObjects) {
    	
    }
}
