/*
 * Generate 22.05.2005 by Birgit Sirocic
 */
package learningVisualizations;

import java.awt.Color;
import java.awt.Font;
import java.beans.*;
import java.io.*;
import java.util.*;
import java.util.Map.Entry;

/*
 * Class:		leviConfiguration
 *
 * Description:	Containing the system configuration which is saved in the LS_config.xml.
 * 				It contains for example the path of the jar file, the current color settings,
 * 				the language selection and more.
 *
 * Date: 		22.05.2005
 * @author      sirocic
 **/
public class LeviConfiguration {

	public static final String COLOR_COMPONENT_BACKGROUND = "ColorComponentBackground";
    public static final String COLOR_COMPONENT_FOREGROUND = "ColorComponentForeground";
    public static final String COLOR_COMPONENT_HIGHLIGHT = "ColorComponentHighlight";
    public static final String COLOR_MODULE_BACKGROUND = "ColorModuleBackground";
    public static final String COLOR_MODULE_FOREGROUND = "ColorModuleForeground";
    public static final String COLOR_MODULE_HIGHLIGHT = "ColorModuleHighlight";
    public static final String COLOR_BUTTON_BACKGROND = "ColorButtonBackground";
    public static final String FONT_LABELS = "FontLabel";
    public static final String FONT_SMALL = "FontSmallLabel";
    public static final String KEYDEFSCENARIO="DefaultScenario";
    public static final String KEYDEFMODEL="DefaultModel";
    public static final String FONT_MENU= "FontMenu";

    protected LeviModule learningModule;
    
    // contains the configuration settings
    // DF 20060608 1) no need to make this static. 2) Added types
    private Hashtable<String,Object> settings= new Hashtable<String,Object>();
    private Hashtable<String,Object> infos = new Hashtable<String,Object>();

    // DF 20060608 no need to make this static. It was accessed with this. anyway
    private String configFile="";

    // DF 20060608 Added support for propagating property changes
    private PropertyChangeSupport changeSupport = new PropertyChangeSupport(this);

    // Constructor
    public LeviConfiguration() {
        // DF 20060608 empty construtor comment
    }
    public LeviConfiguration(LeviModule learningModule, String name)
    {
        infos.put("module",name);
    }


    // MUTATOR METHODES
    public void setModuleName(String name) {
        infos.put("module",name);
    }
    public void setLearningModule(LeviModule learningModule) {
        this.learningModule = learningModule;
    }
    /*
     * Methode:		loadConfiguration
     *
     * Description:	Loading the settings from the xml configuration file. The name of the file
     * 				is given by the parameter <configFilename>. The file is searched
     * 				in the same directory where the jar file is placed.
     *
     * Date: 		    22.05.2005
     * @author        sirocic
     **/
    public boolean loadConfiguration(String configFile)
    {
        Hashtable<String,Object> newInfos;
        String path;

        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("CALL leviConfiguration.loadConfiguration: file name="+configFile);

        // Filename merken
        this.configFile = configFile;
        path = getWorkingDir()+ this.configFile;

        Object result2 = null;
        Object result1 = null;

        try {

            XMLDecoder d = new XMLDecoder(
                    new BufferedInputStream(
                            new FileInputStream(path)));
            result1 = d.readObject();
            result2 = d.readObject();
            d.close();

            // check if the config file is the right one
            newInfos = (Hashtable<String,Object>)result1;
            if (newInfos.containsKey("module")) {
                if (((String)newInfos.get("module")).equals(infos.get("module"))) {
                    newInfos = (Hashtable<String, Object>)result2;
                }
            } 
            else if (newInfos.containsKey("component")) {
                if (((String)newInfos.get("component")).equals(infos.get("module"))) {
                    newInfos = (Hashtable<String, Object>)result2;
                }
            }
            // fire the property changes
            for( Entry<String,Object> e : newInfos.entrySet()) {
                Object oldValue = infos.get(e.getKey());
                if( !e.getValue().equals(oldValue)) {
                    changeSupport.firePropertyChange(e.getKey(), oldValue, e.getValue());
                }
            }

            // take over the results
            settings = newInfos;

        } catch (Exception e) {
            // Fehler beim Lesen der Konfiguration
            return false;
        }

        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("END leviConfiguration.loadConfiguration");

        return true;

    } // END loadConfiguration


    /*
     * Methode:		saveConfiguration
     *
     * Description:	Saving the settings into the xml configuration file. The name of the file
     * 				is saved in the Attribute <configFile>. The file is written
     * 				in the same directory where the jar file is placed.
     *
     * Date: 		    22.05.2005
     * @author        sirocic
     **/
    public boolean saveConfiguration()
    {
        String path = getWorkingDir()+configFile;

        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("CALL leviConfiguration.saveConfiguration:");

        try {
            XMLEncoder e = new XMLEncoder(
                    new BufferedOutputStream(
                            new FileOutputStream(path)));
            e.writeObject(infos);
            e.writeObject(settings);
            e.close();

        } catch (Exception e) {
           // Can not write configuration file
            return false;
        };

        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("END leviConfiguration.saveConfiguration");

        return true;

    } // END saveConfiguration

    /**
     * Methode:		createDefaultProperties
     *
     * Description:	Generates the default properties of the common module.
     *
     * Date: 		    08.09.2005
     * @author        sirocic
     */
      public void createDefaultProperties() {

          // TRACE OUTPUT
          if (LeviModule.isTraceActive()) System.out.println("CALL leviConfiguration.createDefaultProperties:");

            // language
            putSettingByKey("Language", Locale.getDefault().getLanguage());
            // Colors
            putSettingByKey("ColorModuleBackground", Color.WHITE);
            putSettingByKey("ColorModuleForeground", Color.BLACK);
            putSettingByKey("ColorModuleHighlight", Color.RED);
            putSettingByKey("ColorButtonBackground", new Color(240, 240, 240));

            // Font
            putSettingByKey("FontLabel", new Font("SansSerif", Font.BOLD, 14));
            putSettingByKey("FontSmallLabel", new Font("SansSerif", Font.PLAIN, 12));
            putSettingByKey("FontMenu", new Font("SansSerif", Font.BOLD, 12));

            // Modus
            putSettingByKey("Help", Boolean.FALSE);
            putSettingByKey("Tooltips", Boolean.TRUE);

            // Simulation
            putSettingByKey("SleepFactor", 15);

          // TRACE OUTPUT
          if (LeviModule.isTraceActive()) System.out.println("END leviConfigurationonent.createDefaultProperties");
      }

    /**
     * Methode:		putSettingByKey
     *
     * Description:	Changing the value of a parameter. The parameter is addressed
     * 				by its key.
     *              This method will fire a propertyChange if appropriate.
     *
     * Date: 		    22.05.2005
     * @author        sirocic
     **/
    public boolean putSettingByKey(String sKey, Object newValue)
    {
        Object oldValue = settings.get(sKey);
        if((newValue != null) && (!newValue.equals(oldValue))) {
        	// wird ein Pfad abgelegt z.B. DefaultScenario werden in ihm alle Pfadtrennzeichen durch "/"
        	// ersetzt. Das Zeichen funktioniert sowohl unter Window als auch unter Unix. 
        	// Vermeidet beim Plattformwechsel, dass z.B. das eingetragene DefaultScenario nicht geladen werden
        	// kann!!!! 
        	if (sKey.equals(KEYDEFSCENARIO) || sKey.equals(KEYDEFMODEL)) 
        		newValue = changeSeperators(newValue.toString());
        	
        	// jetzt eintragen
            settings.put(sKey,newValue);
            changeSupport.firePropertyChange(sKey, oldValue, newValue);
        }
        if (LeviModule.isTraceActive()) System.out.println("END leviConfiguration.putSettingByKey key="+sKey);
        return true;

    }
    /**
     * Methode:		changeSeperators
     *
     * Description:	Tauscht alle Pfadtrennzeichen durch "/" aus. Dieses Zeichen funktioniert als Trennzeichen
     * 				unter Windows und Unix. 
     *
     * Date: 		22.11.2007
     * @author        sirocic
     **/
    private Object changeSeperators(String path)
    {
        String retString="", separator = "/";
        
        if (LeviModule.isTraceActive()) System.out.println("CALL leviConfiguration.changeSeperators");
        
        if (path != null) 
        	retString = path.replace(System.getProperty("file.separator"), separator );
        	
        if (LeviModule.isTraceActive()) System.out.println("END leviConfiguration.changeSeperators");
        
        return (Object) retString;

    }

    /**
     * Methode:		getSettingByKey
     *
     * Description:	Getting the value of a parameter. The parameter is addressed
     * 				by its key.
     *
     * Date: 		    22.05.2005
     * @author        sirocic
     **/
    public Object getSettingByKey(String sKey)
    {
        // DF 2006-06-08 - "if"-block removed.
        // get() returns null anyway if the key is not found.
        return settings.get(sKey);
    }

    /**
     * Methode:		removeSettingByKey
     *
     * Description:	Removes the value of a parameter. The parameter is addressed
     * 				by its key.
     *
     * Date: 		    22.05.2005
     * @author        sirocic
     **/
    public void removeSettingByKey(String sKey)
    {
        	settings.remove(sKey);
    }
    /**
     * Methode:		keyExist
     *
     * Description:	Test if the given key exist in the settings or not.
     *
     * Date: 		    13.04.2006
     * @author        sirocic
     **/
    public boolean keyExist(String sKey)
    {
        // DF 2006-06-08 - "if"-block removed.
        // containsKey() already produces the desired result
        return settings.containsKey(sKey);
    }

    /**
     * Methode:		getKeysOfSettings
     *
     * Description:	Getting all keys of the settings.
     *
     * Date: 		    08.09.2005
     * @author        sirocic
     **/
    public Enumeration<String> getKeysOfSettings()
    {
        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("CALL leviConfiguration.getKeysOfSettings:");

        if (!settings.isEmpty()) {
            if (LeviModule.isTraceActive()) System.out.println("END leviModule.getKeysOfSettings");
            return settings.keys();
        }

        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("END leviModule.getKeysOfSettings return=null");

        return null;
    }

    /*
     * Methode:		getWorkingDir
     *
     * Description:	Finding out the path of the jar file. This is the
     * 				current working directory.
     *
     * Date: 		    22.05.2005
     * @author        sirocic
     **/
    protected String getWorkingDir()
    {
        int i;
        Properties sys = System.getProperties();
        String path = sys.getProperty("user.dir");

        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("CALL leviConfiguration.getWorkingDir:");

        // convert all backslashes to forward slashes as usual in unix
        while ((i=path.indexOf("\\"))>=0) {
            if (i<path.length()-1) {
                path=path.substring(0,i)+"/"+path.substring(i+1);
            } else {
                path=path.substring(0,i)+"/";
            }
        }

        if (!path.endsWith("/")) path=path+"/";

        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("END leviModule.getWorkingDir path="+path);

        return path;

    } // END getWorkingDir

    /*
     * ---- property change support
     */

    /*
     * (non-javadoc)
     * @see java.beans.PropertyChangeSupport#addPropertyChangeListener(java.beans.PropertyChangeListener)
     */
    /**
     * Method for implementing property change support: adding listeners
     *
     * @param listener  The PropertyChangeListener to be added
     * @see java.beans.PropertyChangeSupport#addPropertyChangeListener(PropertyChangeListener)
     */
    public void addPropertyChangeListener(PropertyChangeListener listener) {
        changeSupport.addPropertyChangeListener(listener);
    }


    /**
     * Method for implementing property change support: adding listeners
     *
     * @param propertyName  The name of the property to listen on.
     * @param listener  The PropertyChangeListener to be added
     * @see java.beans.PropertyChangeSupport#addPropertyChangeListener(String, PropertyChangeListener)
     */
    public void addPropertyChangeListener(String propertyName, PropertyChangeListener listener) {
        changeSupport.addPropertyChangeListener(propertyName,listener);
    }


    /**
     * Method for implementing property change support: removing listeners
     *
     * @param listener The PropertyChangeListener to be removed
     * @see PropertyChangeSupport#removePropertyChangeListener(PropertyChangeListener);
     */
    public void removePropertyChangeListener(PropertyChangeListener listener) {
        changeSupport.removePropertyChangeListener(listener);
    }


    /**
     * Method for implementing property change support: removing listeners
     *
     * @param propertyName The name of the property that was listened on
     * @param listener The PropertyChangeListener to be removed
     * @see PropertyChangeSupport#removePropertyChangeListener(String, PropertyChangeListener)
     */
    public void removePropertyChangeListener(String propertyName, PropertyChangeListener listener) {
        changeSupport.removePropertyChangeListener(propertyName, listener);
    }



}
