/* leviModule.java
 *
 * $Id: LeviModule.java,v 1.8 2009/04/28 04:49:16 sirocic Exp $
 *
 * Created on 24.06.2005
 * @author sirocic
 */
package learningVisualizations;

import java.awt.*;
import java.io.*;
import java.net.*;
import java.util.*;

import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import javax.xml.*;
import javax.xml.parsers.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;
import javax.xml.validation.*;

import learningVisualizations.editor.LeviEditor;
import learningVisualizations.editor.LeviEditorInterface;
import learningVisualizations.gui.LeviColorChooseDialog;
import learningVisualizations.gui.LeviColorDialog;
import learningVisualizations.gui.LeviDefaultScenarioDialog;
import learningVisualizations.gui.LeviFontChooseDialog;
import learningVisualizations.gui.LeviFontDialog;
import learningVisualizations.gui.LeviFrame;
import learningVisualizations.gui.LeviQuitDialog;
import learningVisualizations.gui.actions.LeviAbstractAction;
import learningVisualizations.help.LeviHelpInterface;
import learningVisualizations.scenario.LeviScenario;
import learningVisualizations.scenario.LeviScenarioInterface;
import learningVisualizations.simulator.LeviSimulator;
import learningVisualizations.simulator.LeviSimulatorInterface;
import learningVisualizations.visualizer.LeviVisualizerInterface;

import org.w3c.dom.*;
import org.xml.sax.SAXException;

/**
 * @author sirocic
 *
 * Class: leviModule
 *
 * Description: The class "leviModule" of the general learning module.
 *
 * Date:	26.06.2005
 */
public abstract class LeviModule {

    // CONSTANTS
	public static final String 	VERSION = "Version 0.1";
	public static final String MODULENAME = "LeVi module ";
    public static final String DEFAULTCONFIGFILE = "leviConfig.xml";
    public static final int LECTUREVIEW = 1;
    public static final int SELFSTUDYVIEW = 2;
    public static final int EDITORMODE = 3;
    public static final int SIMULATIONMODE = 4;
    public static final String XMLTAGTYPE = "Type";
    public static final String XMLSCREEN = "Screen";
    public static final String XMLSCREENWIDTH = "ScreenWidth";
    public static final String XMLSCREENHEIGHT = "ScreenHeight";
    public static String TYPE = "XXX";

    // GENERAL LEARNING Module ATTRIBUTES
    public String version;
    private String name;
    private LeviConfiguration config;
    private LeviResourceManager ressources;
    private String language;
    protected int view=SELFSTUDYVIEW; // protected to allow derived classes to change it
    private int mode=EDITORMODE;
    private boolean helpVisible=true;
    private boolean tooltipsVisible=true;
    private boolean defaultScenarioExist=false;
    private int loadedScreenWidth, loadedScreenHeight;

    // GENERAL LEARNING Module PARTS
    private LeviHelpInterface help;
    private LeviVisualizerInterface visualizer;
    private LeviEditorInterface editor;
    private LeviSimulatorInterface simulator;
    private ArrayList<LeviScenarioInterface> scenarios;
    private int currentScenarioIndex = -1;
    private File xmlScenarioFile;
    private Document xmlScenarioDoc;

    // THE GENERAL GUI
    private LeviFrame frame;

	// FILE OPEN AND SAVE DIALOG
    private JFileChooser fileOpenSaveDialog;
    private LeviColorChooseDialog colorChooseDialog;
    private LeviColorDialog colorConfigDialog;
    private LeviFontChooseDialog fontChooseDialog;
    private LeviFontDialog fontConfigDialog;
    private LeviDefaultScenarioDialog defaultScenarioConfigDialog;
    private LeviQuitDialog quitDialog;

    // HELP VARIABLES
    private HashMap toolbarButtons;

    // TRACE VARIABLE
    private static boolean trace=false;

    public LeviModule()
    {
        this.name = "noname";
        this.config = new LeviConfiguration();
        this.ressources = new LeviResourceManager();
        this.language = "de";
        this.xmlScenarioFile = null;
        this.xmlScenarioDoc = null;
        this.scenarios = new ArrayList<LeviScenarioInterface>();
        newScenario();
        this.fileOpenSaveDialog = new JFileChooser();
        this.colorChooseDialog = new LeviColorChooseDialog();
        this.colorConfigDialog = new LeviColorDialog();
        this.fontChooseDialog = new LeviFontChooseDialog();
        this.fontConfigDialog = new LeviFontDialog();
        this.loadedScreenHeight =0;
        this.loadedScreenWidth =0;

    }
    /**
     * Short hand for <tt>leviModule( name, config )</tt>, using
     * the default config file.
     *
     * @param cname name of the module
     */
    public LeviModule(String name)
    {
        this(name, "learningVisualizations.leviConfiguration", DEFAULTCONFIGFILE, VERSION);
    }

    /**
     *
     * Methode:       Contructors
     *
     * Description:   Reading the configuration file and the general resources.
     *                Setting the feature like title, language etc. to the values
     *                readed from the configuration and resources, respectively.
     *
     * Date:          24.06.2005
     * @author        sirocic
     * @param cname
     * @param configFile
     */
    public LeviModule(String name, String configName, String configFileName, String version) {

  	   String scenarioName;
  	   Boolean tmp2;
  	  
        // TRACE OUTPUT
        if (isTraceActive()) System.out.println("CALL leviModule.Contructors(cname, configFileName)");

        // SETTING THE Module NAME ETC.
        this.version = version;
        this.name = name;
        this.loadedScreenHeight =0;
        this.loadedScreenWidth =0;

        // GENERATING THE GENERAL RESSOURCE OBJECTS WITH THE DEFAULT LANGUAGE OF THE SYSTEM
        // USE FOR THE ERROR MESSAGES WHILE READING THE CONFIGURATION FILE !!!
        ressources = new LeviResourceManager(this.getClass(),Locale.getDefault());

        // GENERATING THE GENERAL CONFIGURATION OBJECTS 
        try {
        	config = (LeviConfiguration) Class.forName(configName).newInstance();
        	config.setModuleName(name);
        	config.setLearningModule(this);

        	// READING THE CONFIGURATION FROM FILE ...
        	if (!config.loadConfiguration(configFileName)) {

	          // the reading of the configuration was not successful!
        	  getFrame().showWarningMessageDialog("dlg.errortext.6", configFileName, config.getWorkingDir(), "leviModule");
	          
		      // generate the default configuration settings and write it to a new configuration file
		      config.createDefaultProperties();
	          if (!config.saveConfiguration()) {
	        	  // Konfiguration konnte nicht ins File geschrieben werden.
	        	  getFrame().showErrorMessageDialog("dlg.errortext.9", configFileName, config.getWorkingDir(), "leviModule");
	        	 
	          }
	
	        }
	
	        // BESTIMMEN DER SPRACHWAHL AUS DER EINGELESENEN KONFIGURATION
	        if (config.keyExist("Language")) {
		        // Sprachwahl einstellen.
		        Object tmp  = config.getSettingByKey("Language");
		        language = (String)tmp;
		        Locale.setDefault(new Locale(language));
	        }
	        else {
	        	// Angabe fr die Sprache fehlt im Konfigurationsfile
	        	getFrame().showErrorMessageDialog("dlg.errortext.5", configFileName, "leviModule");  
	        }
	
	        // NOW GENERATE THE GENERAL RESSOURCE OBJECTS WITH THE LANGUAGE GIVEN IN THE CONFIGURATION
	        ressources = new LeviResourceManager(this.getClass(),Locale.getDefault());
	        
	        // BESTIMMEN DER EINSTELLUNG FR DEN AVATAR AUS DER EINGELESENEN KONFIGURATION
	        // setting the Visibility of the help panel
	        if (config.keyExist("Help") || config.keyExist("Avatar")) {
	        	if (config.keyExist("Help"))
	        		tmp2  = (Boolean) config.getSettingByKey("Help");
	        	else
	        		tmp2  = (Boolean) config.getSettingByKey("Avatar");
		        setHelpVisible(tmp2.booleanValue());
	        }
	        else {
	        	// Angabe fr die Hilfe fehlt im Konfigurationsfile
	        	getFrame().showErrorMessageDialog("dlg.errortext.7", configFileName, "leviModule");  
	        	
	  	        // Wenn nichts in der Konfiguration steht, Help erst mal sichtbar machen.
	        	setHelpVisible(true);
	        	config.putSettingByKey("Help", true);
	        }
	        // setting the Visibility of the tooltips
	        if (config.keyExist("Tooltips")) {
	        	tmp2  = (Boolean) config.getSettingByKey("Tooltips");
		        setTooltipsVisible(tmp2.booleanValue());
	        }
	        else {
	        	// Angabe fr die Tooltips fehlt im Konfigurationsfile
	  	        // Wenn nichts in der Konfiguration steht, Tooltip erst mal sichtbar machen.
	        	setTooltipsVisible(true);
	        	config.putSettingByKey("Tooltips", true);
	        }

	        // UIManger aktualisieren fr die Standarddialoge Opne, Save, ColorChooser
	        updateUIManager();
	          
	        // Sammlung mit allen toolbarButtons initialisieren.
	        toolbarButtons = new HashMap();
	
	        // SETTING DEFAULT ATTRIBUTES WHICH ARE NOT CONTAINED IN THE CONFIGURATION
	        setMode(EDITORMODE);
	        setView(SELFSTUDYVIEW);
	
	        // GENERATE THE DEFAULT FRAME
	        frame = new LeviFrame(this);
	        frame.composeFrame();
	
	        // GENERATE THE DEFAULT STANDARD PART OF THE Module
	        scenarios = new ArrayList<LeviScenarioInterface>();

	        // LOADING THE DEFAULT scenario
	        if (config.keyExist(LeviConfiguration.KEYDEFSCENARIO)) {
	            if (config.getSettingByKey(LeviConfiguration.KEYDEFSCENARIO).toString().length() != 0) {
	                // an entry for the default scenario exists in the configuration
	            	scenarioName = config.getSettingByKey(LeviConfiguration.KEYDEFSCENARIO).toString();
	                File scenarioFile = new File(scenarioName);
	                if (scenarioFile.exists()) {                    
	                	defaultScenarioExist = true;
	                    setXmlScenarioFile(scenarioFile);
	                }
	                else {
	                    // the given default file do not exist.
	                	getFrame().showErrorMessageDialog("dlg.errortext.8", scenarioName, "leviModule");  
	                }
	            }
	        }
	        else if (config.keyExist(LeviConfiguration.KEYDEFMODEL)) {
	            if (config.getSettingByKey(LeviConfiguration.KEYDEFMODEL).toString().length() != 0) {
	                // an entry for the default scenario exists in the configuration
	            	scenarioName = config.getSettingByKey(LeviConfiguration.KEYDEFMODEL).toString();
	                File scenarioFile = new File(scenarioName);
	                if (scenarioFile.exists()) {                    
	                	defaultScenarioExist = true;
	                    setXmlScenarioFile(scenarioFile);
	                }
	                else {
	                    // the given default file do not exist.
	                	getFrame().showErrorMessageDialog("dlg.errortext.8", scenarioName, "leviModule");  
	                   
	                }
	            }
	        }
	        
        } 
        catch (Exception e) {
        	// Es gibt vermutlich die Configurationsklasse nicht...
        	if  (getFrame()!=null)
        		getFrame().showErrorMessageDialog("dlg.errortext.0", configName, "leviModule");  
        	else {
        		// Falls Exceptions auftreten bevor die Frame erzeugt wurde.
        		JOptionPane.showMessageDialog(new JFrame(),"Interner Fehler beim Konfigurieren des Lernmoduls." , "Fehlermeldung", JOptionPane.ERROR_MESSAGE);	
        	}
        }

        // TRACE OUTPUT
        if (isTraceActive()) System.out.println("END leviModule.Contructors(cname, configFileName)");

  } // End Constructor

    /**
     * Destroying the relevant instance fields of the class.
     *
     * @author sirocic
     */
    public void dispose()
    {
        colorConfigDialog.dispose();
        colorChooseDialog.dispose();
        fontConfigDialog.dispose();
        fontChooseDialog.dispose();
        defaultScenarioConfigDialog.dispose();
        quitDialog.dispose();
        
    }

  // ABSTRACT METHODES the specific methode has to defined in the sub module.
  /*
   * Methode:		loadSchemaFile
   *
   * Description:	Ladend des modulspezifischen Schema-Files z.b. RealTimeScheduling.xsd aus den Ressourcen.
   *
  * Date: 		17.02.2007
   * @author      sirocic
   */
  public abstract URL loadSchemaFile() ;

  /*
   * Methode:		disableCommands
   *
   * Description:	Abschalten von Kommandos (GUI-Elementen), die z.B. whrend der Simulation nicht
   * 				erlaubt sind. Hier sollten die fr alle Lernmodule gltigen Kommandos disabled werden.
   * 				In den lernmodule-spezifischen Klassen sollte dann die lernmodul-spezifischen
   * 				Kommandos disabled werden.
   *
   * Date: 		    27.02.2007
   * @author        sirocic
   */
  public abstract void disableCommands() ;

  /*
   * Methode:		enableCommands
   *
   * Description:	Anschalten von Kommandos (GUI-Elementen), die z.B. whrend der Simulation nicht
   * 				erlaubt sind. Hier sollten die fr alle Lernmodule gltigen Kommandos enabled werden.
   * 				In den lernmodule-spezifischen Klassen sollte dann die lernmodul-spezifischen
   * 				Kommandos enabled werden.
   *
   * Date: 		    27.02.2007
   * @author        sirocic
   */
  public abstract void enableCommands() ;

  /*
  * Methode:		openScenario
  *
  * Description:	Opens the scenario in the given file.
  *
  * Date: 		22.12.2005
  * @author      sirocic
  */
  public abstract void openScenario(File scenarioFile) ;
  
  /*
   * Methode:		newScenario
   *
   * Description:	Generates a new and empty scenario.
   *
   * Date: 		22.12.2005
   * @author      sirocic
   */
  public abstract void newScenario() ;

  /**
   * Add a scenario to the list of scenarios and make it the current scenario
   * @param scenario The scenario to add.
   */
  public void addScenario(LeviScenario scenario) {
	  LeviScenarioInterface previousActiveScenario;
	  
      if(scenarios.size() > 11 ) return;

      scenarios.add(scenario);
      previousActiveScenario = null;

      if( currentScenarioIndex > -1) {
          previousActiveScenario = scenarios.get(currentScenarioIndex);
          previousActiveScenario.setOpen(false);
      }

      currentScenarioIndex = scenarios.indexOf(scenario);
      scenario.setOpen(true);

      // Neue Szenario in Menu View einfgen
      frame.getMenubar().addToScenarioView(scenario);
      frame.getMenubar().updateLanguageColorFonts();
      //Auch ins Popup-Menu damit ...
      frame.getPopupMenu().addToScenarioView(scenario);
      frame.getPopupMenu().updateLanguageColorFonts();
 
      //The added scenario should listen to property changes.
      setActiveScenario(scenario);
	  
  }
  
  /**
   * Methode:		getCurrentOrNewScenario
   *
   * Description:	Returns the current scenario or an empty one.
   * 
   * Date: 		19.04.2006
   * @author    sirocic
   */
  public LeviScenarioInterface getCurrentOrNewScenario() {
	  
	  if( currentScenarioIndex < 0 ) newScenario();
      return scenarios.get( currentScenarioIndex );
      
  }
  
  /**
   * Methode:		getScenario
   *
   * Description:	Returns the current scenario. 
   * 
   * Date: 		19.04.2006
   * @author    sirocic
   */
  public LeviScenarioInterface getScenario() {
	  
	  if( currentScenarioIndex >= 0 )
		  return scenarios.get( currentScenarioIndex );
	  else
		  return null;
      
  }
  
  /**
   * Methode:		setActiveScenario
   *
   * Description:	Sets the current scenario. 
   * 
   * Date: 		19.04.2006
   * @author    sirocic
   */
  public void setActiveScenario(LeviScenarioInterface scenario) {
	  int scenarioIndex;
	  
	  // Ist die Komponente im SIMULAITONSMODUS d.h. luft gerade die Visualisierung???
      if (getMode() == LeviModule.SIMULATIONMODE) {
      	// SIMULATION STOPPEN !!!
			// Wenn es einen ActionButton gibt der disabled ist, 
			// wird dieser wieder auf enable gesetzt.
    	    LeviAbstractAction.resetActiveAction();
			
	        if (getVisualizer().isBusy()) {
	        	// Es sind noch Animationen akiv vom vorherigen Schritt.
	        	// Die mssen abgebrochen werden. Der Endzustand des vorherigen Schritts muss
	        	// eingestellt werden.
		        try {
				       EventQueue.invokeLater( new Runnable() {
			                  public void run() {
			                	  getVisualizer().getScenarioVisualizer().cancelAnimation(true);
			                  }
			           }); 
				}
			    catch (Exception exception)
			    {
			        // Irgendwas ist mit dem Repaint wohl nicht richtig gelaufen!!!
			    	System.out.println("ERROR (setActiveScenario)"+exception.getMessage());
			    }
	        }
	        
			getSimulator().stopSimulation();
			
			if ( getView() == LeviModule.SELFSTUDYVIEW) {
				// In der Selbststudiumssicht muss die Komponente zurck in den Editormodus.
				setMode(LeviModule.EDITORMODE);
				getFrame().changeMode();
			}
      }
      
      // Szenario wechseln
      if (getScenario()!=null) {
    	  getScenario().setOpen(false);
      }
	  scenarioIndex = scenarios.indexOf( scenario );
      if( scenarioIndex > -1) {
          currentScenarioIndex = scenarioIndex;
          scenario.setOpen(true);
          if (getEditor().getScenarioEditor()!=null)
        	  getEditor().getScenarioEditor().createGraphicScenario();
          if (getVisualizer().getScenarioVisualizer()!=null)
        	  getVisualizer().getScenarioVisualizer().createGraphicScenario();
          frame.refresh();
      }

      // Mens aktualisieren
      getFrame().getMenubar().updateViewMenu();
      getFrame().getPopupMenu().updateViewMenu();
     
  } // switchScenario()


  /**
   * Methode:		closeScenario
   *
   * Description:	Closes the current scenario. If there are other open
   * 				scenarios, the next in the scenarios array list 
   * 				will be the current one.
   * 
   * Date: 		19.04.2006
   * @author    sirocic
   */
   public void closeScenario() {
	   LeviScenarioInterface scenario = getScenario();
	   int index = scenarios.indexOf(scenario);

	   scenario.setOpen(false);
	   scenarios.remove(scenario);

      if(index > (scenarios.size()-1)) index--;
      if( index >= 0 ) {
          currentScenarioIndex = -1; // does no longer exist
          setActiveScenario(scenarios.get(index));
          getScenario().setOpen(true);
      } else {
          currentScenarioIndex = -1;
          newScenario();
      }

      frame.getMenubar().removeFromScenarioView(scenario);
      frame.getMenubar().updateViewMenu();
      frame.getPopupMenu().removeFromScenarioView(scenario);
      frame.getPopupMenu().updateViewMenu();
      
   }

  /**
   * Methode:		saveScenario
   *
   * Description:	Saves the current scenario into the specified xml scenario file. This methode creates the general
   *                DOM structure of the scenario. It calls the buildDOMElements methode from
   *                the current scenario. This method returns the specific DOM structure of the
   *                scenario which depends on the current module type.
   *                At the end of the method the dom structur is saved as a xml file.
   *                The variables <xmlScenarioFile> und <xmlScenariodoc> will be updated.
   *
   * Date: 		19.04.2006
   * @author    sirocic
   */
   public void saveScenario() {
       DocumentBuilderFactory factory;
       DocumentBuilder builder;
       Document xmlDoc;
       Element scenario, screenElement, elem;
       Source source;
       Result result;
       Transformer xformer;

       // TRACE OUTPUT
       if (LeviModule.isTraceActive()) System.out.println("CALL leviModule.saveScenario: ");

       // DOM DOCUMENT AUFBAUEN ...
       factory = DocumentBuilderFactory.newInstance();
       try {
           builder = factory.newDocumentBuilder();
           xmlDoc = builder.newDocument();
           scenario = xmlDoc.createElement(LeviScenario.XMLTAGSCENARIO);

	   	    // ELEMENT FR DEN KOMPONENTENTYP ERZEUGEN
	   	    elem = xmlDoc.createElement(XMLTAGTYPE);
	   	    elem.setTextContent(TYPE);
	   	    scenario.appendChild(elem);
	   	   
	        // Element fr die Bildschirmgre erzeugen
	   	    screenElement = xmlDoc.createElement(XMLSCREEN);
	   	    elem = xmlDoc.createElement(XMLSCREENWIDTH);
	           elem.setTextContent(String.valueOf((int)frame.getFullscreenSize().getWidth()));
	   	    screenElement.appendChild(elem);
	   	    elem = xmlDoc.createElement(XMLSCREENHEIGHT);
	           elem.setTextContent(String.valueOf((int)frame.getFullscreenSize().getHeight()));
	   	    screenElement.appendChild(elem);
	   	    scenario.appendChild(screenElement);
   	    
           // AB HIER WEISS NUR DAS Szenario SELBER, WAS WIE GESPEICHERT WIRD.
           scenario = getScenario().buildDOMElements(scenario, xmlDoc);

           xmlDoc.appendChild(scenario);
           setXmlScenarioDoc(xmlDoc);

       } catch (ParserConfigurationException e) {
           // Das DOM-Dokument konnte nicht erzeugt werden!
    	   getFrame().showErrorMessageDialog("dlg.errortext.12", e.getMessage(),"leviModule");
           
		   return;
       }

          // XML FILE SCHREIBEN...
        try {
            // DOM Dokument zum Schreiben vorbereiten
            source = new DOMSource(xmlScenarioDoc);
            // Output-File vorbereiten
            result = new StreamResult(xmlScenarioFile);

            // DOM Dokument in XML-Output-File schreiben
            xformer = TransformerFactory.newInstance().newTransformer();
            xformer.setOutputProperty(OutputKeys.INDENT, "yes");
            xformer.transform(source, result);

        } catch (TransformerConfigurationException e) {
        } catch (TransformerException e) {
            // Das XML-file konnte nicht geschrieben werden!
        	getFrame().showErrorMessageDialog("dlg.errortext.13", e.getMessage(),"leviModule");
        	
        }

          // TRACE OUTPUT
          if (LeviModule.isTraceActive()) System.out.println("END leviModule.saveScenario");


      } // End saveScenario
   
   /**
    * Methode:		reset
    *
    * Description:	Zurcksetzen der Komponente. Alle Variable, die etwas mit einem geladenen Szenario zu
    *               tun haben werden wieder zurckgesetzt. Editor und Visualizer werden als leer
    *               angezeigt.
    *
    * Date: 		 05.05.2006
    * @author        sirocic
    */
     public void reset() {

        // Variablen fr das Szenariofile zurcksetzen.
        setXmlScenarioDoc(null);
        setXmlScenarioFile(null);

        // Editor subern ...
        getEditor().cleanMe();

        // Viualizer subern ...
       getVisualizer().cleanMe();

     }


  /*
   * Methode:		changeMode
   *
   * Description:	Zum Wechsel in eine andere View (Lecture, Selfstudy) muss die Frame-Decoration
   *                sichtbar bzw. unsichtbar gemacht werden. Diese Methode macht die Frame not displayable
   *                durch einen Aufruf der Methode dispose. Dann kann die Frame-Decoration verndert werden.
   *                Anschliessend braucht die Frame nur mit einem setVisable auf true wieder sichtbar gemacht
   *                werden.
   *
   * Date: 		29.01.2006
   * @author sirocic
   */
  public void changeMode()
  {

        //	 TRACE OUTPUT
        if (isTraceActive()) System.out.println("CALL leviModule.changeMode:");

        // Das Anzeigen und Verstecken der Frame-Titelzeile geht nur, wenn die Frame vorher
        // per dispose aus der Anzeigehierarchie entfernt wurde.
        if (frame!=null)
        {
            // Frame nach den aktuellen Attributen aufbauen
            frame.composeFrame();
            frame.refresh();

        }

        //	TRACE OUTPUT
        if (isTraceActive()) System.out.println("END leviModule.changeMode");

  }

  
  /**
   * Methode:		createModuleDialogs
   *
   * Description:	Erzeugt die Dialoge fr die Konfiguration (Farbe, Fonts), Szenario speichern
   * 				und Szenario laden.
   * 
   * Date: 		    24.06.2005
   * @author        sirocic
   * @param frame the frame
   */
    public void createModuleDialogs(LeviFrame frame) {

        // TRACE OUTPUT
        if (isTraceActive()) System.out.println("CALL leviModule.createModuleDialogs: ");
        
        // GENERATING THE FILE DIALOG BOX
        fileOpenSaveDialog = new JFileChooser();
        fileOpenSaveDialog.setCurrentDirectory(new File("."));
        fileOpenSaveDialog.setFileFilter(new FileFilter()
                {
                    public boolean accept(File f)
                    {
                        return f.isDirectory() || f.getName().toLowerCase().endsWith(".xml");
                    }
                    public String getDescription()
                    {
                        return "XML-Dateien";
                    }
                });
        // Standard-Java-Dialog an die aktuellen Farbeneinstellungen anpassen...
        fileOpenSaveDialog.setOpaque(true);
        setDialogColor(fileOpenSaveDialog);

        //	  GENERATING THE COLOR CONFIGURATION DIALOG
        colorConfigDialog = new LeviColorDialog(frame);
        
        //GENERATING THE COLOR CHOOSE DIALOG
        colorChooseDialog = new LeviColorChooseDialog(frame);

        // GENERATING THE FONT CONFIGURATION DIALOG
        fontConfigDialog = new LeviFontDialog(frame);
        
        //GENERATING THE FONT CHOOSE DIALOG
        fontChooseDialog = new LeviFontChooseDialog(frame);
        
        //  GENERATING THE DEFAULT scenario CONFIGURATION DIALOG
        defaultScenarioConfigDialog = new LeviDefaultScenarioDialog(frame);

        // GENERATE QUIT DIALOG
        quitDialog = new LeviQuitDialog(frame);
        
        // TRACE OUTPUT
        if (isTraceActive()) System.out.println("END leviModule.createModuleDialogs");

    }
    /**
     * Methode:		updateDialogsColorLanguageFonts
     *
     * Description:	Aktualisiert bei den LeVi-Dialogen die Farben und Textbausteine gem der aktuellen
     * 				Konfiguration bzw. Spracheinstellung (Ressourcen).
     * 
     * Date: 	    16.03.2007
     * @author        sirocic
     * @param frame the frame
     */
      public void updateDialogsColorLanguageFonts() {

          // TRACE OUTPUT
          if (isTraceActive()) System.out.println("CALL leviModule.updateDialogsColorLanguageFonts: ");

          // REFRESH bzw REGENERATE THE FILE DIALOG BOX
          
          // Jetzt den Dialog neu erzeugen!
          fileOpenSaveDialog = new JFileChooser();
          fileOpenSaveDialog.setCurrentDirectory(new File("."));
          fileOpenSaveDialog.setFileFilter(new FileFilter()
                  {
                      public boolean accept(File f)
                      {
                          return f.isDirectory() || f.getName().toLowerCase().endsWith(".xml");
                      }
                      public String getDescription()
                      {
                          return "XML-Dateien";
                      }
                  });
          // Standard-Java-Dialog an die aktuellen Farbeneinstellungen anpassen...
          fileOpenSaveDialog.setOpaque(true);
          setDialogColor(fileOpenSaveDialog);
          
          // REFRESH THE COLOR CONFIGURATION DIALOG
          colorConfigDialog.updateLanguageColorFonts();
          
          //REFRESH THE COLOR CHOOSE DIALOG
          colorChooseDialog.updateLanguageColorFonts();
          
          // REFRESH THE FONT CONFIGURATION DIALOG
          fontConfigDialog.updateLanguageColorFonts();
          
          //REFRESH THE FONT CHOOSE DIALOG
          fontChooseDialog.updateLanguageColorFonts();

          //  REFRESH THE DEFAULT scenario CONFIGURATION DIALOG
          defaultScenarioConfigDialog.updateLanguageColorFonts();

          // REFRESH QUIT DIALOG
          quitDialog.updateLanguageColorFonts();
          
          // TRACE OUTPUT
          if (isTraceActive()) System.out.println("END leviModule.updateDialogsColorLanguageFonts");

      }
      
  /**
   * Methode:		updateUIManager
   *
   * Description:	Aktualisiert bei den LeVi-Dialogen die Farben und Textbausteine gem der aktuellen
   * 				Konfiguration bzw. Spracheinstellung (Ressourcen).
   * 
   * Date: 	    16.03.2007
   * @author        sirocic
   * @param frame the frame
   */
    public void updateUIManager() {

        // TRACE OUTPUT
        if (isTraceActive()) System.out.println("CALL leviModule.updateUIManager: ");

        // Textbausteine im UIManger neusetzen aus den Ressourcen. Die werden von den Standarddialogen
        // verwendet.
        UIManager.put("FileChooser.lookInLabelText", getRessources().getResourceByKey("FileChooser.lookInLabelText"));
        UIManager.put("FileChooser.saveInLabelText", getRessources().getResourceByKey("FileChooser.saveInLabelText"));
        UIManager.put("FileChooser.fileNameLabelText", getRessources().getResourceByKey("FileChooser.fileNameLabelText"));
        UIManager.put("FileChooser.filesOfTypeLabelText", getRessources().getResourceByKey("FileChooser.filesOfTypeLabelText"));
        UIManager.put("FileChooser.cancelButtonText", getRessources().getResourceByKey("FileChooser.cancelButtonText"));
        UIManager.put("FileChooser.openButtonText", getRessources().getResourceByKey("FileChooser.openButtonText"));
        UIManager.put("FileChooser.saveButtonText", getRessources().getResourceByKey("FileChooser.saveButtonText"));
        UIManager.put("FileChooser.upFolderToolTipText", getRessources().getResourceByKey("FileChooser.upFolderToolTipText"));
        UIManager.put("FileChooser.homeFolderToolTipText", getRessources().getResourceByKey("FileChooser.homeFolderToolTipText"));
        UIManager.put("FileChooser.newFolderToolTipText", getRessources().getResourceByKey("FileChooser.newFolderToolTipText"));
        UIManager.put("FileChooser.listViewButtonToolTipTextlist", getRessources().getResourceByKey("FileChooser.listViewButtonToolTipTextlist"));
        UIManager.put("FileChooser.detailsViewButtonToolTipText", getRessources().getResourceByKey("FileChooser.detailsViewButtonToolTipText"));
        UIManager.put("FileChooser.saveButtonToolTipText", getRessources().getResourceByKey("FileChooser.saveButtonToolTipText"));
        UIManager.put("FileChooser.openButtonToolTipText", getRessources().getResourceByKey("FileChooser.openButtonToolTipText"));
        UIManager.put("FileChooser.cancelButtonToolTipText", getRessources().getResourceByKey("FileChooser.cancelButtonToolTipText"));
        UIManager.put("ColorChooser.previewText", getRessources().getResourceByKey("ColorChooser.previewText"));
        UIManager.put("ColorChooser.swatchesRecentText", getRessources().getResourceByKey("ColorChooser.swatchesRecentText"));
        UIManager.put("ColorChooser.sampleText", getRessources().getResourceByKey("ColorChooser.sampleText"));
        UIManager.put("TitledBorder.font", getConfiguration().getSettingByKey("FontMenu"));
        if (getConfiguration().getSettingByKey("ColorComponentForeground")!=null) {
	        UIManager.put("TitledBorder.titleColor", getConfiguration().getSettingByKey("ColorComponentForeground"));
	        UIManager.put("TextField.foreground", getConfiguration().getSettingByKey("ColorComponentForeground"));
        }
        else {
        	UIManager.put("TitledBorder.titleColor", getConfiguration().getSettingByKey("ColorModuleForeground"));
            UIManager.put("TextField.foreground", getConfiguration().getSettingByKey("ColorModuleForeground"));
        }
        UIManager.put("ComboBox.font", getConfiguration().getSettingByKey("FontMenu"));
        
        // TRACE OUTPUT
        if (isTraceActive()) System.out.println("END leviModule.updateUIManager");

    }
        
    /**
     * Methode:		setDialogColor
     *
     * Description:	Setzt beim ber Parameter bergebenen Dialog die Hintergrund-, Vordergrundfarbe 
     *              und den Font, wenn mgliche, auf die Werte aus der Konfiguration.
     * 
     * Date: 		08.11.2006
     * @author        sirocic
     * 
     */
      public void setDialogColor(Container dialog) {
          Color backgroundColor, foregroundColor;
          Component[] components;
          int i;
          
          // TRACE OUTPUT
          if (isTraceActive()) System.out.println("CALL leviModule.setDialogColor: ");

          if (getConfiguration().getSettingByKey("ColorComponentBackground")!=null) {
	          backgroundColor  = (Color)getConfiguration().getSettingByKey("ColorComponentBackground");
	          foregroundColor = (Color)getConfiguration().getSettingByKey("ColorComponentForeground");
          }
          else {
        	  backgroundColor  = (Color)getConfiguration().getSettingByKey("ColorModuleBackground");
              foregroundColor = (Color)getConfiguration().getSettingByKey("ColorModuleForeground");
          }

          dialog.setBackground(backgroundColor);
          dialog.setForeground(foregroundColor);
          components =dialog.getComponents();
          
          // alle Komponenten des Kontainers durchlaufen.
          for (i=0; i < dialog.getComponentCount(); i++) {
              if ((components[i] instanceof JPanel))
            	  setModulesColor(components[i], backgroundColor, foregroundColor );
              else if (components[i] instanceof JTabbedPane) {
            	  ((JTabbedPane)components[i]).setOpaque(true);
            	  setModulesColor(components[i], backgroundColor, foregroundColor );
              }
              
              components[i].setBackground(backgroundColor);
              components[i].setForeground(foregroundColor);

          }

          // TRACE OUTPUT
          if (isTraceActive()) System.out.println("END leviModule.setDialogColor");
          
      }
  /**
   * Methode:		setModulesColor
   *
   * Description:	Durchluft alle Unterkomponenten des ber Parameter angebenen Panels
   * 				und versucht die Hintergrund-, Vordergrundfarbe und den Font auf die
   * 				aktuellen Konfigurationswerte zu setzen.
   * 				Handelt es sich bei der Untekomponent wiederum um einen Panel, wird die
   * 				Methde rekursiv aufgerufen.
   * 
   * Date: 			09.11.2006
   * @author        sirocic
   * 
   */
    public void setModulesColor(Component panel, Color bColor, Color fColor) {
        Component[] components;
        int i;
        
        // TRACE OUTPUT
        if (isTraceActive()) System.out.println("CALL leviModule.setModulesColor: ");

    	components = ((Container)panel).getComponents();
      	for (i=0; i < ((Container)panel).getComponentCount(); i++) {
      		if ((components[i] instanceof JPanel))
      			setModulesColor(components[i], bColor, fColor);
      		components[i].setBackground(bColor);
      		components[i].setForeground(fColor);
      	}
      	
        // TRACE OUTPUT
        if (isTraceActive()) System.out.println("END leviModule.setModulesColor");
        
    }


  /*
   * Methode:		loadScenarioFromFile
   *
   * Description:	Load the xml-File which contains the scenario and parse it. It creats a internal
   *                scenario object.
   *
   * Date: 		    13.04.2006
   * @author        sirocic
   */
    public Document loadScenarioFromFile(File scenarioFile) {
    	URL loadedSchemaFile;
    	SchemaFactory scenarioSchemaFactory;
    	Schema scenarioSchema;
        DocumentBuilder builder;
        LeviParserErrorHandler parserErrorHandler;
        Document xmlDoc;

        // TRACE OUTPUT
        if (isTraceActive()) System.out.println("CALL leviModule.loadScenarioFromFile: file="+scenarioFile.getPath());

        // reading and parsing xml file
        try
        {
        	// parsing scenario schema
        	loadedSchemaFile = loadSchemaFile();
        	scenarioSchemaFactory = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
        	scenarioSchema = scenarioSchemaFactory.newSchema(loadedSchemaFile);
        	
            // generating the xml parser
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setSchema(scenarioSchema);
            factory.setValidating(false);
            factory.setIgnoringElementContentWhitespace(true);
            parserErrorHandler = new LeviParserErrorHandler(this);
            builder = factory.newDocumentBuilder();
            builder.setErrorHandler(parserErrorHandler);
            
            // parsing the xml file ...
            xmlDoc = builder.parse(scenarioFile);
            
        }
        catch(IOException exception)
        {
            JOptionPane.showMessageDialog((LeviFrame)frame, exception);
            return null;
        }
        catch (ParserConfigurationException exception)
        {
            JOptionPane.showMessageDialog((LeviFrame)frame, exception);
            return null;
        }
        catch (SAXException exception)
        {
        	// Der Parser hat Fehler gefunden und das Szenario kann nicht geffnet werden.
        	JOptionPane.showMessageDialog((LeviFrame)frame, exception);
        	return null;
        }

        // TRACE OUTPUT
        if (isTraceActive()) System.out.println("END leviModule.loadScenarioFromFile");

        if (parserErrorHandler.foundError())
        	return null;
        else
        	return xmlDoc;

    }
    
/*
 * Methode:		saveLoadedScreenSize
 * 
 * Description:	List die in der XML-Szenariodatei enthaltenen Werte fr die Bildschirmweite und-
 * 				-hhe ein und speichert sie auf die globalen Variablen.
 * 
 * Parameters:   xmlScreen = xml element of the screen
 * 
 * Date: 		20.06.2007
 * @author       sirocic
 */
 public void saveLoadedScreenSize(Element xmlScreen) {
     NodeList screenChildren;
 	Node screenChildNode;
 	Element screenChild;
 	String text;
 	int j;
 	int value;
 	
  	// TRACE OUTPUT
  	if (LeviModule.isTraceActive()) System.out.println("CALL leviModule.saveLoadedScreenSize: tag="+xmlScreen.getTagName());

    // get the Data from the xml description 
  	screenChildren = xmlScreen.getChildNodes();
	for (j=0; j < screenChildren.getLength(); j++) {
		screenChildNode = screenChildren.item(j);
	    if (screenChildNode instanceof Element) {
		   screenChild = (Element) screenChildNode;					
		   if (screenChild.getTagName().equals(XMLSCREENWIDTH)) {
					// get the buffer name
					text = screenChild.getFirstChild().getNodeValue().trim();
					if (text!=null) {
						value = Integer.parseInt(text);
						setLoadedScreenWidth(value);
					}
					else {
						getFrame().showWarningMessageDialog("dlg.errortext.100", "leviModule");
					}
			}
			else if (screenChild.getTagName().equals(XMLSCREENHEIGHT)) {
					// get the buffer id
					text = screenChild.getFirstChild().getNodeValue().trim();
                     if (text!=null) {
                    	 value = Integer.parseInt(text);
                    	 setLoadedScreenHeight(value);
                     }
                     else {
                    	 getFrame().showWarningMessageDialog("dlg.errortext.101", "leviModule");
				        
                     }
			}
			
	    } // if (screenChildNode instanceof Element)
	} // for (j=0; j < screenChildren.getLength(); j++)
 
	// TRACE OUTPUT
    if (LeviModule.isTraceActive()) System.out.println("END leviModule.saveLoadedScreenSize");

 }
 
 /*
  * Methode:		updateCoordinate
  * 
  * Description:	Passt den ber Parameter angegebenen Koordinaten Wert auf die aktuelle Bildschirmgre 
  * 				and.
  * 
  * Parameters:   oldValue = Bildschirmkoordinate oder Wert fr Breit bzw. Hhe eines Objekts.
  * 
  * Date: 		20.06.2007
  * @author       sirocic
  */
  public int updateXCoordinate(int oldValue) {
	  int x;
	  
	  x = oldValue;
	  if (getLoadedScreenWidth() > 0) {
    	  if (getLoadedScreenWidth() != getFrame().getFullscreenSize().getWidth()) 
    		  // Wert anpassen
    		  x = (oldValue * (int) getFrame().getFullscreenSize().getWidth()) / getLoadedScreenWidth();
	  }
	  
	  return x;
  }
  public int updateYCoordinate(int oldValue) {
 	  int y;
 	  
 	 y = oldValue;
 	 if (getLoadedScreenHeight() > 0) {
     	 if (getLoadedScreenHeight() != getFrame().getFullscreenSize().getHeight()) 
   		  // Wert anpassen
   		  y = (oldValue * (int) getFrame().getFullscreenSize().getHeight() ) / getLoadedScreenHeight();
 	 }
 	 
 	  return y;
  }
  

  /**
   * Methode:		askUserToSave
   *
   * Description:	Erzeugt eine Dialog mit dem der Benutzer gefragt wird, ob die aktuellen
   * 				nderungen im Szenario abgespeichert werden sollen oder nicht.
   * 				Abhngig von der Antword, wird das aktuelle Szenario abgespeichert oder nicht.
   *
   * Date: 		05.05.2006
   * @author    sirocic
   */
   public void askUserToSave() {
        int answer;

        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("CALL leviModule.askUserToSave: ");

        answer = JOptionPane.showConfirmDialog((LeviFrame)frame,
                getRessources().getResourceByKey("dlg.asktosave.text"),
                getRessources().getResourceByKey("dlg.asktosave.title"),
                JOptionPane.YES_NO_OPTION,
                JOptionPane.WARNING_MESSAGE
            );

        // Anwort auswerten ...
        switch (answer){
        case JOptionPane.YES_OPTION :
                    // Szenario SPEICHERN !!!!
                    if (xmlScenarioFile == null)
                        askUserForFile();
                    else {
                        saveScenario();
                        
                        // Szenarionderungen wurden gerade gespeichert.
                        getScenario().setUnchanged();
                    }
                    break;
        case JOptionPane.NO_OPTION :
                    // NICHT SPEICHERN
                    break;
        }

        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("END leviModule.askUserToSave");

   }


   /**
    * Methode:		askUserForFile
    *
    * Description:	Erzeugt eine Dialog mit dem der Benutzer zur Angabe eines Szenariofiles
    *               aufgefordert wird. Wenn der Benutzer ein File angegeben hat, wird das
    *               Speichern des Szenarien veranlat.
    *
    * Date: 		05.05.2006
    * @author       sirocic
    */
    public void askUserForFile() {
        int answer;
        File scenarioFile = null;
        String fileName;
        boolean save = false;
        boolean canceled = false;

        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("CALL leviModule.askUserForFile: ");

        while (!save && !canceled) {
            // Show the file open dialog and read the given file name
        	getFileOpenSaveDialog().setDialogTitle(getRessources().getResourceByKey("FileChooser.saveButtonText"));
            answer=getFileOpenSaveDialog().showSaveDialog((LeviFrame)frame);
            if (answer == JFileChooser.APPROVE_OPTION) {
                // File has been selected
                scenarioFile = getFileOpenSaveDialog().getSelectedFile();

                if (!scenarioFile.toString().endsWith(".xml")) {
                	fileName = scenarioFile.toString()+".xml";
                	scenarioFile = new File(fileName);
                }
                	
                // TRACE OUTPUT
                if (LeviModule.isTraceActive()) System.out.println("scenarioFile="+scenarioFile);

                // EXISTIERT DAS FILE SCHON ???
                if (scenarioFile.exists()) {
                    // Das File gibt es schon. berschreiben ???
                    answer = JOptionPane.showConfirmDialog((LeviFrame)frame,
                        getRessources().getResourceByKey("dlg.overwrite.text"),
                        getRessources().getResourceByKey("dlg.overwrite.title"),
                        JOptionPane.YES_NO_CANCEL_OPTION,
                        JOptionPane.WARNING_MESSAGE
                    );

                    // Anwort auswerten ...
                    switch (answer){
                    case JOptionPane.YES_OPTION :
                                // FILE BERSCHREIBEN !!!!
                                // Vorhandenes File lschen
                                scenarioFile.delete();
                                save = true;
                                break;
                    case JOptionPane.CANCEL_OPTION :
                                // SPEICHERN ABBRECHEN !!!
                                canceled = true;
                                break;
                    case JOptionPane.NO_OPTION :
                                // NICHT BERSCHREIBEN, NOCHMAL EIN FILE AUSWHLEN
                                break;
                    }

                } // END if (scenarioFile.exists())
                else
                    // FILE GIBT ES NOCH NICHT ... ALSO EINFACHE SPEICHERN
                    save = true;

            } // END if (answer == JFileChooser.APPROVE_OPTION)
            else {
                // ES WURDE KEIN FILE ANGEGEBEN !!!
                canceled = true;
            }

        } // END While

        if (save) {
            // NEUES FILE ERZEUGEN UND Szenario IM XML-FORMAT RAUSSCHREIBEN
            try {
                // neues File anlegen.
                scenarioFile.createNewFile();

                if (scenarioFile.canWrite()) {
                    // Das File darf auch beschrieben werden.
                    // xml-Szenario-File der Komponente aktualisieren.
                    setXmlScenarioFile(scenarioFile);

                    // Aktuelles Szenario in XML-Format rausschreiben...
                    saveScenario();
                    
                    // Filename als Szenarioname bernehmen
                    getScenario().setName(scenarioFile.getName());
                    
                    // Szenarionderungen wurden gerade gespeichert.
                    getScenario().setUnchanged();
                    
                    // Titelleiste der Komponente aktualisieren...
                    getFrame().setTitle(getRessources().getResourceByKey("frame.title")+ " "+
            				getVersion()+ " - " + getXmlScenarioFile().getName());
                }
                else {
                    // FEHLER KEINE SCHREIBRECHT AUF DEM FILE
                	getFrame().showErrorMessageDialog("dlg.errortext.14", scenarioFile.getName(),"leviModule");
                }

            }
            catch (IOException e) {
                // File konnte nicht geschrieben werden.
            	getFrame().showErrorMessageDialog("dlg.errortext.15", scenarioFile.getName(),"leviModule");
            }
        }

        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("END leviModule.askUserForFile");


    }

    public void exit() {
        getSimulator().requestStop();
        dispose();
        getFrame().dispose();
        
        System.exit(0);
        
    }
    

  /**
   * Methode:		setView
   *
   * Description:	Set the view mode to the given value. For changing the view mode look of the frame
   *                the frame has to be destroyed and create new. This is necessary because of the
   *                change of the frame decoration.
   *
   * Date: 		    24.06.2005
   * @author        sirocic
   * @param v The view mode. Possible values: SELFSTUDYVIEW (1), LECTUREVIEW (2)
   */
    public void setView(int v) {

        // TRACE OUTPUT
        if (isTraceActive()) System.out.println("CALL leviModule.setView: v="+v);

        // View mode get the new value
        this.view=v;

        // TRACE OUTPUT
        if (isTraceActive()) System.out.println("END leviModule.setView");
    }

  /**
   * Methode:		setMode
   *
   * Description:	Set the mode to the given value. The module is in the simulation mode, if the
   *                simulation has been started. It is in the scenarioPanel mode else.
   *                In the selfstudy view for changing in the simulation mode the scenarioPanel have to close
   *                and  the visualizer have to open.
   *
   * Date: 		    28.09.2005
   * @author        sirocic
   * @param m simulation mode
   */
    public void setMode(int m) {

        // TRACE OUTPUT
        if (isTraceActive()) System.out.println("CALL leviModule.setMode: new="+m);

        // mode get the new value
        this.mode=m;

        // TRACE OUTPUT
        if (isTraceActive()) System.out.println("END leviModule.setMode");

    }

  /*
   * Methode:		setHelpVisible
   *
   * Description:	Depending on the parameter value the methode set the help panel visible or unvisible.
   *
   * Date: 		    24.06.2005
   * @author        sirocic
   */
    public void setHelpVisible(boolean val) {

        // TRACE OUTPUT
        if (isTraceActive()) System.out.println("CALL leviModule.setHelpVisible: val="+val);

        // neuen Wert merken
        this.helpVisible=val;

        // TRACE OUTPUT
        if (isTraceActive()) System.out.println("END leviModule.setHelpVisible");

    }

    /*
     * Methode:		setTooltipsVisible
     *
     * Description:	Depending on the parameter value the methode set the help panel visible or unvisible.
     *
     * Date: 		    24.06.2005
     * @author        sirocic
     */
      public void setTooltipsVisible(boolean val) {

          // TRACE OUTPUT
          if (isTraceActive()) System.out.println("CALL leviModule.setTooltipsVisible: val="+val);

          // neuen Wert merken
          tooltipsVisible=val;

          // TRACE OUTPUT
          if (isTraceActive()) System.out.println("END leviModule.setTooltipsVisible");

      }
          
    public String getVersion() {
        return version;
    }
    public String getName() {
          return name;
    }
    public int getLoadedScreenWidth() {
        return loadedScreenWidth;
    }
    public int getLoadedScreenHeight() {
        return loadedScreenHeight;
    }

    public void setName(String n) {
        name=n;
    }
    public void setLoadedScreenWidth(int width) {
        loadedScreenWidth = width;
    }
    public void setLoadedScreenHeight(int height) {
        loadedScreenHeight = height;
    }
    public LeviResourceManager getRessources() {
          return ressources;
    }
    public void setRessources(LeviResourceManager r) {
       ressources=r;
    }
    public LeviConfiguration getConfiguration() {
          return config;
    }
    public void setConfiguration(LeviConfiguration c) {
        config=c;
    }
    public String getLanguage() {
          return language;
    }
    public void setLanguage(String l) {
        language=l;
    }
    public int getView() {
          return view;
    }
    public int getMode() {
          return mode;
    }
    public LeviHelpInterface getHelp() {
          return help;
    }
    public void setHelp(LeviHelpInterface a) {
        help=a;
    }
    public LeviVisualizerInterface getVisualizer() {
          return visualizer;
    }
    public void setVisualizer(LeviVisualizerInterface v) {
        visualizer=v;
    }
    public LeviEditorInterface getEditor() {
          return editor;
    }
    public void setEditor(LeviEditorInterface e) {
        editor=e;
    }
    public LeviSimulatorInterface getSimulator() {
          return simulator;
    }
    public void setSimulator(LeviSimulatorInterface s) {
        simulator=s;
    }
    public LeviFrame getFrame() {
          return frame;
    }
    public void setFrame(LeviFrame f) {
        frame=f;
    }
    public boolean isHelpVisible() {
          return helpVisible;
    }
    public boolean isTooltipsVisible() {
        return tooltipsVisible;
  }
    public boolean hasDefaultScenario() {
          return defaultScenarioExist;
    }
    public JFileChooser getFileOpenSaveDialog() {
          return fileOpenSaveDialog;
    }

    public LeviFontChooseDialog getFontChooseDialog() {
        return fontChooseDialog;
    }
    public LeviFontDialog getFontConfigDialog() {
          return fontConfigDialog;
    }
    public LeviColorChooseDialog getColorChooseDialog() {
        return colorChooseDialog;
    }
    public LeviColorDialog getColorConfigDialog() {
          return colorConfigDialog;
    }
    public LeviQuitDialog getQuitDialog() {
        return quitDialog;
  }

    public LeviDefaultScenarioDialog getDefaultScenarioConfigDialog() {
          return defaultScenarioConfigDialog;
    }
    public File getXmlScenarioFile() {
          return xmlScenarioFile;
    }
    public void setXmlScenarioFile(File xmlfile) {
        xmlScenarioFile=xmlfile;
    }
    public Document getXmlScenarioDoc() {
          return xmlScenarioDoc;
    }
    public void setXmlScenarioDoc(Document xmldoc) {
        xmlScenarioDoc=xmldoc;
    }
    static public boolean isTraceActive() {
          return trace;
    }
    static public void setTrace(boolean value) {
        trace=value;
    }
    public HashMap getToolbarButtons() {
          return toolbarButtons;
    }
    public void setToolbarButtons(HashMap buttons) {
        toolbarButtons=buttons;
    }

    public ArrayList<LeviScenarioInterface> getScenarios() {
    	return scenarios;
    }

    public int getCurrentScenarioIndex() {
		return currentScenarioIndex;
	}
	public void setCurrentScenarioIndex(int currentScenarioIndex) {
		this.currentScenarioIndex = currentScenarioIndex;
	}

    

}
