/* leviFrame.java
 *
 * Created on 11.02.2005
 * Autor: Birgit Sirocic
 *
 */
package learningVisualizations;

import java.awt.Toolkit;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.lang.reflect.Field;
import java.util.*;

import javax.swing.event.SwingPropertyChangeSupport;

/**
 * @author sirocic
 *
 * Class: leviResourceManager
 *
 * Description: The class "leviResourceManager" manages the resourcen. It reads the
 * 				resource files and provides the founded data.
 *
 * Date:	11.02.2005
 */
public class LeviResourceManager {


    public static final String PROPERTY_LOCALE = "Locale";

    private String resourcePrefix = "resources";
    private String refClass;
    private String refPacket;
    private Locale refLocale;
    private ResourceBundle res;
    private ResourceBundle defaultRes;

    /**
     * If any <code>PropertyChangeListeners</code> have been registered,
     * the <code>changeSupport</code> field describes them.
     *
     * @see #addPropertyChangeListener
     * @see #removePropertyChangeListener
     * @see #firePropertyChange
     */
    private SwingPropertyChangeSupport changeSupport;

    /**
    *
    * Methode:  Constructors
    *
    * Date: 11.02.05
    *
    * @author sirocic
    *
    */
    public LeviResourceManager()
    {
        this.defaultRes = null;
        this.res = null;
        this.refClass = "";
        this.refLocale = null;
    }

    public LeviResourceManager(Class<? extends Object> clazz)
    {
      this( clazz, Locale.getDefault() );
    }

    public LeviResourceManager(Class<? extends Object> clazz, Locale loc)
    {
        this.refClass = clazz.getSimpleName();
        this.refPacket = clazz.getPackage().getName();
        this.refLocale = loc;
        setLocale( refLocale );

    }


    /**
     * Methode:     getResourceByKey
     *
     * Description: Returns the resource string which is addressed by the
     *              parameter <key>.
     *
     * Date:        22.04.2005
     *
     * @author sirocic
     * @param key
     * @return the String resource found by the given key.
     */
    public String getResourceByKey( String key ) {

        String resStr;
        boolean found = false;

        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("CALL leviResourceManager.getResourceByKey: key="+key);

        // DF: Complete rewrite to make it easier to correct problems
        // with the resource bundle.
        try {
            resStr = res.getString( key );
            found = true;
        } catch(  MissingResourceException mre ) {
            resStr = key + " (no key)";
        } catch ( NullPointerException npe ) {
            resStr = key + " (no bundle)";
        }

        // DF 2006-05-19: If nothing is found, try to look up the defaults
        if( !found ) {
            try {
                resStr = defaultRes.getString( key );
                found = true;
            } catch(  MissingResourceException mre ) {
                // remain silent
            } catch ( NullPointerException npe ) {
                System.err.println("Warning: No default resource bundle!");
            }
        }

         // TRACE OUTPUT
         if (LeviModule.isTraceActive()) System.out.println("END leviResourceManager.getResourceByKey return"+resStr);

         return resStr;
    }

    /**
     * Methode:     keyAvailable
     *
     * Description: Tests if the key which is given by parameter <key> is available.
     *
     * Date:        22.04.2005
     *
     * @author sirocic
     * @param key
     * @return true if the given key was found in the loaded resource bundle, false if not.
     */
    public boolean keyAvailable( String key ) {
        boolean ret = false;

        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("CALL leviResourceManager.keyAvailable: key="+key);

        try {
            res.getString( key );
            ret = true;
        } catch ( MissingResourceException mre ) {
            // key not found
        } catch ( NullPointerException npe ) {
            // bundle is nll
        }

        if( !ret ) {
            try {
                defaultRes.getString( key );
                ret = true;
            } catch ( MissingResourceException mre ) {
                // key not found
            } catch ( NullPointerException npe ) {
                // default bundle is nll
            }
        }

        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("END leviResourceManager.keyAvailable retunr="+ret);

        return ret;
    }

    /**
     * @author sirocic
     *
     * Methode:		getLocale
     *
     * Description:	Returns the Local class.
     *
     * Date: 		22.04.2005
     * @return the locale of the resource
     */
    public Locale getLocale() {
        return refLocale;
    }

    /**
     * @author sirocic
     *
     * Methode:		setLocale
     *
     * Description:	Set the Locale for the resource bundle and try
     *              to reload it.
     *
     * Date: 		22.04.2005
     *
     * @param loc the Locale to set for the resources
     * @return true if the bundle for the set Locale could be loaded, false otherwise.
     */
    public boolean setLocale(Locale loc) {
        boolean ret = true;

        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("CALL leviResourceManager.setLocale: loc=" + loc.toString());

        Locale prevLocale = refLocale;
        Locale newLocale = loc;

        /*
         * DF 2006-05-19: Try to load default resource bundle
         *
         * Currently, this does not change "ret", because if the
         * default bundle is missing, we could still get on with the app
         * specific bundle (i.e. languages supported by just one application
         * but not by the whole framework)
         */
        try {
            // this works out to be
            // "resources.learningVisualizations.leviModule"
            this.defaultRes =
                ResourceBundle.getBundle( resourcePrefix + "." +
                                          LeviModule.class.getPackage().getName() + "." +
                                          LeviModule.class.getSimpleName(), loc );
        } catch (Exception e) {
            System.err.println( resourcePrefix + "." +
                    LeviModule.class.getPackage().getName() + "." +
                    LeviModule.class.getSimpleName() + ", Locale " + loc + " not found" );
        }

        /*
         * Now try to load the application specific bundle.
         */
        try {
            // this works out to be
            // "resources." + the path of the derived application Module
            this.res =
                ResourceBundle.getBundle( resourcePrefix + "." + refPacket + "." + refClass, loc );
        } catch (Exception e) {
            System.err.println( resourcePrefix + "." + refPacket + "." + refClass + ", Locale " + loc + " not found" );
            System.err.println( "changing language is not possible" );
            ret = false;
        }

        /*
         * Application specific bundle was found
         */
        if( res != null ) {
            // given locale is new locale
            refLocale = loc;
            /*
             * DF: What exactly should the follwing do?
             */
            if (!keyAvailable("locale.Language") ||
                    res.getString("locale.Language").equals("") ) {
                System.err.println( "locale.Language not defined in " + refClass + " bundle. Trying preferred Language");
                if (keyAvailable("locale.preferredLanguage") &&
                        !res.getString("locale.preferredLanguage").equals("") ) {
                    // locale not present, recursively change to preferred locale
                    System.err.println( "Changing to preferredLanguage '" +
                                        res.getString("locale.preferredLanguage") + "'");
                    ret = setLocale( new Locale( res.getString("locale.preferredLanguage") ) );
                }
            }
        }

        /*
         * DF 20060607: Added workaround for bug #6292739
         * See http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=6292739
         * This may not work with specific JREs.
         */
        try {
            Field field = Toolkit.class.getDeclaredField("resources");
            field.setAccessible(true);
            field.set(null, ResourceBundle.getBundle("sun.awt.resources.awt"));
        } catch(NoSuchFieldException nsfe) {
            // ignored - what can be done anyway?
        } catch(IllegalAccessException iae ) {
            // ignored - what can be done anyway?
        }

        /*
         * DF: Fire a property change if the locale was changed
         */
        if( ret ) firePropertyChange(PROPERTY_LOCALE, prevLocale, newLocale);

        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("END leviResourceManager.setLocale return="+ret);

        return ret;
    }


    // propertyChange support, mainly copied from java.awt.Component

    /**
     * Adds a PropertyChangeListener to the listener list. The listener is
     * registered for all bound properties of this class, including the
     * following:
     * <ul>
     *    <li>this Component's Locale ("name")</li>
     * </ul>
     * Note that if this <code>Component</code> is inheriting a bound property, then no
     * event will be fired in response to a change in the inherited property.
     * <p>
     * If <code>listener</code> is <code>null</code>,
     * no exception is thrown and no action is performed.
     *
     * @param    listener  the property change listener to be added
     *
     * @see #removePropertyChangeListener
     * @see #getPropertyChangeListeners
     * @see #addPropertyChangeListener(java.lang.String, java.beans.PropertyChangeListener)
     */
    public synchronized void addPropertyChangeListener(PropertyChangeListener listener) {
        if (listener == null) {
            return;
        }
        if (changeSupport == null) {
            changeSupport = new SwingPropertyChangeSupport(this);
        }
        changeSupport.addPropertyChangeListener(listener);
    }

    /**
     * Removes a PropertyChangeListener from the listener list. This method
     * should be used to remove PropertyChangeListeners that were registered
     * for all bound properties of this class.
     * <p>
     * If listener is null, no exception is thrown and no action is performed.
     *
     * @param listener the PropertyChangeListener to be removed
     *
     * @see #addPropertyChangeListener
     * @see #getPropertyChangeListeners
     * @see #removePropertyChangeListener(java.lang.String,java.beans.PropertyChangeListener)
     */
    public synchronized void removePropertyChangeListener(PropertyChangeListener listener) {
        if (listener == null || changeSupport == null) {
            return;
        }
        changeSupport.removePropertyChangeListener(listener);
    }

    /**
     * Returns an array of all the property change listeners
     * registered on this module.
     *
     * @return all of this module's <code>PropertyChangeListener</code>s
     *         or an empty array if no property change
     *         listeners are currently registered
     *
     * @see      #addPropertyChangeListener
     * @see      #removePropertyChangeListener
     * @see      #getPropertyChangeListeners(java.lang.String)
     * @see      java.beans.PropertyChangeSupport#getPropertyChangeListeners
     */
    public synchronized PropertyChangeListener[] getPropertyChangeListeners() {
        if (changeSupport == null) {
            return new PropertyChangeListener[0];
        }
        return changeSupport.getPropertyChangeListeners();
    }

    /**
     * Adds a PropertyChangeListener to the listener list for a specific
     * property. The specified property may be user-defined, or one of the
     * following:
     * <ul>
     *    <li>this Component's Locale</li>
     * </ul>
     * Note that if this <code>Component</code> is inheriting a bound property, then no
     * event will be fired in response to a change in the inherited property.
     * <p>
     * If <code>propertyName</code> or <code>listener</code> is <code>null</code>,
     * no exception is thrown and no action is taken.
     *
     * @param propertyName one of the property names listed above
     * @param listener the property change listener to be added
     *
     * @see #removePropertyChangeListener(java.lang.String, java.beans.PropertyChangeListener)
     * @see #getPropertyChangeListeners(java.lang.String)
     * @see #addPropertyChangeListener(java.lang.String, java.beans.PropertyChangeListener)
     */
    public synchronized void addPropertyChangeListener(String propertyName,
                                                       PropertyChangeListener listener) {
        if (listener == null) {
            return;
        }
        if (changeSupport == null) {
            changeSupport = new SwingPropertyChangeSupport(this);
        }
        changeSupport.addPropertyChangeListener(propertyName, listener);
    }

    /**
     * Removes a <code>PropertyChangeListener</code> from the listener
     * list for a specific property. This method should be used to remove
     * <code>PropertyChangeListener</code>s
     * that were registered for a specific bound property.
     * <p>
     * If <code>propertyName</code> or <code>listener</code> is <code>null</code>,
     * no exception is thrown and no action is taken.
     *
     * @param propertyName a valid property name
     * @param listener the PropertyChangeListener to be removed
     *
     * @see #addPropertyChangeListener(java.lang.String, java.beans.PropertyChangeListener)
     * @see #getPropertyChangeListeners(java.lang.String)
     * @see #removePropertyChangeListener(java.beans.PropertyChangeListener)
     */
    public synchronized void removePropertyChangeListener(String propertyName,
                                                          PropertyChangeListener listener) {
        if (listener == null || changeSupport == null) {
            return;
        }
        changeSupport.removePropertyChangeListener(propertyName, listener);
    }

    /**
     * Returns an array of all the listeners which have been associated
     * with the named property.
     *
     * @return all of the <code>PropertyChangeListener</code>s associated with
     *         the named property; if no such listeners have been added or
     *         if <code>propertyName</code> is <code>null</code>, an empty
     *         array is returned
     *
     * @see #addPropertyChangeListener(java.lang.String, java.beans.PropertyChangeListener)
     * @see #removePropertyChangeListener(java.lang.String, java.beans.PropertyChangeListener)
     * @see #getPropertyChangeListeners
     */
    public synchronized PropertyChangeListener[] getPropertyChangeListeners(String propertyName) {
        if (changeSupport == null) {
            return new PropertyChangeListener[0];
        }
        return changeSupport.getPropertyChangeListeners(propertyName);
    }

    /**
     * Support for reporting bound property changes for Object properties.
     * This method can be called when a bound property has changed and it will
     * send the appropriate PropertyChangeEvent to any registered
     * PropertyChangeListeners.
     *
     * @param propertyName the property whose value has changed
     * @param oldValue the property's previous value
     * @param newValue the property's new value
     */
    protected void firePropertyChange(String propertyName,
                                      Object oldValue, Object newValue) {
        PropertyChangeSupport changeSupport = this.changeSupport;
        if (changeSupport == null ||
            (oldValue != null && newValue != null && oldValue.equals(newValue))) {
            return;
        }
        changeSupport.firePropertyChange(propertyName, oldValue, newValue);
    }

    /**
     * Support for reporting bound property changes for boolean properties.
     * This method can be called when a bound property has changed and it will
     * send the appropriate PropertyChangeEvent to any registered
     * PropertyChangeListeners.
     *
     * @param propertyName the property whose value has changed
     * @param oldValue the property's previous value
     * @param newValue the property's new value
     */
    protected void firePropertyChange(String propertyName,
                                      boolean oldValue, boolean newValue) {
        PropertyChangeSupport changeSupport = this.changeSupport;
        if (changeSupport == null || oldValue == newValue) {
            return;
        }
        changeSupport.firePropertyChange(propertyName, oldValue, newValue);
    }

    /**
     * Support for reporting bound property changes for integer properties.
     * This method can be called when a bound property has changed and it will
     * send the appropriate PropertyChangeEvent to any registered
     * PropertyChangeListeners.
     *
     * @param propertyName the property whose value has changed
     * @param oldValue the property's previous value
     * @param newValue the property's new value
     */
    protected void firePropertyChange(String propertyName,
                                      int oldValue, int newValue) {
        PropertyChangeSupport changeSupport = this.changeSupport;
        if (changeSupport == null || oldValue == newValue) {
            return;
        }
        changeSupport.firePropertyChange(propertyName, oldValue, newValue);
    }

    /**
     * Reports a bound property change.
     *
     * @param propertyName the programmatic name of the property
     *          that was changed
     * @param oldValue the old value of the property (as a byte)
     * @param newValue the new value of the property (as a byte)
     * @see #firePropertyChange(java.lang.String, java.lang.Object,
     *          java.lang.Object)
     */
    public void firePropertyChange(String propertyName, byte oldValue, byte newValue) {
        if (changeSupport == null || oldValue == newValue) {
            return;
        }
        firePropertyChange(propertyName, new Byte(oldValue), new Byte(newValue));
    }

    /**
     * Reports a bound property change.
     *
     * @param propertyName the programmatic name of the property
     *          that was changed
     * @param oldValue the old value of the property (as a char)
     * @param newValue the new value of the property (as a char)
     * @see #firePropertyChange(java.lang.String, java.lang.Object,
     *          java.lang.Object)
     */
    public void firePropertyChange(String propertyName, char oldValue, char newValue) {
        if (changeSupport == null || oldValue == newValue) {
            return;
        }
        firePropertyChange(propertyName, new Character(oldValue), new Character(newValue));
    }

    /**
     * Reports a bound property change.
     *
     * @param propertyName the programmatic name of the property
     *          that was changed
     * @param oldValue the old value of the property (as a short)
     * @param newValue the old value of the property (as a short)
     * @see #firePropertyChange(java.lang.String, java.lang.Object,
     *          java.lang.Object)
     */
    public void firePropertyChange(String propertyName, short oldValue, short newValue) {
        if (changeSupport == null || oldValue == newValue) {
            return;
        }
        firePropertyChange(propertyName, new Short(oldValue), new Short(newValue));
    }


    /**
     * Reports a bound property change.
     *
     * @param propertyName the programmatic name of the property
     *          that was changed
     * @param oldValue the old value of the property (as a long)
     * @param newValue the new value of the property (as a long)
     * @see #firePropertyChange(java.lang.String, java.lang.Object,
     *          java.lang.Object)
     */
    public void firePropertyChange(String propertyName, long oldValue, long newValue) {
        if (changeSupport == null || oldValue == newValue) {
            return;
        }
        firePropertyChange(propertyName, new Long(oldValue), new Long(newValue));
    }

    /**
     * Reports a bound property change.
     *
     * @param propertyName the programmatic name of the property
     *          that was changed
     * @param oldValue the old value of the property (as a float)
     * @param newValue the new value of the property (as a float)
     * @see #firePropertyChange(java.lang.String, java.lang.Object,
     *          java.lang.Object)
     */
    public void firePropertyChange(String propertyName, float oldValue, float newValue) {
        if (changeSupport == null || oldValue == newValue) {
            return;
        }
        firePropertyChange(propertyName, new Float(oldValue), new Float(newValue));
    }

    /**
     * Reports a bound property change.
     *
     * @param propertyName the programmatic name of the property
     *          that was changed
     * @param oldValue the old value of the property (as a double)
     * @param newValue the new value of the property (as a double)
     * @see #firePropertyChange(java.lang.String, java.lang.Object,
     *          java.lang.Object)
     */
    public void firePropertyChange(String propertyName, double oldValue, double newValue) {
        if (changeSupport == null || oldValue == newValue) {
            return;
        }
        firePropertyChange(propertyName, new Double(oldValue), new Double(newValue));
    }



}
