/* leviEditor.java
 *
 * Created on 23.06.2005
 * @author sirocic
 */
package learningVisualizations.editor;

import java.awt.*;

import javax.swing.*;

import learningVisualizations.LeviModule;
import learningVisualizations.gui.LeviToolbar;


/**
 * @author sirocic
 *
 * Class: leviEditor
 *
 * Description: The class "leviEditor" is the general graphic scenarioPanel.
 *
 * Date:	23.06.2005
 */
public class LeviEditor extends JPanel implements LeviEditorInterface {

    // CONSTANTS
    final static public int LABELOFFSET=20;

    private LeviModule learningModule; // the parent module for the data transfer between the module parts
    
    private Dimension editorSize;
    
    // 	GUI ELEMENTS
    private LeviToolbar toolbar;
    protected MenuContainer editorContentPanel;
    private LeviScenarioEditorInterface scenarioEditor; // Panel frs Szenario
    /**
     * This constructor must not be used! leviEditor does not work without
     * learningModule.
     * Making this private gives an appropriate compiler error as a hint.
     *
     * TODO: Delete the constructor's contents after approval.
     */
    private LeviEditor() {
          super();
          this.learningModule = null;
          setToolbar( new LeviToolbar( null ) );
          this.editorContentPanel = null;
          this.scenarioEditor = null;
   }

   /**
    * Construct a leviEditor for the learningModule.
    * @param learningModule
    */
   public LeviEditor(LeviModule learningModule) {

      super();

      // TRACE OUTPUT
      if (LeviModule.isTraceActive()) System.out.println("CALL leviEditor.Contructors(learningModule)");

      // setting the module
      this.learningModule = learningModule;

      // init the scenarioPanel view
      setToolbar( new LeviToolbar(this.learningModule) );
      this.editorContentPanel = null;
      this.scenarioEditor = null;
      setLayout(new BorderLayout());

      // FARBWECHSEL ...
 	  setPanelColors();

	  // FONTWECHSEL
	  setPanelFont();
	  
      // TRACE OUTPUT
      if (LeviModule.isTraceActive()) System.out.println("END leviEditor.Contructors(learningModule)");

   }

   /*
    * Methode:		setDialogTexte
    * 
    * Description:	Setz alle Text-Bausteine des Dialoges neu. Wichtig fr Sprachwechsel.
    *    
    * Parameters:   
    * 
    * Date: 		 23.04.2007
    * @author        sirocic
    */	
   protected void setPanelTexte() { }
   
   
   // ACCESSOR METHODES...
   public LeviModule getLearningModule()
   {
       return learningModule;
   }
   public void setLearningModule(LeviModule c)
   {
      learningModule=c;
   }
   public LeviToolbar getToolbar()
   {
       return toolbar;
   }
   public MenuContainer getEditorContentPanel()
   {
       return editorContentPanel;
   }
   public LeviScenarioEditorInterface getScenarioEditor()
   {
       return scenarioEditor;
   }
   // MUTATOR METHODES...
   /*
    * Methode:		paintComponent
    *
    * Description:	Calls the paintComponent methode of the JPanel.
    *
    * Parameters:
    *
    * Date: 		 06.09.2005
    * @author        sirocic
    */
   public void paintComponent(Graphics g) {
	   
      // TRACE OUTPUT
      if (LeviModule.isTraceActive()) System.out.println("CALL leviEditor.paintComponent:");

      // here calling the paint methode of the JPanel
      super.paintComponent(g);
  
      // TRACE OUTPUT
      if (LeviModule.isTraceActive()) System.out.println("END leviEditor.paintComponent");
        
   }

   /*
    * Methode:		cleanMe
    *
    * Description:	Subert den Editor bevor ein neues Szenario geffnet wird. 
    *
    * Parameters:
    *
    * Date: 		 21.10.2006
    * @author        sirocic
    */
   public void cleanMe() {
	   getScenarioEditor().cleanMe();
   }
   
   /*
    * Methode:		setDialogFont
    * 
    * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
    * 				Werte. 
    *    
    * Parameters:   Aktuelle Font fr Labels und Textfelder.
    * 
    * Date: 		 23.04.2007
    * @author        sirocic
    */	
   private void setPanelFont() {
	   Font labelFont;
	   
	   labelFont = (Font)learningModule.getConfiguration().getSettingByKey("FontLabel");
	   
	   setFont(labelFont);
	   	  
   }
   
   /*
    * Methode:		setDialogColors
    * 
    * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
    * 				Werte. 
    *    
    * Parameters:   Aktuelle Farbwerte fr Hintergrund und Vordergrund.
    * 
    * Date: 		 23.04.2007
    * @author        sirocic
    */	
   private void setPanelColors() {
	   Color backgroundColor,  foregroundColor;
		 
	   if (learningModule.getConfiguration().getSettingByKey("ColorComponentBackground")!=null)
	   {
		   backgroundColor = (Color)learningModule.getConfiguration().getSettingByKey("ColorComponentBackground");
		   foregroundColor = (Color)learningModule.getConfiguration().getSettingByKey("ColorComponentForeground");
	   }
	   else {
		   backgroundColor = (Color)learningModule.getConfiguration().getSettingByKey("ColorModuleBackground");
		   foregroundColor = (Color)learningModule.getConfiguration().getSettingByKey("ColorModuleForeground");
	   }
	   
	   // 	 Hintergrund und Zeichenfarbe auf aktuelle Werte setzen
       setBackground(backgroundColor);
	   setForeground(foregroundColor);
     
   }
   
   /*
    * Methode:		updateLanguageColorFonts
    *
    * Description:	Ldt sprachabhngige Teile neu. In der Toolbar knnen z.B. sprachabhngige
    * 				Buttons sein 
    *
    * Date: 		12.03.2007
    * 
    * @author sirocic
    */
   public void updateLanguageColorFonts()
   {
	   
	  //	 SUBKOMPONENTE UPDATEN...
	  getToolbar().updateLanguageColorFonts();
	  if (scenarioEditor != null) {
		  scenarioEditor.updateLanguageColorFonts();
      }
   
	   // 	 FARBWECHSEL ...
	   setPanelColors();

	   // FONTWECHSEL
	   setPanelFont();
	  
	   // SPRACHWECHSEL ...
	   setPanelTexte();
	   
   }
   
   public void setToolbar(LeviToolbar t)
   {
       if( toolbar != null ) {
           if(learningModule.getConfiguration() != null) {
               learningModule.getConfiguration().removePropertyChangeListener(toolbar);
           }
       }
       toolbar=t;
       if(learningModule.getConfiguration() != null) {
           learningModule.getConfiguration().addPropertyChangeListener(toolbar);
       }
   }
   public void setEditorContentPanel(MenuContainer panel)
   {
       editorContentPanel=panel;
   }
   public void setScenarioEditor(LeviScenarioEditorInterface scenarioPanel)
   {
       this.scenarioEditor=scenarioPanel;
   }

}
