/* leviScenarioEditorMouseListener.java
 * 
 * Created on 17.10.2006
 * @author sirocic
 */
package learningVisualizations.editor;

import java.awt.*;
import java.awt.event.*;

import learningVisualizations.LeviModule;
import learningVisualizations.gui.LeviScenarioPanel;
import learningVisualizations.visualizer.LeviShapeMaker;

/**
 * @author sirocic
 *
 * Class: leviScenarioEditorMouseListener
 * 
 * Description: Handles the mouse events for the scenario scenarioPanel or visualizer, respectivly.
 * 
 * Date:	17.10.2006
 */
public class LeviScenarioEditorMouseListener extends MouseAdapter {
	
	protected LeviScenarioPanel scenarioPanel;
	protected LeviShapeMaker selected;
	
	/* 
    * Methode:	Constructors
    * 
    * Date: 18.03.2005
    * @author sirocic
    */
	public LeviScenarioEditorMouseListener()
	{
		this.scenarioPanel = null;
		this.selected = null;
	}
	public LeviScenarioEditorMouseListener(LeviScenarioPanel scenarioPanel)
	{
		this();
		this.scenarioPanel = scenarioPanel;
	}
	
	//	 MUTATOR METHODES ...
	
	/*
	 * Methode:		mousePressed
	 * 
	 * Description:	This method will be called if the user press a mouse key.
	 *              
	 * 
	 * Date: 		17.10.2006
	 * @author      sirocic
	 */
	public void mousePressed(MouseEvent event) {

		// TRACE OUTPUT
	  	if (LeviModule.isTraceActive()) System.out.println("CALL leviScenarioEditorMouseListener.mousePressed:");
		  
	  	pressedAction(event);
	     
	  	// TRACE OUTPUT
	  	if (LeviModule.isTraceActive()) System.out.println("END leviScenarioEditorMouseListener.mousePressed");
  	
	}
	
	/*
	 * Methode:		mouseReleased
	 * 
	 * Description:	Diese Methode wird aufgerufen, wenn der Benutzer die Maustaste wieder loslt.
	 *              Beim Verschieben eines Objektes mu nichts weiter gemacht werden an der Stelle.
	 *              
	 * Date: 		17.10.2006
	 * @author      sirocic
	 */
	public void mouseReleased(MouseEvent event) {
	
		// TRACE OUTPUT
	  	if (LeviModule.isTraceActive()) System.out.println("CALL leviScenarioEditorMouseListener.mouseReleased:");
		  
	  	
	  	// TRACE OUTPUT
	  	if (LeviModule.isTraceActive()) System.out.println("END leviScenarioEditorMouseListener.mouseReleased");
	
	}
	
  /*
   * Methode:		mouseClicked
   * 
   * Description:	This method contains the actions which will take place if the user has
   *                clicked the mouse. If a buffer or process is founded at the mouse positionl,
   *                this will be the currently selected object in the scenarioPanel.
   * 
   * Date: 		    18.03.2005
   * @author        sirocic
   */
	public void mouseClicked(MouseEvent event) {
		
	  // TRACE OUTPUT
	  if (LeviModule.isTraceActive()) System.out.println("CALL leviScenarioEditorMouseListener.mouseClicked:");
	 
	  clickedAction(event);
		
  	  // TRACE OUTPUT
  	  if (LeviModule.isTraceActive()) System.out.println("END leviScenarioEditorMouseListener.mouseClicked");

	}
		
		
	/*
	 * Methode:		pressedAction
	 * 
	 * Description:	Diese Methode ist fr die allgemeine Eventbehandlung beim Drcken einer Maustaste
	 *              verantwortlich, die fr alle Lernmodule identisch ist. Das sind z.B. der Szenarioeditor
	 *              bekommt den Fokus und das eventuell selektierte Objekt finden. Wenn das bereits
	 *              zuvor ausgewhlte Objekt selektiert wurde, wird der Editor in den MOVE Modus gesetzt,
	 *              da so ein Verschieben des Ojekts eingeleitet wird.  
	 *              
	 *              Diese Methode dient beim Entwickeln von Subklassen dazu doppelten Code zu vermeiden
	 * 
	 * Date: 		17.10.2006
	 * @author      sirocic
	 */
	public void pressedAction(MouseEvent event) {
		
		// TRACE OUTPUT
	  	if (LeviModule.isTraceActive()) System.out.println("CALL leviScenarioEditorMouseListener.pressedAction:");
	
	  	// scenario Editor gets the focus ...
	  	scenarioPanel.getScenarioParent().getParentPanel().requestFocus();
	  	
	  	// Is there a object ??
	  	selected = scenarioPanel.find(event.getPoint());
		
	  	if (selected != null) {
	  		// Der Benutzer hat ein Objekt selektiert !!!
	  		if (scenarioPanel.getScenarioParent().getCurrentSelected()!= null) {
	  			// Es existiert bereits ein selektiertes Objekt !!!
		  		if ((scenarioPanel.getScenarioParent().getModus() == scenarioPanel.getScenarioParent().IDLE) && 
		  				(scenarioPanel.getScenarioParent().getCurrentSelected().equals(selected))	) {
		  			// Der Editor ist gerade im Normalmodus und Benutzer hat auf das bereits selektierte Objekt
		  			// geklickt. Das Verschieben wird eingeleitet, d.h. der Editor wird auf den Modus MOVE gesetzt.
				  	scenarioPanel.getScenarioParent().setModus(scenarioPanel.getScenarioParent().MOVE);
		  		}
	  		}
	  	}
	  
	  	// TRACE OUTPUT
	  	if (LeviModule.isTraceActive()) System.out.println("END leviScenarioEditorMouseListener.pressedAction");
  	
	}
	
	/*
	 * Methode:		clickedAction
	 * 
	 * Description:	Diese Methode ist fr die allgemeine Eventbehandlung beim Klicken mit der Maustaste
	 *              verantwortlich, die fr alle Lernmodule identisch ist. Das sind z.B. der Szenarioeditor
	 *              bekommt den Fokus und das eventuell selektierte Objekt finden. Wenn das bereits
	 *              zuvor ausgewhlte Objekt selektiert wurde, wird der Editor in den MOVE Modus gesetzt,
	 *              da so ein Verschieben des Ojekts eingeleitet wird.  
	 *              
	 *              Diese Methode dient beim Entwickeln von Subklassen dazu doppelten Code zu vermeiden
	 * 
	 * Date: 		17.10.2006
	 * @author      sirocic
	 */
	public void clickedAction(MouseEvent event) {
		
		// TRACE OUTPUT
	  	if (LeviModule.isTraceActive()) System.out.println("CALL leviScenarioEditorMouseListener.clickedAction:");

	  	if (selected != null) {
	  		// ES WURDE EIN OBJEKT ANGEKLICKT !!!
		  
	  		// Markierung des alten selektierten Objekts zurcksetzen ...
	  		if (scenarioPanel.getScenarioParent().getCurrentSelected()!= null)
	  			scenarioPanel.getScenarioParent().getCurrentSelected().setSelected(false);
	     
	  		if (!selected.equals(scenarioPanel.getScenarioParent().getCurrentSelected())) {
	  			// Neues Objekt ist nicht identisch mit dem vorher selektierten Objekt.
    		 	// Neues Objekt als selektiert markieren...
	  			selected.setSelected(true);
     
	  			//	Selektiertes Objekt im Editor merken ...
	  			scenarioPanel.getScenarioParent().updateCurrentSelection(selected);
	  			
	  			//Maus ist ber dem selektierten Objekt; Cursor ndern ...
		    	scenarioPanel.setCursor(Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR));
	  			
	  		}
	  		else {
	  			// Neues Objekt ist identisch mit dem vorher selektierten Objekt.
	  			// Selektion zurck nehmen.
	  			scenarioPanel.getScenarioParent().updateCurrentSelection(null);
	  			
	  			//	Maus ist nicht mehr auf selektiertem Objekt; Default Cursor wieder einstellen.
	  			scenarioPanel.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
	  		}
	     
	   }		
	   else {
		   // ES WURDE KEIN OBJEKT ANGEKLICKT !!!
		   if (scenarioPanel.getScenarioParent().getModus() == scenarioPanel.getScenarioParent().MOVE)
			  // set the scenarioPanel modus back to idle
			   scenarioPanel.getScenarioParent().setModus(scenarioPanel.getScenarioParent().IDLE);
			
		   // Cursor zurcksetzen auf Pfeil.
	 	   scenarioPanel.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
	   }
	  
	   // TRACE OUTPUT
	   if (LeviModule.isTraceActive()) System.out.println("END leviScenarioEditorMouseListener.clickedAction");
  			 
	}
	
	/*
	 * Methode:		releaseAction
	 * 
	 * Description:	Diese Methode ist fr die allgemeine Eventbehandlung beim Loslassen der Maustaste
	 *              verantwortlich, die fr alle Lernmodule identisch ist. Das ist z.B. das Zurcksetzen
	 *              des selektieren Objekts auf die Default-Layer-Ebene im Szenarioeditorpanel.
	 *              
	 *              Diese Methode dient beim Entwickeln von Subklassen dazu doppelten Code zu vermeiden
	 * 
	 * Date: 		28.02.2007
	 * @author      sirocic
	 */
	public void releaseAction(MouseEvent event) {
		
		// TRACE OUTPUT
	  	if (LeviModule.isTraceActive()) System.out.println("CALL leviScenarioEditorMouseListener.releaseAction:");

	   // TRACE OUTPUT
	   if (LeviModule.isTraceActive()) System.out.println("END leviScenarioEditorMouseListener.releaseAction");
  			 
	}
	
   public void setScenarioPanel(LeviScenarioPanel parentScenarioPanel)
   {
   		this.scenarioPanel = parentScenarioPanel;
   }
   
   // ACCESSOR METHODES ...
   public LeviScenarioPanel getScenarioPanel()
   {
   		return scenarioPanel;
   }
	
}
