/* leviScenarioEditorMouseMotionListener.java
 * 
 * Created on 17.10.2006
 * @author sirocic
 */
package learningVisualizations.editor;
import java.awt.*;
import java.awt.event.*;

import javax.swing.JLayeredPane;

import learningVisualizations.LeviModule;
import learningVisualizations.gui.LeviScenarioPanel;
import learningVisualizations.visualizer.LeviShapeMaker;

/**
 * @author sirocic
 *
 * Class: leviScenarioEditorMouseMotionListener
 * 
 * Description: Handles the mouse motions for the scenario scenarioPanel.
 * 
 * Date:	17.10.2006
 */
public class LeviScenarioEditorMouseMotionListener implements MouseMotionListener {

	protected LeviScenarioPanel scenarioPanel;
	protected int xOffset, yOffset;
	
	/* 
    * Methode:	Constructors
    * 
    * Date: 17.10.2006
    * @author sirocic
    */
	public LeviScenarioEditorMouseMotionListener()
	{
		this.scenarioPanel = null;
	}
	public LeviScenarioEditorMouseMotionListener(LeviScenarioPanel scenarioPanel)
	{
		this.scenarioPanel = scenarioPanel;
	}
	
	//	 MUTATOR METHODES ...
	
	/*
	 * Methode:		mouseMoved
	 * 
	 * Description:	This method will be called if the user move the mouse cursor.
	 *              
	 * 
	 * Date: 		17.10.2006
	 * @author      sirocic
	 */
	public void mouseMoved(MouseEvent event) {
		
		  // TRACE OUTPUT
		  if (LeviModule.isTraceActive()) System.out.println("CALL leviScenarioEditorMouseMotionListener.mouseMoved:");

		  movedAction(event);
		  
	  	  // TRACE OUTPUT
	  	  if (LeviModule.isTraceActive()) System.out.println("END leviScenarioEditorMouseMotionListener.mouseMoved");
	  	
	}

	/*
	 * Methode:		mouseDragged
	 * 
	 * Description:	This method will be called if the user drag the mouse.
	 *              
	 * 
	 * Date: 		17.10.2006
	 * @author      sirocic
	 */
	public void mouseDragged(MouseEvent event) {
		
	    // TRACE OUTPUT
		if (LeviModule.isTraceActive()) System.out.println("CALL leviScenarioEditorMouseMotionListener.mouseDragged:");
		  
		draggedAction(event);
		
	  	// TRACE OUTPUT
	  	if (LeviModule.isTraceActive()) System.out.println("END leviScenarioEditorMouseMotionListener.mouseDragged");
	  	
	}
	
	/*
	 * Methode:		movedAction
	 * 
	 * Description:	Diese Methode ist fr die allgemeine Eventbehandlung beim Drcken einer Maustaste
	 *              verantwortlich, die fr alle Lernmodule identisch ist. 
	 *              
	 *              Diese Methode dient beim Entwickeln von Subklassen dazu doppelten Code zu vermeiden
	 * 
	 * Date: 		17.10.2006
	 * @author      sirocic
	 */
	public void movedAction(MouseEvent event) {
		
		// TRACE OUTPUT
	  	if (LeviModule.isTraceActive()) System.out.println("CALL leviScenarioEditorMouseMotionListener.movedAction:");
	
	    if (scenarioPanel.getScenarioParent().getCurrentSelected() != null) {
	      if (((LeviShapeMaker)scenarioPanel.getScenarioParent().getCurrentSelected()).contains(event.getPoint())) {
	    	
            // Maus ist ber dem selektierten Objekt; Cursor ndern ...
		    scenarioPanel.setCursor(Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR));
		    	
	    	// Offset des Mauszeigers ber dem Ojekt merken. 
	    	if (scenarioPanel.getScenarioParent().getCurrentSelected().getLeviObject() != null) {
			    xOffset = event.getX() - scenarioPanel.getScenarioParent().getCurrentSelected().getLeviObject().getLeftX();
			    yOffset = event.getY() - scenarioPanel.getScenarioParent().getCurrentSelected().getLeviObject().getTopY();
	    	}
	    	else {
	    		xOffset = event.getX();
			    yOffset = event.getY();
	    	}
		    
	      }
	      else {
	    	// maus ist woanders !!! ; Default Cursor wieder einstellen.
	    	scenarioPanel.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
	      }
	      
	    }
	    else {
	    	// Falls vorher ein Objekt selektiert war und der Cursor auf MOVE steht.
	    	// Jetzt ist kein Objekt selektiert also auf jeden Fall den Normalen Cursor einstellen.
	    	// Default Cursor wieder einstellen.
	    	scenarioPanel.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
	     }
		  
	  	// TRACE OUTPUT
	  	if (LeviModule.isTraceActive()) System.out.println("END leviScenarioEditorMouseMotionListener.movedAction");
	  
	}
	
	/*
	 * Methode:		draggedAction
	 * 
	 * Description:	Diese Methode ist fr die allgemeine Eventbehandlung beim Drcken einer Maustaste
	 *              verantwortlich, die fr alle Lernmodule identisch ist. 
	 *              
	 *              Diese Methode dient beim Entwickeln von Subklassen dazu doppelten Code zu vermeiden
	 * 
	 * Date: 		17.10.2006
	 * @author      sirocic
	 */
	public void draggedAction(MouseEvent event) {
		
		// TRACE OUTPUT
	  	if (LeviModule.isTraceActive()) System.out.println("CALL leviScenarioEditorMouseMotionListener.draggedAction:");
	
		if ( (scenarioPanel.getScenarioParent().getCurrentSelected() != null) && (scenarioPanel.getScenarioParent().getModus() == LeviScenarioEditorVisualizer.MOVE) ) {
			// Objekt verschieben an neue Position ...
			scenarioPanel.getScenarioParent().getCurrentSelected().getLeviObject().setLeftX(event.getX() - xOffset);
			scenarioPanel.getScenarioParent().getCurrentSelected().getLeviObject().setTopY(event.getY() - yOffset);
			scenarioPanel.getScenarioParent().getCurrentSelected().setLocation(event.getX() - xOffset, event.getY() - yOffset);
		
			scenarioPanel.getScenarioParent().fitPanel();
		}
		
		// Szenario wurde verndert
		scenarioPanel.getScenarioParent().getLearningModule().getScenario().setChanged();
        
	  	// TRACE OUTPUT
	  	if (LeviModule.isTraceActive()) System.out.println("END leviScenarioEditorMouseMotionListener.draggedAction");
	  
	}
	
   public void setScenarioPanel(LeviScenarioPanel scenarioPanel)
   {
   		this.scenarioPanel = scenarioPanel;
   }
   
   // ACCESSOR METHODES ...
   public LeviScenarioPanel getScenarioPanel()
   {
   		return scenarioPanel;
   }
   
   
}
