/* leviScenarioEditor.java
 * 
 * Created on 17.10.2006
 * @author sirocic
 */
package learningVisualizations.editor;

import java.awt.Color;
import java.awt.Font;
import java.awt.event.MouseListener;

import javax.swing.*;

import learningVisualizations.LeviModule;
import learningVisualizations.gui.LeviScenarioPanel;
import learningVisualizations.visualizer.LeviScenarioVisualizerInterface;
import learningVisualizations.visualizer.LeviShapeMaker;


/**
 * @author sirocic
 *
 * Class: leviScenarioEditor
 * 
 * Description: The class "leviScenarioEditor" is the part of the scenarioPanel (panel) in which 
 *              the scenario will be painted.
 * 
 * Date:	17.10.2006
 */
public abstract class LeviScenarioEditorVisualizer extends JScrollPane 
					implements LeviScenarioEditorInterface,
							   LeviScenarioVisualizerInterface {

	  static final long serialVersionUID=1;
	  
	  final static int MAXDRAWWIDTH=2000;
	  final static int MAXDRAWHEIGHT=2000;
	  
	  //Szenarioeditorzustnde ...
	  final static public int IDLE=1;  // Editor is waiting on user action
	  final static public int MOVE=2;  // User is moving an object.
	  
	  protected JPanel parentPanel;
	  protected LeviModule myModule;
	  protected LeviScenarioPanel scenarioPanel;
	  public MouseListener popupListener; // Mouse-Listener fr den Visualizer. Nur aktiv wenn die Komponente im Vorlesungsmodus ist.
	  private int modus; // aktueller Szenarioeditorzustand z.B. idle, move ...
	  
	  // EDITOR CONTENT
	  private LeviShapeMaker currentSelected; // currently selected object
		
	  /*
	   * Methode:		Constructors
	   * 
	   * Date: 		    17.10.2006
	   * @author        sirocic
	   */
	   public LeviScenarioEditorVisualizer() {
		  super();
		  parentPanel = null;
		  myModule = null;
          this.modus = IDLE;
          this.currentSelected = null;
	      this.popupListener = null;
	   }
	   public LeviScenarioEditorVisualizer(JPanel myParent, LeviModule myComponent) {
	   	
		  super();
		  
		  // TRACE OUTPUT
		  if (LeviModule.isTraceActive()) System.out.println("CALL leviScenarioEditor.Contructors(edt)");
		  this.myModule = myComponent;
		  this.parentPanel = myParent;
		  this.setFocusable(true);
	      this.currentSelected = null;
	      this.popupListener = null;
	      this.modus = IDLE;
      
	      // FARBWECHSEL ...
	 	  setPanelColors();

		  // FONTWECHSEL
		  setPanelFont();
		  
		  // TRACE OUTPUT
		  if (LeviModule.isTraceActive()) System.out.println("END leviScenarioEditor.Contructors(edt)");
	   }

	   /*
	    * Methode:		createGraphicScenario
	    *
	    * Description:	Creates the graphic elements for the current scenario and repaints the panel.
	    * 				This have to be done in the component subclass.
	    *
	    * Parameters:
	    *
	    * Date: 		 18.10.2006
	    * @author        sirocic
	    */
	   public abstract void createGraphicScenario();
	   
	   /*
	    * Methode:		fitPanel
	    * 
	    * Description:	Bestimmen der bentigten Panelgre fr's aktuelle Szenario. 
	    *               Es werden alle grafischen Elemente (Shapes) durchlaufen und jeweils
	    *               der rechte untere Punkt bestimmt. Die am weitesten rechts und unten
	    *               liegenden Koordinaten werden sich gemerkt. 
	    *    
	    *    			Leider sind die Objekte des Editors noch nicht verallgemeinert sondern
	    *               bestehen aus verschiedenen ArrayList(s). Daher kann dies hier nur in der
	    *               abgeleiteten Klasse durchgefhrt werden. TODO ????
	    *               
	    * Parameters:    
	    * 
	    * Date: 		 17.10.2006
	    * @author        sirocic
	    */	
	   public abstract void fitPanel() ;
	   
	   /*
	    * Methode:		cancelAnimation
	    *
	    * Description:	Abrechen der aktuellen Animationen. Springen in den nchsten Zustand ohne 
	    *               das die Animation vollstndig durchgelaufen ist.
	    *
	    * Date: 		 12.10.2006
	    * @author        sirocic
	    */
	   public abstract void cancelAnimation(boolean cancelByBackOrStop) ;
	 
	   /*
	    * Methode:		cleanMe
	    *
	    * Description:	Deletes all graphical element from the scenarioPanel.
	    *
	    * Parameters:
	    *
	    * Date: 		 05.05.2006
	    * @author        sirocic
	    */
	   public void cleanMe() {

	      // TRACE OUTPUT
	      if (LeviModule.isTraceActive()) System.out.println("CALL leviScenarioEditor.cleanMe:");

	      currentSelected = null;
	      modus = IDLE;

	      // TRACE OUTPUT
	      if (LeviModule.isTraceActive()) System.out.println("END leviScenarioEditor.cleanMe");

	   }
	   public void setCurrentSelected(LeviShapeMaker selectedObject)
	   {
	       currentSelected=selectedObject;
	   }
	   public void setModus(int newModus)
	   {
	       modus=newModus;
	   }
	   public void removeSelectedObject()
	   {
	       
	       // Szenario wurde verndert
	       getLearningModule().getScenario().setChanged();
	       
	       // selektion des gelschten Objekts zurcksezten
	       updateCurrentSelection(null);
	       
	       // Panelausmasse neu bestimmen
	       fitPanel();
	       
	   }
	   
	   /*
	    * Methode:		updateCurrentSelection
	    *
	    * Description:	Updating the current selected object . 
	    *
	    * Date: 		17.10.2006
	    * @author       sirocic
	    */
	   public void updateCurrentSelection(LeviShapeMaker newSelection) {

	         // TRACE OUTPUT
	         if (LeviModule.isTraceActive()) System.out.println("CALL leviScenarioEditor.updateCurrentSelection:");

	         //	Selektiertes Objekt merken ...
	         setCurrentSelected(newSelection);

	         // TRACE OUTPUT
	         if (LeviModule.isTraceActive()) System.out.println("END leviScenarioEditor.updateCurrentSelection");

	   }
	   public void setPopupListener(MouseListener listener) {
	       popupListener = listener;
	       addMouseListener(listener);
	   }
	   public void removePopupListener() {
		   
	        this.removeMouseListener(popupListener);
	        addMouseListener(null);
	        
	        // Da jetzt nichts mehr selektiert werden kann
	        // sollte das eventuell gerade selektierte Objekt wieder zurckgesetzt werden.
	  		// Markierung des alten selektierten Objekts zurcksetzen ...
	  		if (getCurrentSelected()!= null)
	  			getCurrentSelected().setSelected(false);
	  		updateCurrentSelection(null);
	  		
	   }

   /*
    * Methode:		setDialogFont
    * 
    * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
    * 				Werte. 
    *    
    * Parameters:   Aktuelle Font fr Labels und Textfelder.
    * 
    * Date: 		 23.04.2007
    * @author        sirocic
    */	
   private void setPanelFont() {
	   Font labelFont;
	   
	   labelFont = (Font)myModule.getConfiguration().getSettingByKey("FontLabel");
	   
	   setFont(labelFont);
	   	  
   }
   
   /*
    * Methode:		setDialogTexte
    * 
    * Description:	Setz alle Text-Bausteine des Dialoges neu. Wichtig fr Sprachwechsel.
    *    
    * Parameters:   
    * 
    * Date: 		 23.04.2007
    * @author        sirocic
    */	
   private void setPanelTexte() {}
   
   /*
    * Methode:		setDialogColors
    * 
    * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
    * 				Werte. 
    *    
    * Parameters:   Aktuelle Farbwerte fr Hintergrund und Vordergrund.
    * 
    * Date: 		 23.04.2007
    * @author        sirocic
    */	
   private void setPanelColors() {
	   Color backgroundColor,  foregroundColor;
		 
	   if (myModule.getConfiguration().getSettingByKey("ColorComponentBackground")!=null)
	   {
		   backgroundColor = (Color)myModule.getConfiguration().getSettingByKey("ColorComponentBackground");
		   foregroundColor = (Color)myModule.getConfiguration().getSettingByKey("ColorComponentForeground");
	   }
	   else {
		   backgroundColor = (Color)myModule.getConfiguration().getSettingByKey("ColorModuleBackground");
		   foregroundColor = (Color)myModule.getConfiguration().getSettingByKey("ColorModuleForeground");
	   }
	   
	   // 	 Hintergrund und Zeichenfarbe auf aktuelle Werte setzen
       setBackground(backgroundColor);
	   setForeground(foregroundColor);   
   }
	   
   /*
    * Methode:		updateLanguageColorFonts
    *
    * Description:	Ldt sprachabhngige Teile neu. In der Toolbar knnen z.B. sprachabhngige
    * 				Buttons sein 
    *
    * Date: 		12.03.2007
    * 
    * @author sirocic
    */
   public void updateLanguageColorFonts() {
	   
	   if (scenarioPanel != null) {
		   scenarioPanel.updateLanguageColorFonts();
	   }
	   setPanelTexte();
	   setPanelFont();
	   setPanelColors();
	   
   }
   
   public void setParentPanel(JPanel myEditor)
   {
   		this.parentPanel = myEditor;
   }
   public JPanel getParentPanel()
   {
   	return parentPanel;
   }
   public LeviModule getLearningModule()
   {
   	return myModule;
   }
   public JLayeredPane getScenarioPanel()
   {
   	return scenarioPanel;
   }
   public LeviShapeMaker getCurrentSelected()
   {
       return currentSelected;
   }
   public int getModus()
   {
       return modus;
   }

   public MouseListener getPopupListener()
   {
          return popupListener;
   }
   
}
