/* leviButton.java
 * 
 * Created on 12.10.2005
 * @author sirocic
 */
package learningVisualizations.gui;



import java.awt.*;
import java.net.URL;

import javax.swing.*;

import learningVisualizations.LeviModule;



/**
 * @author Alexandra
 *
 * Class: leviButton
 * 
 * Description: 
 * 
 * Date 18.09.2005
 */
public class LeviButton  extends JButton {

	private static final long serialVersionUID=1;
	public static final int BUTTONSIZE = 31;
    private static final Insets margins = new Insets(0,0,0,0);
    
    private LeviModule module;
    private String keyLabel; // identifies the button
   
    public LeviButton() {
    	super();
    	setText("leviButton");
    	this.keyLabel = "";
    }
    /**
     * @param icon
     */
    public LeviButton(ImageIcon icon) {
   
    	super(icon);
    	
    	this.keyLabel = "";
    	
        // TODO Auto-generated constructor stub
        setMargin(margins);
        setVerticalTextPosition(BOTTOM);
        setHorizontalTextPosition(CENTER);
    }

	/**
	 * @param text
	 */
	public LeviButton(String text) {
	    this(new ImageIcon(text));
	}

	/**
	 * @param imageFile
	 * @param text
	 */
    public LeviButton(String imageFile, String text) {
        this(new ImageIcon(imageFile));
        setText(text);
    }
    public LeviButton(String keyLabel, LeviModule module) {
 
    	super();
    	
    	this.keyLabel = keyLabel;
    	this.module = module;
    
    	if (module != null) {
	        // Hintergrundfarbe setzen
	    	setPanelColors();
	    	
	    	// bentigte Bilder setzen
	    	URL iconurl, piconurl, diconurl;
	    	iconurl = LeviModule.class.getResource(module.getRessources().getResourceByKey(keyLabel+".icon"));
	 	    piconurl = LeviModule.class.getResource(module.getRessources().getResourceByKey(keyLabel+".picon"));
	 	    diconurl = LeviModule.class.getResource(module.getRessources().getResourceByKey(keyLabel+".dicon"));
	    	if (iconurl!= null)  
	    		setIcon(new ImageIcon(iconurl));
	    	if (piconurl!= null) setPressedIcon(new ImageIcon(piconurl));
	    	if (piconurl!= null) setRolloverIcon(new ImageIcon(piconurl));
	    	if (diconurl!= null) setDisabledIcon(new ImageIcon(diconurl));
        
	        // Farbe, Tooltips und Font setzen
	        setPanelTexte();
	        setPanelFont();
	        setPanelColors();
    	}
	        
        // Abstnde und Positionen setzen
        setMargin(margins);
        setVerticalTextPosition(BOTTOM);
        setHorizontalTextPosition(CENTER);
        
       
    }
    public LeviButton(String keyLabel, LeviModule module, Action action) {
    	 
    	this(keyLabel, module);
    	
    	this.addActionListener(action);
    	
    }

    
    // MUTATOR METHODES ...
    public void setKey(String key) {
    	keyLabel = key;
    }
    
   /**
    * Methode:		updateLanguageColorFonts
    *
    * Description:	Sollte aufgerufen werden, wenn ein Sprach- oder Farbwechsel stattgefunden hat.
    * 				Texte und Icons werden neu aus den Resourcen geladen und die Farbe neu nach den
    * 				aktuellen Werten gesetzt.
    *
    * Parameters:
    *
    * Date: 		13.09.2005
    * @author      sirocic
    */
    public void updateLanguageColorFonts() {
    	URL iconurl, piconurl, diconurl;
    	Color bcolor;
    	
        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("CALL leviButton.updateLanguageColorFonts:");

        if (module != null) {
	        
        	setPanelColors();
        	
	        if (module.getFrame() != null) {
		    	// bentigte Bilder setzen
		    	iconurl = LeviModule.class.getResource(module.getRessources().getResourceByKey(keyLabel+".icon"));
		 	    piconurl = LeviModule.class.getResource(module.getRessources().getResourceByKey(keyLabel+".picon"));
		 	    diconurl = LeviModule.class.getResource(module.getRessources().getResourceByKey(keyLabel+".dicon"));
		    	if (iconurl!= null)  setIcon(new ImageIcon(iconurl));
		    	if (piconurl!= null) setPressedIcon(new ImageIcon(piconurl));
		    	if (piconurl!= null) setRolloverIcon(new ImageIcon(piconurl));
		    	if (diconurl!= null) setDisabledIcon(new ImageIcon(diconurl));
	        }
	        
	        setPanelTexte();
	        setPanelFont();
        }
        
        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("END leviButton.updateLanguageColorFonts");

    }
    
    /*
     * Methode:		setDialogFont
     * 
     * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
     * 				Werte. 
     *    
     * Parameters:   Aktuelle Font fr Labels und Textfelder.
     * 
     * Date: 		 23.04.2007
     * @author        sirocic
     */	
    public void setPanelFont() {
 	   Font labelFont;
 	   
 	   labelFont = (Font)module.getConfiguration().getSettingByKey("FontLabel");
 	   
 	   setFont(labelFont);
 	   	  
    }
    
    /*
     * Methode:		setDialogTexte
     * 
     * Description:	Setz alle Text-Bausteine des Dialoges neu. Wichtig fr Sprachwechsel.
     *    
     * Parameters:   
     * 
     * Date: 		 23.04.2007
     * @author        sirocic
     */	
    public void setPanelTexte() {
 	   
    	// 	 Tooltip setzen
    	if (module.isTooltipsVisible()) 
    		 setToolTipText(module.getRessources().getResourceByKey(keyLabel+".tip"));
    	else
    		setToolTipText(null);
    	
    }
    
    /*
     * Methode:		setDialogColors
     * 
     * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
     * 				Werte. 
     *    
     * Parameters:   Aktuelle Farbwerte fr Hintergrund und Vordergrund.
     * 
     * Date: 		 23.04.2007
     * @author        sirocic
     */	
    public void setPanelColors() {
 	   Color bcolor;
 	    	  
 	   // 	 Hintergrundfarbe setzen
 	   bcolor =(Color)module.getConfiguration().getSettingByKey("ColorButtonBackground");
       if (bcolor != null) setBackground(bcolor);
   	
    }
    
    //  ACCESSOR METHODES ...
    public String getKey() {
    	return keyLabel;
    }

}
