/* leviColorChooseDialog.java
 * 
 * Created on 09.11.2006
 * @author sirocic
 */
package learningVisualizations.gui;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.colorchooser.*;

import learningVisualizations.LeviModule;



/**
 * @author sirocic
 *
 * Class: leviColorChooseDialog
 * 
 * Description: The class "leviColorChooseDialog" of the general color choose dialog. 
 * 				
 * Date:	09.11.2006
 */
public class LeviColorChooseDialog extends LeviDialog {

	static final long serialVersionUID=1;

	private LeviColorTargetInterface colorTarget;
	private JLabel targetPropertyLabel;
	private JColorChooser farbenCC;
	
  /*
   * Methode:		Contructors
   * 
   * Description:	Displays the current color setting from the configuration.
   *				Provides the user with the possiblity to change the color settings
   *                for special elements like module background, process foreground etc.
   *                The selected colors are store by the background color of the preview panels.
   *                The configuration will be updated if the confirm button was selected.
   * 
   * Date: 		    07.09.2005
   * @author        sirocic
   */
	public LeviColorChooseDialog() {
	   super();
   	   // INIT
	   
  	   // SET DIALOG PROPERTIES
   	   setSize(WIDTH, HEIGHT);
	}
   public LeviColorChooseDialog(LeviFrame frame) {
   	   super(frame,"dlg.color.title");
   	   
   	   // Lokale Variablen
       AbstractColorChooserPanel[] oldPanels;

	   // TRACE OUTPUT
	   if (LeviModule.isTraceActive()) System.out.println("CALL leviColorChooseDialog.leviColorDialog:");

  	   // INIT
   	   // GUI Dialogelemente erzeugen ...
       farbenCC = new JColorChooser();
       oldPanels = farbenCC.getChooserPanels();
       farbenCC.removeChooserPanel(oldPanels[0]);
       farbenCC.removeChooserPanel(oldPanels[2]);
       
   	   // create the buttons for closing and cancling the dialog 
	   okButton.addActionListener(new 
		        ActionListener() 
				{
	   					public void actionPerformed(ActionEvent e)
	   					{
	   						Color selectedColor;
	   		   	            
	   						selectedColor = farbenCC.getColor();
	   						
	   						// Farbe bernehmen .....
	   						colorTarget.setSelectedColor(selectedColor);
	   						
	   		   	   		    // Dialog unsichtbar machen
	   						showDialog(false);
	   					}
		
				} );

       // in die vertikale Box ...
       contentVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
       contentVB.add(farbenCC);
       contentVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
	   contentVB.add(buttonHB);
	   contentVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
  
	   // FARBEN, FONTS UND TEXTE SETZEN ...
 	   updateLanguageColorFonts();
 	 
		// TRACE OUTPUT
		if (LeviModule.isTraceActive()) System.out.println("END leviColorChooseDialog.leviColorDialog");

   }
   
   // MUTATOR METHODES ...
   
   /*
    * Methode:		showDialog
    * 
    * Description:	Setzt die aktuellen Farben aus der Konfiguration bevor der Dialog angezeigt bzw.
    * 				wieder versteckt wird.
    *    
    * Parameters:    
    * 
    * Date: 		 11.04.2006
    * @author        sirocic
    */	
   public void showDialog(boolean visible, Color selectedColor, JLabel propertyLabel, LeviColorTargetInterface colorTarget) {
	   String aktDialogTitel;
	   
 	  // TRACE OUTPUT
 	  if (LeviModule.isTraceActive()) System.out.println("CALL leviColorChooseDialog.showDialog:");
 	  
 	  if (visible) {
 		  // Dialog wird sichtbar gemacht. 
 		  if (propertyLabel != null) {
 			 targetPropertyLabel = propertyLabel;
 			 aktDialogTitel = myFrame.getLearningModule().getRessources().getResourceByKey("dlg.color.choose.title") + " - " + propertyLabel.getText();
 			 setTitle(aktDialogTitel);
 		  }
 		  
          if (selectedColor != null)
        	  farbenCC.setColor(selectedColor);
          
          myFrame.getLearningModule().setDialogColor(farbenCC);
       
          if (colorTarget != null)
        	  this.colorTarget = colorTarget;
 	  }
   	   		  
 	  super.showDialog(visible);
 	  
  	  // TRACE OUTPUT
  	  if (LeviModule.isTraceActive()) System.out.println("END leviColorChooseDialog.showDialog");

   }
   
   /*
    * Methode:		fitDialog
    * 
    * Description:	Bestimmen der bentigten Panelgre fr das Anzeigen des vollstndigen Dialogs. 
    *    
    * Parameters:    
    * 
    * Date: 		 16.01.2007
    * @author        sirocic
    */	
   public void fitDialog() {
	  Graphics2D g2;
  	  int maxWidth, maxHeight;
  	  
  	  // TRACE OUTPUT
  	  if (LeviModule.isTraceActive()) System.out.println("CALL leviColorChooseDialog.fitDialog:");
  	
  	  // init
  	  g2 = (Graphics2D) getGraphics();
  	  maxWidth = 0;
	  maxHeight =0;
	  
	  if (g2 != null)  {
		  // Bentigte Zeilenbreite und -hhe bestimmen anhand der Textausmasse bei aktuelle
	      // eingestellten Font ...
		  g2.setFont(getFont());

	  	  // BENTIGTE CONTENT-PANEL-BREITE BESTIMMEN ....
	  	  // Breiteste Zeile aus Schriftlabel (Font-Label), Preview-Panel und Fontauswahl-Buttone bestimmen.
	  	  maxWidth =  farbenCC.getWidth();
		  
	  	  // BENTIGTE CONTENT-PANEL-HOEHE BESTIMMEN ....
	  	  maxHeight = LeviFrame.LINEOFFSET + farbenCC.getHeight() ;
	  	  
	  	  //Panelgre aktualisieren ...
	  	  getRootPane().setSize(maxWidth,maxHeight);
	  	  getRootPane().setMaximumSize(new Dimension(maxWidth,maxHeight));
	  	  getRootPane().setPreferredSize(new Dimension(maxWidth,maxHeight));
	  }
	  
	  super.fitDialog();
	  
  	  // TRACE OUTPUT
  	  if (LeviModule.isTraceActive()) System.out.println("END leviColorChooseDialog.fitDialog");
  	
   }
   
   /*
    * Methode:		setDialogFont
    * 
    * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
    * 				Werte. 
    *    
    * Parameters:   Aktuelle Font fr Labels und Textfelder.
    * 
    * Date: 		 17.01.2007
    * @author        sirocic
    */	
   protected void setDialogFont() {
	   Font menuFont;
	   AbstractColorChooserPanel[] oldPanels;
	   Color color;
	   
	   menuFont = (Font)myFrame.getLearningModule().getConfiguration().getSettingByKey("FontMenu");

	   super.setDialogFont("FontMenu");
	   
	   // aktuell eingestellte Farbe merken.
	   color = farbenCC.getColor();
	   // Das Farbauswahl-Panel muss neu erzeugt werden, damit die Sprachnderungen wirksam werden.
	   // Vorher wurden die entsprechenden Textbauteile im UIManager aktualisiert. 
	   // (siehe leviModule.updateUIManager)
       farbenCC = new JColorChooser();
       farbenCC.setFont(menuFont);
       oldPanels = farbenCC.getChooserPanels();
       farbenCC.removeChooserPanel(oldPanels[0]);
       farbenCC.removeChooserPanel(oldPanels[2]);  
       farbenCC.setColor(color);
       // Vertikale Box neuaufbauen damit das neue Farbauswahl-Panel auch sichtbar wird.
       contentVB.removeAll();
       contentVB.add(Box.createVerticalStrut(5));
       contentVB.add(farbenCC);
	   contentVB.add(Box.createVerticalStrut(15));
	   contentVB.add(buttonHB);
  	   contentVB.add(Box.createVerticalStrut(5));
  	   validate();
  	   
   }
   
   /*
    * Methode:		setDialogTexte
    * 
    * Description:	Setz alle Text-Bausteine des Dialoges neu. Wichtig fr Sprachwechsel.
    *    
    * Parameters:   
    * 
    * Date: 		 26.01.2007
    * @author        sirocic
    */	
   protected void setDialogTexte() {
	   AbstractColorChooserPanel[] oldPanels;
	   Color color;
	   String title;
	     
	   if (targetPropertyLabel != null) 
		   title=myFrame.getLearningModule().getRessources().getResourceByKey("dlg.color.choose.title") + " - " + targetPropertyLabel.getText();
	   else
		   title=myFrame.getLearningModule().getRessources().getResourceByKey("dlg.color.choose.title");

	   super.setDialogTexte(title);
	 
	   // aktuell eingestellte Farbe merken.
	   color = farbenCC.getColor();
	  // Das Farbauswahl-Panel muss neu erzeugt werden, damit die Sprachnderungen wirksam werden.
	  // Vorher wurden die entsprechenden Textbauteile im UIManager aktualisiert. 
	  // (siehe leviModule.updateUIManager)
      farbenCC = new JColorChooser();
      oldPanels = farbenCC.getChooserPanels();
      farbenCC.removeChooserPanel(oldPanels[0]);
      farbenCC.removeChooserPanel(oldPanels[2]);  
      farbenCC.setColor(color);
      // Vertikale Box neuaufbauen damit das neue Farbauswahl-Panel auch sichtbar wird.
      contentVB.removeAll();
      contentVB.add(Box.createVerticalStrut(5));
      contentVB.add(farbenCC);
	  contentVB.add(Box.createVerticalStrut(15));
	  contentVB.add(buttonHB);
  	  contentVB.add(Box.createVerticalStrut(5));
  	  validate();
  	  
   }
   
   /*
    * Methode:		setDialogColors
    * 
    * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
    * 				Werte. 
    *    
    * Parameters:   Aktuelle Farbwerte fr Hintergrund und Vordergrund.
    * 
    * Date: 		 17.01.2007
    * @author        sirocic
    */	
   protected void setDialogColors() {
	   Color buttonBackgroundColor;
	   Color backgroundColor,  foregroundColor;
		 
	   if (myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentBackground")!=null)
	   {
		   backgroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentBackground");
		   foregroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentForeground");
	   }
	   else {
		   backgroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorModuleBackground");
		   foregroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorModuleForeground");
	   }
	   buttonBackgroundColor  = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorButtonBackground");
	 	  
	   super.setDialogColors();
	 
       farbenCC.setOpaque(true);
       farbenCC.setBackground(backgroundColor);
       farbenCC.setForeground(foregroundColor);
 
       myFrame.getLearningModule().setDialogColor(farbenCC);
      
	 	  
   }
   /*
    * Methode:		updateLanguageColorFonts
    * 
    * Description:	Setzt die aktuellen Farben aus der Konfigurationbzw. und Texte aus den Ressourcen.
    * 				Wird aufgerufen in erster Linie nach einem dynamischen Sprach- oder Farbwechsel.
    *    
    * Date: 		 16.03.2007
    * @author        sirocic
    */	
   public void updateLanguageColorFonts() {
      
 	  // TRACE OUTPUT
 	  if (LeviModule.isTraceActive()) System.out.println("CALL leviColorChooseDialog.updateLanguageColorFonts:");
 	  
 	  // SUBKOMPONENTE UPDATEN...
  
      //    SPRACHWECHSEL ...
      super.updateLanguageColorFonts();
   	  
  	  // TRACE OUTPUT
  	  if (LeviModule.isTraceActive()) System.out.println("END leviColorChooseDialog.updateLanguageColorFonts");

   }

}
