/* leviColorChooserPanel.java
 * 
 * Created on 21.03.2007
 * @author sirocic
 */
package learningVisualizations.gui;

import java.awt.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.colorchooser.*;
import javax.accessibility.*;

import learningVisualizations.LeviModule;




/**
 * @author sirocic
 *
 * Class: leviSwatchChooserPanel
 *
 * Description: Entspricht im wesentlichen der Klasse DefaultPreviewPanel. Da aber bei der
 * 				Farbumstellung von levi die Hintergrundfarbe im der ursprnglichen Swingklasse nicht umgesetzt
 * 				wurden, wurde diese Klasse in letzter Konsequenz neu geschrieben. 
 *
 * Date:	21.03.2007
 */
public class LeviColorChooserPanel extends JPanel implements Accessible {

	public static final String      SELECTION_MODEL_PROPERTY = "selectionModel";
    public static final String      PREVIEW_PANEL_PROPERTY = "previewPanel";
    public static final String      CHOOSER_PANELS_PROPERTY = "chooserPanels";

    private ColorSelectionModel selectionModel;
    private LeviPreviewPanel previewPanel;
    private LeviSwatchChooserPanel chooserPanel;
    private JPanel borderPanel;
    private boolean dragEnabled;
    
    private LeviModule module;

    /*
     * Methode:		Contructors
     * 
     * Description:	
     * Date: 		 21.03.2007
     * @author        sirocic
     */
    public LeviColorChooserPanel(LeviModule module) {
        this(module, Color.white);
    }
    public LeviColorChooserPanel(LeviModule module, Color initialColor) {
    	this(module, new DefaultColorSelectionModel(initialColor) );
    }
    public LeviColorChooserPanel(LeviModule module, ColorSelectionModel scenario) {
    	super();
    	
    	this.module=module;
    	selectionModel = scenario;
        dragEnabled = false;
        
        setLayout(new BorderLayout());
        previewPanel = new LeviPreviewPanel(module);
        chooserPanel = new LeviSwatchChooserPanel(module, selectionModel, previewPanel);
        
        Border targetEtchedB = BorderFactory.createEtchedBorder();
        Border targetTitledB = BorderFactory.createTitledBorder(targetEtchedB,
				  UIManager.getString("ColorChooser.previewText"));
        
        borderPanel = new JPanel();
        borderPanel.setBorder(targetTitledB);
        borderPanel.add(previewPanel);
		
        // 	 FARBWECHSEL ...
 	   setPanelColors();

 	   // FONTWECHSEL
 	   setPanelFont();
 	  
 	   // SPRACHWECHSEL ...
 	   setPanelTexte();
 	   
        add(chooserPanel, BorderLayout.CENTER);
        add(borderPanel, BorderLayout.SOUTH);
        
    }

    public Color getColor() {
        return selectionModel.getSelectedColor();
    }

    public void setColor(Color color) {
        selectionModel.setSelectedColor(color);

    }

    public void setColor(int r, int g, int b) {
        setColor(new Color(r,g,b));
    }


    public void setColor(int c) {
        setColor((c >> 16) & 0xFF, (c >> 8) & 0xFF, c & 0xFF);
    }


    public void setDragEnabled(boolean b) {
        if (b && GraphicsEnvironment.isHeadless()) {
            throw new HeadlessException();
        }
        dragEnabled = b;
    }

    public boolean getDragEnabled() {
    	return dragEnabled;
    }

    public void setPreviewPanel(LeviPreviewPanel preview) {

        if (previewPanel != preview) {
        LeviPreviewPanel oldPreview = previewPanel;
        previewPanel = preview;
            firePropertyChange(JColorChooser.PREVIEW_PANEL_PROPERTY, oldPreview, preview);
        }
    }

    public JComponent getPreviewPanel() {
        return previewPanel;
    }

    public void setChooserPanel( LeviSwatchChooserPanel panel) {
       
        chooserPanel = panel;
        firePropertyChange(CHOOSER_PANELS_PROPERTY, chooserPanel, panel);
    }

    public LeviSwatchChooserPanel getChooserPanel() {
        return chooserPanel;
    }

    public ColorSelectionModel getSelectionScenario() {
        return selectionModel;
    }

    public void setSelectionScenario(ColorSelectionModel newModel ) {
        ColorSelectionModel oldModel = selectionModel;
        selectionModel = newModel;
        firePropertyChange(JColorChooser.SELECTION_MODEL_PROPERTY, oldModel, newModel);
    }

    protected String paramString() {
    	StringBuffer chooserPanelsString = new StringBuffer("");

        chooserPanelsString.append("[" + chooserPanel.toString()
                       + "]");
        String previewPanelString = (previewPanel != null ?
                     previewPanel.toString() : "");

        return super.paramString() +
        ",chooserPanels=" + chooserPanelsString.toString() +
        ",previewPanel=" + previewPanelString;
    }

    protected AccessibleContext accessibleContext = null;

    public AccessibleContext getAccessibleContext() {
        if (accessibleContext == null) {
            accessibleContext = new AccessibleJColorChooser();
        }
        return accessibleContext;
    }

    /*
     * Methode:		setDialogFont
     * 
     * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
     * 				Werte. 
     *    
     * Parameters:   Aktuelle Font fr Labels und Textfelder.
     * 
     * Date: 		 23.04.2007
     * @author        sirocic
     */	
    private void setPanelFont() {
 	   Font labelFont;
 	   
 	   labelFont = (Font)module.getConfiguration().getSettingByKey("FontLabel");
 	   
 	   setFont(labelFont);
 	   	  
    }
    
    /*
     * Methode:		setDialogTexte
     * 
     * Description:	Setz alle Text-Bausteine des Dialoges neu. Wichtig fr Sprachwechsel.
     *    
     * Parameters:   
     * 
     * Date: 		 23.04.2007
     * @author        sirocic
     */	
    private void setPanelTexte() {
    	
    	Border targetEtchedB = BorderFactory.createEtchedBorder();
        Border targetTitledB = BorderFactory.createTitledBorder(targetEtchedB,
				  UIManager.getString("ColorChooser.previewText"));
        borderPanel.setBorder(targetTitledB);
	   	
    }
    
    /*
     * Methode:		setDialogColors
     * 
     * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
     * 				Werte. 
     *    
     * Parameters:   Aktuelle Farbwerte fr Hintergrund und Vordergrund.
     * 
     * Date: 		 23.04.2007
     * @author        sirocic
     */	
    private void setPanelColors() {
 	   Color backgroundColor,  foregroundColor;
 	   
 	  if (module.getConfiguration().getSettingByKey("ColorComponentBackground")!=null)
	   {
		   backgroundColor = (Color)module.getConfiguration().getSettingByKey("ColorComponentBackground");
		   foregroundColor = (Color)module.getConfiguration().getSettingByKey("ColorComponentForeground");
	   }
 	   else {
 		    backgroundColor = (Color)module.getConfiguration().getSettingByKey("ColorModuleBackground");
 	    	foregroundColor = (Color)module.getConfiguration().getSettingByKey("ColorModuleForeground");
 	   }
 	 	  
 	   // 	 Hintergrund und Zeichenfarbe auf aktuelle Werte setzen
       setBackground(backgroundColor);
 	   setForeground(foregroundColor);
 	   borderPanel.setBackground(backgroundColor);
       borderPanel.setForeground(foregroundColor);
      
    }
    
    /*
     * Methode:		updateColorLanguageFonts
     * 
     * Description:	Setzt die aktuellen Farben aus der Konfigurationbzw. und Texte aus den Ressourcen.
     * 				Wird aufgerufen in erster Linie nach einem dynamischen Sprach- oder Farbwechsel.
     *    
     * Date: 		16.03.2007
     * @author      sirocic
     */	
    public void updateColorLanguageFonts() {
     
        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("CALL leviColorChooserPanel.updateColorLanguageFonts:");

        // SUBKOMPONENTE UPDATEN...
        chooserPanel.updateColorLanguageFonts();
        previewPanel.updateColorLanguageFonts();
        

        // 	 FARBWECHSEL ...
 	   setPanelColors();

 	   // FONTWECHSEL
 	   setPanelFont();
 	  
 	   // SPRACHWECHSEL ...
 	   setPanelTexte();
        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("END leviColorChooserPanel.updateColorLanguageFonts:");

    }
    protected class AccessibleJColorChooser extends AccessibleJComponent {

        public AccessibleRole getAccessibleRole() {
            return AccessibleRole.COLOR_CHOOSER;
        }

    } // inner class AccessibleJColorChooser
}



