/* leviColorDialog.java
 * 
 * Created on 07.09.2005
 * @author sirocic
 */
package learningVisualizations.gui;

import java.awt.*;
import java.awt.event.*;
import java.awt.font.FontRenderContext;
import java.awt.geom.Rectangle2D;
import java.util.*;

import javax.swing.*;

import learningVisualizations.LeviModule;



/**
 * @author sirocic
 *
 * Class: leviColorDialog
 * 
 * Description: The class "leviColorDialog" of the general color configuration dialog. 
 * 				
 * Date:	07.09.2005
 */
public class LeviColorDialog extends LeviDialog implements LeviColorTargetInterface {
	
    public static final int PREVIEWSIZE = 15;

    private LeviColorTargetInterface myself=this;
	private LeviChooseButton selectedButton;
	private Enumeration keys; // keys of all properties 
	private String key;  // current key
	
	private ArrayList<LeviChooseButton> colorButtons;
	private ArrayList<JPanel> colorButtonPanels;
	private ArrayList<JLabel> colorLabels;
	private ArrayList<JPanel> colorLabelPanels;
	private ArrayList<JPanel> previews;
	private ArrayList<JPanel> previewPanels;
	private Box colorsHB, colorLabelsVB, previewPanelsVB, colorButtonsVB;
	JPanel preview;
	
	
  /*
   * Methode:		Contructors
   * 
   * Description:	Displays the current color setting from the configuration.
   *				Provides the user with the possiblity to change the color settings
   *                for special elements like module background, process foreground etc.
   *                The selected colors are store by the background color of the preview panels.
   *                The configuration will be updated if the confirm button was selected.
   * 
   * Date: 		    07.09.2005
   * @author        sirocic
   */
	public LeviColorDialog() {
	   super();
   	   // INIT
   	   this.selectedButton=null;
   	   this.colorLabels = new ArrayList<JLabel>();  
   	   this.colorLabelPanels = new ArrayList<JPanel>(); 
	   this.colorButtons = new ArrayList<LeviChooseButton>();
	   this.colorButtonPanels = new ArrayList<JPanel>();
	   this.previewPanels = new ArrayList<JPanel>();
	   this.previews = new ArrayList<JPanel>();
	   this.colorsHB= Box.createHorizontalBox();
	   this.colorLabelsVB = Box.createVerticalBox(); 
	   this.previewPanelsVB = Box.createVerticalBox(); 
	   this.colorButtonsVB = Box.createVerticalBox(); 
	   this.key = "";
	   this.keys = null;

  	   // SET DIALOG PROPERTIES
   	   setSize(WIDTH, HEIGHT);
	}
   public LeviColorDialog(LeviFrame frame) {
   	   super(frame, "dlg.color.title");
   	   
   	   // Lokale Variablen
   	   String labelStr;
   	   JLabel label;
   	   JPanel panel, buttonP, previewPanel;
   	   LeviChooseButton button;   
   	   int i;
   	   Color colorSetting;
   	   String keyNeu;
   	   
   	   
	   // TRACE OUTPUT
	   if (LeviModule.isTraceActive()) System.out.println("CALL leviColorDialog.leviColorDialog:");

  	   // INIT
	   this.colorLabels = new ArrayList<JLabel>();  
   	   this.colorLabelPanels = new ArrayList<JPanel>(); 
	   this.colorButtons = new ArrayList<LeviChooseButton>();
	   this.colorButtonPanels = new ArrayList<JPanel>();
	   this.previewPanels = new ArrayList<JPanel>();
	   this.previews = new ArrayList<JPanel>();
	   this.colorsHB= Box.createHorizontalBox();
	   this.colorLabelsVB = Box.createVerticalBox(); 
	   this.previewPanelsVB = Box.createVerticalBox(); 
	   this.colorButtonsVB = Box.createVerticalBox(); 
   	   
   	   // KORREKTUR ALTE BEZEICHNUNGEN DER FARBENWERTE
   	   keys=myFrame.getLearningModule().getConfiguration().getKeysOfSettings();
	   	while (keys.hasMoreElements()) {
	   		key=(String)keys.nextElement();
	   		if (key.startsWith("ColorComponent")) {
				  // Alte Bezeichnung fr die Modulfarben gefunden
				  // Aktualisieren in Konfiguration
				  keyNeu = key.replace("Component", "Module");
				  colorSetting = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey(key);
				  myFrame.getLearningModule().getConfiguration().removeSettingByKey(key);
				  myFrame.getLearningModule().getConfiguration().putSettingByKey(keyNeu, colorSetting);
			  }
	   	}
	   	
	   // FOR EACH COLOR PROBERTY ...
	   keys=myFrame.getLearningModule().getConfiguration().getKeysOfSettings();
   	   i=0;
   	   while (keys.hasMoreElements()) {
   		  key=(String)keys.nextElement();
   	   	  
   	   	  if (key.startsWith("Color")) {
   	   		  // color property was found !!!
   	   		  
   			  // TRACE OUTPUT
   			  if (LeviModule.isTraceActive()) System.out.println("	key="+key);
   			  
   	   	      // Labelelement fr die Bezeichung erzeugen und in die vertikale box fr alle Labels einfgen
   			  labelStr = myFrame.getLearningModule().getRessources().getResourceByKey("dlg.color."+key)+":";
	   	      label =new JLabel(labelStr);
	   	      panel = new JPanel();
	   	      panel.setLayout(new BorderLayout());
	   	      panel.add(label, BorderLayout.EAST);
	   	      colorLabels.add(label);
	   	      colorLabelPanels.add(panel);
	   	      colorLabelsVB.add(panel);
	   	      colorLabelsVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
 
   	   	      // Preview-Panel fr Die Farbe erzeugen und in die vertikale box fr alle Previews einfgen
   	   	      preview = new JPanel();
   	   	      if (myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentForeground")!=null)
   	   	    	  preview.setBorder(BorderFactory.createLineBorder((Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentForeground"), 1));
   	   	      else
   	   	    	  preview.setBorder(BorderFactory.createLineBorder((Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorModuleForeground"), 1));
   	   	      preview.setPreferredSize(new Dimension(PREVIEWSIZE, PREVIEWSIZE));
   	   	      preview.setMaximumSize(preview.getPreferredSize());
   	   	      preview.setMinimumSize(preview.getPreferredSize());
   	   	      preview.setBackground((Color)myFrame.getLearningModule().getConfiguration().getSettingByKey(key));
   	   	      previewPanel = new JPanel();
   	   	      previewPanel.setAlignmentX(Component.CENTER_ALIGNMENT);
   	   	      previewPanel.setAlignmentY(Component.CENTER_ALIGNMENT);
		   	  previewPanel.setPreferredSize(new Dimension(LeviButton.BUTTONSIZE, LeviButton.BUTTONSIZE));
		   	  previewPanel.setMaximumSize(previewPanel.getPreferredSize());
		   	  previewPanel.setMinimumSize(previewPanel.getPreferredSize());
		   	  previewPanel.add(preview);
   	   	      previews.add(preview);
   	   	      previewPanels.add(previewPanel);
	   	      previewPanelsVB.add(previewPanel);
	   	      previewPanelsVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
   	   	      
   	   	      // add the color select button 
   	   	      button = new LeviChooseButton("button.color", myFrame.getLearningModule(), preview, label);
   	   	      button.addActionListener(new 
   		        ActionListener() 
			    {
   					public void actionPerformed(ActionEvent e)
   					{
   						
   						// TRACE OUTPUT
   						if (LeviModule.isTraceActive()) System.out.println("CALL ActionListener.actionPerformed: source="+e.getSource().toString());
                        
   						// get the selected button
   						selectedButton= (LeviChooseButton) e.getSource();
   								
   						// show the color choose dialog
   						myFrame.getLearningModule().getColorChooseDialog().showDialog(true,
   				                selectedButton.getPreview().getBackground(), selectedButton.getPropertyLabel(), myself);
   						
   			        }
	
			   }
	          );
   	   	      buttonP = new JPanel();
	   	      buttonP.setPreferredSize(new Dimension(LeviButton.BUTTONSIZE, LeviButton.BUTTONSIZE));
	   	      buttonP.setMaximumSize(buttonP.getPreferredSize());
	   	      buttonP.setMinimumSize(buttonP.getPreferredSize());
   	   	      buttonP.setLayout(new BorderLayout());
	   	      buttonP.add(button, BorderLayout.CENTER);
	   	      colorButtons.add(button);
	   	      colorButtonPanels.add(buttonP);
	   	      colorButtonsVB.add(buttonP);
	   	      colorButtonsVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
	   	      
   	          i++;
   	   	  }
   	   	  
   	   }
   	   
   	   // Labels, Previews und Colorauswahl-Buttons in eine Zeile packen.
	   colorsHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
	   colorsHB.add(colorLabelsVB);
	   colorsHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
	   colorsHB.add(previewPanelsVB);
	   colorsHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
	   colorsHB.add(colorButtonsVB);
	   colorsHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
	      
   	   // create the buttons for closing and cancling the dialog 
	   okButton.addActionListener(new 
		        ActionListener() 
				{
	   					public void actionPerformed(ActionEvent e)
	   					{
	   						int j=0;
	   						
	   						// put the current color values into the configuration
	   			   	        keys=myFrame.getLearningModule().getConfiguration().getKeysOfSettings();
	   		   	   
	   		   	            while (keys.hasMoreElements()) {
	   		   	   	          key=(String)keys.nextElement();
	   		   	   	  
	   		   	   	          if (key.startsWith("Color")) {
	   		   	   	        	  if (key.startsWith("ColorComponent"))
	   		   	   	        		  	key = key.replace("Component", "Module");
	   		   	   	             myFrame.getLearningModule().getConfiguration().putSettingByKey(key, (Color)((JPanel)previews.get(j)).getBackground());
	   		   	   	             j++;
	   		   	   	          }
	   		   	   	          
	   		   	   	        }
	   		   	            
	   		   	            // save the configuration ...
	   		   	            myFrame.getLearningModule().getConfiguration().saveConfiguration();
	   		   	            
	   		   	            // repaint the module
	   		   	            myFrame.updateLanguageColorFonts();
	   		   	            
	   		   	   		    // color property was found !!!
	   		   	            showDialog(false);
	   					}
		
				} );
 
	   //	 jetzt die komplette vertikal Box zusammenbauen
	   contentVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
	   contentVB.add(colorsHB);
	   contentVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
	   contentVB.add(buttonHB);
	   contentVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
  
	   // FARBEN, FONTS UND TEXTE SETZEN ...
 	   updateLanguageColorFonts();
 	   
		// TRACE OUTPUT
		if (LeviModule.isTraceActive()) System.out.println("END leviColorDialog.leviColorDialog");

   }
   
   // MUTATOR METHODES ...
   
   /*
    * Methode:		showDialog
    * 
    * Description:	Setzt die aktuellen Farben aus der Konfiguration bevor der Dialog angezeigt bzw.
    * 				wieder versteckt wird.
    *    
    * Parameters:    
    * 
    * Date: 		 11.04.2006
    * @author        sirocic
    */	
   public void showDialog(boolean visible) {
      int i;
      
 	  // TRACE OUTPUT
 	  if (LeviModule.isTraceActive()) System.out.println("CALL leviColorDialog.showDialog:");
 	  
 	  if (visible) {
 		  // Dialog wird sichtbar gemacht. 
	   	  i=0;
	   	  keys=this.myFrame.getLearningModule().getConfiguration().getKeysOfSettings();
	   	  while (keys.hasMoreElements()) {
	   		  key=(String)keys.nextElement();
	   	   	  
	   	   	  if (key.startsWith("Color")) {
	   	   		  // color property was found !!!
	   	   		  if (key.startsWith("ColorComponent"))
	   	   			  key = key.replace("Component", "Module");
 			  
	   	   		  // Vorschau fr die Farbeigenschaft wieder auf den Konfigurationswert setzen!
	   	   		  ((JPanel)previews.get(i)).setBackground((Color)myFrame.getLearningModule().getConfiguration().getSettingByKey(key));
	   	   		  i++;
	   	   	  }
	   	  }

 	  }

 	  super.showDialog(visible);
 	  
  	  // TRACE OUTPUT
  	  if (LeviModule.isTraceActive()) System.out.println("END leviColorDialog.showDialog");

   }
   
   /*
    * Methode:		setDialogFont
    * 
    * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
    * 				Werte. 
    *    
    * Parameters:   Aktuelle Font fr Labels und Textfelder.
    * 
    * Date: 		 17.01.2007
    * @author        sirocic
    */	
   protected void setDialogFont() {
	   Font menuFont;
	   int i;
	   
	   menuFont = (Font)myFrame.getLearningModule().getConfiguration().getSettingByKey("FontMenu");

	   super.setDialogFont("FontMenu");
	   
	   for (i=0; i < (int)colorLabels.size(); i++) 
		   	  ((JLabel) colorLabels.get(i)).setFont(menuFont);

   }
   
   /*
    * Methode:		fitDialog
    * 
    * Description:	Bestimmen der bentigten Panelgre fr das Anzeigen des vollstndigen Dialogs. 
    *    
    * Parameters:    
    * 
    * Date: 		 18.07.2007
    * @author        sirocic
    */	
   public void fitDialog() {
	  Graphics2D g2;
  	  int maxWidth, maxHeight;
  	  int maxLabelWidth;
  	  Rectangle2D boundsLabel;
	  FontRenderContext context;
	  int i;
  	  
  	  // TRACE OUTPUT
  	  if (LeviModule.isTraceActive()) System.out.println("CALL leviColorDialog.fitDialog:");
  	
  	  // init
  	  g2 = (Graphics2D) getGraphics();
  	  maxWidth = 0;
	  maxHeight =0;
	  maxLabelWidth = 0;
	  
	  if (g2 != null)  {
		  // Bentigte Zeilenbreite und -hhe bestimmen anhand der Textausmasse bei aktuelle
	      // eingestellten Font ...
		  g2.setFont(getFont());
	      context = g2.getFontRenderContext();

	  	  // BENTIGTE CONTENT-PANEL-BREITE BESTIMMEN ....
	  	  
	  	  // Breite des breitesten Labels bestimmen.
	  	  for (i=0; i < colorLabels.size(); i++) {
		  	  boundsLabel = getFont().getStringBounds(((JLabel)colorLabels.get(i)).getText(), context);
		  	  if (maxLabelWidth < (int)boundsLabel.getWidth())
		  	  	 maxLabelWidth = (int)boundsLabel.getWidth();
	  	  }
	  	  
	  	  // daraus jetzt die maximale Breite einer Zeile bestimmen.
	  	  maxWidth = maxLabelWidth + 4*LeviFrame.ATTRIBUTOFFSET 
	  	  			+ 2*LeviButton.BUTTONSIZE;

	  	  // BENTIGTE CONTENT-PANEL-HOEHE BESTIMMEN ....
	  	  maxHeight = LeviFrame.LINEOFFSET + ((int)colorLabels.size()*LeviButton.BUTTONSIZE)
	  	        + ((int)colorLabels.size()*LeviFrame.LINEOFFSET) ;
	  	  
	  	  // Panelgre aktualisieren ...
	  	  getRootPane().setSize(maxWidth,maxHeight);
	  	  getRootPane().setMaximumSize(new Dimension(maxWidth,maxHeight));
	  	  getRootPane().setPreferredSize(new Dimension(maxWidth,maxHeight));
	  }
	  
	  super.fitDialog();
	  
  	  // TRACE OUTPUT
  	  if (LeviModule.isTraceActive()) System.out.println("END leviColorDialog.fitDialog");
  	
   }
   
   /*
    * Methode:		setDialogTexte
    * 
    * Description:	Setz alle Text-Bausteine des Dialoges neu. Wichtig fr Sprachwechsel.
    *    
    * Parameters:   
    * 
    * Date: 		 26.01.2007
    * @author        sirocic
    */	
   protected void setDialogTexte() {
	   int i;
	   String labelStr;
	   
	   super.setDialogTexte(myFrame.getLearningModule().getRessources().getResourceByKey("dlg.color.title"));
	   for (LeviChooseButton button : colorButtons){
		   button.setPanelTexte();
	   }
	   
   	   i=0;
   	   keys=this.myFrame.getLearningModule().getConfiguration().getKeysOfSettings();
   	   while (keys.hasMoreElements()) {
   		  key=(String)keys.nextElement();
   	   	  
   	   	  if (key.startsWith("Color")) {
   	   		  if (key.startsWith("ColorComponent"))
   	   			  	key = key.replace("Component", "Module");
   	   		  // color property was found !!!
   	   		  labelStr = myFrame.getLearningModule().getRessources().getResourceByKey("dlg.color."+key)+":";
   	   		  ((JLabel) colorLabels.get(i)).setText(labelStr);
   	   		  i++;
   	   	  }
   	   }
	   	  
   }
   
   /*
    * Methode:		setDialogColors
    * 
    * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
    * 				Werte. 
    *    
    * Parameters:   Aktuelle Farbwerte fr Hintergrund und Vordergrund.
    * 
    * Date: 		 17.01.2007
    * @author        sirocic
    */	
   protected void setDialogColors() {
	   Color backgroundColor,  foregroundColor, buttonBackgroundColor;
	   int i;
	   
	   if (myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentBackground")!=null){
		   backgroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentBackground");
		   foregroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentForeground");
	   }
	   else {
		   backgroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorModuleBackground");
		   foregroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorModuleForeground");
	   }
	   buttonBackgroundColor  = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorButtonBackground");
	 	  
	   super.setDialogColors();
	   
   	   for (i=0; i < (int)colorLabels.size(); i++) {  
  	      
   		  colorLabelPanels.get(i).setBackground(backgroundColor);
	   	  colorLabelPanels.get(i).setForeground(foregroundColor);
	   	  colorButtonPanels.get(i).setBackground(backgroundColor);
	   	  colorButtonPanels.get(i).setForeground(foregroundColor);
	   	  previewPanels.get(i).setForeground(foregroundColor);
	   	  previewPanels.get(i).setBackground(backgroundColor);
	   	  if (myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentForeground")!=null)
	   		  previews.get(i).setBorder(BorderFactory.createLineBorder((Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentForeground"), 1));
	   	  else
	   		  previews.get(i).setBorder(BorderFactory.createLineBorder((Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorModuleForeground"), 1));
	   	  colorLabels.get(i).setBackground(backgroundColor);
	   	  colorLabels.get(i).setForeground(foregroundColor);
	   	  colorButtons.get(i).setBackground(buttonBackgroundColor);
   	   }
	 	 
	 	  
   }
   /*
    * Methode:		updateLanguageColorFonts
    * 
    * Description:	Setzt die aktuellen Farben aus der Konfigurationbzw. und Texte aus den Ressourcen.
    * 				Wird aufgerufen in erster Linie nach einem dynamischen Sprach- oder Farbwechsel.
    *    
    * Date: 		 16.03.2007
    * @author        sirocic
    */	
   public void updateLanguageColorFonts() {
      int i;
      
 	  // TRACE OUTPUT
 	  if (LeviModule.isTraceActive()) System.out.println("CALL leviColorDialog.updateLanguageColorFonts:");
 	  
   	  // SUBKOMPONENTE UPDATEN...
      for (i=0;i < colorButtons.size(); i++)
    	  ((LeviChooseButton)colorButtons.get(i)).updateLanguageColorFonts();
      
      super.updateLanguageColorFonts();
      
  	  // TRACE OUTPUT
  	  if (LeviModule.isTraceActive()) System.out.println("END leviColorDialog.updateLanguageColorFonts");

   }
 
   public void setSelectedColor(Color color) {
	   if (selectedButton != null)
		   selectedButton.getPreview().setBackground(color);
   }

}

