/* LeviDefaultScenarioDialog.java
 *
 * Created on 07.04.2006
 * @author sirocic
 */
package learningVisualizations.gui;

import java.awt.*;
import java.awt.event.*;
import java.awt.font.FontRenderContext;
import java.awt.geom.Rectangle2D;
import java.net.*;

import javax.swing.*;

import java.io.File;

import learningVisualizations.LeviConfiguration;
import learningVisualizations.LeviModule;



/**
 * @author sirocic
 *
 * Class: LeviDefaultScenarioDialog
 *
 * Description: The class "LeviDefaultScenarioDialog" of the default scenario setting dialog.
 *
 * Date:	07.04.2006
 */
public class LeviDefaultScenarioDialog extends LeviDialog {

    public static final int MAXPATHLEN=255;
    public static final int PATHLEN=100;
    
    private JLabel textLabel;
    private JTextField defaultScenarioTF;
    private LeviButton scenarioSelectButton;
    private Box inputHB, textHB;


    /*
     * Methode:		Contructors
     *
     * Description:	Displays the current filename and path of the default scenario from the
     * 				configuration.
     *				Provides the user with the possiblity to change the default scenario.
     *                The configuration will be updated if the confirm button was selected.
     *                During the start of the learning module the default scenario will be
     *                automatically loaded into the scenarioPanel or visualizer, respectively.
     *
     * Date: 		    07.09.2005
     * @author        sirocic
     */
    public LeviDefaultScenarioDialog() {
        super();
        // INIT
        this.textLabel = new JLabel();
        this.defaultScenarioTF = new JTextField(MAXPATHLEN);
        this.defaultScenarioTF.setMaximumSize(defaultScenarioTF.getPreferredSize());
        this.scenarioSelectButton = new LeviButton();
        this.inputHB = Box.createHorizontalBox();
        this.textHB = Box.createHorizontalBox();
        
        // SET DIALOG PROPERTIES
        setSize(WIDTH, HEIGHT);
    }

    public LeviDefaultScenarioDialog(LeviFrame frame) {
        super(frame, "dlg.defscenario.title");

        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("CALL LeviDefaultScenarioDialog.constructor:");

        // INIT
        contentVB.setAlignmentX(Box.LEFT_ALIGNMENT);
        inputHB = Box.createHorizontalBox();
        inputHB.setAlignmentX(Box.LEFT_ALIGNMENT);
        textHB = Box.createHorizontalBox();
        textHB.setAlignmentX(Box.LEFT_ALIGNMENT);
        buttonHB.setAlignmentX(Box.LEFT_ALIGNMENT);

        // CREATING THE TEXT OF THE DIALOG ...
        textLabel = new JLabel(this.myFrame.getLearningModule().getRessources().getResourceByKey("dlg.defscenario.text1"));
        textHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        textHB.add(textLabel);
        
        // CREATING THE INPUT ELEMENTS FOR THE DEFAULT scenario FILE...
        // loading the icon of the file select button.
        defaultScenarioTF = new JTextField(PATHLEN);
        
        // add the scenario select button
        scenarioSelectButton = new LeviButton( "button.open", this.myFrame.getLearningModule());
        scenarioSelectButton.addActionListener(new
                ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                int result;
                File scenarioFile;

                // Show the file open dialog and read the given file name
                result=myFrame.getLearningModule().getFileOpenSaveDialog().showOpenDialog(myFrame);
                if (result == JFileChooser.APPROVE_OPTION) {
                    // File has been selected
                	scenarioFile = myFrame.getLearningModule().getFileOpenSaveDialog().getSelectedFile();
                    // Filename im Dialog anzeigen.
                    // DF 2006-06-08: Discard the path if it's the working
                    // directory. This makes config portable between systems if
                    // scenarios in the current dir are used. Better would be an
                    // abstract storage of relative pathnames only
                    StringBuffer nameTest = new StringBuffer();
                    nameTest.append(System.getProperty("user.dir"));
                    nameTest.append(System.getProperty("file.separator"));
                    // Ausgesucht File liegt im Arbeitsverzeichnis der Komponente???
                    if( scenarioFile.getPath().equals(nameTest+scenarioFile.getName())) {
                    	defaultScenarioTF.setText(scenarioFile.getName());
                    } else {
                    	if (scenarioFile.getPath().startsWith(nameTest.toString())) {
                    		defaultScenarioTF.setText(scenarioFile.getPath().substring(nameTest.length()));
                    	}
                    	else
                    		defaultScenarioTF.setText(scenarioFile.getPath());
                    }
                }
            }
        } );

        inputHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        inputHB.add(defaultScenarioTF);
        inputHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
        inputHB.add(scenarioSelectButton);
        inputHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
       
        // CREATING THE OK AND CANCEL BUTTON ...
        okButton.addActionListener(new
                ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                String fileName;
                String errorMsg, errorTitle;
                File scenarioFile;

                // Angegebenen Filename bernehmen.
                fileName =  defaultScenarioTF.getText();
                scenarioFile = new File(fileName);

                // Ist die Eingabe ein gltiger Filename ???
                if ((scenarioFile.exists()) || (fileName.length() == 0)) {
                    // Neuen Filename in die Konfiguration bernehmen.
	                    myFrame.getLearningModule().getConfiguration().putSettingByKey(
	                    		LeviConfiguration.KEYDEFSCENARIO, fileName);
	
	                    // save the configuration ...
	                    myFrame.getLearningModule().getConfiguration().saveConfiguration();
	
	                    //	Dialog schliessen!
	                    showDialog(false);

                }
                else
                {
                    // Angegebenes File existiert nicht!!!!
      	            errorMsg = String.format(myFrame.getLearningModule().getRessources().getResourceByKey("dlg.errortext.8"),fileName);
      		        errorTitle = myFrame.getLearningModule().getRessources().getResourceByKey("dlg.error.title")+ " (LeviDefaultScenarioDialog).";
      		        JOptionPane.showMessageDialog(myFrame,errorMsg, errorTitle, JOptionPane.ERROR_MESSAGE);
                }
            }
        } );

        // ADD IT IN THE VERTICAL CONTENT BOX
        contentVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
        contentVB.add(textHB);
        contentVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
        contentVB.add(inputHB);
        contentVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
        contentVB.add(buttonHB);
        contentVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));

        //  FARBEN, FONTS UND TEXTE SETZEN ...
  	    updateLanguageColorFonts();
  	   
        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("END LeviDefaultScenarioDialog.leviColorDialog");

    }

    // MUTATOR METHODES ...

    /*
     * Methode:		showDialog
     *
     * Description:	Setzt die aktuellen Farben aus der Konfiguration bevor der Dialog angezeigt bzw.
     * 				wieder versteckt wird.
     *
     * Parameters:
     *
     * Date: 		 11.04.2006
     * @author        sirocic
     */
    public void showDialog(boolean visible) {
        String scenarioName;

        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("CALL LeviDefaultScenarioDialog.showDialog:");

        if (visible) {
            // Dialog wird sichtbar gemacht. 

            // das Texteingabefeld fr den Default-Szenarionamen auf den aktuellen
            // Konfigurationwert setzen.
        	if (myFrame.getLearningModule().getConfiguration().getSettingByKey(LeviConfiguration.KEYDEFSCENARIO) != null) {
        		scenarioName = myFrame.getLearningModule().getConfiguration().getSettingByKey(LeviConfiguration.KEYDEFSCENARIO).toString();
	            defaultScenarioTF.setText(scenarioName);
        	}
        	else if (myFrame.getLearningModule().getConfiguration().getSettingByKey(LeviConfiguration.KEYDEFMODEL) != null) {
        		scenarioName = myFrame.getLearningModule().getConfiguration().getSettingByKey(LeviConfiguration.KEYDEFMODEL).toString();
	            defaultScenarioTF.setText(scenarioName);
        	}
            
            validate();
            
        }

        // jetzt anzeigen ...
        super.showDialog(visible);

        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("END LeviDefaultScenarioDialog.showDialog");

    }
    
    /*
     * Methode:		fitDialog
     * 
     * Description:	Bestimmen der bentigten Panelgre fr das Anzeigen des vollstndigen Dialogs. 
     *    
     * Parameters:    
     * 
     * Date: 		 29.06.2007
     * @author        sirocic
     */	
    public void fitDialog() {
 	  Graphics2D g2;
   	  int maxWidth, tmp, maxHeight;
   	  Rectangle2D boundsLabel;
 	  FontRenderContext context;
 	  int textfieldHeight;
   	  
   	  // TRACE OUTPUT
   	  if (LeviModule.isTraceActive()) System.out.println("CALL LeviDefaultScenarioDialog.fitDialog:");
   	
   	  // init
   	  g2 = (Graphics2D) getGraphics();
   	  maxWidth = 0;
   	  tmp = 0;
 	  maxHeight =0;
 	  
 	  if (g2 != null)  {
 		  // Bentigte Zeilenbreite und -hhe bestimmen anhand der Textausmasse bei aktuelle
 	      // eingestellten Font ...
 		  g2.setFont(getFont());
 	      context = g2.getFontRenderContext();

 	      // Eingabetextfeld an aktuelle Fontsize und Textinhalt anpassen ...
 	      boundsLabel = getFont().getStringBounds(textLabel.getText(), context);
 	      textfieldHeight=(int)boundsLabel.getHeight()+LeviFrame.TFOFFSET;
 	      defaultScenarioTF.setSize(new Dimension((int)boundsLabel.getWidth(),textfieldHeight));
          defaultScenarioTF.setPreferredSize(defaultScenarioTF.getSize());
          defaultScenarioTF.setMaximumSize(defaultScenarioTF.getSize());
          defaultScenarioTF.setMinimumSize(defaultScenarioTF.getSize());
          
         
 	  	  // BENTIGTE DIALOG-BREITE BESTIMMEN ....
 	  	  // Breiteste Zeile  bestimmen.
          maxWidth = (int)boundsLabel.getWidth() + scenarioSelectButton.getWidth() + 4*LeviFrame.ATTRIBUTOFFSET;
          
 	  	  // BENTIGTE DIALOG-HHE BESTIMMEN ....
 	  	  maxHeight = 3*LeviFrame.LINEOFFSET + (int)boundsLabel.getHeight(); 
 	  	  if (scenarioSelectButton.getHeight() > defaultScenarioTF.getHeight())
			  tmp = scenarioSelectButton.getHeight();
		  else
			 tmp = defaultScenarioTF.getHeight();
 	  	  maxHeight = maxHeight + tmp ; 
 	  	
	  	  // Panelgre aktualisieren ...
	  	  getRootPane().setSize(maxWidth,maxHeight);
	  	  getRootPane().setMaximumSize(new Dimension(maxWidth,maxHeight));
	  	  getRootPane().setPreferredSize(new Dimension(maxWidth,maxHeight));
 	  }
 	  
	  super.fitDialog();
	 
   	  // TRACE OUTPUT
   	  if (LeviModule.isTraceActive()) System.out.println("END LeviDefaultScenarioDialog.fitDialog");
   	
    }
    
    /*
     * Methode:		setDialogFont
     * 
     * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
     * 				Werte. 
     *    
     * Parameters:   Aktuelle Font fr Labels und Textfelder.
     * 
     * Date: 		 17.01.2007
     * @author        sirocic
     */	
    protected void setDialogFont() {
 	   Font menuFont;
 	   
 	   menuFont = (Font)myFrame.getLearningModule().getConfiguration().getSettingByKey("FontMenu");
 	   
 	   super.setDialogFont("FontMenu");
 	  
 	   textLabel.setFont(menuFont);
 	   defaultScenarioTF.setFont(menuFont);

    }
    
    /*
     * Methode:		setDialogTexte
     * 
     * Description:	Setz alle Text-Bausteine des Dialoges neu. Wichtig fr Sprachwechsel.
     *    
     * Parameters:   
     * 
     * Date: 		 26.01.2007
     * @author        sirocic
     */	
    protected void setDialogTexte() {
 	   
 	   super.setDialogTexte(myFrame.getLearningModule().getRessources().getResourceByKey("dlg.defscenario.title"));
 	   
 	   textLabel.setText(myFrame.getLearningModule().getRessources().getResourceByKey("dlg.defscenario.text1"));
      
    }
    
    /*
     * Methode:		setDialogColors
     * 
     * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
     * 				Werte. 
     *    
     * Parameters:   Aktuelle Farbwerte fr Hintergrund und Vordergrund.
     * 
     * Date: 		 17.01.2007
     * @author        sirocic
     */	
    protected void setDialogColors() {
 	   Color backgroundColor,  foregroundColor, buttonBackgroundColor;
 	   
 	   if (myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentBackground")!=null) {
	 	   backgroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentBackground");
	 	   foregroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentForeground");
 	   }
 	   else {
 		  backgroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorModuleBackground");
 	 	  foregroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorModuleForeground");
 	   }
 	   buttonBackgroundColor  = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorButtonBackground");
 	 	  
	   super.setDialogColors();
	   
	   defaultScenarioTF.setBackground(backgroundColor);
	   defaultScenarioTF.setForeground(foregroundColor);
	   textLabel.setForeground(foregroundColor);
	   scenarioSelectButton.setBackground(buttonBackgroundColor);

    }
    
    /*
     * Methode:		updateLanguageColorFonts
     * 
     * Description:	Setzt die aktuellen Farben aus der Konfigurationbzw. und Texte aus den Ressourcen.
     * 				Wird aufgerufen in erster Linie nach einem dynamischen Sprach- oder Farbwechsel.
     *    
     * Date: 		 16.03.2007
     * @author        sirocic
     */	
    public void updateLanguageColorFonts() {
      
        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("CALL LeviDefaultScenarioDialog.updateLanguageColorFonts:");

        //      SUBKOMPONENTE UPDATEN...
        scenarioSelectButton.updateLanguageColorFonts();
        
        super.updateLanguageColorFonts();
     	  
        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("END LeviDefaultScenarioDialog.updateLanguageColorFonts");

    }

 



}

