/* leviDialog.java
 * 
 * Created on 18.10.2007
 * @author sirocic
 */
package learningVisualizations.gui;

import java.awt.*;

import javax.swing.*;

import learningVisualizations.LeviModule;
import learningVisualizations.gui.actions.LeviActionCancelDialog;





/**
 * @author sirocic
 *
 * Class: leviDialog
 * 
 * Description: Basisklasse fr alle Dialoge. Implementiert einheitliche Dialogelemente. 
 * 				
 * Date:	18.10.2007
 */
public class LeviDialog extends JDialog {

	static final long serialVersionUID=1;
	
	public static final int WIDTH = 400;
	public static final int HEIGHT = 400;
	public static final int OFFSET = 40;

    public static final int LOCATIONX = 200;
    public static final int LOCATIONY = 200;
    
    protected LeviDialog myself;
    protected LeviFrame myFrame; // the parent frame; connection to the learning module
    protected Container contentPane;
    protected Box contentBox;
    protected JScrollPane contentPaneSP;
	private LeviActionCancelDialog cancelAction;
	protected LeviButton okButton, cancelButton;
	protected Box contentVB, contentHB, buttonHB;
	int locationX, locationY;
	String title, dlgTitleKey, dialogFontKey;
	
	  /*
	   * Methode:		Contructors
	   * 
	   * Description:	Erzeugt die allgemeinen Dialogelemente.
	   * 
	   * Date: 		    18.10.2007
	   * @author        sirocic
	   */
		public LeviDialog() {
		   super();
	   	   // INIT
	   	   this.myFrame= null;
	   	   myself = this;
	   	   this.contentPane=getContentPane(); 
	   	   this.contentPaneSP=new JScrollPane();
		   this.contentVB = Box.createVerticalBox(); 
		   this.contentHB = Box.createHorizontalBox(); 
		   this.buttonHB = Box.createHorizontalBox(); 
		   this.cancelButton = new LeviButton(); 
		   this.okButton = new LeviButton();
		   this.cancelAction=null;
		   this.locationX= LOCATIONX;
	       this.locationY= LOCATIONY;
		   
	  	   // SET DIALOG PROPERTIES
	   	   setSize(WIDTH, HEIGHT);
		}
	   public LeviDialog(LeviFrame frame, String dlgTitleKey) {
	   	   super((JFrame) frame, frame.getLearningModule().getRessources().getResourceByKey(dlgTitleKey));
	   	   
		   // TRACE OUTPUT
		   if (LeviModule.isTraceActive()) System.out.println("CALL leviDialog.leviDialog:");

	  	   // INIT
	   	   myFrame= frame;
           myself = this;
           this.dlgTitleKey=dlgTitleKey;
	   	   locationX= LOCATIONX;
	       locationY= LOCATIONY;
	       contentPane=getContentPane(); 
	   	   contentPaneSP=new JScrollPane(contentPane);
	   	   contentPaneSP.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
	   	   contentPaneSP.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_NEVER);
		   contentVB = Box.createVerticalBox(); 
		   contentHB = Box.createHorizontalBox();
		   buttonHB = Box.createHorizontalBox();
		   
	   	   // create the buttons for closing and cancling the dialog 
		   okButton = new LeviButton("button.ok", myFrame.getLearningModule());
		   cancelAction = new LeviActionCancelDialog(this);
		   cancelButton = new LeviButton("button.cancel", myFrame.getLearningModule(), cancelAction);
		   
		   // in eine horizontale Box ...
		   buttonHB = Box.createHorizontalBox();
		   buttonHB.add(Box.createHorizontalGlue());
		   buttonHB.add(okButton);
		   buttonHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
		   buttonHB.add(cancelButton);
		   buttonHB.add(Box.createHorizontalGlue());
		   
	   	   // add to the content panel of the dialog
		   contentBox = contentVB;
	   	   contentPane.setLayout(new BorderLayout());
	   	   contentPane.add(contentBox, BorderLayout.CENTER);
	   	   setContentPane(contentPaneSP);
	    
			// TRACE OUTPUT
			if (LeviModule.isTraceActive()) System.out.println("END leviDialog.leviDialog");

	   }
	   
	   // MUTATOR METHODES ...
	   
	   /*
	    * Methode:		showDialog
	    * 
	    * Description:	Setzt die aktuellen Farben aus der Konfiguration bevor der Dialog angezeigt bzw.
	    * 				wieder versteckt wird.
	    *    
	    * Parameters:    
	    * 
	    * Date: 		 22.10.2007
	    * @author        sirocic
	    */	
	   public void showDialog(boolean visible) {
	      int i;
	      
	 	  // TRACE OUTPUT
	 	  if (LeviModule.isTraceActive()) System.out.println("CALL leviDialog.showDialog:");
	 	  
	 	  if (visible) {
	 		  
		   	 setLocation(locationX, locationY);
		   	 
		   	 // Dialog an anzuzeigende Colors anpassen
		 	  pack();
		 	  fitDialog();
	 	  }
	   	   		  
	 	  // ... so jetzt den Dialog anzeigen bzw. verstecken.
	   	  setVisible(visible);
	   	  
	  	  // TRACE OUTPUT
	  	  if (LeviModule.isTraceActive()) System.out.println("END leviDialog.showDialog");

	   }
	   
	   /*
	    * Methode:		setDialogFont
	    * 
	    * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
	    * 				Werte. 
	    *    
	    * Parameters:   Aktuelle Font fr Labels und Textfelder.
	    * 
	    * Date: 		 22.10.2007
	    * @author        sirocic
	    */	
	   protected void setDialogFont(String dialogFontKey) {
		   Font dialogFont;
		   
		   dialogFont = (Font)myFrame.getLearningModule().getConfiguration().getSettingByKey(dialogFontKey);
		   
		   this.dialogFontKey=dialogFontKey;
		   
		   setFont(dialogFont);

	   }
	   protected void setDialogFont() {
		   Font menuFont;
		   
		   menuFont = (Font)myFrame.getLearningModule().getConfiguration().getSettingByKey("FontMenu");
		   
		   setFont(menuFont);
	   }
	   
	   /*
	    * Methode:		fitDialog
	    * 
	    * Description:	Bestimmen der bentigten Panelgre fr das Anzeigen des vollstndigen Dialogs. 
	    *    
	    * Parameters:    
	    * 
	    * Date: 		 22.10.2007
	    * @author        sirocic
	    */	
	   public void fitDialog() {
		  Graphics2D g2;
	  	  int maxWidth, maxHeight;
	  	  
	  	  // TRACE OUTPUT
	  	  if (LeviModule.isTraceActive()) System.out.println("CALL leviDialog.fitDialog:");
	  	
	  	  // init
	  	  g2 = (Graphics2D) getGraphics();
	  	  maxWidth = 0;
		  maxHeight =0;
		  
		  if (g2 != null)  {
			  // Bentigte Zeilenbreite und -hhe bestimmen anhand der Textausmasse bei aktuelle
		      // eingestellten Font ...
			  g2.setFont(getFont());

			  //  explizit setzen, da sonst evtl. ButtenBox viel zu lang sein kann???!!!
	          buttonHB.setSize(new Dimension(getRootPane().getWidth(),okButton.getHeight()));
	          buttonHB.setPreferredSize(buttonHB.getSize());
	          buttonHB.setMaximumSize(buttonHB.getSize());
	          buttonHB.setMinimumSize(buttonHB.getSize());
	          
		  	  // BENTIGTE DIALOG-BREITE BESTIMMEN ....
		  	  // Breiteste Zeile aus Schriftlabel (Font-Label), Preview-Panel und Fontauswahl-Buttone bestimmen.
		  	  maxWidth = getInsets().left ;

		  	  // daraus jetzt die maximale Breite einer Zeile bestimmen.
		  	  maxWidth = maxWidth + getRootPane().getWidth();
		  	  
			  // noch den Abstand zum rechten Rahmen addieren. 
		  	  maxWidth = maxWidth + getInsets().right;
		  	  
		  	  // BENTIGTE DIALOG-HHE BESTIMMEN ....
		  	  maxHeight = getInsets().top ; 
		  	  maxHeight = maxHeight + getRootPane().getHeight();
		  	  maxHeight = maxHeight + okButton.getHeight() + 2*LeviFrame.LINEOFFSET;
		  	  maxHeight = maxHeight + getInsets().bottom;
		  	
		  	  // Einfach Dialoggre setzen ...
		  	  setSize(maxWidth,maxHeight);
		  	  setMaximumSize(new Dimension(maxWidth,maxHeight));
		  	  setPreferredSize(new Dimension(maxWidth,maxHeight));
		  	  
		  	  // Wenn der Dialog gleich gro ist mit dem Componentenfenster, die Scrollbar aktivieren.
		  	  if ((maxHeight >= (int)myFrame.getNormalSize().getHeight() - (int)myFrame.getNormalSize().getHeight()/5)
		  			  || (maxWidth >= (int)myFrame.getNormalSize().getWidth() - (int)myFrame.getNormalSize().getWidth()/5)) {
		  		  contentPaneSP.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		  	  	  contentPaneSP.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		  	  }
		  	  else {
			  	  // Scrollbar wieder ausschalten.
			  	  contentPaneSP.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_NEVER);
			  	  contentPaneSP.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		  	  }
		  	  
		  }
		  
		  validate();
		  
	  	  // TRACE OUTPUT
	  	  if (LeviModule.isTraceActive()) System.out.println("END leviDialog.fitDialog");
	  	
	   }
	   
	   /*
	    * Methode:		setDialogTexte
	    * 
	    * Description:	Setz alle Text-Bausteine des Dialoges neu. Wichtig fr Sprachwechsel.
	    *    
	    * Parameters:   
	    * 
	    * Date: 		 22.10.2007
	    * @author        sirocic
	    */	
	   protected void setDialogTexte(String title) {
		   
		   this.title = title;
		   setTitle(title);
		   	  
	   }
	   protected void setDialogTexte() {
		   title = myFrame.getLearningModule().getRessources().getResourceByKey(dlgTitleKey);
		   setTitle(title);	  
	   }
	   
	   /*
	    * Methode:		setDialogColors
	    * 
	    * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
	    * 				Werte. 
	    *    
	    * Parameters:   Aktuelle Farbwerte fr Hintergrund und Vordergrund.
	    * 
	    * Date: 		 22.10.2007
	    * @author        sirocic
	    */	
	   protected void setDialogColors() {
		   Color backgroundColor,  foregroundColor, buttonBackgroundColor;
	 	   
	 	   if (myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentBackground")!=null) {
		 	   backgroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentBackground");
		 	   foregroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentForeground");
	 	   }
	 	   else {
			   backgroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorModuleBackground");
			   foregroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorModuleForeground");
	 	   }
		   buttonBackgroundColor  = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorButtonBackground");
		 	  
		   setBackground(backgroundColor);
	 	   setForeground(foregroundColor);
	       contentPane.setBackground(backgroundColor);
	       contentPane.setForeground(foregroundColor);
	       contentVB.setBackground(backgroundColor);
	       contentVB.setForeground(foregroundColor);
		   buttonHB.setBackground(backgroundColor);
		   buttonHB.setForeground(foregroundColor);
	   	   cancelButton.setBackground(buttonBackgroundColor);
	       okButton.setBackground(buttonBackgroundColor);
		 	 
		 	  
	   }
	   /*
	    * Methode:		updateLanguageColorFonts
	    * 
	    * Description:	Setzt die aktuellen Farben aus der Konfigurationbzw. und Texte aus den Ressourcen.
	    * 				Wird aufgerufen in erster Linie nach einem dynamischen Sprach- oder Farbwechsel.
	    *    
	    * Date: 		 22.10.2007
	    * @author        sirocic
	    */	
	   public void updateLanguageColorFonts() {
	      
	 	  // TRACE OUTPUT
	 	  if (LeviModule.isTraceActive()) System.out.println("CALL leviDialog.updateLanguageColorFonts:");
	 	 
	 	  // SUBKOMPONENTE UPDATEN...
	      okButton.updateLanguageColorFonts();
	      cancelButton.updateLanguageColorFonts();
    
	   	  // SPRACHWECHSEL ...
	   	  setDialogTexte();
	   	  
	   	  // FONTWECHSEL
	   	  setDialogFont();
	   	  
	      //    FARBWECHSEL ...
	 	  setDialogColors();

	      fitDialog();
	      
	  	  // TRACE OUTPUT
	  	  if (LeviModule.isTraceActive()) System.out.println("END leviDialog.updateLanguageColorFonts");

	   }
	   
	   public void setFrame(LeviFrame frame) {
		   myFrame = frame;
	   }
	   public void setLocationX(int x) {
		   locationX = x;
	   }
	   public void setLocationY(int y) {
		   locationY = y;
	   }
	   public void setContentBox(Box contentBox) {
		   this.contentBox = contentBox;
		   contentPane.removeAll();
		   contentPane.add(contentBox, BorderLayout.CENTER);
	   }
	   
	   // ACCESSOR METHODES
	   public LeviFrame getFrame() {
		   return myFrame;
	   }
	   public int getLocationX() {
		   return locationX;
	   }
	   public int getLocationY() {
		   return locationY;
	   }
	   
}
