/* leviFontChooseDialog.java
 * 
 * Created on 22.06.2007
 * @author sirocic
 */
package learningVisualizations.gui;

import java.awt.*;
import java.awt.event.*;
import java.awt.font.FontRenderContext;
import java.awt.geom.Rectangle2D;

import javax.swing.*;
import javax.swing.text.*;

import learningVisualizations.LeviModule;



/**
 * @author sirocic
 *
 * Class: leviFontChooseDialog
 * 
 * Description: The class "leviFontChooseDialog" of the general font choose dialog. 
 * 				
 * Date:	22.06.2007
 */
public class LeviFontChooseDialog extends LeviDialog implements ActionListener{
	
	public static final int PREVIEWWEITE = 350;
	
	private JLabel targetPropertyLabel;
	protected LeviFontTargetInterface fontTarget;
	private JComboBox fontName;
	private JCheckBox fontBold, fontItalic;
	private JTextField fontSize;
	private JLabel fontNameLabel, fontSizeLabel, previewLabel;
	private JPanel previewPanel;
	private SimpleAttributeSet attributes;
	protected Font newFont;
	private Box fontHB, previewHB;
	private String listOfFonts[];
	
	
  /*
   * Methode:		Contructors
   * 
   * Description:	Displays the current color setting from the configuration.
   *				Provides the user with the possiblity to change the color settings
   *                for special elements like module background, process foreground etc.
   *                The selected colors are store by the background color of the preview panels.
   *                The configuration will be updated if the confirm button was selected.
   * 
   * Date: 		    07.09.2005
   * @author        sirocic
   */
	public LeviFontChooseDialog() {
	   super();
   	   // INIT
   	   this.fontName= new JComboBox();
   	   this.fontBold = new JCheckBox(); 
   	   this.fontItalic = new JCheckBox();
   	   this.fontSize = new JTextField();
   	   this.previewLabel = new JLabel();
	   this.fontNameLabel = new JLabel();
	   this.fontSizeLabel = new JLabel();
   	   this.attributes = new SimpleAttributeSet(); 
	   this.fontHB = Box.createHorizontalBox(); 
	   
  	   // SET DIALOG PROPERTIES
   	   setSize(WIDTH, HEIGHT);
	}
   public LeviFontChooseDialog(LeviFrame frame) {
   	   super(frame, "dlg.font.title");
   	   
	   // TRACE OUTPUT
	   if (LeviModule.isTraceActive()) System.out.println("CALL leviFontChooseDialog.konstruktor:");
	   
  	   // INIT
   	   attributes = new SimpleAttributeSet();
	   fontHB = Box.createHorizontalBox(); 
	   fontHB.setAlignmentX(LEFT_ALIGNMENT);
	   previewHB = Box.createHorizontalBox();
	   previewHB.setAlignmentX(LEFT_ALIGNMENT);
	   buttonHB.setAlignmentX(LEFT_ALIGNMENT);

   	   // GUI Dialogelemente erzeugen ...
	   fontNameLabel = new JLabel();
	   listOfFonts = GraphicsEnvironment.getLocalGraphicsEnvironment().getAvailableFontFamilyNames();
		
	   fontName = new JComboBox(listOfFonts);
	   fontName.setSelectedIndex(1);
	   fontName.addActionListener(this);
	   fontSizeLabel = new JLabel();
	   fontSize = new JTextField("12", 4);
	   fontSize.setHorizontalAlignment(SwingConstants.RIGHT);
	   fontSize.addActionListener(this);
	   fontBold = new JCheckBox();
	   fontBold.setSelected(true);
	   fontBold.addActionListener(this);
	   fontItalic = new JCheckBox();
	   fontItalic.addActionListener(this);
	   
	   //	 Preview-Anzeige erzeugen
	   previewPanel = new JPanel(new BorderLayout());
 	   if (myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentBackground")!=null) 
 		  previewPanel.setBorder(BorderFactory.createLineBorder((Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentForeground"), 1));
 	   else
 		   previewPanel.setBorder(BorderFactory.createLineBorder((Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorModuleForeground"), 1));
	   previewLabel = new JLabel(myFrame.getLearningModule().getRessources().getResourceByKey("dlg.font.chooser.sampletext"));
	   previewPanel.add(previewLabel, BorderLayout.CENTER);
	   
	   // in eine horizontale Box damit
	   previewHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
	   previewHB.add(previewPanel);
	   previewHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));

	   // Elemente zur Auswahl des Fonts in eine horizontale Box platzieren
	   fontHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
	   fontHB.add(fontNameLabel);
	   fontHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
	   fontHB.add(fontName);
	   fontHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
	   fontHB.add(fontSizeLabel);
	   fontHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
	   fontHB.add(fontSize);
	   fontHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
	   fontHB.add(fontBold);
	   fontHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
	   fontHB.add(fontItalic);
	   fontHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));

   	   // create the buttons for closing and cancling the dialog 
	   okButton.addActionListener(new 
		        ActionListener() 
				{
	   					public void actionPerformed(ActionEvent e)
	   					{
	   						Font selectedFont;
	   		   	            
		   				     // Check the name of the font
		   				     if (!StyleConstants.getFontFamily(attributes)
		   				                        .equals(fontName.getSelectedItem())) {
		   				       StyleConstants.setFontFamily(attributes, 
		   				                                    (String)fontName.getSelectedItem());
		   				     }
		   				     // Check the font size (no error checking yet)
		   				     if (StyleConstants.getFontSize(attributes) != 
		   				                                    Integer.parseInt(fontSize.getText())) {
		   				       StyleConstants.setFontSize(attributes, 
		   				                                  Integer.parseInt(fontSize.getText()));
		   				     }
		   				     // Check to see if the font should be bold
		   				     if (StyleConstants.isBold(attributes) != fontBold.isSelected()) {
		   				       StyleConstants.setBold(attributes, fontBold.isSelected());
		   				     }
		   				     // Check to see if the font should be italic
		   				     if (StyleConstants.isItalic(attributes) != fontItalic.isSelected()) {
		   				       StyleConstants.setItalic(attributes, fontItalic.isSelected());
		   				     }
		   				         
		   				     // and update our preview label
		   				     updateFont();
		   				     
	   						// Font bernehmen .....
	   						fontTarget.setSelectedFont(newFont);
	   						
	   		   	   		    // Dialog unsichtbar machen
	   						showDialog(false);
	   					}
		
				} );

       // in die vertikale Box ...
       contentVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
       contentVB.add(fontHB);
       contentVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
       contentVB.add(previewHB);
       contentVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
       contentVB.add(buttonHB);
       contentVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
  
       //     FARBEN, FONTS UND TEXTE SETZEN ...
 	   updateLanguageColorFonts();
 
		// TRACE OUTPUT
		if (LeviModule.isTraceActive()) System.out.println("END leviFontChooseDialog.Konstructor");

   }
   
   /*
    * Methode:		actionPerformed
    * 
    * Description:	Implementiert den ActionListener fr die Fonteingabe-Elemente. Wenn sich an der
    * 				Einstellung der Eingabeelemente etwas ndert, wird dies im Font aktualisiert und
    * 				die Preview-Anzeige wird aktualisiert.
    *    
    * Parameters:    
    * 
    * Date: 		 04.07.2007
    * @author        sirocic
    */	
   public void actionPerformed(ActionEvent ae) {
     // Check the name of the font
     if (!StyleConstants.getFontFamily(attributes)
                        .equals(fontName.getSelectedItem())) {
       StyleConstants.setFontFamily(attributes, 
                                    (String)fontName.getSelectedItem());
     }
     // Check the font size (no error checking yet)
     if (StyleConstants.getFontSize(attributes) != 
                                    Integer.parseInt(fontSize.getText())) {
       StyleConstants.setFontSize(attributes, 
                                  Integer.parseInt(fontSize.getText()));
     }
     // Check to see if the font should be bold
     if (StyleConstants.isBold(attributes) != fontBold.isSelected()) {
       StyleConstants.setBold(attributes, fontBold.isSelected());
     }
     // Check to see if the font should be italic
     if (StyleConstants.isItalic(attributes) != fontItalic.isSelected()) {
       StyleConstants.setItalic(attributes, fontItalic.isSelected());
     }
         
     // and update our preview label
     updateFont();
   }
   
   /*
    * Methode:		updatePreviewFont
    * 
    * Description:	Aktualisiert die Preview-Anzeige im Dialog.
    *    
    * Date: 		 04.07.2007
    * @author        sirocic
    */	
   protected void updateFont() {
	    String name = StyleConstants.getFontFamily(attributes);
	    boolean bold = StyleConstants.isBold(attributes);
	    boolean ital = StyleConstants.isItalic(attributes);
	    int size = StyleConstants.getFontSize(attributes);

	    newFont = new Font(name, (bold ? Font.BOLD : 0) +
	                            (ital ? Font.ITALIC : 0), size);
	    previewLabel.setFont(newFont);
	    
	  }
   
   // MUTATOR METHODES ...
   
   /*
    * Methode:		showDialog
    * 
    * Description:	Setzt die aktuellen Farben aus der Konfiguration bevor der Dialog angezeigt bzw.
    * 				wieder versteckt wird.
    *    
    * Parameters:    
    * 
    * Date: 		 11.04.2006
    * @author        sirocic
    */	
   public void showDialog(boolean visible, Font selectedFont, JLabel propertyLabel, LeviFontTargetInterface fontTarget) {
	  String aktDialogTitel;
	   
 	  // TRACE OUTPUT
 	  if (LeviModule.isTraceActive()) System.out.println("CALL leviFontChooseDialog.showDialog:");
 	  
 	  if (visible) {
 		  // Dialog wird sichtbar gemacht. 
 		 if (propertyLabel != null) {
 			 targetPropertyLabel = propertyLabel;
 			 aktDialogTitel = myFrame.getLearningModule().getRessources().getResourceByKey("dlg.font.chooser.title") + " - " + propertyLabel.getText();
 			 setTitle(aktDialogTitel);
 		  }
 		 
          if (selectedFont != null)  {
        	  // Check the name of the font
        	  StyleConstants.setFontFamily(attributes, selectedFont.getFamily());
        	  fontName.setSelectedItem(selectedFont.getFamily());
        	  // Check the font size 
        	  StyleConstants.setFontSize(attributes, selectedFont.getSize());
        	  fontSize.setText(String.valueOf(selectedFont.getSize()));
        	  // Check to see if the font should be bold
        	  StyleConstants.setBold(attributes, selectedFont.isBold());
        	  fontBold.setSelected(selectedFont.isBold());
        	  // Check to see if the font should be italic
        	  StyleConstants.setItalic(attributes, selectedFont.isItalic());
        	  fontItalic.setSelected(selectedFont.isItalic());

        	  updateFont();
           }
           
           if (fontTarget != null)
        	  this.fontTarget = fontTarget;

 	  }
   	   		  
 	  // ... so jetzt den Dialog anzeigen bzw. verstecken.
 	  super.showDialog(visible);
   	  
  	  // TRACE OUTPUT
  	  if (LeviModule.isTraceActive()) System.out.println("END leviFontChooseDialog.showDialog");

   }
   /*
    * Methode:		fitDialog
    * 
    * Description:	Bestimmen der bentigten Panelgre fr das Anzeigen des vollstndigen Dialogs. 
    *    
    * Parameters:    
    * 
    * Date: 		 16.01.2007
    * @author        sirocic
    */	
   public void fitDialog() {
	  Graphics2D g2;
  	  int maxWidth, maxHeight;
  	  Rectangle2D boundsLabel;
  	  FontRenderContext context;
  	  int textfieldHeight, textfieldWidth;
  	  int i;
  	  
  	  // TRACE OUTPUT
  	  if (LeviModule.isTraceActive()) System.out.println("CALL leviFontChooseDialog.fitDialog:");
  	
  	  // init
  	  g2 = (Graphics2D) getGraphics();
  	  maxWidth = 0;
	  maxHeight =0;
	  
	  if (g2 != null)  {
		  // Bentigte Zeilenbreite und -hhe bestimmen anhand der Textausmasse bei aktuelle
	      // eingestellten Font ...
		  g2.setFont(getFont());
 	      context = g2.getFontRenderContext();

		  // GUI-ELEMENTE AN AKTUELLE FONTSIZE UND INHALTE ANPASSEN ...
 	      // Auswahlbox fr den Schriftsatz ...
		  boundsLabel = getFont().getStringBounds((String)fontName.getSelectedItem(), context);
 	      textfieldHeight=(int)boundsLabel.getHeight()+LeviFrame.TFOFFSET;
 	      // Weite der Auswahlbox richtet sich nach dem lngsten String in der Liste...
 	      boundsLabel = getFont().getStringBounds(listOfFonts[0], context);
 	      textfieldWidth = (int)boundsLabel.getWidth()+LeviFrame.COMBOBOXOFFSET;
 	      for (i=1; i < listOfFonts.length; i++) {
 	    	 boundsLabel = getFont().getStringBounds(listOfFonts[i], context);
 	    	 if (textfieldWidth < (int)boundsLabel.getWidth())
 	    		textfieldWidth = (int)boundsLabel.getWidth()+LeviFrame.COMBOBOXOFFSET;
 	      }
 	      fontName.setSize(new Dimension(textfieldWidth,textfieldHeight));
 	      fontName.setPreferredSize(fontName.getSize());
 	      fontName.setMaximumSize(fontName.getSize());
 	      fontName.setMinimumSize(fontName.getSize());
 	      
 	      // Texteingabefeld fr die Fontsize ...
 	      boundsLabel = getFont().getStringBounds(fontSize.getText(), context);
	      textfieldWidth = LeviFrame.NUMBERTFWIDTH;
	      if (textfieldWidth < (int)boundsLabel.getWidth())
	    	  textfieldWidth = (int)boundsLabel.getWidth();
	      fontSize.setSize(new Dimension(textfieldWidth,textfieldHeight));
	      fontSize.setPreferredSize(fontSize.getSize());
	      fontSize.setMaximumSize(fontSize.getSize());
	      fontSize.setMinimumSize(fontSize.getSize());
	      
	      // Preview-Anzeige fr den ausgewhlen Font ...
	      g2.setFont(previewLabel.getFont());
 	      context = g2.getFontRenderContext();
	      boundsLabel = getFont().getStringBounds(previewLabel.getText(), context);
	      textfieldHeight=LeviFontDialog.PREVIEWHOEHE;
 	      textfieldWidth = PREVIEWWEITE;
 	      if (textfieldHeight < (int)boundsLabel.getHeight()+LeviFrame.TFOFFSET)
 	    	  textfieldHeight=(int)boundsLabel.getHeight()+LeviFrame.TFOFFSET;
 	      if (textfieldWidth < (int)boundsLabel.getWidth())
 	    	  textfieldWidth = (int)boundsLabel.getWidth()+LeviFrame.TFOFFSET;
	      previewPanel.setSize(new Dimension(textfieldWidth, textfieldHeight));
		  previewPanel.setMinimumSize(previewPanel.getSize());
		  previewPanel.setMaximumSize(previewPanel.getSize());
		  previewPanel.setPreferredSize(previewPanel.getSize());
		  
		  // BENTIGTE CONTENT-PANEL-BREITE BESTIMMEN ....
	  	  // Breiteste Zeile  bestimmen.
	  	  maxWidth = LeviFrame.ATTRIBUTOFFSET ;
	  	  if (previewPanel.getWidth() > (fontNameLabel.getWidth() + fontName.getWidth() + fontSizeLabel.getWidth() + fontSize.getWidth() + 
	  			  fontBold.getWidth() + fontItalic.getWidth()+ 5*LeviFrame.ATTRIBUTOFFSET))
	  		  maxWidth = maxWidth + previewPanel.getWidth() + 2*LeviFrame.ATTRIBUTOFFSET;
	  	  else
	  		maxWidth = maxWidth + fontNameLabel.getWidth() + fontName.getWidth()+ fontSize.getWidth() 
	  							+ fontSizeLabel.getWidth() + fontBold.getWidth() 
	  							+ fontItalic.getWidth()+ 5*LeviFrame.ATTRIBUTOFFSET;
	  	  
	  	  // BENTIGTE CONTENT-PANEL-HHE BESTIMMEN ....
	  	  maxHeight =  LeviFrame.LINEOFFSET; 
	  	  maxHeight = maxHeight + fontName.getHeight() + LeviFrame.LINEOFFSET + previewPanel.getHeight();
	  	
	  	 // Panelgre aktualisieren ...
	  	  getRootPane().setSize(maxWidth,maxHeight);
	  	  getRootPane().setMaximumSize(new Dimension(maxWidth,maxHeight));
	  	  getRootPane().setPreferredSize(new Dimension(maxWidth,maxHeight));
	  }
	  
	  super.fitDialog();
	  
  	  // TRACE OUTPUT
  	  if (LeviModule.isTraceActive()) System.out.println("END leviFontChooseDialog.fitDialog");
  	
   }
   
   /*
    * Methode:		setDialogFont
    * 
    * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
    * 				Werte. 
    *    
    * Parameters:   Aktuelle Font fr Labels und Textfelder.
    * 
    * Date: 		 17.01.2007
    * @author        sirocic
    */	
   protected void setDialogFont() {
	   Font menuFont;
	   
	   menuFont = (Font)myFrame.getLearningModule().getConfiguration().getSettingByKey("FontMenu");
	   
	   super.setDialogFont("FontMenu");
	   
	   fontNameLabel.setFont(menuFont);
	   fontName.setFont(menuFont);
	   fontSizeLabel.setFont(menuFont);
	   fontSize.setFont(menuFont);
       fontBold.setFont(menuFont);
       fontItalic.setFont(menuFont);

   }
   
   /*
    * Methode:		setDialogTexte
    * 
    * Description:	Setz alle Text-Bausteine des Dialoges neu. Wichtig fr Sprachwechsel.
    *    
    * Parameters:   
    * 
    * Date: 		 05.07.2007
    * @author        sirocic
    */	
   protected void setDialogTexte() {
      String titleKey;
      
	  if (targetPropertyLabel != null) 
		  titleKey ="dlg.font.chooser.title" + " - " + targetPropertyLabel.getText();
	   else
		  titleKey = "dlg.font.chooser.title";

	  super.setDialogTexte(myFrame.getLearningModule().getRessources().getResourceByKey(titleKey));
	  
	  fontNameLabel.setText(myFrame.getLearningModule().getRessources().getResourceByKey("dlg.font.chooser.name")+":");
	  fontSizeLabel.setText(myFrame.getLearningModule().getRessources().getResourceByKey("dlg.font.chooser.size")+":");
	  fontBold.setText(myFrame.getLearningModule().getRessources().getResourceByKey("dlg.font.chooser.bold"));
	  fontItalic.setText(myFrame.getLearningModule().getRessources().getResourceByKey("dlg.font.chooser.italic"));
	  previewLabel.setText(myFrame.getLearningModule().getRessources().getResourceByKey("dlg.font.chooser.sampletext"));
   }
   
   /*
    * Methode:		setDialogColors
    * 
    * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
    * 				Werte. 
    *    
    * Parameters:   Aktuelle Farbwerte fr Hintergrund und Vordergrund.
    * 
    * Date: 		 17.01.2007
    * @author        sirocic
    */	
   protected void setDialogColors() {
	   Color backgroundColor,  foregroundColor, buttonBackgroundColor;
	   
	   if (myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentBackground")!=null) {
		   backgroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentBackground");
		   foregroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentForeground");
	   }
	   else {
		   backgroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorModuleBackground");
		   foregroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorModuleForeground");
	   }
	   buttonBackgroundColor  = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorButtonBackground");
	 	  
	   super.setDialogColors();
	   
	   fontHB.setBackground(backgroundColor);
	   fontHB.setForeground(foregroundColor);
       fontName.setBackground(backgroundColor);
       fontName.setForeground(foregroundColor);
       fontNameLabel.setBackground(backgroundColor);
       fontNameLabel.setForeground(foregroundColor);
       fontBold.setBackground(backgroundColor);
       fontBold.setForeground(foregroundColor);
       fontItalic.setBackground(backgroundColor);
       fontItalic.setForeground(foregroundColor);
       fontSize.setBackground(backgroundColor);
       fontSize.setForeground(foregroundColor);
       fontSizeLabel.setBackground(backgroundColor);
       fontSizeLabel.setForeground(foregroundColor);
       previewLabel.setBackground(backgroundColor);
       previewLabel.setForeground(foregroundColor);
       previewPanel.setBackground(backgroundColor);
       previewPanel.setForeground(foregroundColor);
       if (myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentForeground")!=null)
    	   previewPanel.setBorder(BorderFactory.createLineBorder((Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentForeground"), 1));
       else
    	   previewPanel.setBorder(BorderFactory.createLineBorder((Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorModuleForeground"), 1));
   }
   /*
    * Methode:		updateLanguageColorFonts
    * 
    * Description:	Setzt die aktuellen Farben aus der Konfigurationbzw. und Texte aus den Ressourcen.
    * 				Wird aufgerufen in erster Linie nach einem dynamischen Sprach- oder Farbwechsel.
    *    
    * Date: 		 16.03.2007
    * @author        sirocic
    */	
   public void updateLanguageColorFonts() {
      
 	  // TRACE OUTPUT
 	  if (LeviModule.isTraceActive()) System.out.println("CALL leviFontChooseDialog.updateLanguageColorFonts:");
 	  
 	  // SUBKOMPONENTE UPDATEN...
 	  
 	  super.updateLanguageColorFonts();
 
  	  // TRACE OUTPUT
  	  if (LeviModule.isTraceActive()) System.out.println("END leviFontChooseDialog.updateLanguageColorFonts");

   }
   
   
}
