/* leviFontDialog.java
 * 
 * Created on 22.06.2007
 * @author sirocic
 */
package learningVisualizations.gui;

import java.awt.*;
import java.awt.event.*;
import java.awt.font.*;
import java.awt.geom.*;
import java.util.*;

import javax.swing.*;

import learningVisualizations.LeviModule;




/**
 * @author sirocic
 *
 * Class: leviFontDialog
 * 
 * Description: Realisiert den Einstellungsdialog fr die benutzten Fonts. 
 * 				
 * Date:	22.06.2007
 */
public class LeviFontDialog extends LeviDialog implements LeviFontTargetInterface {

	public static final int PREVIEWHOEHE = LeviFrame.TFHEIGHT + 15;
	public static final int PREVIEWWIDTH = 150;

    private LeviFontTargetInterface myself=this;
	private LeviChooseButton selectedButton;
	private Enumeration keys; // keys of all properties 
	private String key;  // current key
	
	private ArrayList fontButtons, fontButtonPanels, fontLabels, fontLabelPanels, previewPanels, previewLabels;
	private Box  fontsHB, fontLabelsVB, previewPanelsVB, fontButtonsVB;

	JLabel previewLabel;
	
	
  /*
   * Methode:		Contructors
   * 
   * Description:	Displays the current font setting from the configuration.
   *				Provides the user with the possiblity to change the font settings
   *                for special elements like normal text or headers etc.
   *                The selected fonts are store by the preview panels.
   *                The configuration will be updated if the confirm button was selected.
   * 
   * Date: 		    22.06.2007
   * @author        sirocic
   */
	public LeviFontDialog() {
	   super();
   	   // INIT
   	   this.selectedButton=null;
   	   this.contentPane=getContentPane(); 
   	   this.fontLabels = new ArrayList();  
	   this.fontButtons = new ArrayList();
	   this.fontButtonPanels = new ArrayList();
	   this.previewPanels = new ArrayList();
	   this.previewLabels = new ArrayList();
	   this.contentVB = Box.createVerticalBox(); 
	   this.fontsHB= Box.createHorizontalBox();
	   this.fontLabelsVB = Box.createVerticalBox(); 
	   this.previewPanelsVB = Box.createVerticalBox(); 
	   this.fontButtonsVB = Box.createVerticalBox(); 
	   this.key = "";
	   this.keys = null;
	   
  	   // SET DIALOG PROPERTIES
   	   setSize(WIDTH, HEIGHT);
	}
   public LeviFontDialog(LeviFrame frame) {
   	   super(frame, "dlg.font.title");
   	   
   	   // Lokale Variablen
   	   String labelStr;
   	   JLabel label;
   	   JPanel panel;
   	   LeviChooseButton button;   
   	   JPanel preview, buttonP;
   	   int i;
   	   
	   // TRACE OUTPUT
	   if (LeviModule.isTraceActive()) System.out.println("CALL leviFontDialog.konstruktor:");

  	   // INIT
   	   this.fontLabels = new ArrayList();
   	   this.fontLabelPanels = new ArrayList();
	   this.fontButtons = new ArrayList();
	   this.fontButtonPanels = new ArrayList();
	   this.previewPanels = new ArrayList();
	   this.previewLabels = new ArrayList();
	   this.fontsHB= Box.createHorizontalBox();
	   this.fontLabelsVB = Box.createVerticalBox(); 
	   this.previewPanelsVB = Box.createVerticalBox(); 
	   this.fontButtonsVB = Box.createVerticalBox();
	 
   	   // FOR EACH FONT PROBERTY ...
   	   // Die Bezeichnungen, Vorschau-Panels und Fontauswahl-Buttons erzeugen.
   	   keys=this.myFrame.getLearningModule().getConfiguration().getKeysOfSettings();
   	   i=0;
   	   while (keys.hasMoreElements()) {
   		  key=(String)keys.nextElement();
   	   	  
   	   	  if (key.startsWith("Font")) {
   	   		  // font property was found !!!
   	   		  
   			  // TRACE OUTPUT
   			  if (LeviModule.isTraceActive()) System.out.println("	key="+key);   			
   	   	      
   	   	      // Labelelement fr die Bezeichung erzeugen und in die vertikale box fr alle Labels einfgen
   	   	      labelStr = myFrame.getLearningModule().getRessources().getResourceByKey("dlg.font."+key)+":";
	   	      label =new JLabel(labelStr);
	   	      panel = new JPanel();
	   	      panel.setLayout(new BorderLayout());
	   	      panel.add(label, BorderLayout.EAST);
	   	      fontLabels.add(label);
	   	      fontLabelPanels.add(panel);
	   	      fontLabelsVB.add(panel);
	   	      fontLabelsVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
   	   	      
   	   	      // Preview-Panel fr den Font erzeugen und in die vertikale box fr alle Previews einfgen
   	   	      preview = new JPanel();
   	   	      if (myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentForeground")!=null)
   	   	    	  preview.setBorder(BorderFactory.createLineBorder((Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentForeground"), 1));
   	   	      else
   	   	    	  preview.setBorder(BorderFactory.createLineBorder((Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorModuleForeground"), 1));
   	   	      previewLabel = new JLabel("AaBbYyZz");
   	   	      previewLabel.setFont((Font)myFrame.getLearningModule().getConfiguration().getSettingByKey(key));
   	   	      preview.add(previewLabel);
   	   	      previewPanels.add(preview);
	   	      previewLabels.add(previewLabel);
	   	      previewPanelsVB.add(preview);
	   	      previewPanelsVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
   	   	      
   	   	      // add the color select button 
   	   	      button = new LeviChooseButton("button.font", myFrame.getLearningModule(), previewLabel, label);
   	   	      button.addActionListener(new 
   		        ActionListener() 
			    {
   					public void actionPerformed(ActionEvent e)
   					{
   						
   						// TRACE OUTPUT
   						if (LeviModule.isTraceActive()) System.out.println("CALL ActionListener.actionPerformed: source="+e.getSource().toString());
                        
   						// get the selected button
   						selectedButton= (LeviChooseButton) e.getSource();
   						
   						// show the font choose dialog
   						myFrame.getLearningModule().getFontChooseDialog().showDialog(true,
   				                selectedButton.getPreview().getFont(), selectedButton.getPropertyLabel(), myself);
   						
   			        }
	
			   }
	          );
   	   	      buttonP = new JPanel();
   	   	      button.setAlignmentX(Component.CENTER_ALIGNMENT);
   	   	      button.setAlignmentY(Component.CENTER_ALIGNMENT);
   	   	      buttonP.add(button);
   	   	      fontButtons.add(button);
   	   	      fontButtonPanels.add(buttonP);
   	   	      fontButtonsVB.add(buttonP);
   	   	      fontButtonsVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
   	   	      
   	          i++;
   	   	  }
   	   	  
   	   }
   	   
   	   // Labels, Previews und Fontauswahl-Buttons in eine Zeile packen.
	   fontsHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
   	   fontsHB.add(fontLabelsVB);
	   fontsHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
   	   fontsHB.add(previewPanelsVB);
	   fontsHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
   	   fontsHB.add(fontButtonsVB);
	   fontsHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
	      
   	   // create the buttons for closing and cancling the dialog 
	   okButton.addActionListener(new 
		        ActionListener() 
				{
	   					public void actionPerformed(ActionEvent e)
	   					{
	   						int j=0;
	   						
	   						// put the current color values into the configuration
	   			   	        keys=myFrame.getLearningModule().getConfiguration().getKeysOfSettings();
	   		   	   
	   		   	            while (keys.hasMoreElements()) {
	   		   	   	          key=(String)keys.nextElement();
	   		   	   	  
	   		   	   	          if (key.startsWith("Font")) {
	   		   	   	             myFrame.getLearningModule().getConfiguration().putSettingByKey(key, (Font)((JLabel)previewLabels.get(j)).getFont());
	   		   	   	             j++;
	   		   	   	          }
	   		   	   	          
	   		   	   	        }
	   		   	            
	   		   	            // save the configuration ...
	   		   	            myFrame.getLearningModule().getConfiguration().saveConfiguration();
	   		   	            
	   		   	            // repaint the module
	   		   	            myFrame.updateLanguageColorFonts();
	   		   	            
	   		   	   		    // color property was found !!!
	   		   	            showDialog(false);
	   					}
		
				} );
  
   	   // jetzt die komplette vertikal Box zusammenbauen
	   contentVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
   	   contentVB.add(fontsHB);
	   contentVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
	   contentVB.add(buttonHB);
   	   contentVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
  
   	   //     FARBEN, FONTS UND TEXTE SETZEN ...
 	   updateLanguageColorFonts();
 	   
		// TRACE OUTPUT
		if (LeviModule.isTraceActive()) System.out.println("END leviFontDialog.konstruktor");

   }
   
   // MUTATOR METHODES ...
   
   /*
    * Methode:		showDialog
    * 
    * Description:	Setzt die aktuellen Farben aus der Konfiguration bevor der Dialog angezeigt bzw.
    * 				wieder versteckt wird.
    *    
    * Parameters:    
    * 
    * Date: 		 11.04.2006
    * @author        sirocic
    */	
   public void showDialog(boolean visible) {
      int i;
      
 	  // TRACE OUTPUT
 	  if (LeviModule.isTraceActive()) System.out.println("CALL leviFontDialog.showDialog:");
 	  
 	  if (visible) {
 		  // Dialog wird sichtbar gemacht. 
	   	  i=0;
	   	  keys=this.myFrame.getLearningModule().getConfiguration().getKeysOfSettings();
	   	  while (keys.hasMoreElements()) {
	   		  key=(String)keys.nextElement();
	   	   	  
	   	   	  if (key.startsWith("Font")) {
	   	   		  // font property was found !!!
	   	   		  // Vorschau fr die Fonteigenschaft wieder auf den Konfigurationswert setzen!
	   	   		  ((JLabel)previewLabels.get(i)).setFont((Font)myFrame.getLearningModule().getConfiguration().getSettingByKey(key));
	   	   		  i++;
	   	   	  }
	   	  }
	 	  
 	  }
   	   		  
 	  // ... so jetzt den Dialog anzeigen bzw. verstecken.
 	  super.showDialog(visible);
   	  
  	  // TRACE OUTPUT
  	  if (LeviModule.isTraceActive()) System.out.println("END leviFontDialog.showDialog");

   }
   
   /*
    * Methode:		fitDialog
    * 
    * Description:	Bestimmen der bentigten Panelgre fr das Anzeigen des vollstndigen Dialogs. 
    *    
    * Parameters:    
    * 
    * Date: 		 16.01.2007
    * @author        sirocic
    */	
   public void fitDialog() {
	  Graphics2D g2;
  	  int maxWidth, maxHeight;
  	  int maxLabelWidth;
  	  Rectangle2D boundsLabel;
	  FontRenderContext context;
	  JPanel previewP, buttonP;
	  int i;
  	  
  	  // TRACE OUTPUT
  	  if (LeviModule.isTraceActive()) System.out.println("CALL leviFontDialog.fitDialog:");
  	
  	  // init
  	  g2 = (Graphics2D) getGraphics();
  	  maxWidth = 0;
	  maxHeight =0;
	  maxLabelWidth = 0;
	  
	  if (g2 != null)  {
		  // Bentigte Zeilenbreite und -hhe bestimmen anhand der Textausmasse bei aktuelle
	      // eingestellten Font ...
		  g2.setFont(getFont());
	      context = g2.getFontRenderContext();

	      // GUI-ELEMENTE AN AKTUELLE FONTSIZE UND INHALTE ANPASSEN ...
	      for (i=0; i < previewPanels.size(); i++) {
	    	  previewP = ((JPanel)previewPanels.get(i));
	    	  previewP.setSize(new Dimension(PREVIEWWIDTH, PREVIEWHOEHE));
	    	  previewP.setPreferredSize(previewP.getSize());
	    	  previewP.setMaximumSize(previewP.getSize());
	    	  previewP.setMinimumSize(previewP.getSize());
	    	  
	    	  buttonP = ((JPanel)fontButtonPanels.get(i));
	    	  buttonP.setSize(new Dimension(LeviButton.BUTTONSIZE, PREVIEWHOEHE));
	    	  buttonP.setPreferredSize(buttonP.getSize());
   	   	      buttonP.setMaximumSize(buttonP.getSize());
   	   	      buttonP.setMinimumSize(buttonP.getSize());
   	   	     
	      }
	      
	      // BENTIGTE CONTENT-PANEL-BREITE BESTIMMEN ....
	  	  // Breiteste Zeile aus Schriftlabel (Font-Label), Preview-Panel und Fontauswahl-Button bestimmen.
	  	  maxWidth = LeviFrame.ATTRIBUTOFFSET ;
	  	  // Breite des breitesten Labels bestimmen.
	  	  for (i=0; i < fontLabels.size(); i++) {
		  	  boundsLabel = getFont().getStringBounds(((JLabel)fontLabels.get(i)).getText(), context);
		  	  if (maxLabelWidth < (int)boundsLabel.getWidth())
		  	  	 maxLabelWidth = (int)boundsLabel.getWidth();
	  	  }
	  	  fontLabelsVB.validate();
	  	  // daraus jetzt die maximale Breite einer Zeile bestimmen.
	  	  maxWidth = maxWidth + maxLabelWidth + 4*LeviFrame.ATTRIBUTOFFSET + PREVIEWWIDTH 
	  	  			+ ((JButton)fontButtons.get(0)).getWidth();
 	  
	  	  // BENTIGTE CONTENT-PANEL-HHE BESTIMMEN ....
	  	  maxHeight = LeviFrame.LINEOFFSET; 
	  	  maxHeight = maxHeight + (previewLabels.size()*PREVIEWHOEHE) 
	  	        + (previewLabels.size()*LeviFrame.LINEOFFSET);
	  	  
	  	  // Panelgre aktualisieren ...
	  	  getRootPane().setSize(maxWidth,maxHeight);
	  	  getRootPane().setMaximumSize(new Dimension(maxWidth,maxHeight));
	  	  getRootPane().setPreferredSize(new Dimension(maxWidth,maxHeight));
	  	  
	  }
	  
	  super.fitDialog();
	
  	  // TRACE OUTPUT
  	  if (LeviModule.isTraceActive()) System.out.println("END leviFontDialog.fitDialog");
  	
   }
   
   /*
    * Methode:		setDialogFont
    * 
    * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
    * 				Werte. 
    *    
    * Parameters:   Aktuelle Font fr Labels und Textfelder.
    * 
    * Date: 		 17.01.2007
    * @author        sirocic
    */	
   protected void setDialogFont() {
	   Font menuFont;
	   int i;
	   
	   menuFont = (Font)myFrame.getLearningModule().getConfiguration().getSettingByKey("FontMenu");
	   
	   super.setDialogFont("FontMenu");
	   
	   for (i=0; i < (int)fontLabels.size(); i++) 
		   ((JLabel)fontLabels.get(i)).setFont(menuFont);

   }
   
   /*
    * Methode:		setDialogTexte
    * 
    * Description:	Setz alle Text-Bausteine des Dialoges neu. Wichtig fr Sprachwechsel.
    *    
    * Parameters:   
    * 
    * Date: 		 26.01.2007
    * @author        sirocic
    */	
   protected void setDialogTexte() {
	   int i;
	   String labelStr;
	   
	   super.setDialogTexte(myFrame.getLearningModule().getRessources().getResourceByKey("dlg.font.title"));
	   
   	   i=0;
   	   keys=this.myFrame.getLearningModule().getConfiguration().getKeysOfSettings();
   	   while (keys.hasMoreElements()) {
   		  key=(String)keys.nextElement();
   	   	  
   	   	  if (key.startsWith("Font")) {
   	   		  // color property was found !!!
   	   		  labelStr = myFrame.getLearningModule().getRessources().getResourceByKey("dlg.font."+key)+":";
   	   		  ((JLabel) fontLabels.get(i)).setText(labelStr);
   	   		  i++;
   	   	  }
   	   }
	   	  
   }
   
   /*
    * Methode:		setDialogColors
    * 
    * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
    * 				Werte. 
    *    
    * Parameters:   Aktuelle Farbwerte fr Hintergrund und Vordergrund.
    * 
    * Date: 		 17.01.2007
    * @author        sirocic
    */	
   protected void setDialogColors() {
	   Color backgroundColor,  foregroundColor, buttonBackgroundColor;
	   int i;
	   
	   if (myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentBackground")!=null) {
		   backgroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentBackground");
		   foregroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentForeground");
	   }
	   else {
		   backgroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorModuleBackground");
		   foregroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorModuleForeground");
		   
	   }
	   buttonBackgroundColor  = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorButtonBackground");
	 	  
	   super.setDialogColors();
	   
   	   fontsHB.setBackground(backgroundColor);
   	   fontsHB.setForeground(foregroundColor);
   	   fontLabelsVB.setBackground(backgroundColor);
   	   fontLabelsVB.setForeground(foregroundColor);
   	   previewPanelsVB.setBackground(backgroundColor);
   	   previewPanelsVB.setForeground(foregroundColor);
   	   fontButtonsVB.setBackground(backgroundColor);
   	   fontButtonsVB.setForeground(foregroundColor);
   	   for (i=0; i < (int)previewPanels.size(); i++) {  
	   	  ((JLabel) fontLabels.get(i)).setBackground(backgroundColor);
	   	  ((JLabel) fontLabels.get(i)).setForeground(foregroundColor);
	   	  ((JPanel) fontLabelPanels.get(i)).setBackground(backgroundColor);
	   	  ((JPanel) fontLabelPanels.get(i)).setForeground(foregroundColor);
	   	  ((JLabel) previewLabels.get(i)).setBackground(backgroundColor);
	   	  ((JLabel) previewLabels.get(i)).setForeground(foregroundColor);
	   	  ((JPanel) previewPanels.get(i)).setBackground(backgroundColor);
	   	  ((JPanel) previewPanels.get(i)).setForeground(foregroundColor);
	   	  ((JPanel) fontButtonPanels.get(i)).setBackground(backgroundColor);
	   	  ((JPanel) fontButtonPanels.get(i)).setForeground(foregroundColor);
   	   }
	 	 
	 	  
   }
   /*
    * Methode:		updateLanguageColorFonts
    * 
    * Description:	Setzt die aktuellen Farben aus der Konfigurationbzw. und Texte aus den Ressourcen.
    * 				Wird aufgerufen in erster Linie nach einem dynamischen Sprach- oder Farbwechsel.
    *    
    * Date: 		 16.03.2007
    * @author        sirocic
    */	
   public void updateLanguageColorFonts() {
      int i;
      
 	  // TRACE OUTPUT
 	  if (LeviModule.isTraceActive()) System.out.println("CALL leviColorDialog.updateLanguageColorFonts:");
 	  
 	 // SUBKOMPONENTE UPDATEN...
  	 super.updateLanguageColorFonts();
   	  
   	  // SUBKOMPONENTE UPDATEN...
      for (i=0;i < fontButtons.size(); i++)
    	  ((LeviChooseButton)fontButtons.get(i)).updateLanguageColorFonts();
      
  	  // TRACE OUTPUT
  	  if (LeviModule.isTraceActive()) System.out.println("END leviColorDialog.updateLanguageColorFonts");

   }
   

   public void setSelectedFont(Font font) {
	   if (selectedButton != null)
		   selectedButton.getPreview().setFont(font);
   }
   // ACCESSOR METHODES
   
}