/* leviFrame.java
 *
 * Created on 11.02.2005
 * Autor: Birgit Sirocic
 *
 */
package learningVisualizations.gui;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

import learningVisualizations.LeviModule;
import learningVisualizations.gui.actions.LeviAbstractAction;
import learningVisualizations.gui.actions.LeviActionQuit;
import learningVisualizations.help.LeviHelp;


/**
 * @author sirocic
 *
 * Class: leviFrame
 *
 * Description: The class "leviFrame" is the frame of the general learning module.
 *
 * Date:	11.02.2005
 */
public class LeviFrame extends JFrame {

    // CONSTANTS
    static final long serialVersionUID=1;
    public static final int DIVIDERSIZE = 10;
    public static final int DIVIDEROFFSET = 9;
	public static final int BUTTONOFFSET = 20;
	public static final int RADIOBUTTONWIDTH=30;
	public static final int COMBOBOXOFFSET = 40;
	public static final int ATTRIBUTOFFSET = 5;
	public static final int LINEOFFSET = 10;
	public static final int TFOFFSET = 10;
	public static final int TEXTTFWIDTH=200;
	public static final int NAMETFWIDTH=100;
	public static final int MAXTEXTTFWIDTH=400;
	public static final int TFHEIGHT=22;
	public static final int NUMBERTFWIDTH=50;
	public static final int NUMBERTFOFFSET=10;
	

    // CONTENT ATTRIBUTES
    public LeviModule learningModule;
    private LeviFrame me=this;

    // DISPLAYING ATTRIBUTES
    private Dimension normalSize;
    private Dimension fullscreenSize;
    private int locationX, locationY;
    private int selfstudyDividerLocation;


    // GENERAL GUI ELEMENTS
    private LeviMenu menubar;
    private LeviPopup popupMenu;
    private JSplitPane SelfstudyPanel;


  /*
   * @author sirocic
   *
   * Methode:		Contructors
   *
   * Description:	Reading the configuration file and the general resources.
   *				Setting the feature like title, language etc. to the values
   *				readed from the configuration and resources, respectively.
   * 				Building the gui elements of the general learning module.
   *
   * Date: 		11.02.2005
   */
    public LeviFrame() {
        super();

          // COMPUTING THE WINDOW SIZES
        int scrX=0, scrY=0;
        try {
          scrX = (int)getToolkit().getScreenSize().getWidth();
          scrY = (int)getToolkit().getScreenSize().getHeight();
        } catch (HeadlessException he) {};
        if (scrX<=0) scrX = 1024;
        if (scrY<=0) scrY = 1024;
        fullscreenSize = new Dimension(scrX,scrY);
        normalSize = new Dimension(scrX-(scrX/10),scrY-(scrY/10));
        locationX = 0;
        locationY = 0;

        menubar = new LeviMenu();
        popupMenu = new LeviPopup();

        // building the content panels (use in lecture or/and selfstudy view)
        SelfstudyPanel = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
        this.selfstudyDividerLocation = 0;

        // SET THE CLOSE OPERATION
        setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        addWindowListener(new WindowAdapter () {
            public void windowClosing(WindowEvent e)
            {
                // DF: 20060530 - quit using the action, so the "quit" command will
                // always do the same as the window closing action using the [x] icon.
                LeviActionQuit laq = new LeviActionQuit();
                laq.putValue("lModule", getLearningModule()); // this alone is not sufficient
                laq.setLearningModule(getLearningModule());
                laq.actionPerformed( new ActionEvent( this, ActionEvent.ACTION_PERFORMED, "Quit" ) );
            }
        });

        // SET THE RESIZE OPERATION
        addComponentListener(new ComponentAdapter () {
            public void componentResized(ComponentEvent e)
            {

                // TRACE OUTPUT
                if (LeviModule.isTraceActive()) {
                    System.out.println("CALL leviFrame.componentResized normalSize=("+normalSize.getWidth()+", "+normalSize.getHeight()+")");
                    System.out.println("       fullscreenSize=("+fullscreenSize.getWidth()+", "+fullscreenSize.getHeight()+")");
                }

                // Neue Fenstergre merken in "normalSize" fr's Umschalten der Views
                if (!getSize().equals(fullscreenSize))
                    normalSize.setSize(getWidth(),getHeight());

                // TRACE OUTPUT
                if (LeviModule.isTraceActive())  {
                    System.out.println("END leviFrame.componentResized normalSize=("+normalSize.getWidth()+", "+normalSize.getHeight()+")");
                    System.out.println("       fullscreenSize=("+fullscreenSize.getWidth()+", "+fullscreenSize.getHeight()+")");
                }
            }
            public void componentMoved(ComponentEvent e)
            {

                // TRACE OUTPUT
                if (LeviModule.isTraceActive()) System.out.println("CALL leviFrame.componentMoved ");

                if (me.isShowing()) {
	                // Neue Position der Frame merken ...
	                if ((me.getLocationOnScreen().getX() != 0) ||
	                    (me.getLocationOnScreen().getY() != 0))
	                {
	                    locationX = (int) me.getLocationOnScreen().getX();
	                    locationY = (int) me.getLocationOnScreen().getY();
	                }
                }

                // TRACE OUTPUT
                if (LeviModule.isTraceActive())  System.out.println("END leviFrame.componentMoved locationX=("+locationX+ " locationY="+locationY);
            }
        });

        this.learningModule = null;

    }
    public LeviFrame(LeviModule learningModule) {

        this();

        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("CALL leviFrame.Contructors(c)");

        // SAVE PARENT COMPONENT
        this.learningModule = learningModule;

        // CREATING THE GUI

        // building the menu bar
        menubar = new LeviMenu(this);
        menubar.updateLanguageColorFonts();

        //CREATING THE GUI (Use in lecture view!)
        // building the popup menu
        popupMenu = new LeviPopup(this);
        popupMenu.updateLanguageColorFonts();

        //	  GENERATING THE DIALOGS ...
        learningModule.createModuleDialogs(this);

        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("END leviFrame.Contructors(c)");


    } // End Constructor


  /*
   *
   * @author sirocic
   *
   * Methode:		composeFrame
   *
   * Description:	Composes the frame according the current component attributes.
   *
   * Date: 		11.05.2005
   */
  public void composeFrame()
  {
        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("CALL leviFrame.composeFrame:");

        // setting the title line
    	if (learningModule.getXmlScenarioFile() != null)
    		setTitle(learningModule.getRessources().getResourceByKey("frame.title")+ " "+
    				learningModule.getVersion()+ " - " + learningModule.getXmlScenarioFile().getName());
    	else
    		setTitle(learningModule.getRessources().getResourceByKey("frame.title")+ " "+
    				learningModule.getVersion());
    	

        // Frame zurcksetzen bevor neue View aufgebaut werden kann.
        setJMenuBar(null);
        getContentPane().removeAll();

        switch (learningModule.getView()) {
            case (LeviModule.SELFSTUDYVIEW): {
                // COMPOSE THE CONTENT PARTS OF THE SELFSTUDY VIEW
                setSize(normalSize);
                setLocation(locationX, locationY);
                if (!this.isDisplayable())
                  setUndecorated(false); // show the title bar
                  setJMenuBar(menubar);
                  
                  if ((getLearningModule().getVisualizer() != null) && 
                		  (getLearningModule().getVisualizer().getScenarioVisualizer()!= null))
                	  getLearningModule().getVisualizer().getScenarioVisualizer().removePopupListener();
                  
                  SelfstudyPanel.setSize((int)normalSize.getWidth(),(int)normalSize.getHeight()-this.getJMenuBar().getHeight());
                  
                  if (learningModule.getMode()==LeviModule.EDITORMODE) {
                      SelfstudyPanel.setLeftComponent((JComponent) learningModule.getEditor());
                      // Kommandos, die whrend der Simulation nicht erlaubt waren, wieder aktivieren.
                      getLearningModule().enableCommands();
                  }
                  else if (learningModule.getMode()==LeviModule.SIMULATIONMODE) {
                      SelfstudyPanel.setLeftComponent((JComponent)learningModule.getVisualizer());
                      //Kommandos, die whrend der Simulation nicht erlaubt waren, wieder aktivieren.
                      getLearningModule().disableCommands();
                  }
                  
                  SelfstudyPanel.setRightComponent((JComponent)learningModule.getHelp());
                  getContentPane().add(SelfstudyPanel, BorderLayout.CENTER);

                  // SplitPanel-Divider zwischen Content und Help positionieren
                  if (learningModule.getHelp()!= null) {
                	  // Falls die Weite des Help-Panels und des Dividers noch nicht festge-
                	  // setzt wurde, dies hier jetzt tun.
                	  if (learningModule.getHelp().getWidth() == 0)
                		  learningModule.getHelp().setSize(LeviHelp.HELPWIDTH, SelfstudyPanel.getHeight());
                	  if(SelfstudyPanel.getDividerSize() == 0)
                		  SelfstudyPanel.setDividerSize(DIVIDERSIZE);
                	  // Position des Dividers berechnet sich aus der Breite des Split Panels
                      // verringert um die Breite des Help-Panels, des Dividers.
                	  selfstudyDividerLocation = SelfstudyPanel.getWidth()
                                               - SelfstudyPanel.getDividerSize()
                                               - learningModule.getHelp().getWidth()
                                               - DIVIDEROFFSET;
                	  
                  }

                  // CONSIDER THE ATTRIBUTE VALUES
                  if (learningModule.getHelp()!=null) {
                     if (!learningModule.isHelpVisible())
                         // makes the Help invisible
                         setHelpPanelVisible(false);
                     else
                         setHelpPanelVisible(true); 
                  }
                  
                  break;
            }
            case (LeviModule.LECTUREVIEW): {
                // COMPOSE THE CONTENT PARTS OF THE LECTURE VIEW.
                setSize(fullscreenSize);
                setLocation(0,0);
                setUndecorated(true); // hide the title bar
                learningModule.setHelpVisible(false);
                learningModule.setTooltipsVisible(false);
                if ((getLearningModule().getVisualizer() != null) &&
                		(getLearningModule().getVisualizer().getScenarioVisualizer() != null)) {
                	// Kontextmenu fr den Visualizer im Vorlsungsmodus aktiv machen.
                	// Wird von einer Komponente der ScenarioVisualizer nicht verwendet, muss die
                	// Komponente selber fr das Kontextmenu sorgen.
                	learningModule.getVisualizer().getScenarioVisualizer().setPopupListener(
                		learningModule.getVisualizer().getScenarioVisualizer().getPopupListener());
                	// Falls sich das scenario gendert hat, hier noch mal die grafischen Elemente des Visualizers
                	// neu aufbauen.
                	learningModule.getVisualizer().getScenarioVisualizer().createGraphicScenario();

                }
                
                // KOMMANDOS DEAKTIVIEREN WHREND DER SIMULATION UND DANACH WIEDER AKTIVIEREN ...
                if (learningModule.getMode()==LeviModule.EDITORMODE) {
                    // Kommandos, die whrend der Simulation nicht erlaubt waren, wieder aktivieren.
                    getLearningModule().enableCommands();
                }
                else if (learningModule.getMode()==LeviModule.SIMULATIONMODE) {
                    //Kommandos, die whrend der Simulation nicht erlaubt waren, wieder aktivieren.
                    getLearningModule().disableCommands();
                }
                
                getContentPane().add((JComponent)learningModule.getVisualizer(), BorderLayout.CENTER);
                
                break;
            }
        }

        refresh();

        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("END leviFrame.composeFrame");

  } // END composeFrame

  /*
   * Methode:		changeView
   *
   * Description:	Zum Wechsel in eine andere View (Lecture, Selfstudy) muss die Frame-Decoration
   *                sichtbar bzw. unsichtbar gemacht werden. Diese Methode macht die Frame not displayable
   *                durch einen Aufruf der Methode dispose. Dann kann die Frame-Decoration verndert werden.
   *                Anschliessend braucht die Frame nur mit einem setVisable auf true wieder sichtbar gemacht
   *                werden.
   *
   * Date: 		29.01.2006
   * @author sirocic
   */
  public void changeView()
  {

        //	 TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("CALL leviFrame.changeView:");

        // Das Anzeigen und Verstecken der Frame-Titelzeile geht nur, wenn die Frame vorher
        // per dispose aus der Anzeigehierarchie entfernt wurde.
        try
        {
            dispose();

            // Frame nach den aktuellen Attributen aufbauen
            composeFrame();
            setVisible(true);

        }
        catch (Exception e)
        {
            System.out.println("Internal Error ("+this.toString()+"):" + e.getMessage());
        }

        //	TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("END leviFrame.changeView");

  }

  /*
   * Methode:		changeMode
   *
   * Description:	Wechsel zwischen Editormodus und Simulationsmodus. Hier sind Buttons bzw. 
   * 				Menueintrge zu aktivieren oder deaktivieren usw..
   *
   * Date: 		29.01.2006
   * @author sirocic
   */
  public void changeMode()
  {

        //	 TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("CALL leviFrame.changeMode:");

        // Das Anzeigen und Verstecken der Frame-Titelzeile geht nur, wenn die Frame vorher
        // per dispose aus der Anzeigehierarchie entfernt wurde.

        // Frame nach den aktuellen Attributen aufbauen
        if (learningModule.getView() != LeviModule.LECTUREVIEW)
            composeFrame();
        else {
        	// In der Vorlesungsansicht braucht die Frame nicht neu aufgebaut werden, da
            // hier immer der Visualizer angezeigt wird!!!
        	// Es mssen aber eventuell Kommandos deaktiviert oder wieder aktiviert werden.
        	
            // KOMMANDOS DEAKTIVIEREN WHREND DER SIMULATION UND DANACH WIEDER AKTIVIEREN ...
            if (learningModule.getMode()==LeviModule.EDITORMODE) {
                // Kommandos, die whrend der Simulation nicht erlaubt waren, wieder aktivieren.
                getLearningModule().enableCommands();
            }
            else if (learningModule.getMode()==LeviModule.SIMULATIONMODE) {
                //Kommandos, die whrend der Simulation nicht erlaubt sind,  deaktivieren.
                getLearningModule().disableCommands();
            }
            
            refresh();
            
        }

        

        //	TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("END leviFrame.changeMode");

  }


  /*
   * @author sirocic
   *
   * Methode:		changeItemAction
   *
   * Description:	Changes the action of a menu item.
   *
   * Parameters: <menu> the JMenu where a item should be changed
   * 			 <oldtxt> the old text of the item
   * 			 <newtxt> the new text of the item
   *
   * Date: 		17.07.2005
   */
  public void changeItemAction(JMenu menu, String key, Action newAction)
  {
      int i;

    // TRACE OUTPUT
    if (LeviModule.isTraceActive()) System.out.println("CALL leviFrame.changeItemAction: menu="+menu.toString()
            + " key="+key+" action="+newAction.getClass().getName());

    for (i=0; i<menu.getItemCount(); i++) {
        if (menu.getItem(i)!= null) {
            // Item is not a seperator!!!
        	if (menu.getItem(i) instanceof LeviSubmenu)
        		changeItemAction((LeviSubmenu)menu.getItem(i), key, newAction);
        	else {
	            if (menu.getItem(i).getText().equals(getLearningModule().getRessources().getResourceByKey(key))) {
	                menu.getItem(i).setAction(newAction);
	            }
        	}
        }
    } // end for

    // TRACE OUTPUT
    if (LeviModule.isTraceActive()) System.out.println("END leviFrame.changeItemAction");

  }
  


  /*
   * @author sirocic
   *
   * Methode:		changeItemText
   *
   * Description:	Changes the text of a menu item.
   *
   * Parameters: <menu> the JMenu where a item should be changed
   * 			 <oldtxt> the old text of the item
   * 			 <newtxt> the new text of the item
   *
   * Date: 		17.07.2005
   */
  public void changeItemText(JMenu menu, String oldkey, String newkey)
  {
      int i;

    // TRACE OUTPUT
    if (LeviModule.isTraceActive()) System.out.println("CALL leviFrame.changeItemText: menu="+menu.toString()
            + " old="+oldkey+" new="+newkey);

    for (i=0; i<menu.getItemCount(); i++) {
        if (menu.getItem(i)!= null) {
            // Item is not a seperator!!!
        	if (menu.getItem(i) instanceof LeviSubmenu)
        		changeItemText((LeviSubmenu)menu.getItem(i), oldkey, newkey);
        	else {
	            if (menu.getItem(i).getText().equals(getLearningModule().getRessources().getResourceByKey(oldkey))) {
	                menu.getItem(i).setText(getLearningModule().getRessources().getResourceByKey(newkey));
	                (( LeviAbstractAction) menu.getItem(i).getAction()).setKeyLabel(newkey);
	            }
        	}
        }
    } // end for

    // TRACE OUTPUT
    if (LeviModule.isTraceActive()) System.out.println("END leviFrame.changeItemText");

  }
  
  /*
   * @author sirocic
   *
   * Methode:		setMenueItemEnabled
   *
   * Description:	Set the menu item enabled or disabled depending on the value.
   *
   * Parameters: <menu> the JMenu where a item should be changed
   * 			 <key> the key of the item
   * 			 <value> true if the menu item should be enabled, false otherwise.
   *
   * Date: 		30.08.2006
   */
  public void setMenueItemEnabled(JMenu menu, String key, boolean value)
  {
      int i;

    // TRACE OUTPUT
    if (LeviModule.isTraceActive()) System.out.println("CALL leviFrame.setMenueItemEnabled: menu="+menu.toString()
            + " key="+key+" value="+value);

    for (i=0; i<menu.getItemCount(); i++) {
        if (menu.getItem(i)!= null) {
            // Item is not a seperator!!!
        	if (( LeviAbstractAction) menu.getItem(i).getAction() != null) {
        		// Item ist kein Untermenu!!!
        		if (key.equals((( LeviAbstractAction) menu.getItem(i).getAction()).getKeyLabel())) 
        			menu.getItem(i).setEnabled(value);
        	}
        }
    } // end for

    // TRACE OUTPUT
    if (LeviModule.isTraceActive()) System.out.println("END leviFrame.setMenueItemEnabled");

  }

  /*

   * Methode:		refresh
   *
   * Description:	Veranlasst, das die Komponente neu gezeichnet wird.
   *
   * Date: 		17.07.2005
   * @author sirocic
   */
  public void refresh()
  {

        paintAll(getGraphics());

  }
  
  /*

   * Methode:		updateLanguageColorFonts
   *
   * Description:	Setzt die Attribute fr Farbe, Sprache und smallLabelFonts auf die aktuellen Werte aus
   * 				der Konfiguration bzw. Ressourcen. Diese Methode sollte aufgerufen werden, wenn
   * 				sich in den Einstellungen etwas gendert hat. 
   *
   * Date: 			19.07.2005
   * @author sirocic
   */
  public void updateLanguageColorFonts()
  {

	    // Teile des Lernmoduls neu laden, wegen evtl. Sprach-, Farbwechsel usw..
	    getLearningModule().updateUIManager();
	  	if (getLearningModule().getHelp() != null)
	  		getLearningModule().getHelp().updateLanguageColorFonts();
	  	if (getLearningModule().getEditor()!= null)
	  		getLearningModule().getEditor().updateLanguageColorFonts();
	  	if (getLearningModule().getVisualizer()!= null)
	  		getLearningModule().getVisualizer().updateLanguageColorFonts();
	  	getMenubar().updateLanguageColorFonts();
	  	getPopupMenu().updateLanguageColorFonts();
	  	getLearningModule().updateDialogsColorLanguageFonts();
	  	
        paintAll(getGraphics());

  }
  
  // ACCESS METHODES...
  public LeviModule getLearningModule()
  {
      return learningModule;
  }
  public void setLearningModule(LeviModule c)
  {
     learningModule=c;
  }

  public LeviMenu getMenubar()
  {
      return menubar;
  }
  public void setMenubar(LeviMenu m)
  {
      menubar=m;
  }

  public LeviPopup getPopupMenu()
  {
      return popupMenu;
  }

  public void setPopupMenu(LeviPopup p)
  {
      popupMenu=p;
  }

  public JSplitPane getSelfstudyPanel()
  {
      return SelfstudyPanel;
  }
  public void setSelfstudyPanel(JSplitPane s)
  {
      SelfstudyPanel=s;
  }
  public int getSelfstudyDividerLocation()
  {
      return selfstudyDividerLocation;
  }
  public void setSelfstudyDividerLocation(int s)
  {
      selfstudyDividerLocation=s;
      SelfstudyPanel.setDividerLocation(selfstudyDividerLocation);
  }

  /*
   * @author sirocic
   *
   * Methode:     setHelpPanelVisible
   *
   * Description:	Shows or hides the panel of the help. Divider size have to be set to null
   * 				or
   *
   * Parameters: <v> true for showing the help panel
   *
   * Date: 		30.07.2005
   */
  public void setHelpPanelVisible(boolean v)
  {
      // TRACE OUTPUT
      if (LeviModule.isTraceActive()) System.out.println("CALL leviFrame.setHelpPanelVisible: v="+v);

      learningModule.getHelp().setVisible(v);

        if (v) {
          // setting the size of the help panel
          getLearningModule().getHelp().setSize(LeviHelp.HELPWIDTH, SelfstudyPanel.getHeight());

            // divider location must set now, if the help was unvisible!
            SelfstudyPanel.setDividerLocation(this.selfstudyDividerLocation);
            //make divider visible again
            SelfstudyPanel.setDividerSize(DIVIDERSIZE);

        } else {
            // make divider unvisible!
            SelfstudyPanel.setDividerSize(0);

        }


      // TRACE OUTPUT
      if (LeviModule.isTraceActive()) System.out.println("END leviFrame.setHelpPanelVisible");

  }

  public Dimension getFullscreenSize() {
      return fullscreenSize;
  }
  public Dimension getNormalSize() {
      return normalSize;
  }
  public int getDefaultDividerOffset() {
      return DIVIDEROFFSET;
  }

  public void showErrorMessageDialog(String errkey, String data1, String methode)
  {
	  String errorMsg, errorTitle;
	  
	  errorMsg =String.format(learningModule.getRessources().getResourceByKey(errkey),data1);
	  errorTitle = learningModule.getRessources().getResourceByKey("dlg.error.title")+ " ("+methode+")";
	  JOptionPane.showMessageDialog(this,errorMsg, errorTitle, JOptionPane.ERROR_MESSAGE);	
	  
  }
  public void showErrorMessageDialog(String errkey, String methode)
  {
	  String errorMsg, errorTitle;
	  
	  errorMsg =learningModule.getRessources().getResourceByKey(errkey);
	  errorTitle = learningModule.getRessources().getResourceByKey("dlg.error.title")+ " ("+methode+")";
	  JOptionPane.showMessageDialog(this,errorMsg, errorTitle, JOptionPane.ERROR_MESSAGE);	
	  
  }
  public void showErrorMessageDialog(String errkey, String data1, String data2, String methode)
  {
	  String errorMsg, errorTitle;
	  
	  errorMsg =String.format(learningModule.getRessources().getResourceByKey(errkey),data1, data2);
	  errorTitle = learningModule.getRessources().getResourceByKey("dlg.error.title")+ " ("+methode+")";
	  JOptionPane.showMessageDialog(this,errorMsg, errorTitle, JOptionPane.ERROR_MESSAGE);	
	  
  }
  public void showErrorMessageDialog(String errkey, String data1, String data2, String data3, String methode)
  {
	  String errorMsg, errorTitle;
	  
	  errorMsg =String.format(learningModule.getRessources().getResourceByKey(errkey),data1, data2, data3);
	  errorTitle = learningModule.getRessources().getResourceByKey("dlg.error.title")+ " ("+methode+")";
	  JOptionPane.showMessageDialog(this,errorMsg, errorTitle, JOptionPane.ERROR_MESSAGE);	
	  
  }
  public void showErrorMessageDialog(String errkey, String data1, String data2, String data3, String data4, String data5, String methode)
  {
	  String errorMsg, errorTitle;
	  
	  errorMsg =String.format(learningModule.getRessources().getResourceByKey(errkey),data1, data2, data3, data4, data5);
	  errorTitle = learningModule.getRessources().getResourceByKey("dlg.error.title")+ " ("+methode+")";
	  JOptionPane.showMessageDialog(this,errorMsg, errorTitle, JOptionPane.ERROR_MESSAGE);	
	  
  }
  public void showErrorMessageDialog(String errkey, String data1, String data2, String data3, String data4, String methode)
  {
	  String errorMsg, errorTitle;
	  
	  errorMsg =String.format(learningModule.getRessources().getResourceByKey(errkey),data1, data2, data3, data4);
	  errorTitle = learningModule.getRessources().getResourceByKey("dlg.error.title")+ " ("+methode+")";
	  JOptionPane.showMessageDialog(this,errorMsg, errorTitle, JOptionPane.ERROR_MESSAGE);	
	  
  }
  public void showWarningMessageDialog(String errkey, String methode)
  {
	  String errorMsg, errorTitle;
	  
	  errorMsg =String.format(learningModule.getRessources().getResourceByKey(errkey));
	  errorTitle = learningModule.getRessources().getResourceByKey("dlg.warning.title")+ " ("+methode+")";
	  JOptionPane.showMessageDialog(this,errorMsg, errorTitle, JOptionPane.WARNING_MESSAGE);	
	  
  }
  public void showOkOptionMessageDialog(String message, String title)
  {
	  JOptionPane.showMessageDialog(this, message, title, JOptionPane.OK_OPTION);
	  
  }
  public void showInfoMessageDialog(String errkey)
  {
	  String errorMsg;
	  
	  errorMsg =String.format(learningModule.getRessources().getResourceByKey(errkey));
	  JOptionPane.showMessageDialog(this,errorMsg, "", JOptionPane.INFORMATION_MESSAGE);	
	  
  }
  public void showWarningMessageDialog(String errkey, String data1, String data2, String methode)
  {
	  String errorMsg, errorTitle;
	  
	  errorMsg =String.format(learningModule.getRessources().getResourceByKey(errkey),data1, data2);
	  errorTitle = learningModule.getRessources().getResourceByKey("dlg.warning.title")+ " ("+methode+")";
	  JOptionPane.showMessageDialog(this,errorMsg, errorTitle, JOptionPane.WARNING_MESSAGE);	
	  
  }
  public void showWarningMessageDialog(String errkey, String data1, String methode)
  {
	  String errorMsg, errorTitle;
	  
	  errorMsg =String.format(learningModule.getRessources().getResourceByKey(errkey),data1);
	  errorTitle = learningModule.getRessources().getResourceByKey("dlg.warning.title")+ " ("+methode+")";
	  JOptionPane.showMessageDialog(this,errorMsg, errorTitle, JOptionPane.WARNING_MESSAGE);	
	  
  }
}

