/* leviFrame.java
 * 
 * Created on 12.05.2005
 * Autor: Birgit Sirocic
 * 
 */
package learningVisualizations.gui;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

import learningVisualizations.LeviModule;
import learningVisualizations.gui.actions.LeviAbstractAction;
import learningVisualizations.gui.actions.LeviActionSetActiveScenario;
import learningVisualizations.scenario.LeviScenario;
import learningVisualizations.scenario.LeviScenarioInterface;

/**
 * @author sirocic
 *
 * Class: leviPopup
 * 
 * Description: The class "leviPopup" is the general class for a
 * 				popup menu. 
 * 
 * Date:	12.05.2005
 */
public class LeviPopup extends JPopupMenu
{
	static final long serialVersionUID=1;
	
	private LeviFrame myFrame;
	InputMap imap;
    ActionMap amap;

	// Submenus 
	private LeviSubmenu myMenuScenario;
	private LeviSubmenu myMenuView;
	private LeviSubmenu myMenuSimulation;
	private LeviSubmenu myMenuConfiguration;
	private LeviSubmenu mySubmenuLanguage;
	private LeviSubmenu mySubmenuTempo;
	private LeviSubmenu mySubmenuConfigSimulator;
	private LeviSubmenu mySubmenuConfigScenario;
	
  /* @author sirocic
  *
  * Methode:	Constructors
  * 
  * Date: 11.10.2005
  *
  */
  public LeviPopup()
  { 
	  	super();
	  	this.myFrame = null;
	  	this.myMenuScenario = new LeviSubmenu();
	  	this.myMenuSimulation = new LeviSubmenu();
	  	this.myMenuView = new LeviSubmenu();
	  	this.myMenuConfiguration = new LeviSubmenu();
    	this.mySubmenuLanguage = new LeviSubmenu();
    	this.mySubmenuTempo = new LeviSubmenu();
    	this.mySubmenuConfigSimulator = new LeviSubmenu();
    	this.mySubmenuConfigScenario = new LeviSubmenu();
  }
  public LeviPopup(LeviFrame myFrame)
  { 
  	super();
  	
  	this.myFrame = myFrame;

    imap = myFrame.getRootPane().getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
    amap = myFrame.getRootPane().getActionMap();
    
  	// BUILD SUBMENU "Scenario" 
  	myMenuScenario = new LeviSubmenu(this.myFrame, "menu.scenario");
  	addToMenuScenario("learningVisualizations.gui.actions.LeviActionOpenScenario", "scenario.open", "key.open", null, null);
  	addToMenuScenario("learningVisualizations.gui.actions.LeviActionQuit", "scenario.quit", "key.quit",  null, null);
	this.add(myMenuScenario);
	
 	// BUILD SUBMENU "SIMULATION" 
  	myMenuSimulation = new LeviSubmenu(this.myFrame,"menu.simulation");
  	addToMenuSimulation("learningVisualizations.gui.actions.LeviActionPlaySimulation", "sim.play", "key.sim.play",  null, null);
  	addToMenuSimulation("learningVisualizations.gui.actions.LeviActionForwardSimulation", "sim.forward", "key.sim.forward", null, null);
  	addToMenuSimulation("learningVisualizations.gui.actions.LeviActionPauseSimulation", "sim.pause", "key.sim.pause",  null, null);
  	addToMenuSimulation("learningVisualizations.gui.actions.LeviActionStopSimulation", "sim.stop", "key.sim.stop",  null, null);
  	mySubmenuTempo = new LeviSubmenu(this.myFrame, "sim.tempo");
  	addToSubmenuTempo("learningVisualizations.gui.actions.LeviActionChangeTempo", "sim.faster", "key.sim.faster",  null, null);
  	addToSubmenuTempo("learningVisualizations.gui.actions.LeviActionChangeTempo", "sim.slower", "key.sim.slower",  null, null);
  	addSubmenuToMenuSimulation(mySubmenuTempo);
  	this.add(myMenuSimulation);
  	
 	// BUILD SUBMENU "VIEW" 
  	myMenuView = new LeviSubmenu(this.myFrame,"menu.viewmode");
  	addToMenuView("learningVisualizations.gui.actions.LeviActionChangeView", "viewmode.study", "key.mode.study",  null, null);
  	myMenuView.addSeparator();
  	this.add(myMenuView);
  	
  	// BUILD SUBMENU "KONFIGURATION" 
  	myMenuConfiguration = new LeviSubmenu(this.myFrame,"menu.konfig");
  	
  	mySubmenuConfigScenario = new LeviSubmenu(this.myFrame, "konfig.scenario");
  	addToSubmenuScenario("learningVisualizations.gui.actions.LeviActionSelectDefaultScenario", "konfig.default", "key.default", null, null);
  	addSubmenuToMenuConfiguration(mySubmenuConfigScenario);
  	
  	mySubmenuConfigSimulator = new LeviSubmenu(this.myFrame, "konfig.sim");
  	addToSubmenuSimulator("learningVisualizations.gui.actions.LeviActionSaveSleepFactor", "konfig.sleeping", "key.sleeping",  null, null);
  	addSubmenuToMenuConfiguration(mySubmenuConfigSimulator);
  	
  	mySubmenuLanguage = new LeviSubmenu(this.myFrame, "konfig.language");
  	addToSubmenuLanguage("learningVisualizations.gui.actions.LeviActionLanguageKonfiguration", "konfig.language.de", "key.konfig.language.de", null, null,"de");
  	addToSubmenuLanguage("learningVisualizations.gui.actions.LeviActionLanguageKonfiguration", "konfig.language.en", "key.konfig.language.en", null, null, "en");
  	addSubmenuToMenuConfiguration(mySubmenuLanguage);
  	
  	addToMenuConfiguration("learningVisualizations.gui.actions.LeviActionColorKonfiguration", "konfig.color", "key.konfig.color", null, null);
  	addToMenuConfiguration("learningVisualizations.gui.actions.LeviActionFontKonfiguration", "konfig.font", "key.konfig.font", null, null);
  	
	this.add(myMenuConfiguration);

	// FARBEN SETZEN ...
	setPanelColors();

	// FONTS SETZEN ...
    setPanelFont();
	  

  }
  
  /*
   * Methode:		paintComponent
   * 
   * Description:	Calls the paintComponent methode of the JPanel.
   *    
   * Parameters:    
   * 
   * Date: 		    13.10.2005
   * @author        sirocic
   */	
  public void paintComponent(Graphics g) {
     
	  // TRACE OUTPUT
	  if (LeviModule.isTraceActive()) System.out.println("CALL leviPopup.paintComponent:");
	  
	  // here calling the paint methode of the JPanel
	  super.paintComponent(g);
	  
 	  // TRACE OUTPUT
 	  if (LeviModule.isTraceActive()) System.out.println("END leviPopup.paintComponent");

  }
  /*
   * Methode:		setDialogFont
   * 
   * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
   * 				Werte. 
   *    
   * Parameters:   Aktuelle Font fr Labels und Textfelder.
   * 
   * Date: 		 23.04.2007
   * @author        sirocic
   */	
  private void setPanelFont() {
	  Font font;
	   
 	 font = (Font)myFrame.getLearningModule().getConfiguration().getSettingByKey("FontMenu");
 	   
 	 setFont(font);
	   	  
  }
  
  /*
   * Methode:		setDialogTexte
   * 
   * Description:	Setz alle Text-Bausteine des Dialoges neu. Wichtig fr Sprachwechsel.
   *    
   * Parameters:   
   * 
   * Date: 		 23.04.2007
   * @author        sirocic
   */	
  private void setPanelTexte() {

	  //		 Wichtig beim dynamischen Sprachwechsel!
	  // Textbausteine fr die Untermenus neu aus den Ressourcen holen und setzen.
	  // Die Texte fr die Menueintrge werden in der paintComponent-Methode der Klasse
	  // leviSubmenu gesetzt.
	  myMenuScenario.setText(myFrame.getLearningModule().getRessources().getResourceByKey("menu.scenario"));
	  myMenuView.setText(myFrame.getLearningModule().getRessources().getResourceByKey("menu.viewmode"));
	  myMenuSimulation.setText(myFrame.getLearningModule().getRessources().getResourceByKey("menu.simulation"));
	  myMenuConfiguration.setText(myFrame.getLearningModule().getRessources().getResourceByKey("menu.konfig"));
	  mySubmenuLanguage.setText(myFrame.getLearningModule().getRessources().getResourceByKey("konfig.language"));
	  mySubmenuTempo.setText(myFrame.getLearningModule().getRessources().getResourceByKey("sim.tempo"));
	  
  }
  
  /*
   * Methode:		setDialogColors
   * 
   * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
   * 				Werte. 
   *    
   * Parameters:   Aktuelle Farbwerte fr Hintergrund und Vordergrund.
   * 
   * Date: 		 23.04.2007
   * @author        sirocic
   */	
  private void setPanelColors() {
	  Color backgroundColor,  foregroundColor;
 	   
 	   if (myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentBackground")!=null) {
 		   backgroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentBackground");
 		   foregroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentForeground");
 	   }
 	   else {
 		   backgroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorModuleBackground");
 		   foregroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorModuleForeground");
 		   
 	   }  
 	   
	   // 	 Hintergrund und Zeichenfarbe auf aktuelle Werte setzen
      setBackground(backgroundColor);
	  setForeground(foregroundColor);

  }
  
  /*
   * Methode:		updateLanguageColorFonts
   *
   * Description:	Ldt sprachabhngige Teile neu. In der Toolbar knnen z.B. sprachabhngige
   * 				Buttons sein 
   *
   * Date: 		12.03.2007
   * 
   * @author sirocic
   */
  public void updateLanguageColorFonts()
  {
	  int i;
      Component item;
      
	   //	 SUBKOMPONENTE UPDATEN...
	   for(i=0; i < this.getComponentCount(); i++) {
          item = this.getComponent(i);
          if (item != null)  {
             // bei den Menueintrgen, die selber Untermens sind, muss upgedatet werden
             if (item instanceof LeviSubmenu)
                 ((LeviSubmenu)item).updateLanguageColorFonts();
          }
       }
	  
	   // 	 FARBWECHSEL ...
	   setPanelColors();

	   // FONTWECHSEL
	   setPanelFont();
	  
	   // SPRACHWECHSEL ...
	   setPanelTexte();
	   
  }
  
  /*
  *
  * Methode:		createMenuAction
  * 
  * Description:	Creates a new action object for a menu item specified by the parameter. 
  *                 <actionName> is the name of the action class.
  *                 <keyLabel> is the ressource key of the new menu item label. <keyStroke> is the
  *                 ressource key of the accelerator key. <inputEvent> is the integer code of the event. 
  *                 <icon> is the icon which will be displayed next to the menu item and <tip> is the tool tip
  *                 of the new menu item.
  * 
  * Date: 		    29.08.2005
  * @author         sirocic
  */
  private Action createMenuAction(String actionName, String keyLabel, String keyStroke, Icon icon, String tip)
  {
  	Action action;
  	
	// TRACE OUTPUT
	if (LeviModule.isTraceActive()) System.out.println("CALL leviPopup.createMenuAction: actionName="+actionName);

	try
	{
		action = (Action) Class.forName(actionName).newInstance();
		action.putValue(Action.NAME, myFrame.getLearningModule().getRessources().getResourceByKey(keyLabel));
		((LeviAbstractAction) action).setLearningModule(myFrame.getLearningModule());
		((LeviAbstractAction) action).setKeyLabel(keyLabel);
		if (myFrame.getLearningModule().getRessources().getResourceByKey(keyStroke).length() != 0)
		   action.putValue(Action.ACCELERATOR_KEY, 
				   KeyStroke.getKeyStroke(myFrame.getLearningModule().getRessources().getResourceByKey(keyStroke)));
        if ( icon != null)
        	action.putValue(Action.SMALL_ICON, icon);
        if ( tip != null)
            action.putValue(Action.SHORT_DESCRIPTION, tip);
        
        // Jetzt noch dieselbe Action an die Tastenkombi hngen fr den Visualizer
        if (myFrame.getLearningModule().getRessources().getResourceByKey(keyStroke).length() != 0) {
	        imap.put(KeyStroke.getKeyStroke(myFrame.getLearningModule().getRessources().getResourceByKey(keyStroke)), keyLabel);
	        amap.put(keyLabel, action);
        }
        
		// TRACE OUTPUT
		if (LeviModule.isTraceActive()) System.out.println("END leviPopup.createMenuAction");
	
        return action;

	}
	catch (Exception e)
	{
		System.out.println("Internal Error (leviPopup.createMenuAction):" + e.toString());
		
		// TRACE OUTPUT
		if (LeviModule.isTraceActive()) System.out.println("END leviPopup.createMenuAction with error");
	
		return null;
	}
		
  }
  
  /*
  *
  * Methode:		createMenuScenarioAction
  * 
  * Description:	Creates a new action object for a menu item specified by the parameter. 
  *                 <actionName> is the name of the action class.
  *                 <keyLabel> is the ressource key of the new menu item label. <keyStroke> is the
  *                 ressource key of the accelerator key. <inputEvent> is the integer code of the event. 
  *                 <icon> is the icon which will be displayed next to the menu item and <tip> is the tool tip
  *                 of the new menu item.
  * 
  * Date: 		    29.08.2005
  * @author         sirocic
  */
  private Action createMenuScenarioAction(String actionName, LeviScenario scenario)
  {
  	Action action;
  	
	// TRACE OUTPUT
	if (LeviModule.isTraceActive()) System.out.println("CALL leviMenu.createMenuScenarioAction: actionName="+actionName);

	try
	{
		action = (Action) Class.forName(actionName).newInstance();
		action.putValue(Action.NAME, scenario.getName());
		action.putValue("SCENARIO", scenario);
		((LeviAbstractAction) action).setLearningModule(myFrame.getLearningModule());
		((LeviAbstractAction) action).setKeyLabel("");
        action.putValue(Action.SHORT_DESCRIPTION, "");
        
		// TRACE OUTPUT
		if (LeviModule.isTraceActive()) System.out.println("END leviMenu.createMenuAction");
	
        return action;

	}
	catch (Exception e)
	{
		System.out.println("Internal Error (leviMenu.createMenuScenarioAction):" + e.toString());
		
		// TRACE OUTPUT
		if (LeviModule.isTraceActive()) System.out.println("END leviMenu.createMenuScenarioAction with error");
	
		return null;
	}
		
  }
  /*
  *
  * Methode:		addSubmenu
  * 
  * Description:	Adds a new submenu to the menu given by the parameter <menu>. 
  *                 The submenu is given by the parameter <submenu>.
  * 
  * Date: 		    29.08.2005
  * @author         sirocic
  */
  private void addSubmenu(JMenu menu, JMenu submenu)
  {
  	
	// TRACE OUTPUT
	if (LeviModule.isTraceActive()) System.out.println("CALL leviPopup.addSubmenu: menu="+menu.getName());
	
	menu.add(submenu);

	// TRACE OUTPUT
	if (LeviModule.isTraceActive()) System.out.println("END leviPopup.addSubmenu");
	
  }
  
  // !!!!
  // scenario MENU
  
  public JMenu getMenuScenario()
  {
  	return myMenuScenario;
  }
  /*
  *
  * Methode:		setMenuScenarioLabel
  * 
  * Description:	Set the label of the scenario menu to the given string <newName>.
  * 
  * Date: 		    29.08.2005
  * @author         sirocic
  */
  public void setMenuScenarioLabel(String newName)
  {
	// TRACE OUTPUT
	if (LeviModule.isTraceActive()) System.out.println("CALL leviPopup.setMenuScenarioLabel: newName="+newName);
	
  	myMenuScenario.setText(newName);
  	
	// TRACE OUTPUT
	if (LeviModule.isTraceActive()) System.out.println("END leviPopup.setMenuScenarioLabel");
	
  }
  
  /*
  *
  * Methode:		addToMenuScenario
  * 
  * Description:	Adds a new item to the Scenario menu. The menu item will execute the action given by <actionName>.
  *                 <keyLabel> is the ressource key of the new menu item label. <keyStroke> is the
  *                 ressource key of the accelerator key. <inputEvent> is the integer code of the event. 
  *                 <icon> is the icon which will be displayed next to the menu item and <tip> is the tool tip
  *                 of the new menu item.
  * 
  * Date: 		    29.08.2005
  * @author         sirocic
  */
  public void addToMenuScenario(String actionName, String keyLabel, String keyStroke, Icon icon, String tip)
  {
  	Action action;
  	
	// TRACE OUTPUT
	if (LeviModule.isTraceActive()) System.out.println("CALL leviPopup.addToMenuScenario: actionName="+actionName);

    action = createMenuAction(actionName, keyLabel, keyStroke, icon, tip);

    if (action != null) 
		// append to the Scenario menu 
		myMenuScenario.add(action);

  	
	// TRACE OUTPUT
	if (LeviModule.isTraceActive()) System.out.println("END leviPopup.addToMenuScenario");
	
  }
  
  /*
  *
  * Methode:		addSubmenuToMenuScenario
  * 
  * Description:	Adds a new submenu to the Scenario menu. The submenu is given by the parameter <submenu>
  * 
  * Date: 		    29.08.2005
  * @author         sirocic
  */
  public void addSubmenuToMenuScenario(JMenu submenu)
  {  	
	// TRACE OUTPUT
	if (LeviModule.isTraceActive()) System.out.println("CALL leviPopup.addSubmenuToMenuScenario: submenu="+submenu.getName());
	
	addSubmenu(myMenuScenario, submenu);
	
	// TRACE OUTPUT
	if (LeviModule.isTraceActive()) System.out.println("END leviPopup.addSubmenuToMenuScenario");
  }
  
  
  // !!!!!
  // VIEW MENU
  
  public JMenu getMenuView()
  {
  	return myMenuView;
  }
  
  /*
  *
  * Methode:		addToMenuView
  * 
  * Description:	Adds a new item to the view menu. The menu item will execute the action given by <actionName>.
  *                 <keyLabel> is the ressource key of the new menu item label. <keyStroke> is the
  *                 ressource key of the accelerator key. <inputEvent> is the integer code of the event. 
  *                 <icon> is the icon which will be displayed next to the menu item and <tip> is the tool tip
  *                 of the new menu item.
  * 
  * Date: 		    29.08.2005
  * @author         sirocic
  */
  public void addToMenuView(String actionName, String keyLabel, String keyStroke, Icon icon, String tip)
  {
    Action action;
    JMenuItem item;
    int i;
    boolean trennlinie;
  	
	// TRACE OUTPUT
	if (LeviModule.isTraceActive()) System.out.println("CALL leviPopup.addToMenuView: actionName="+actionName);

    action = createMenuAction(actionName, keyLabel, keyStroke, icon, tip);

    if (action != null) {
		// append to the Scenario menu 
    	// Da der neue eintrag kein Szenario ist, muss er vor der 
    	// Trennlinie eingefgt werden.
    	i=0;
    	trennlinie=false;
    	while ( (!trennlinie) && (i < myMenuView.getItemCount())) {
            item = myMenuView.getItem(i);
            if (item != null) {
            	if (item.getAction()==null)
            		trennlinie=true;
            }
            i++;
        }
    	if (trennlinie) {
    		item = new JMenuItem();
    		item.setAction(action);
    		myMenuView.add(item, i);
    	}
    	else
    		myMenuView.add(action);
    }
	// TRACE OUTPUT
	if (LeviModule.isTraceActive()) System.out.println("END leviPopup.addToMenuView");
	
  }
  
  /*
  *
  * Methode:		addSubmenuToMenuView
  * 
  * Description:	Adds a new submenu to the view menu. The submenu is given by the parameter <submenu>
  * 
  * Date: 		    29.08.2005
  * @author         sirocic
  */
  public void addSubmenuToMenuView(JMenu submenu)
  {
  	
	// TRACE OUTPUT
	if (LeviModule.isTraceActive()) System.out.println("CALL leviPopup.addSubmenuToMenuView: submenu="+submenu.getName());
	
	addSubmenu(myMenuView, submenu);
  	
	// TRACE OUTPUT
	if (LeviModule.isTraceActive()) System.out.println("END leviPopup.addSubmenuToMenuView");
  }

  /*
  *
  * Methode:		addToScenarioView
  * 
  * Description:	Adds a new item for a scenario in the view menu. 
  * 				The relevant scenario is given by the parameter.
  * Date: 		    15.05.2006
  * @author         sirocic
  */
  public void addToScenarioView(LeviScenario scenario)
  {
    Action action;
  	
	// TRACE OUTPUT
	if (LeviModule.isTraceActive()) System.out.println("CALL leviMenu.addToScenarioView: scenarioName="+scenario.getName());

    action = createMenuScenarioAction("learningVisualizations.gui.actions.LeviActionSetActiveScenario",scenario);

    if (action != null) 
		// append to the Scenario menu 
    	myMenuView.add(action);
	
	// TRACE OUTPUT
	if (LeviModule.isTraceActive()) System.out.println("END leviMenu.addToScenarioView");
	
  }
  
  /*
  *
  * Methode:		removeFromScenarioView
  * 
  * Description:	Removes the item for the givven scenario from the view menu. 
  * 				The relevant scenario is given by the parameter.
  * Date: 		    15.05.2006
  * @author         sirocic
  */
  public void removeFromScenarioView(LeviScenarioInterface scenario)
  {
  	int i;
  	JMenuItem item;
  	
	// TRACE OUTPUT
	if (LeviModule.isTraceActive()) System.out.println("CALL leviMenu.addToScenarioView: scenarioName="+scenario.getName());

	if (myMenuView.getItemCount() > 0) {
		i=0;
		while (i < myMenuView.getItemCount()) {
	        item = myMenuView.getItem(i);
	        if ((item!=null)&& (item.getText()!=null)){
	        	if (item.getText().equals(scenario.getName()))
	    			myMenuView.remove(item);
	        }
	        i++;
	    }
	}
	
	// TRACE OUTPUT
	if (LeviModule.isTraceActive()) System.out.println("END leviMenu.addToScenarioView");
	
  }
  
  /*
  *
  * Methode:		updateViewMenu
  * 
  * Description:	Updates the menu view. All scenarios are enable
  * 				but not the current shown one.
  * 
  * Date: 		    15.05.2006
  * @author         sirocic
  */
  public void updateViewMenu()
  {
	  int i;
      JMenuItem item;
      LeviScenario itemScenario;
      
      // TRACE OUTPUT
      if (LeviModule.isTraceActive()) System.out.println("CALL leviPopup.updateViewMenu:");

      //  ITEMS UPDATEN...
      for(i=0; i < myMenuView.getItemCount(); i++) {
          item = myMenuView.getItem(i);
          if (item != null)  {
             if (item.getAction() instanceof LeviActionSetActiveScenario) {
            	 itemScenario = (LeviScenario) item.getAction().getValue("SCENARIO");
            	 item.setText(itemScenario.getName());
            	 if(myFrame.getLearningModule().getScenario().getName().equals(itemScenario.getName())) {
            		 item.setEnabled(false);
                 } else {
                	 item.setEnabled(true);
                 }
             }
          }
      }
      
    // TRACE OUTPUT
  	if (LeviModule.isTraceActive()) System.out.println("END leviPopup.updateViewMenu");
  	
  }
 

  // !!!!
  // SIMULATION MENU
  
  public JMenu getMenuSimulation()
  {
  	return myMenuSimulation;
  }
  
  /*
  *
  * Methode:		addToMenuSimulation
  * 
  * Description:	Adds a new item to the simulation menu. The menu item will execute the action given by <actionName>.
  *                 <keyLabel> is the ressource key of the new menu item label. <keyStroke> is the
  *                 ressource key of the accelerator key. <inputEvent> is the integer code of the event. 
  *                 <icon> is the icon which will be displayed next to the menu item and <tip> is the tool tip
  *                 of the new menu item.
  * 
  * Date: 		    29.08.2005
  * @author         sirocic
  */
  public void addToMenuSimulation(String actionName, String keyLabel, String keyStroke, Icon icon, String tip)
  {
    Action action;
  	
	// TRACE OUTPUT
	if (LeviModule.isTraceActive()) System.out.println("CALL leviPopup.addToMenuSimulation: actionName="+actionName);

    action = createMenuAction(actionName, keyLabel, keyStroke, icon, tip);

    if (action != null) 
		// append to the Scenario menu 
		myMenuSimulation.add(action);

	// TRACE OUTPUT
	if (LeviModule.isTraceActive()) System.out.println("END leviPopup.addToMenuSimulation");
	
  }
  
  /*
  *
  * Methode:		addSubmenuToMenuSimulation
  * 
  * Description:	Adds a new submenu to the Scenario menu. The submenu is given by the parameter <submenu>
  * 
  * Date: 		    29.08.2005
  * @author         sirocic
  */
  public void addSubmenuToMenuSimulation(JMenu submenu)
  {
  	
	// TRACE OUTPUT
	if (LeviModule.isTraceActive()) System.out.println("CALL leviPopup.addSubmenuToMenuSimulation: submenu="+submenu.getName());
	
	addSubmenu(myMenuSimulation,submenu);
	
	// TRACE OUTPUT
	if (LeviModule.isTraceActive()) System.out.println("END leviPopup.addSubmenuToMenuSimulation");
	
  }
  
  /*
  *
  * Methode:		addToSubmenuTempo
  * 
  * Description:	Adds a new item to the submenu Tempo. The menu item will execute the action given by <actionName>.
  *                 <keyLabel> is the ressource key of the new menu item label. <keyStroke> is the
  *                 ressource key of the accelerator key. <inputEvent> is the integer code of the event. 
  *                 <icon> is the icon which will be displayed next to the menu item and <tip> is the tool tip
  *                 of the new menu item.
  *                 The parameter <language> contains the information about the language which will be
  *                 set by this action. 
  * 
  * Date: 		    15.05.2006
  * @author         sirocic
  */
  public void addToSubmenuTempo(String actionName, String keyLabel, String keyStroke, Icon icon, String tip)
  {
    Action action;
  	
	// TRACE OUTPUT
	if (LeviModule.isTraceActive()) System.out.println("CALL leviPopup.addToSubmenuTempo: actionName="+actionName);

    action = createMenuAction(actionName, keyLabel, keyStroke, icon, tip);

    if (action != null) 
		// append to the Scenario menu 
		mySubmenuTempo.add(action);
	
	// TRACE OUTPUT
	if (LeviModule.isTraceActive()) System.out.println("END leviPopup.addToSubmenuTempo");
	
  }
  /*
  *
  * Methode:		addToSubmenuScenario
  * 
  * Description:	Adds a new item to the submenu Scenario in the configuration menu. The menu item will execute the action given by <actionName>.
  *                 <keyLabel> is the ressource key of the new menu item label. <keyStroke> is the
  *                 ressource key of the accelerator key. <inputEvent> is the integer code of the event. 
  *                 <icon> is the icon which will be displayed next to the menu item and <tip> is the tool tip
  *                 of the new menu item.
  * 
  * Date: 		    18.05.2006
  * @author         sirocic
  */
  public void addToSubmenuScenario(String actionName, String keyLabel, String keyStroke, Icon icon, String tip)
  {
    Action action;
  	
	// TRACE OUTPUT
	if (LeviModule.isTraceActive()) System.out.println("CALL leviPopup.addToSubmenuScenario: actionName="+actionName);

    action = createMenuAction(actionName, keyLabel, keyStroke, icon, tip);

    if (action != null) 
		// append to the Scenario menu 
    	mySubmenuConfigScenario.add(action);
	
	// TRACE OUTPUT
	if (LeviModule.isTraceActive()) System.out.println("END leviPopup.addToSubmenuScenario");
	
  }
  /*
  *
  * Methode:		addToSubmenuSimulator
  * 
  * Description:	Adds a new item to the submenu Scenario in the configuration menu. The menu item will execute the action given by <actionName>.
  *                 <keyLabel> is the ressource key of the new menu item label. <keyStroke> is the
  *                 ressource key of the accelerator key. <inputEvent> is the integer code of the event. 
  *                 <icon> is the icon which will be displayed next to the menu item and <tip> is the tool tip
  *                 of the new menu item.
  * 
  * Date: 		    18.05.2006
  * @author         sirocic
  */
  public void addToSubmenuSimulator(String actionName, String keyLabel, String keyStroke, Icon icon, String tip)
  {
    Action action;
  	
	// TRACE OUTPUT
	if (LeviModule.isTraceActive()) System.out.println("CALL leviPopup.addToSubmenuSimulator: actionName="+actionName);

    action = createMenuAction(actionName, keyLabel, keyStroke, icon, tip);

    if (action != null) 
		// append to the Scenario menu 
    	mySubmenuConfigSimulator.add(action);
	
	// TRACE OUTPUT
	if (LeviModule.isTraceActive()) System.out.println("END leviPopup.addToSubmenuSimulator");
	
  }
  
  // !!!!!
  //KONFIGURATION MENU
  
  public JMenu getMenuKonfiguration()
  {
  	return myMenuConfiguration;
  }
  public LeviSubmenu getSubmenuLanguage()
  {
  	return mySubmenuLanguage;
  }
  
  /*
  *
  * Methode:		addToMenuConfiguration
  * 
  * Description:	Adds a new item to the configuration menu. The menu item will execute the action given by <actionName>.
  *                 <keyLabel> is the ressource key of the new menu item label. <keyStroke> is the
  *                 ressource key of the accelerator key. <inputEvent> is the integer code of the event. 
  *                 <icon> is the icon which will be displayed next to the menu item and <tip> is the tool tip
  *                 of the new menu item.
  * 
  * Date: 		    29.08.2005
  * @author         sirocic
  */
  public void addToMenuConfiguration(String actionName, String keyLabel, String keyStroke, Icon icon, String tip)
  {
    Action action;
  	
	// TRACE OUTPUT
	if (LeviModule.isTraceActive()) System.out.println("CALL leviPopup.addToMenuConfiguration: actionName="+actionName);

    action = createMenuAction(actionName, keyLabel, keyStroke, icon, tip);

    if (action != null) 
		// append to the configuration menu 
		myMenuConfiguration.add(action);
	
	// TRACE OUTPUT
	if (LeviModule.isTraceActive()) System.out.println("END leviPopup.addToMenuKonfiguration");
	
  }
  
  /*
  *
  * Methode:		addSubmenuToMenuKonfiguration
  * 
  * Description:	Adds a new submenu to the Scenario menu. The submenu is given by the parameter <submenu>
  * 
  * Date: 		    29.08.2005
  * @author         sirocic
  */
  public void addSubmenuToMenuConfiguration(JMenu submenu)
  {
  	
	// TRACE OUTPUT
	if (LeviModule.isTraceActive()) System.out.println("CALL leviPopup.addSubmenuToMenuKonfiguration: submenu="+submenu.getName());
	
	addSubmenu(myMenuConfiguration,submenu);
	
	// TRACE OUTPUT
	if (LeviModule.isTraceActive()) System.out.println("END leviPopup.addSubmenuToMenuKonfiguration");
	
  }

  /*
  *
  * Methode:		addToSubmenuLanguage
  * 
  * Description:	Adds a new item to the submenu Language. The menu item will execute the action given by <actionName>.
  *                 <keyLabel> is the ressource key of the new menu item label. <keyStroke> is the
  *                 ressource key of the accelerator key. <inputEvent> is the integer code of the event. 
  *                 <icon> is the icon which will be displayed next to the menu item and <tip> is the tool tip
  *                 of the new menu item.
  *                 The parameter <language> contains the information about the language which will be
  *                 set by this action. 
  * 
  * Date: 		    24.04.2006
  * @author         sirocic
  */
  public void addToSubmenuLanguage(String actionName, String keyLabel, String keyStroke, Icon icon, String tip, String language)
  {
    Action action;
  	
	// TRACE OUTPUT
	if (LeviModule.isTraceActive()) System.out.println("CALL leviMenu.addToSubmenuLanguage: actionName="+actionName);

    action = createMenuAction(actionName, keyLabel, keyStroke,  icon, tip);
    action.putValue("LANGUAGE", language);

    if (action != null) 
		// append to the language menu 
		mySubmenuLanguage.add(action);
	
	// TRACE OUTPUT
	if (LeviModule.isTraceActive()) System.out.println("END leviMenu.addToSubmenuLanguage");
	
  }
  
}

