/* leviPreviewPanel.java
 * 
 * Created on 21.03.2007
 * @author sirocic
 */
package learningVisualizations.gui;

import javax.swing.colorchooser.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.text.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.Serializable;

import learningVisualizations.LeviModule;



/**
 * @author sirocic
 *
 * Class: leviSwatchChooserPanel
 *
 * Description: Entspricht im wesentlichen der Klasse DefaultPreviewPanel. Da aber bei der
 * 				Farbumstellung von levi die Hintergrundfarbe im der ursprnglichen Swingklasse nicht umgesetzt
 * 				wurden, wurde diese Klasse in letzter Konsequenz neu geschrieben. 
 *
 * Date:	21.03.2007
 */
public class LeviPreviewPanel extends JPanel {

    private int squareSize = 25;
    private int squareGap = 5;
    private int innerGap = 5;
  

    private int textGap = 5;
    private Font font = new Font("Dialog", Font.PLAIN, 12);
    private String sampleText = UIManager.getString("ColorChooser.sampleText");

    private int swatchWidth = 50;

    private Color oldColor = null;

    private LeviModule module;

    public LeviPreviewPanel(LeviModule module) {
        super();
        this.module=module;

        // FARBEN ...
        Color backgroundColor,  foregroundColor;
   	   
   	   if (module.getConfiguration().getSettingByKey("ColorComponentBackground")!=null) {
   		   backgroundColor = (Color)module.getConfiguration().getSettingByKey("ColorComponentBackground");
   		   foregroundColor = (Color)module.getConfiguration().getSettingByKey("ColorComponentForeground");
   	   }
   	   else {
   		   backgroundColor = (Color)module.getConfiguration().getSettingByKey("ColorModuleBackground");
   		   foregroundColor = (Color)module.getConfiguration().getSettingByKey("ColorModuleForeground");
   		   
   	   }  
   	   
        // Hintergrund- und Zeichenfarben auf aktuelle Werte setzen
        setBackground(backgroundColor);
        setForeground(foregroundColor);
        
    }

    public Dimension getPreferredSize() {

        FontMetrics fm = Toolkit.getDefaultToolkit().getFontMetrics(getFont());

        int ascent = fm.getAscent();
        int height = fm.getHeight();
        int width = fm.stringWidth(sampleText);

            int y = height*3 + textGap*3;
        int x = squareSize * 3 + squareGap*2 + swatchWidth + width + textGap*3;
            return new Dimension( x,y );
    }
    
    public void paintComponent(Graphics g) {
        if (oldColor == null)
        oldColor = getForeground();

        g.setColor(getBackground());
	    g.fillRect(0,0,getWidth(),getHeight());
	
	    if (this.getComponentOrientation().isLeftToRight()) {
	        int squareWidth = paintSquares(g, 0);
	        int textWidth = paintText(g, squareWidth);
	        paintSwatch(g, squareWidth + textWidth);
	    } else {
	        int swatchWidth = paintSwatch(g, 0);
	        int textWidth = paintText(g, swatchWidth);
	        paintSquares(g , swatchWidth + textWidth);
	
	    }
    }

    private int paintSwatch(Graphics g, int offsetX) {
        int swatchX = offsetX;
    g.setColor(oldColor);
    g.fillRect(swatchX, 0, swatchWidth, (squareSize) + (squareGap/2));
    g.setColor(getForeground());
    g.fillRect(swatchX, (squareSize) + (squareGap/2), swatchWidth, (squareSize) + (squareGap/2) );
    return (swatchX+swatchWidth);
    }

    private int paintText(Graphics g, int offsetX) {
    g.setFont(getFont());
    FontMetrics fm = g.getFontMetrics();

    int ascent = fm.getAscent();
    int height = fm.getHeight();
    int width = fm.stringWidth(sampleText);

    int textXOffset = offsetX + textGap;

        Color color = getForeground();

    g.setColor(color);

    g.drawString(sampleText, textXOffset+(textGap/2), ascent+2);

    g.fillRect(textXOffset,
           ( height) + textGap, 
           width + (textGap),
           height +2);

    g.setColor(Color.black);
    g.drawString(sampleText, 
             textXOffset+(textGap/2), 
             height+ascent+textGap+2);


    g.setColor(Color.white);

    g.fillRect(textXOffset,
           ( height + textGap) * 2, 
           width + (textGap),
           height +2);

    g.setColor(color);
    g.drawString(sampleText,
             textXOffset+(textGap/2), 
             ((height+textGap) * 2)+ascent+2);

    return width + textGap*3;
    
    }

    private int paintSquares(Graphics g, int offsetX) {

    int squareXOffset = offsetX;
        Color color = getForeground();

        g.setColor(Color.white);
    g.fillRect(squareXOffset,0,squareSize,squareSize);
    g.setColor(color);
    g.fillRect(squareXOffset+innerGap, 
           innerGap, 
           squareSize - (innerGap*2), 
           squareSize - (innerGap*2));
    g.setColor(Color.white);
    g.fillRect(squareXOffset+innerGap*2,
           innerGap*2,
           squareSize - (innerGap*4),
           squareSize - (innerGap*4));

        g.setColor(color);
    g.fillRect(squareXOffset,squareSize+squareGap,squareSize,squareSize);

    g.translate(squareSize+squareGap, 0);
        g.setColor(Color.black);
    g.fillRect(squareXOffset,0,squareSize,squareSize);
    g.setColor(color);
    g.fillRect(squareXOffset+innerGap, 
           innerGap, 
           squareSize - (innerGap*2), 
           squareSize - (innerGap*2));
    g.setColor(Color.white);
    g.fillRect(squareXOffset+innerGap*2,
           innerGap*2,
           squareSize - (innerGap*4),
           squareSize - (innerGap*4));
    g.translate(-(squareSize+squareGap), 0);

    g.translate(squareSize+squareGap, squareSize+squareGap);
        g.setColor(Color.white);
    g.fillRect(squareXOffset,0,squareSize,squareSize);
    g.setColor(color);
    g.fillRect(squareXOffset+innerGap, 
           innerGap, 
           squareSize - (innerGap*2), 
           squareSize - (innerGap*2));
    g.translate(-(squareSize+squareGap), -(squareSize+squareGap));
    


    g.translate((squareSize+squareGap)*2, 0);
        g.setColor(Color.white);
    g.fillRect(squareXOffset,0,squareSize,squareSize);
    g.setColor(color);
    g.fillRect(squareXOffset+innerGap, 
           innerGap, 
           squareSize - (innerGap*2), 
           squareSize - (innerGap*2));
    g.setColor(Color.black);
    g.fillRect(squareXOffset+innerGap*2,
           innerGap*2,
           squareSize - (innerGap*4),
           squareSize - (innerGap*4));
    g.translate(-((squareSize+squareGap)*2), 0);

    g.translate((squareSize+squareGap)*2, (squareSize+squareGap));
        g.setColor(Color.black);
    g.fillRect(squareXOffset,0,squareSize,squareSize);
    g.setColor(color);
    g.fillRect(squareXOffset+innerGap, 
           innerGap, 
           squareSize - (innerGap*2), 
           squareSize - (innerGap*2));
    g.translate(-((squareSize+squareGap)*2), -(squareSize+squareGap));

    return (squareSize*3+squareGap*2);

    }

    /*
     * Methode:		setDialogFont
     * 
     * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
     * 				Werte. 
     *    
     * Parameters:   Aktuelle Font fr Labels und Textfelder.
     * 
     * Date: 		 23.04.2007
     * @author        sirocic
     */	
    private void setPanelFont() {
  	   Font labelFont;
  	   
  	   labelFont = (Font)module.getConfiguration().getSettingByKey("FontLabel");
  	   
  	   setFont(labelFont);
  	   	  
    }
    
    /*
     * Methode:		setDialogTexte
     * 
     * Description:	Setz alle Text-Bausteine des Dialoges neu. Wichtig fr Sprachwechsel.
     *    
     * Parameters:   
     * 
     * Date: 		 23.04.2007
     * @author        sirocic
     */	
    private void setPanelTexte() {

    	 sampleText = UIManager.getString("ColorChooser.sampleText");
    }
    
    /*
     * Methode:		setDialogColors
     * 
     * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
     * 				Werte. 
     *    
     * Parameters:   Aktuelle Farbwerte fr Hintergrund und Vordergrund.
     * 
     * Date: 		 23.04.2007
     * @author        sirocic
     */	
    private void setPanelColors() {
    	Color backgroundColor,  foregroundColor;
    	   
	   if (module.getConfiguration().getSettingByKey("ColorComponentBackground")!=null) {
		   backgroundColor = (Color)module.getConfiguration().getSettingByKey("ColorComponentBackground");
		   foregroundColor = (Color)module.getConfiguration().getSettingByKey("ColorComponentForeground");
	   }
	   else {
		   backgroundColor = (Color)module.getConfiguration().getSettingByKey("ColorModuleBackground");
		   foregroundColor = (Color)module.getConfiguration().getSettingByKey("ColorModuleForeground");
		   
	   }  
    	   
  	   // 	 Hintergrund und Zeichenfarbe auf aktuelle Werte setzen
       setBackground(backgroundColor);
  	   setForeground(foregroundColor);
  	  
    }
    

    
    /*
     * Methode:		updateColorLanguageFonts
     * 
     * Description:	Setzt die aktuellen Farben aus der Konfigurationbzw. und Texte aus den Ressourcen.
     * 				Wird aufgerufen in erster Linie nach einem dynamischen Sprach- oder Farbwechsel.
     *    
     * Date: 		16.03.2007
     * @author      sirocic
     */	
    public void updateColorLanguageFonts() {
       
        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("CALL leviPreviewPanel.updateColorLanguageFonts:");

        // SUBKOMPONENTE UPDATEN...
    
    	// FARBEN SETZEN ...
    	setPanelColors();

    	// FONTS SETZEN ...
        setPanelFont();
        
        // Texte setzen ...
        setPanelTexte();

        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("END leviPreviewPanel.updateColorLanguageFonts:");

    }
    
}
