/* leviQuitDialog.java
 * 
 * Created on 09.11.2006
 * @author sirocic
 */
package learningVisualizations.gui;

import java.awt.*;
import java.awt.event.*;
import java.awt.font.FontRenderContext;
import java.awt.geom.Rectangle2D;

import javax.swing.*;

import learningVisualizations.LeviModule;



/**
 * @author sirocic
 *
 * Class: leviQuitDialog
 * 
 * Description: The class "leviQuitDialog" of the general quit dialog which ask
 * 				the user for quit or not. 
 * 				
 * Date:	09.11.2006
 */
public class LeviQuitDialog extends LeviDialog {

	private JLabel questionL;

	private Box contentHB;
	
	
  /*
   * Methode:		Contructors
   * 
   * Description:	Displays the current color setting from the configuration.
   *				Provides the user with the possiblity to change the color settings
   *                for special elements like component background, process foreground etc.
   *                The selected colors are store by the background color of the preview panels.
   *                The configuration will be updated if the confirm button was selected.
   * 
   * Date: 		    07.09.2005
   * @author        sirocic
   */
	public LeviQuitDialog() {
	   super();
   	   // INIT
	   
  	   // SET DIALOG PROPERTIES
   	   setSize(WIDTH, HEIGHT);
	}
   public LeviQuitDialog(LeviFrame frame) {
   	   super( frame, "dlg.quit.title");
   	   
	   // TRACE OUTPUT
	   if (LeviModule.isTraceActive()) System.out.println("CALL leviQuitDialog.leviQuitDialog:");

  	   // INIT
	   contentHB = Box.createHorizontalBox(); 
	   
   	   // GUI Dialogelemente erzeugen ...
	   questionL = new JLabel();
	   questionL.setText(myFrame.getLearningModule().getRessources().getResourceByKey("dlg.quit.text"));
	   
	   //	 in die horizontale Box ...
       contentHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
       contentHB.add(questionL);
       contentHB.add(Box.createRigidArea(new Dimension(LeviFrame.ATTRIBUTOFFSET,0)));
       
   	   // create the buttons for closing and cancling the dialog 
	   okButton.addActionListener(new 
		        ActionListener() 
				{
	   					public void actionPerformed(ActionEvent e)
	   					{				
	   						//	 Dialog unsichtbar machen
	   						showDialog(false);
	   						
	   						// Komponente verlassen ...
	   				        myFrame.getLearningModule().exit();
	   				     
	   					}
		
				} );

       // in die vertikale Box ...
       contentVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
       contentVB.add(contentHB);
	   contentVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
	   contentVB.add(buttonHB);
	   contentVB.add(Box.createRigidArea(new Dimension(0,LeviFrame.LINEOFFSET)));
  
       //     FARBEN, FONTS UND TEXTE SETZEN ...
 	   updateLanguageColorFonts();
 
		// TRACE OUTPUT
		if (LeviModule.isTraceActive()) System.out.println("END leviQuitDialog.leviQuitDialog");

   }
   
   // MUTATOR METHODES ...
   
   /*
    * Methode:		showDialog
    * 
    * Description:	Setzt die aktuellen Farben aus der Konfiguration bevor der Dialog angezeigt bzw.
    * 				wieder versteckt wird.
    *    
    * Parameters:    
    * 
    * Date: 		 11.04.2006
    * @author        sirocic
    */	
   public void showDialog(boolean visible) {

 	  // TRACE OUTPUT
 	  if (LeviModule.isTraceActive()) System.out.println("CALL leviQuitDialog.showDialog:");
 	  
 	  if (visible) {
 		  
 		  // Dialog wird sichtbar gemacht. 

	 	  // Position des Dialogs abhngig von Hauptframe.
	   	  setLocationX(myFrame.getWidth()/2-WIDTH/2);
	   	  setLocationY(myFrame.getHeight()/2-HEIGHT/2);
	   	  
 	  }
   	   		  
 	  // 	 ... so jetzt den Dialog anzeigen bzw. verstecken.
 	  super.showDialog(visible);
   	  
  	  // TRACE OUTPUT
  	  if (LeviModule.isTraceActive()) System.out.println("END leviQuitDialog.showDialog");

   }
   
   /*
    * Methode:		fitDialog
    * 
    * Description:	Bestimmen der bentigten Panelgre fr das Anzeigen des vollstndigen Dialogs. 
    *    
    * Parameters:    
    * 
    * Date: 		 16.01.2007
    * @author        sirocic
    */	
   public void fitDialog() {
	  Graphics2D g2;
  	  int maxWidth,  maxHeight;
  	  Rectangle2D boundsLabel;
	  FontRenderContext context;
  	  
  	  // TRACE OUTPUT
  	  if (LeviModule.isTraceActive()) System.out.println("CALL leviQuitDialog.fitDialog:");
  	
  	  // init
  	  g2 = (Graphics2D) getGraphics();
  	  maxWidth = 0;
	  maxHeight =0;
	  
	  if (g2 != null)  {
		  // Bentigte Zeilenbreite und -hhe bestimmen anhand der Textausmasse bei aktuelle
	      // eingestellten Font ...
		  g2.setFont(getFont());
	      context = g2.getFontRenderContext();

	  	  // BENTIGTE DIALOG-BREITE BESTIMMEN ....
	  	  // Zeilenbreite aus Schriftlabel (Font-Label), Preview-Panel und Fontauswahl-Buttone bestimmen.
	  	  maxWidth =  2* LeviFrame.ATTRIBUTOFFSET ;
	  	  boundsLabel = getFont().getStringBounds(questionL.getText(), context);
	  	  maxWidth = maxWidth + (int)boundsLabel.getWidth();
		  // noch den Abstand zum rechten Rahmen addieren. 
	  	  maxWidth = maxWidth + 2* LeviFrame.ATTRIBUTOFFSET;
	  	  
	  	  // BENTIGTE DIALOG-HHE BESTIMMEN ....
	  	  maxHeight = LeviFrame.LINEOFFSET; 
	  	  maxHeight = maxHeight + + (int)boundsLabel.getHeight();
	  	
	  	  // Panelgre aktualisieren ...
	  	  getRootPane().setSize(maxWidth,maxHeight);
	  	  getRootPane().setMaximumSize(new Dimension(maxWidth,maxHeight));
	  	  getRootPane().setPreferredSize(new Dimension(maxWidth,maxHeight));
	  }
	  
	  super.fitDialog();
	  
  	  // TRACE OUTPUT
  	  if (LeviModule.isTraceActive()) System.out.println("END leviQuitDialog.fitDialog");
  	
   }
   
   /*
    * Methode:		setDialogFont
    * 
    * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
    * 				Werte. 
    *    
    * Parameters:   Aktuelle Font fr Labels und Textfelder.
    * 
    * Date: 		 17.01.2007
    * @author        sirocic
    */	
   protected void setDialogFont() {
	   Font menuFont;
	   
	   menuFont = (Font)myFrame.getLearningModule().getConfiguration().getSettingByKey("FontMenu");
	   
	   super.setDialogFont("FontMenu");
	   
	   questionL.setFont(menuFont);

   }
   
   /*
    * Methode:		setDialogTexte
    * 
    * Description:	Setz alle Text-Bausteine des Dialoges neu. Wichtig fr Sprachwechsel.
    *    
    * Parameters:   
    * 
    * Date: 		 26.01.2007
    * @author        sirocic
    */	
   protected void setDialogTexte() {
	   String labelStr;
	   
	   super.setDialogTexte(myFrame.getLearningModule().getRessources().getResourceByKey("dlg.quit.title"));
	   
	   labelStr = myFrame.getLearningModule().getRessources().getResourceByKey("dlg.quit.text");
	   questionL.setText(labelStr);
	   	  
   }
   
   /*
    * Methode:		setDialogColors
    * 
    * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
    * 				Werte. 
    *    
    * Parameters:   Aktuelle Farbwerte fr Hintergrund und Vordergrund.
    * 
    * Date: 		 17.01.2007
    * @author        sirocic
    */	
   protected void setDialogColors() {
	   Color buttonBackgroundColor;
	   Color backgroundColor,  foregroundColor;
   	   
   	   if (myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentBackground")!=null) {
   		   backgroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentBackground");
   		   foregroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentForeground");
   	   }
   	   else {
   		   backgroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorModuleBackground");
   		   foregroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorModuleForeground");
   		   
   	   }  
	   buttonBackgroundColor  = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorButtonBackground");
	 	  
	   super.setDialogColors();
	   
       questionL.setBackground(backgroundColor);
       questionL.setForeground(foregroundColor);
	 	  
   }
   /*
    * Methode:		updateLanguageColorFonts
    * 
    * Description:	Setzt die aktuellen Farben aus der Konfigurationbzw. und Texte aus den Ressourcen.
    * 				Wird aufgerufen in erster Linie nach einem dynamischen Sprach- oder Farbwechsel.
    *    
    * Date: 		 16.03.2007
    * @author        sirocic
    */	
   public void updateLanguageColorFonts() {
     
 	  // TRACE OUTPUT
 	  if (LeviModule.isTraceActive()) System.out.println("CALL leviQuitDialog.updateLanguageColorFonts:");

 	  // 	 SUBKOMPONENTE UPDATEN...

 	  
 	  super.updateLanguageColorFonts();
   	  
  	  // TRACE OUTPUT
  	  if (LeviModule.isTraceActive()) System.out.println("END leviQuitDialog.updateLanguageColorFonts");

   }
   
   
}
