/* leviScenarioPanel.java
 * 
 * Created on 05.12.2006
 * @author sirocic
 */
package learningVisualizations.gui;

import java.awt.*;

import javax.swing.*;

import learningVisualizations.LeviModule;
import learningVisualizations.editor.LeviScenarioEditorVisualizer;
import learningVisualizations.visualizer.LeviShapeMaker;




/**
 * @author sirocic
 *
 * Class: leviScenarioPanel
 * 
 * Description: The class "leviScenarioPanel" is the part of the scenarioPanel (panel) in which 
 *              the scenario will be painted.
 * 
 * Date:	05.12.2006
 */
public class LeviScenarioPanel extends JLayeredPane {

	protected LeviModule module;
	protected LeviScenarioEditorVisualizer scenarioParent;
	
  /*
   * Methode:		Constructors
   * 
   * Date: 		    05.09.2005
   * @author        sirocic
   */
   public LeviScenarioPanel() {
	  super();
	  this.setOpaque(true);
   }
   public LeviScenarioPanel(LeviModule module) {
	   super();
	   this.module = module;
	   this.setOpaque(true);
   }
   

   /*
    * Methode:		find
    *
    * Description:	Look to a point in the panel, if there is a object.
    *               If a object is found, it will be returned.
    *
    * Parameters:    p = point on the scenarioPanel panel
    *
    * Date: 		 17.10.2006
    * @author        sirocic
    */
   public LeviShapeMaker find(Point p)
   {
	   
	   // TRACE OUTPUT
	   if (LeviModule.isTraceActive()) System.out.println("CALL leviScenarioEditor.find: point=("+p.getX()+","+p.getY()+")");

	   // Das Bestimmen, ob an den Punktkoordinaten ein Objekt ist oder nicht, ist noch
	   // Komponenten abhngig. Die Objekte im Editor werden nach Typen in unterschiedlichen
	   // ArrayList gehalten. Welche und wieviele es davon gibt, ist der Editor-Oberklasse
	   // nicht bekannt. TODO -> Abstraktion in einer Liste von leviObject(s)????
	   
       // TRACE OUTPUT
       if (LeviModule.isTraceActive()) System.out.println("END leviScenarioEditor.find null");

       
	   return null;
	  
   }

   /*
    * Methode:		setDialogFont
    * 
    * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
    * 				Werte. 
    *    
    * Parameters:   Aktuelle Font fr Labels und Textfelder.
    * 
    * Date: 		 23.04.2007
    * @author        sirocic
    */	
   private void setPanelFont() {
	   Font labelFont;
	   
	   labelFont = (Font)module.getConfiguration().getSettingByKey("FontLabel");
	   
	   setFont(labelFont);
	   	  
   }
   
   /*
    * Methode:		setDialogTexte
    * 
    * Description:	Setz alle Text-Bausteine des Dialoges neu. Wichtig fr Sprachwechsel.
    *    
    * Parameters:   
    * 
    * Date: 		 23.04.2007
    * @author        sirocic
    */	
   private void setPanelTexte() {}
   
   /*
    * Methode:		setDialogColors
    * 
    * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
    * 				Werte. 
    *    
    * Parameters:   Aktuelle Farbwerte fr Hintergrund und Vordergrund.
    * 
    * Date: 		 23.04.2007
    * @author        sirocic
    */	
   private void setPanelColors() {
	   Color backgroundColor,  foregroundColor;
		 
	   if (module.getConfiguration().getSettingByKey("ColorComponentBackground")!=null)
	   {
		   backgroundColor = (Color)module.getConfiguration().getSettingByKey("ColorComponentBackground");
		   foregroundColor = (Color)module.getConfiguration().getSettingByKey("ColorComponentForeground");
	   }
	   else {
		   backgroundColor = (Color)module.getConfiguration().getSettingByKey("ColorModuleBackground");
		   foregroundColor = (Color)module.getConfiguration().getSettingByKey("ColorModuleForeground");
	   }
	   
	   // 	 Hintergrund und Zeichenfarbe auf aktuelle Werte setzen
       setBackground(backgroundColor);
	   setForeground(foregroundColor);
	  
	      
   }
	   
   /*
    * Methode:		updateLanguageColorFonts
    *
    * Description:	Ldt sprachabhngige Teile neu. In der Toolbar knnen z.B. sprachabhngige
    * 				Buttons sein 
    *
    * Date: 		12.03.2007
    * 
    * @author sirocic
    */
   public void updateLanguageColorFonts() {
	   
	   setPanelTexte();
	   setPanelFont();
	   setPanelColors();
	   
   }
   
public LeviScenarioEditorVisualizer getScenarioParent() {
	return scenarioParent;
}
public void setScenarioParent(LeviScenarioEditorVisualizer scenarioParent) {
	this.scenarioParent = scenarioParent;
}
   

 
}
