/* leviSubmenu.java
 *
 * Created on 29.08.2005
 * @author birgit
 */
package learningVisualizations.gui;

import java.awt.*;

import javax.swing.*;

import learningVisualizations.LeviModule;
import learningVisualizations.gui.actions.LeviAbstractAction;
import learningVisualizations.gui.actions.LeviActionSetActiveScenario;



/**
 * @author sirocic
 *
 * Class: leviSubmenu
 *
 * Description: The class "leviSubmenu" is the general class for a
 * 				submenu.
 *
 * Date:	11.02.2005
 */
public class LeviSubmenu extends JMenu {

    static final long serialVersionUID=1;

    // parent frame
    private LeviFrame myFrame;

    // Ressourcenschlssel fr den Menu-Label
    private String keyLabel;


    /*
    *
    * Methode:	Constructors
    *
    * Date:     29.08.05
    * @author   sirocic
    */
    public LeviSubmenu()
    {
        super();
        this.myFrame = null;
        this.keyLabel = null;
    }
    public LeviSubmenu(LeviFrame myFrame, String keyLabel)
    {
        super(myFrame.getLearningModule().getRessources().getResourceByKey(keyLabel));
        this.myFrame = myFrame;
        this.keyLabel = keyLabel;
        
        //FARBEN SETZEN ...
    	setPanelColors();

    	// FONTS SETZEN ...
        setPanelFont();
        
        // TEXTE SETZEN ...
        setPanelTexte();
    	  
    }


    // MUTATOR METHODES...

    /*
     * Methode:		setDialogFont
     * 
     * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
     * 				Werte. 
     *    
     * Parameters:   Aktuelle Font fr Labels und Textfelder.
     * 
     * Date: 		 23.04.2007
     * @author        sirocic
     */	
    private void setPanelFont() {
    	 Font font;
    	 int i;
         JMenuItem item;
         
    	   
    	 font = (Font)myFrame.getLearningModule().getConfiguration().getSettingByKey("FontMenu");
    	   
    	 setFont(font);
    	 for(i=0; i < this.getItemCount(); i++) {
             item = this.getItem(i);
             if (item != null)  {
                 // Damit auch JMenu in der Hintergrundfarbe gezeichnet werden.
                 // Eventuell kann die Instanzprfung auch entfallen
                 if(item instanceof JMenu) {
                     item.setOpaque(true);
                 }
             
                // bei den Menueintrgen, die selber keine Untermens sind, muss der Font neu gesetzt
                // werden. Dynamischer Fontwechsel !!!
                if (!(item instanceof LeviSubmenu))
                    // DF 20060511 - nur bei leviAbstractActions den Text ndern.
                    if( item.getAction() instanceof LeviAbstractAction )
                            item.setFont(font);
             }
        }
    }
    
    /*
     * Methode:		setDialogTexte
     * 
     * Description:	Setz alle Text-Bausteine des Dialoges neu. Wichtig fr Sprachwechsel.
     *    
     * Parameters:   
     * 
     * Date: 		 23.04.2007
     * @author        sirocic
     */	
    private void setPanelTexte() {
    	 int i;
         JMenuItem item;
         
    	setText(myFrame.getLearningModule().getRessources().getResourceByKey(keyLabel));
    	
    	for(i=0; i < this.getItemCount(); i++) {
             item = this.getItem(i);
             if (item != null)  {
                 // Damit auch JMenu in der Hintergrundfarbe gezeichnet werden.
                 // Eventuell kann die Instanzprfung auch entfallen
                 if(item instanceof JMenu) {
                     item.setOpaque(true);
                 }
             
                // bei den Menueintrgen, die selber keine Untermens sind, muss der Text neu gesetzt
                // werden. Dynamischer Sprachwechsel !!!
                if (!(item instanceof LeviSubmenu))
                    // DF 20060511 - nur bei leviAbstractActions den Text ndern.
                    if( (item.getAction() instanceof LeviAbstractAction) &&
                    		!(item.getAction() instanceof LeviActionSetActiveScenario))
                            item.setText(myFrame.getLearningModule().getRessources().getResourceByKey(((LeviAbstractAction)item.getAction()).getKeyLabel()));
             }
        }
    }
    
    /*
     * Methode:		setDialogColors
     * 
     * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
     * 				Werte. 
     *    
     * Parameters:   Aktuelle Farbwerte fr Hintergrund und Vordergrund.
     * 
     * Date: 		 23.04.2007
     * @author        sirocic
     */	
    private void setPanelColors() {
  	   int i;
       JMenuItem item;
       Color backgroundColor,  foregroundColor;
   	   
   	   if (myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentBackground")!=null) {
   		   backgroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentBackground");
   		   foregroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentForeground");
   	   }
   	   else {
   		   backgroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorModuleBackground");
   		   foregroundColor = (Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorModuleForeground");
   		   
   	   }  
   	   
  	   // 	 Hintergrund und Zeichenfarbe auf aktuelle Werte setzen
       setBackground(backgroundColor);
  	   setForeground(foregroundColor);
  	   for(i=0; i < this.getItemCount(); i++) {
         item = this.getItem(i);
         if (item != null)  {
             // Damit auch JMenu in der Hintergrundfarbe gezeichnet werden.
             // Eventuell kann die Instanzprfung auch entfallen
             if(item instanceof JMenu) {
                 item.setOpaque(true);
             }
             // Farben neu setzen fr Hinter- und Vordergrund. Dynamischer Farbwechsel !!!
            item.setBackground(backgroundColor);
            item.setForeground(foregroundColor);
         }
       }
  	 
    }
    
    /*
     * Methode:		updateLanguageColorFonts
     *
     * Description:	Ldt sprachabhngige Teile neu. In der Toolbar knnen z.B. sprachabhngige
     * 				Buttons sein 
     *
     * Date: 		12.03.2007
     * 
     * @author sirocic
     */
    public void updateLanguageColorFonts()
    {
  	   int i;
  	   JMenuItem item;
  	   
  	  //	 SUBKOMPONENTE UPDATEN...
      // Fr die Untermenus im Untermenu auch den Update anschubsen!
      for(i=0; i < this.getItemCount(); i++) {
         item = this.getItem(i);
         if (item != null)  {
             // bei den Menueintrgen, die selber keine Untermens sind, muss der Text neu gesetzt
             // werden. Dynamischer Sprachwechsel !!!
             if (item instanceof LeviSubmenu)
                 ((LeviSubmenu)item).updateLanguageColorFonts();
          }
       }
      
  	   // 	 FARBWECHSEL ...
  	   setPanelColors();

  	   // FONTWECHSEL
  	   setPanelFont();
  	  
  	   // SPRACHWECHSEL ...
  	   setPanelTexte();
  	   
    }
  /*
   * Methode:		paintComponent
   *
   * Description:	Calls the paintComponent methode of the JPanel.
   *
   * Parameters:
   *
   * Date: 		    13.09.2005
   * @author        sirocic
   */
   public void paintComponent(Graphics g) {
	   
      // TRACE OUTPUT
      if (LeviModule.isTraceActive()) System.out.println("CALL leviSubmenu.paintComponent: text="+this.getText());

      // here calling the paint methode of the JPanel
      super.paintComponent(g);

       // TRACE OUTPUT
       if (LeviModule.isTraceActive()) System.out.println("END leviSubmenu.paintComponent");
  }


  /*
  *
  * Methode:		createMenuAction
  *
  * Description:	Creates a new action object for a menu item specified by the parameter.
  *                 <actionName> is the name of the action class.
  *                 <keyLabel> is the ressource key of the new menu item label. <keyStroke> is the
  *                 ressource key of the accelerator key. <inputEvent> is the integer code of the event.
  *                 <icon> is the icon which will be displayed next to the menu item and <tip> is the tool tip
  *                 of the new menu item.
  *
  * Date: 		    29.08.2005
  * @author         sirocic
  */
  private Action createMenuAction(String actionName, String keyLabel, String keyStroke, Icon icon, String tip)
  {
      Action action;

    // TRACE OUTPUT
    if (LeviModule.isTraceActive()) System.out.println("CALL leviSubmenu.createMenuAction: actionName="+actionName);

    try
    {
        action = (Action) Class.forName(actionName).newInstance();
        action.putValue(Action.NAME, myFrame.getLearningModule().getRessources().getResourceByKey(keyLabel));
        ((LeviAbstractAction) action).setLearningModule(myFrame.getLearningModule());
        ((LeviAbstractAction) action).setKeyLabel(keyLabel);
        if (myFrame.getLearningModule().getRessources().getResourceByKey(keyStroke).length() != 0)
           action.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(myFrame.getLearningModule().getRessources().getResourceByKey(keyStroke)));
        if ( icon != null)
            action.putValue(Action.SMALL_ICON, icon);
        if ( tip != null)
            action.putValue(Action.SHORT_DESCRIPTION, tip);

        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("END leviSubmenu.createMenuAction");

        return action;

    }
    catch (Exception e)
    {
        System.out.println("Internal Error (leviSubmenu.createMenuAction):" + e.toString());

        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("END leviSubmenu.createMenuAction with error");

        return null;
    }

  }

  /*
  *
  * Methode:		addToMenu
  *
  * Description:	Adds a new item to the submenu. The menu item will execute the action given by <actionName>.
  *                 <keyLabel> is the ressource key of the new menu item label. <keyStroke> is the
  *                 ressource key of the accelerator key. <inputEvent> is the integer code of the event.
  *                 <icon> is the icon which will be displayed next to the menu item and <tip> is the tool tip
  *                 of the new menu item.
  *
  * Date: 		    29.08.2005
  * @author         sirocic
  */
  public void addToMenu(String actionName, String keyLabel, String keyStroke, Icon icon, String tip)
  {
    Action action;
    JMenuItem item;

    // TRACE OUTPUT
    if (LeviModule.isTraceActive()) System.out.println("CALL leviSubmenu.addToMenu: actionName="+actionName);

    action = createMenuAction(actionName, keyLabel, keyStroke, icon, tip);
    if (action != null) {
        // append to the scenario menu
        item = add(action);
        if (myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentBackground")!=null) {
          item.setBackground((Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentBackground"));
          item.setForeground((Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorComponentForeground"));
        }
        else {
        	item.setBackground((Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorModuleBackground"));
            item.setForeground((Color)myFrame.getLearningModule().getConfiguration().getSettingByKey("ColorModuleForeground"));
        }
    }

    // TRACE OUTPUT
    if (LeviModule.isTraceActive()) System.out.println("END leviSubmenu.addToMenu");

  }
  


  public void setFrame(LeviFrame myFrame)
  {
      this.myFrame = myFrame;
  }

  // ACCESSOR METHODES...
  public LeviFrame getFrame()
  {
      return myFrame;
  }

}
