/* leviSwatchChooserPanel.java
 * 
 * Created on 20.03.2007
 * @author sirocic
 */
package learningVisualizations.gui;

import javax.swing.colorchooser.*;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.io.Serializable;

import learningVisualizations.LeviModule;



/**
 * @author sirocic
 *
 * Class: leviSwatchChooserPanel
 *
 * Description: Entspricht im wesentlichen der Klasse DefaultSwatchChooserPanel. Da aber bei der
 * 				Sprachumstellung von levi die Labels im der ursprnglichen Swingklasse nicht umgesetzt
 * 				wurden, wurde diese Klasse in letzter Konsequenz neu geschrieben. 
 *
 * Date:	20.03.2007
 */
public class LeviSwatchChooserPanel extends AbstractColorChooserPanel {

    LeviSwatchPanel swatchPanel;
    LeviRecentSwatchPanel recentSwatchPanel;
    LeviPreviewPanel preview;
    JPanel superHolder;
    JPanel mainHolder;
    JPanel recentHolder;
    JPanel recentLabelHolder;
    JPanel recentHolderHolder;
    JLabel recentLabel;
    MouseListener mainSwatchListener;
    MouseListener recentSwatchListener;
    ColorSelectionModel colorSelectionModel;
    private LeviModule module;

    private String recentStr = UIManager.getString("ColorChooser.swatchesRecentText");

    public LeviSwatchChooserPanel(LeviModule module, ColorSelectionModel colorSelectionModel, LeviPreviewPanel preview) {
        super();
        this.module=module;
        this.colorSelectionModel = colorSelectionModel;
        this.preview = preview;
        
        Color backgroundColor, foregroundColor;
        
        // FARBEN ...
        if (module.getConfiguration().getSettingByKey("ColorComponentBackground")!=null) {
	        backgroundColor = (Color)module.getConfiguration().getSettingByKey("ColorComponentBackground");
	        foregroundColor = (Color)module.getConfiguration().getSettingByKey("ColorComponentForeground");
	    }
	    else {
	    	backgroundColor = (Color)module.getConfiguration().getSettingByKey("ColorModuleBackground");
	        foregroundColor = (Color)module.getConfiguration().getSettingByKey("ColorModuleForeground");
	    }

        // Hintergrund- und Zeichenfarben auf aktuelle Werte setzen
        setBackground(backgroundColor);
        setForeground(foregroundColor);
        
        superHolder = new JPanel(new BorderLayout());
        superHolder.setBackground(backgroundColor);
        superHolder.setForeground(foregroundColor);

        swatchPanel =  new LeviMainSwatchPanel(module);
        swatchPanel.getAccessibleContext().setAccessibleName(getDisplayName());

        recentSwatchPanel = new LeviRecentSwatchPanel(module);
        recentSwatchPanel.getAccessibleContext().setAccessibleName(recentStr);

        mainSwatchListener = new MainSwatchListener();
        swatchPanel.addMouseListener(mainSwatchListener);
        recentSwatchListener = new RecentSwatchListener();
        recentSwatchPanel.addMouseListener(recentSwatchListener);

	    mainHolder = new JPanel(new BorderLayout());
	    Border border = new CompoundBorder( new LineBorder(Color.black),
	                        new LineBorder(Color.white) );
	    mainHolder.setBorder(border);
	    mainHolder.add(swatchPanel, BorderLayout.CENTER);
	    mainHolder.setBackground(backgroundColor);
	    mainHolder.setForeground(foregroundColor);
	    superHolder.add( mainHolder, BorderLayout.CENTER );

	    recentHolder = new JPanel( new BorderLayout() );
	    recentSwatchPanel.addMouseListener(recentSwatchListener);
	    recentHolder.setBorder(border);
	    recentHolder.add(recentSwatchPanel, BorderLayout.CENTER);
	    recentHolder.setBackground(backgroundColor);
	    recentHolder.setForeground(foregroundColor);
	    recentLabelHolder = new JPanel(new BorderLayout());
	    recentLabelHolder.add(recentHolder, BorderLayout.CENTER);
	    recentLabelHolder.setBackground(backgroundColor);
	    recentLabelHolder.setForeground(foregroundColor);
	    
	    recentLabel = new JLabel(recentStr);
	    recentLabel.setLabelFor(recentSwatchPanel);
	    recentLabelHolder.add(recentLabel, BorderLayout.NORTH);
	    recentHolderHolder = new JPanel(new BorderLayout());
	        if (this.getComponentOrientation().isLeftToRight()) {
	        recentHolderHolder.setBorder(new EmptyBorder(2,10,2,2));
	        } else {
	        recentHolderHolder.setBorder(new EmptyBorder(2,2,2,10));
	        }
        recentHolderHolder.setBackground(backgroundColor);
        recentHolderHolder.setForeground(foregroundColor);
	    recentHolderHolder.add(recentLabelHolder, BorderLayout.CENTER);
	    superHolder.add( recentHolderHolder, BorderLayout.AFTER_LINE_ENDS );    
	
	    add(superHolder);
	    
	    // FARBEN SETZEN ...
		setPanelColors();

		// FONTS SETZEN ...
	    setPanelFont();
	    
	    // TEXTE SETZEN ...
	    setPanelTexte();
	    
    }

    public String getDisplayName() {
        return UIManager.getString("ColorChooser.swatchesNameText");
    }

    public Icon getSmallDisplayIcon() {
        return null;
    }

    public Icon getLargeDisplayIcon() {
        return null;
    }
                               
    public void installChooserPanel(JColorChooser enclosingChooser) {
        super.installChooserPanel(enclosingChooser);
    }

    protected void buildChooser() {
 
    }

    public void uninstallChooserPanel(JColorChooser enclosingChooser) {
        super.uninstallChooserPanel(enclosingChooser);
	    swatchPanel.removeMouseListener(mainSwatchListener);
	    recentSwatchPanel.removeMouseListener(recentSwatchListener);
	    swatchPanel = null;
	    recentSwatchPanel = null;
	    mainSwatchListener = null;
	    recentSwatchListener = null;
	    removeAll();  // strip out all the sub-components
    }

    public void updateChooser() {

    }


    class RecentSwatchListener extends MouseAdapter implements Serializable {
        public void mousePressed(MouseEvent e) {
        Color color = recentSwatchPanel.getColorForLocation(e.getX(), e.getY());
        colorSelectionModel.setSelectedColor(color);

    }
    }

    class MainSwatchListener extends MouseAdapter implements Serializable {
        public void mousePressed(MouseEvent e) {
        Color color = swatchPanel.getColorForLocation(e.getX(), e.getY());
        colorSelectionModel.setSelectedColor(color);
        recentSwatchPanel.setMostRecentColor(color);
        preview.setForeground(color);
    }
    }
    
    /*
     * Methode:		setDialogFont
     * 
     * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
     * 				Werte. 
     *    
     * Parameters:   Aktuelle Font fr Labels und Textfelder.
     * 
     * Date: 		 23.04.2007
     * @author        sirocic
     */	
    private void setPanelFont() {
  	   Font labelFont;
  	   
  	   labelFont = (Font)module.getConfiguration().getSettingByKey("FontLabel");
  	   
  	   setFont(labelFont);
  	   	  
    }
    
    /*
     * Methode:		setDialogTexte
     * 
     * Description:	Setz alle Text-Bausteine des Dialoges neu. Wichtig fr Sprachwechsel.
     *    
     * Parameters:   
     * 
     * Date: 		 23.04.2007
     * @author        sirocic
     */	
    private void setPanelTexte() {

        recentStr = UIManager.getString("ColorChooser.swatchesRecentText");
        recentLabel.setText(recentStr);

    }
    
    /*
     * Methode:		setDialogColors
     * 
     * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
     * 				Werte. 
     *    
     * Parameters:   Aktuelle Farbwerte fr Hintergrund und Vordergrund.
     * 
     * Date: 		 23.04.2007
     * @author        sirocic
     */	
    private void setPanelColors() {
    	Color backgroundColor, foregroundColor;
        
        if (module.getConfiguration().getSettingByKey("ColorComponentBackground")!=null) {
	        backgroundColor = (Color)module.getConfiguration().getSettingByKey("ColorComponentBackground");
	        foregroundColor = (Color)module.getConfiguration().getSettingByKey("ColorComponentForeground");
	    }
	    else {
	    	backgroundColor = (Color)module.getConfiguration().getSettingByKey("ColorModuleBackground");
	        foregroundColor = (Color)module.getConfiguration().getSettingByKey("ColorModuleForeground");
	    }
        
  	   // 	 Hintergrund und Zeichenfarbe auf aktuelle Werte setzen
       setBackground(backgroundColor);
  	   setForeground(foregroundColor);
  	   superHolder.setBackground(backgroundColor);
  	   superHolder.setForeground(foregroundColor);
  	   mainHolder.setBackground(backgroundColor);
	   mainHolder.setForeground(foregroundColor);
	   recentHolder.setBackground(backgroundColor);
	   recentHolder.setForeground(foregroundColor);
	   recentLabelHolder.setBackground(backgroundColor);
	   recentLabelHolder.setForeground(foregroundColor);
	   recentHolderHolder.setBackground(backgroundColor);
	   recentHolderHolder.setForeground(foregroundColor);
     
  	 
    }
    /*
     * Methode:		updateColorLanguageFonts
     * 
     * Description:	Setzt die aktuellen Farben aus der Konfigurationbzw. und Texte aus den Ressourcen.
     * 				Wird aufgerufen in erster Linie nach einem dynamischen Sprach- oder Farbwechsel.
     *    
     * Date: 		16.03.2007
     * @author      sirocic
     */	
    public void updateColorLanguageFonts() {

        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("CALL leviSwatchChooserPanel.updateColorLanguageFonts:");

        // SUBKOMPONENTE UPDATEN...
        swatchPanel.updateColorLanguageFonts();
        recentSwatchPanel.updateColorLanguageFonts();
        
        // 	 FARBWECHSEL ...
 	    setPanelColors();

 	    // FONTWECHSEL
 	    setPanelFont();
 	  
 	    // SPRACHWECHSEL ...
 	    setPanelTexte();
 	   
        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("END leviSwatchChooserPanel.updateColorLanguageFonts:");

    }

}








