/* leviSwatchPanel.java
 * 
 * Created on 20.03.2007
 * @author sirocic
 */
package learningVisualizations.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.event.MouseEvent;

import javax.swing.JPanel;
import javax.swing.UIManager;

import learningVisualizations.LeviModule;


class LeviSwatchPanel extends JPanel {

    protected Color[] colors;
    protected Dimension swatchSize;
    protected Dimension numSwatches;
    protected Dimension gap;
    private LeviModule module;

    public LeviSwatchPanel(LeviModule module) {
        initValues();
        initColors();
	    setToolTipText(""); // register for events
	    setOpaque(true);
	    setRequestFocusEnabled(false);
	    
	    this.module=module;
	    
	    //	  FARBEN SETZEN ...
		setPanelColors();

		// FONTS SETZEN ...
	    setPanelFont();
	    
	    // TEXTE SETZEN ...
	    setPanelTexte();
	    
    }

    public boolean isFocusTraversable() {
        return false;
    }

    protected void initValues() {

    }

    public void paintComponent(Graphics g) {
         g.setColor(getBackground());
         g.fillRect(0,0,getWidth(), getHeight());
     for (int row = 0; row < numSwatches.height; row++) {
        for (int column = 0; column < numSwatches.width; column++) {

          g.setColor( getColorForCell(column, row) ); 
        int x;
        if ((!this.getComponentOrientation().isLeftToRight()) &&
            (this instanceof LeviRecentSwatchPanel)) {
            x = (numSwatches.width - column - 1) * (swatchSize.width + gap.width);
        } else {
            x = column * (swatchSize.width + gap.width);
        }
        int y = row * (swatchSize.height + gap.height);
            g.fillRect( x, y, swatchSize.width, swatchSize.height);
        g.setColor(Color.black);
        g.drawLine( x+swatchSize.width-1, y, x+swatchSize.width-1, y+swatchSize.height-1);
        g.drawLine( x, y+swatchSize.height-1, x+swatchSize.width-1, y+swatchSize.height-1);
        }
     }
    }

    public Dimension getPreferredSize() {
        int x = numSwatches.width * (swatchSize.width + gap.width) -1;
    int y = numSwatches.height * (swatchSize.height + gap.height) -1;
        return new Dimension( x, y );
    }

    protected void initColors() {

      
    }

    public String getToolTipText(MouseEvent e) {
        Color color = getColorForLocation(e.getX(), e.getY());
        return color.getRed()+", "+ color.getGreen() + ", " + color.getBlue();
    }

    public Color getColorForLocation( int x, int y ) {
        int column;
        if ((!this.getComponentOrientation().isLeftToRight()) &&
            (this instanceof LeviRecentSwatchPanel)) {
            column = numSwatches.width - x / (swatchSize.width + gap.width) - 1;
        } else {
            column = x / (swatchSize.width + gap.width);
        }
        int row = y / (swatchSize.height + gap.height);
    return getColorForCell(column, row);
    }

    private Color getColorForCell( int column, int row) {
    return colors[ (row * numSwatches.width) + column ]; // (STEVE) - change data orientation here
    }

    /*
     * Methode:		setDialogFont
     * 
     * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
     * 				Werte. 
     *    
     * Parameters:   Aktuelle Font fr Labels und Textfelder.
     * 
     * Date: 		 23.04.2007
     * @author        sirocic
     */	
    private void setPanelFont() {
  	   Font labelFont;
  	   
  	   labelFont = (Font)module.getConfiguration().getSettingByKey("FontLabel");
  	   
  	   setFont(labelFont);
  	   	  
    }
    
    /*
     * Methode:		setDialogTexte
     * 
     * Description:	Setz alle Text-Bausteine des Dialoges neu. Wichtig fr Sprachwechsel.
     *    
     * Parameters:   
     * 
     * Date: 		 23.04.2007
     * @author        sirocic
     */	
    private void setPanelTexte() {

       
    }
    
    /*
     * Methode:		setDialogColors
     * 
     * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
     * 				Werte. 
     *    
     * Parameters:   Aktuelle Farbwerte fr Hintergrund und Vordergrund.
     * 
     * Date: 		 23.04.2007
     * @author        sirocic
     */	
    private void setPanelColors() {
    	Color backgroundColor, foregroundColor;
        
        if (module.getConfiguration().getSettingByKey("ColorComponentBackground")!=null) {
	        backgroundColor = (Color)module.getConfiguration().getSettingByKey("ColorComponentBackground");
	        foregroundColor = (Color)module.getConfiguration().getSettingByKey("ColorComponentForeground");
	    }
	    else {
	    	backgroundColor = (Color)module.getConfiguration().getSettingByKey("ColorModuleBackground");
	        foregroundColor = (Color)module.getConfiguration().getSettingByKey("ColorModuleForeground");
	    }
        
  	   // 	 Hintergrund und Zeichenfarbe auf aktuelle Werte setzen
       setBackground(backgroundColor);
  	   setForeground(foregroundColor);
  	   
    }
    
    /*
     * Methode:		updateColorLanguageFonts
     * 
     * Description:	Setzt die aktuellen Farben aus der Konfigurationbzw. und Texte aus den Ressourcen.
     * 				Wird aufgerufen in erster Linie nach einem dynamischen Sprach- oder Farbwechsel.
     *    
     * Date: 		16.03.2007
     * @author      sirocic
     */	
    public void updateColorLanguageFonts() {

        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("CALL leviSwatchPanel.updateColorLanguageFonts:");

        // FARBWECHSEL ...
 	    setPanelColors();

 	    // FONTWECHSEL
 	    setPanelFont();
 	  
 	    // SPRACHWECHSEL ...
 	    setPanelTexte();
        
        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("END leviSwatchPanel.updateColorLanguageFonts:");

    }


}
