/* leviToolbar.java
 *
 * Created on 30.03.2005
 * Autor: Birgit Sirocic
 *
 */
package learningVisualizations.gui;

import java.awt.*;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.*;

import learningVisualizations.LeviModule;
import learningVisualizations.gui.actions.LeviAbstractAction;





/**
 * @author sirocic
 *
 * Class: leviFrame
 *
 * Description: The class "leviFrame" is the general class for a
 * 				learning module.
 *
 * Date:	30.03.2005
 */
public class LeviToolbar extends JToolBar
                      implements PropertyChangeListener {

    private LeviModule learningModule;
    
    public final static String VIEWBUTTONKEY="button.view";
    public final static String PLAYBUTTONKEY="button.play";
    public final static String PAUSEBUTTONKEY="button.pause";
    public final static String FORWARDBUTTONKEY="button.forward";
    public final static String BACKWARDBUTTONKEY="button.backward";
    public final static String STOPBUTTONKEY="button.stop";
    public final static String DELETEBUTTONKEY="button.delete";
    public final static String PARAMBUTTONKEY="button.param";
    public final static String FASTERBUTTONKEY="button.faster";
    public final static String SLOWERBUTTONKEY="button.slower";

    
    /**
     * This constructor must not be used! leviToolbar does not work without
     * learningModule.
     * Making this private gives an appropriate compiler error as a hint.
     *
     * TODO: Delete this constructor's contents after approval
     */
    private LeviToolbar()
    {
        this(null);
    }

    /**
     * Construct a toolbar for the learningModule
     *
     * @param learningModule
     */
    public LeviToolbar(LeviModule learningModule)
    {
        super();
        this.learningModule = learningModule;
        
        // FARBEN SETZEN ...
    	setPanelColors();

    	// FONTS SETZEN ...
        setPanelFont();
    	  
        // Initiale Orientierung ist vertikal
        setOrientation(JToolBar.VERTICAL); 
    }

    /**
     * Set the colors of toolbar and the buttons.
     */
    private void setPanelColors() {
    	Color backgroundColor, foregroundColor;
        
        if (learningModule.getConfiguration().getSettingByKey("ColorComponentBackground")!=null) {
	        backgroundColor = (Color)learningModule.getConfiguration().getSettingByKey("ColorComponentBackground");
	        foregroundColor = (Color)learningModule.getConfiguration().getSettingByKey("ColorComponentForeground");
	    }
	    else {
	    	backgroundColor = (Color)learningModule.getConfiguration().getSettingByKey("ColorModuleBackground");
	        foregroundColor = (Color)learningModule.getConfiguration().getSettingByKey("ColorModuleForeground");
	    }
        
   	   // 	 Hintergrund und Zeichenfarbe auf aktuelle Werte setzen
       setBackground(backgroundColor);
   	   setForeground(foregroundColor);
   	   
    }


    // MUTATOR METHODES ...
    /*
     * Methode:		setDialogFont
     * 
     * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
     * 				Werte. 
     *    
     * Parameters:   Aktuelle Font fr Labels und Textfelder.
     * 
     * Date: 		 23.04.2007
     * @author        sirocic
     */	
    private void setPanelFont() {
  	   Font labelFont;
  	   
  	   labelFont = (Font)learningModule.getConfiguration().getSettingByKey("FontLabel");
  	   
  	   setFont(labelFont);
  	   	  
    }
    
    /*
     * Methode:		setDialogTexte
     * 
     * Description:	Setz alle Text-Bausteine des Dialoges neu. Wichtig fr Sprachwechsel.
     *    
     * Parameters:   
     * 
     * Date: 		 23.04.2007
     * @author        sirocic
     */	
    private void setPanelTexte() {
    	
    }
    /**
     * Methode:		updateLanguageColorFonts
     *
     * Description:	Sollte aufgerufen werden, wenn ein Sprachwechsel oder hnliches stattgefunden hat.
     * 				Hier werden z.B. die Icons von sprachabhngigen Buttons neu geladen.
     *
     * Parameters:
     *
     * Date: 		13.09.2005
     * @author      sirocic
     */
     public void updateLanguageColorFonts() {
    	 LeviButton button=null;
         int i=0;
    	 
         // TRACE OUTPUT
         if (LeviModule.isTraceActive()) System.out.println("CALL leviToolbar.updateLanguageColorFonts:");

         // SUBKOMPONENTE UPDATEN...
         while (i < this.getComponentCount()) {
        	 if (getComponentAtIndex(0) instanceof LeviButton) {
            	 button = (LeviButton) getComponentAtIndex(i);
            	 button.updateLanguageColorFonts();
        	 }
             i++;
         }
         
    	 // 	 FARBWECHSEL ...
    	 setPanelColors();

    	 // FONTWECHSEL
    	 setPanelFont();
    	  
    	 // SPRACHWECHSEL ...
    	 setPanelTexte();
    	   
         // TRACE OUTPUT
         if (LeviModule.isTraceActive()) System.out.println("END leviToolbar.updateLanguageColorFonts");

     }
     
    /**
    * Methode:		paintComponent
    *
    * Description:	Sets some attributes of the toolbar and calls the paintComponent
    *                methode of the class JToolBar.
    *
    * Parameters:
    *
    * Date: 		    13.09.2005
    * @author        sirocic
    */
    @Override
    public void paintComponent(Graphics g) {
        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("CALL leviToolbar.paintComponent:");

        // here calling the paint methode of the JToolBar
        super.paintComponent(g);

        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("END leviToolbar.paintComponent");

    }

    /**
    *
    * Methode:		enableXXXButton
    *
    * Description:	Veranlasst, das der jeweilige Standardbutton in der Toolbar erscheint. Soll eine
    * 				andere als die Standardaction an den Button gehngt werde, kann diese ber Parameter
    * 				angegeben werden.
    *
    * Date: 		14.03.2007
    * @author       sirocic
    */
    public void enablePlayButton()
    {
    	addButton("learningVisualizations.gui.actions.LeviActionPlaySimulation",PLAYBUTTONKEY);
    }
    public void enablePlayButton(String action)
    {
    	addButton(action,PLAYBUTTONKEY);
    }
    public void enableForwardButton()
    {
    	addButton("learningVisualizations.gui.actions.LeviActionForwardSimulation",FORWARDBUTTONKEY);
    }
    public void enableForwardButton(String action)
    {
    	addButton(action,FORWARDBUTTONKEY);
    }
    public void enableDeleteButton()
    {
    	addButton("learningVisualizations.gui.actions.LeviActionDeleteObject",DELETEBUTTONKEY);
    }
    public void enableDeleteButton(String action)
    {
    	addButton(action,DELETEBUTTONKEY);
    }
    public void enableViewButton()
    {
    	addButton("learningVisualizations.gui.actions.LeviActionChangeView",VIEWBUTTONKEY);
    }
    public void enableViewButton(String action)
    {
    	addButton(action,VIEWBUTTONKEY);
    }
    public void enableParameterButton(String action)
    {
    	addButton(action,PARAMBUTTONKEY);
    }
    public void enableBackwardButton()
    {
    	addButton("learningVisualizations.gui.actions.LeviActionBackSimulation",BACKWARDBUTTONKEY);
    }
    public void enableBackwardButton(String action)
    {
    	addButton(action,BACKWARDBUTTONKEY);
    }
    public void enablePauseButton()
    {
    	addButton("learningVisualizations.gui.actions.LeviActionPauseSimulation",PAUSEBUTTONKEY);
    }
    public void enablePauseButton(String action)
    {
    	addButton(action,PAUSEBUTTONKEY);
    }
    public void enableStopButton()
    {
    	addButton("learningVisualizations.gui.actions.LeviActionStopSimulation",STOPBUTTONKEY);
    }
    public void enableStopButton(String action)
    {
    	addButton(action,STOPBUTTONKEY);
    }
    public void enableFasterButton()
    {
    	addButton("learningVisualizations.gui.actions.LeviActionChangeTempo",FASTERBUTTONKEY);
    }
    public void enableFasterButton(String action)
    {
    	addButton(action,FASTERBUTTONKEY);
    }
    public void enableSlowerButton()
    {
    	addButton("learningVisualizations.gui.actions.LeviActionChangeTempo",SLOWERBUTTONKEY);
    }
    public void enableSlowerButton(String action)
    {
    	addButton(action,SLOWERBUTTONKEY);
    }


    /**
    *
    * Methode:		addButton
    *
    * Description:	Adds a new button to the toolbar. The button will execute the action given by <actionName>.
    *                 <keyLabel> is the resource key of the button. The corresponding icons and tooltip texts
    *                 will be found in the resources by the button key. These will be set by the methods of
    *                 the class leviButton.
    *
    * Date: 		19.10.2005
    * @author         sirocic
     * @param actionName full qualified class name of the action for the button
     * @param keyLabel
    */
    public void addButton(String actionName, String keyLabel)
    {
        Action buttonAction;
        LeviButton button;
        String errorMsg, errorTitle;

        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("CALL leviToolbar.addButton: actionName="+actionName);

        buttonAction = createToolbarAction(actionName, keyLabel);

        if (buttonAction != null) {
            button = new LeviButton(keyLabel, learningModule, buttonAction);

            if (button != null) {
                // append to the toolbar
                add(button);
            }
            else {
                // Action could not be generated!! No button could be added
            	learningModule.getFrame().showErrorMessageDialog("dlg.errortext.10", keyLabel, "leviToolbar");
            }
        }
        else {
            // Action could not be generated!! No button could be added
        	learningModule.getFrame().showErrorMessageDialog("dlg.errortext.11", keyLabel, actionName, "leviToolbar");
 
        }

        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("END leviToolbar.addButton");

   }


    /**
    *
    * Methode:		createToolbarAction
    *
    * Description:	Creates a new action object for a toolbar button specified by the parameter.
    *                 <actionName> is the name of the action class.
    *                 <keyLabel> is the ressource key of the new menu item label.
    *
    * Date: 		    12.10.2005
    * @author         sirocic
    */
    private Action createToolbarAction(String actionName, String keyLabel)
    {
        Action action;

        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("CALL leviToolbar.createToolbarAction: actionName="+actionName);

        try
        {
              action = (Action) Class.forName(actionName).newInstance();
              ((LeviAbstractAction) action).setLearningModule(learningModule);
              ((LeviAbstractAction) action).setKeyLabel(keyLabel);

              // TRACE OUTPUT
              if (LeviModule.isTraceActive()) System.out.println("END leviToolbar.createToolbarAction");

            return action;

        }
        catch (Exception e)
        {
              System.out.println("Internal Error (leviToolbar.createToolbarAction):" + e.toString());

              // TRACE OUTPUT
              if (LeviModule.isTraceActive()) System.out.println("END leviToolbar.createToolbarAction with error");

              return null;
        }

    }

    //ACCESSOR METHODES ...

    /**
    *
    * Methode:		getButtonByKey
    *
    * Description:	Finds the toolbar button with the given key and returns it.
    *
    * Date: 		    14.10.2005
    * @author         sirocic
    */
    public LeviButton getButtonByKey(String key)
    {
        LeviButton button=null;
        int i=0;

        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("CALL leviToolbar.getButtonByKey: key="+key);

        button = (LeviButton) getComponentAtIndex(0);
        while (i < this.getComponentCount() && !button.getKey().equals(key) ) {
            i++;
            button = (LeviButton) getComponentAtIndex(i);
        }

        if ((button != null) && (!button.getKey().equals(key)))
             button=null;

        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("END leviToolbar.getButtonByKey button="+button);

        return button;

    }

    /**
     * Receive property changes and selectively use it to change colors.
     *
     * @param evt
     * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
     */
    public void propertyChange(PropertyChangeEvent evt) {
        if( evt.getPropertyName().startsWith("Color")) {
            setPanelColors();
        }
    }



}

