/* leviActionBackSimulation.java
 *
 * Created on 03.03.2005
 * Autor: Birgit Sirocic
 *
 */
package learningVisualizations.gui.actions;

import java.awt.EventQueue;
import java.awt.event.ActionEvent;
import javax.swing.*;

import learningVisualizations.LeviModule;
import learningVisualizations.scenario.LeviScenario;
import learningVisualizations.scenario.LeviScenarioInterface;



/**
 * @author sirocic
 *
 * Class: leviActionBackSimulation
 *
 * Description: The class "leviActionBackSimulation" is the class which realizes the function of
 * 				the menu item and the button "Step back", respectively.
 *
 * Date:	03.03.2005
 */
public class LeviActionBackSimulation extends LeviAbstractAction
{
    static final long serialVersionUID=1;

    // Constructors
    public LeviActionBackSimulation()
    {
        // Is needed for the instantiation in the class leviMenu!!
    }
    public LeviActionBackSimulation(LeviModule learningModule, String name)
    {
        putValue(Action.NAME, name);
        setLearningModule(learningModule);
    }
    public LeviActionBackSimulation(LeviModule learningModule, String name, Icon icon, String tip)
    {
        putValue(Action.NAME, name);
        putValue(Action.SMALL_ICON, icon);
        putValue(Action.SHORT_DESCRIPTION, tip);
        setLearningModule(learningModule);
    }
    public LeviActionBackSimulation(LeviModule learningModule, String name, KeyStroke acc)
    {
        putValue(Action.NAME, name);
        putValue(Action.ACCELERATOR_KEY, acc);
        setLearningModule(learningModule);
    }
    // END	 Constructors

  /*
   * @author sirocic
   *
   * Methode:		actionPerformed
   *
   * Description:	Realizing the function of the menu item or button "Step back".
   *
   * Date: 		02.03.2005
   */
    public void actionPerformed(ActionEvent event)
    {

        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("CALL leviActionBackSimulation.actionPerformed: event="+event.toString());

        // Wenn es einen ActionButton gibt der disabled ist,
        // wird dieser wieder auf enable gesetzt.
        resetActiveAction();
        
        if (getLearningModule().getVisualizer().isBusy()) {
        	// Es sind noch Animationen akiv vom vorherigen Schritt.
        	// Die mssen abgebrochen werden. Der Endzustand des vorherigen Schritts muss
        	// eingestellt werden.
	        try {
			       EventQueue.invokeLater( new Runnable() {
		                  public void run() {
		                	  getLearningModule().getVisualizer().getScenarioVisualizer().cancelAnimation(true);
		                  }
		           }); 
			}
		    catch (Exception exception)
		    {
		        // Irgendwas ist mit dem Repaint wohl nicht richtig gelaufen!!!
		    	System.out.println("ERROr (leviActionBackSimulation)"+exception.getMessage());
		    }
        }
        
        // Is there a open and correct scenario?
        LeviScenarioInterface scenario = getLearningModule().getScenario();
        // DF 20060607: check if the scenario is null
        if ((scenario != null) && scenario.isOpen()&& scenario.isOk()) {

            if (getLearningModule().getMode() == LeviModule.EDITORMODE) {
                   // set the module in the simulation mode
                getLearningModule().setMode(LeviModule.SIMULATIONMODE);
                getLearningModule().getFrame().changeMode();
            }

            getLearningModule().getSimulator().stepBackwardSimulation();

        }

        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("END leviActionBackSimulation.actionPerformed");


    }
}

