/* leviActionLanguageKonfiguration.java
 *
 * Created on 04.03.2005
 * Autor: Birgit Sirocic
 *
 */
package learningVisualizations.gui.actions;

import java.awt.event.ActionEvent;
import java.util.Locale;

import javax.swing.*;

import learningVisualizations.LeviModule;
import learningVisualizations.LeviResourceManager;



/**
 * @author sirocic
 *
 * Class: leviActionLanguageKonfiguration
 *
 * Description: The class "leviActionLanguageKonfiguration" is the class which realizes the function of
 * 				the menu item and the button "Set Language", respectively.
 *
 * Date:	04.03.2005
 */
public class LeviActionLanguageKonfiguration extends LeviAbstractAction
{
    static final long serialVersionUID=1;

    private String newLanguage;
    
    //	 Constructors
    public LeviActionLanguageKonfiguration()
    {
        // Is needed for the instantiation in the class leviMenu!!
    }
    public LeviActionLanguageKonfiguration(LeviModule learningModule)
    {
        setLearningModule(learningModule);
    }
    public LeviActionLanguageKonfiguration(LeviModule learningModule, String name)
    {
        putValue(Action.NAME, name);
        setLearningModule(learningModule);
        
    }
    public LeviActionLanguageKonfiguration(LeviModule learningModule, String name, Icon icon, String tip)
    {
        putValue(Action.NAME, name);
        putValue(Action.SMALL_ICON, icon);
        putValue(Action.SHORT_DESCRIPTION, tip);
        setLearningModule(learningModule);

    }
    public LeviActionLanguageKonfiguration(LeviModule learningModule, String name, KeyStroke acc)
    {
        putValue(Action.NAME, name);
        putValue(Action.ACCELERATOR_KEY, acc);
        setLearningModule(learningModule);

    }
    // END	 Constructors

  /*
   * @author sirocic
   *
   * Methode:		actionPerformed
   *
   * Description:	Realizing the function of the menu item or button "Set Language".
   *
   * Date: 		04.03.2005
   */
    public void actionPerformed(ActionEvent event)
    {
        LeviResourceManager resources = getLearningModule().getRessources();
        Locale oldLocale = resources.getLocale();

        // get the messages for the dialog _before_ changing locales
        String title = resources.getResourceByKey("dlg.language.notset.title");
        String message = resources.getResourceByKey("dlg.language.notset.message");

        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("CALL leviActionLanguageKonfiguration.actionPerformed: event="+event.toString());

        // neue Sprache bestimmen;
        // beim Anlegen des Mens muss fr jede Sprache ein Krzel (z.B. de, en) mitangegeben werden.
        newLanguage = getValue("LANGUAGE").toString();

        // Sprache in den Ressourcen ndern
        Locale.setDefault(new Locale(newLanguage));
        if( resources.setLocale(Locale.getDefault()) ) {
        	// Neue Sprache in der Komponente bernehmen
        	getLearningModule().setLanguage(newLanguage);
        	getLearningModule().getFrame().setLocale(Locale.getDefault());
        	
      	    // Titel der Frame aktualisieren, da es ein Sprachwechsel gegeben hat.
          	if (getLearningModule().getXmlScenarioFile() != null)
          		getLearningModule().getFrame().setTitle(getLearningModule().getRessources().getResourceByKey("frame.title")+ " "+
        				getLearningModule().getVersion()+ " - " + getLearningModule().getXmlScenarioFile().getName());
        	else
        		getLearningModule().getFrame().setTitle(getLearningModule().getRessources().getResourceByKey("frame.title")+ " "+
        				getLearningModule().getVersion());
        	
          	// bei allen Subkomponenten und Dialogen die Texte anpassen an die neue Sprache.
            getLearningModule().getFrame().updateLanguageColorFonts();
            
            // die Sprache wird als aktuelle Sprache in der Konfiguration eingetragen.
            getLearningModule().getConfiguration().putSettingByKey("Language", newLanguage);
            // Konfiguration abspeichern...
            getLearningModule().getConfiguration().saveConfiguration();
        } else {
            // roll back - resource bundle not found!
            Locale.setDefault( oldLocale );
            if( !resources.setLocale( oldLocale ) ) {
                System.err.println( "Internal error: Cannot reset Locale");
            }
            // show a dialog indicating that the language could not be changed.
            getLearningModule().getFrame().showOkOptionMessageDialog(message, title);
           
        }

        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("END leviActionLanguageKonfiguration.actionPerformed");

    }
}


