/* leviActionNewScenario.java
 *
 * Created on 02.03.2005
 * Autor: Birgit Sirocic
 *
 */
package learningVisualizations.gui.actions;

import java.awt.EventQueue;
import java.awt.event.*;

import javax.swing.*;

import learningVisualizations.LeviModule;
import learningVisualizations.scenario.LeviScenario;
import learningVisualizations.scenario.LeviScenarioInterface;




/**
 * @author sirocic
 *
 * Class: leviActionNewScenario
 *
 * Description: The class "leviActionNewScenario" is the class which realizes the function of
 * 				the menu item and the button "New Scenario", respectively.
 *
 * Date:	02.03.2005
 */
public class LeviActionNewScenario extends LeviAbstractAction
{
    
    //	 Constructors
    public LeviActionNewScenario()
    {
        // Is needed for the instantiation in the class leviMenu!!
    }
    public LeviActionNewScenario(LeviModule learningModule, String name, Icon icon, String tip)
    {
        putValue(Action.NAME, name);
        putValue(Action.SMALL_ICON, icon);
        putValue(Action.SHORT_DESCRIPTION, tip);
        setLearningModule(learningModule);
    }
    public LeviActionNewScenario(LeviModule learningModule, String name, KeyStroke acc)
    {
        putValue(Action.NAME, name);
        putValue(Action.ACCELERATOR_KEY, acc);
        setLearningModule(learningModule);
    }
    // END	 Constructors

  /*
   * @author sirocic
   *
   * Methode:		actionPerformed
   *
   * Description:	Realizing the function of the menu item or button "New Scenario".
   *
   * Date: 		02.03.2005
   */
    public void actionPerformed(ActionEvent event)
    {

        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("CALL leviActionNewScenario.actionPerformed: event="+event.toString());

        // Ist die Komponente im SIMULAITONSMODUS d.h. luft gerade die Visualisierung???
        if (getLearningModule().getMode() == LeviModule.SIMULATIONMODE) {
        	// SIMULATION STOPPEN !!!
			// Wenn es einen ActionButton gibt der disabled ist, 
			// wird dieser wieder auf enable gesetzt.
			resetActiveAction();
			
	        if (getLearningModule().getVisualizer().isBusy()) {
	        	// Es sind noch Animationen akiv vom vorherigen Schritt.
	        	// Die mssen abgebrochen werden. Der Endzustand des vorherigen Schritts muss
	        	// eingestellt werden.
		        try {
				       EventQueue.invokeLater( new Runnable() {
			                  public void run() {
			                	  getLearningModule().getVisualizer().getScenarioVisualizer().cancelAnimation(true);
			                  }
			           }); 
				}
			    catch (Exception exception)
			    {
			        // Irgendwas ist mit dem Repaint wohl nicht richtig gelaufen!!!
			    	System.out.println("ERROR (leviActionStopSimulation)"+exception.getMessage());
			    }
	        }
	        
			getLearningModule().getSimulator().stopSimulation();
			
			if ( getLearningModule().getView() == LeviModule.SELFSTUDYVIEW) {
				// In der Selbststudiumssicht muss die Komponente zurck in den Editormodus.
				 getLearningModule().setMode(LeviModule.EDITORMODE);
				 getLearningModule().getFrame().changeMode();
			}
        }
		
        LeviScenarioInterface scenario = getLearningModule().getScenario();
        if ((scenario != null) && scenario.hasChanged()) {
            // Es gibt noch nicht abgespeicherte nderungen im Szenario!!!
            // Nachfragen, ob die abgespeichert werden sollen...
            // Antwortet der Benutzer mit Ja, wird in der Methode direkt das Speichern
            // veranlasst.
            getLearningModule().askUserToSave();
        }

        //create new empty scenario object
        getLearningModule().newScenario();
        
        // Oberflchst neuzeichnen ...
        getLearningModule().getFrame().refresh();

        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("END leviActionNewScenario.actionPerformed");

    }
}
