/* leviActionOpenScenario.java
 *
 * Created on 02.03.2005
 * Autor: Birgit Sirocic
 *
 */
package learningVisualizations.gui.actions;

import java.awt.Color;
import java.awt.EventQueue;
import java.awt.event.*;
import java.io.File;

import javax.swing.*;

import learningVisualizations.LeviModule;
import learningVisualizations.gui.LeviFrame;
import learningVisualizations.scenario.LeviScenario;
import learningVisualizations.scenario.LeviScenarioInterface;



/**
 * @author sirocic
 *
 * Class: leviActionOpenScenario
 *
 * Description: The class "leviActionOpenScenario" is the class which realizes the function of
 * 				the menu item and the button "Open scenario", respectively.
 *
 * Date:	02.03.2005
 */
public class LeviActionOpenScenario extends LeviAbstractAction
{

    //	 Constructors
    public LeviActionOpenScenario()
    {
        // Is needed for the instantiation in the class leviMenu!!
    }
    public LeviActionOpenScenario(LeviModule learningModule, String name)
    {
        putValue(Action.NAME, name);
        setLearningModule(learningModule);
    }
    public LeviActionOpenScenario(LeviModule learningModule, String name, Icon icon, String tip)
    {
        putValue(Action.NAME, name);
        putValue(Action.SMALL_ICON, icon);
        putValue(Action.SHORT_DESCRIPTION, tip);
        setLearningModule(learningModule);
    }
    public LeviActionOpenScenario(LeviModule learningModule, String name, KeyStroke acc)
    {
        putValue(Action.NAME, name);
        putValue(Action.ACCELERATOR_KEY, acc);
        setLearningModule(learningModule);
    }
    // END	 Constructors

  /*
   * @author sirocic
   *
   * Methode:		actionPerformed
   *
   * Description:	Realizing the function of the menu item or button "Open Scenario".
   *
   * Date: 		02.03.2005
   */
    public void actionPerformed(ActionEvent event)
    {
        int result;
        File scenarioFile;

        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("CALL leviActionOpenScenario.actionPerformed: ");

        // Ist die Komponente im SIMULAITONSMODUS d.h. luft gerade die Visualisierung???
        if (getLearningModule().getMode() == LeviModule.SIMULATIONMODE) {
        	// SIMULATION STOPPEN !!!
			// Wenn es einen ActionButton gibt der disabled ist, 
			// wird dieser wieder auf enable gesetzt.
			resetActiveAction();
			
	        if (getLearningModule().getVisualizer().isBusy()) {
	        	// Es sind noch Animationen akiv vom vorherigen Schritt.
	        	// Die mssen abgebrochen werden. Der Endzustand des vorherigen Schritts muss
	        	// eingestellt werden.
		        try {
				       EventQueue.invokeLater( new Runnable() {
			                  public void run() {
			                	  getLearningModule().getVisualizer().getScenarioVisualizer().cancelAnimation(true);
			                  }
			           }); 
				}
			    catch (Exception exception)
			    {
			        // Irgendwas ist mit dem Repaint wohl nicht richtig gelaufen!!!
			    	System.out.println("ERROR (leviActionStopSimulation)"+exception.getMessage());
			    }
	        }
	        
			getLearningModule().getSimulator().stopSimulation();
			
			if ( getLearningModule().getView() == LeviModule.SELFSTUDYVIEW) {
				// In der Selbststudiumssicht muss die Komponente zurck in den Editormodus.
				 getLearningModule().setMode(LeviModule.EDITORMODE);
				 getLearningModule().getFrame().changeMode();
			}
        }
        
        // JETZT FFNEN EINLEITEN
        LeviScenarioInterface scenario = getLearningModule().getScenario();
        if ((scenario != null) && scenario.hasChanged()) {
            // Es gibt noch nicht abgespeicherte nderungen im scenario!!!
            // Nachfragen, ob die abgespeichert werden sollen...
            // Antwortet der Benutzer mit Ja, wird in der Methode direkt das Speichern
            // veranlasst.
            getLearningModule().askUserToSave();
        }

        // Show the file open dialog and read the given file name
        getLearningModule().getFileOpenSaveDialog().setDialogTitle(getLearningModule().getRessources().getResourceByKey("FileChooser.openButtonText"));
        result=getLearningModule().getFileOpenSaveDialog().showOpenDialog((LeviFrame)getLearningModule().getFrame());
        if (result != JFileChooser.APPROVE_OPTION) return;

        // ES WURDE EIN scenario AUSGEWHLT !!!        
        // JETZT WIRD DAS NEUE scenario GEFFNET!
        
        // File has been selected
        scenarioFile = getLearningModule().getFileOpenSaveDialog().getSelectedFile();

        // create new scenario object
        getLearningModule().openScenario(scenarioFile);
        
        // Oberflchst neuzeichnen ...
        getLearningModule().getFrame().refresh();
        
        // TRACE OUTPUT
        if (LeviModule.isTraceActive()) System.out.println("END leviActionOpenScenario.actionPerformed");

    }

}
