/* leviHelp.java
 * 
 * Created on 23.06.2005
 * @author sirocic
 */
package learningVisualizations.help;

import java.awt.*;

import javax.swing.*;

import java.io.*;
import java.net.*;

import learningVisualizations.LeviModule;




/**
 * @author sirocic
 *
 * Class: leviHelp
 * 
 * Description: The class "leviHelp" is the general help.
 * 
 * Date:	23.06.2005
 */
public abstract class LeviHelp extends JPanel implements LeviHelpInterface {

	static final long serialVersionUID=1;
	
	public static final int HELPWIDTH=300;
	
	protected LeviModule learningModule;
	
	// Infos zum Finden der richtigen HTML-Datei
	// base = Ressource Basisverzeichnis fr die HTML-Dateien
    protected String base;
    protected String loadedPage;


  /*
   * Methode:	Contructors
   * 
   * Date: 		23.06.2005
   * @author    sirocic
   */
   public LeviHelp() {
	  super();
	  this.learningModule = null;
	  base = null;
	  loadedPage = null;
	  
   }
   public LeviHelp(LeviModule learningModule) {
	   
	  super();
	  
	  URL test;
	  File testFile;
	  String errorMsg, errorTitle;
	  
	  // TRACE OUTPUT
	  if (LeviModule.isTraceActive()) System.out.println("CALL leviHelp.Contructors(learningModule)");
		  		
	  // setting the module
	  this.learningModule = learningModule;
 	  
	  // init the help view
	  setLayout(new BorderLayout());
	  
	  loadedPage = null;
	  if (learningModule.getConfiguration().keyExist("BasisHTMLHelp")) 
		  base = (String) learningModule.getConfiguration().getSettingByKey("BasisHTMLHelp");
	  else if (learningModule.getConfiguration().keyExist("BasisHTMLAvatar")) 
		  base = (String) learningModule.getConfiguration().getSettingByKey("BasisHTMLAvatar");
	  else
		  base = "";
	  
	  //FARBWECHSEL ...
 	  setPanelColors();

	  // FONTWECHSEL
	  setPanelFont();
	  
	  // TRACE OUTPUT
	  if (LeviModule.isTraceActive()) System.out.println("END leviHelp.Contructors(learningModule)");
	
   }
   
   // Ldt den Inhalt des Files in den Help-Panel. Wird vom jeweiligen Komponenten-Help
   // implementiert.
   public abstract void setNewContent(String file);
   
 
   public LeviModule getLearningModule()
   {
   	return learningModule;
   }
   public String getLoadedPage() {
   	return loadedPage;
   }

   // MUTATOR METHODES...
   public void setLearningModule(LeviModule learningModule)
   {
      this.learningModule=learningModule;
   }
   public void setLoadedPage(String loadedPage) {
	   	this.loadedPage = loadedPage;
   }
	   
   /*
    * Methode:		paintComponent
    * 
    * Description:	Calls the paintComponent methode of the JPanel.
    *    
    * Parameters:    
    * 
    * Date: 		 13.09.2005
    * @author        sirocic
    */	
    public void paint(Graphics g) {
      
    	
	  // TRACE OUTPUT
	  if (LeviModule.isTraceActive()) System.out.println("CALL leviHelp.paintComponent:");
	  	  
	  // here calling the paint methode of the JPanel
	  super.paint(g);
	  
  	  // TRACE OUTPUT
  	  if (LeviModule.isTraceActive()) System.out.println("END leviHelp.paintComponent");

  	
   }
   
    /*
     * Methode:		setDialogFont
     * 
     * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
     * 				Werte. 
     *    
     * Parameters:   Aktuelle Font fr Labels und Textfelder.
     * 
     * Date: 		 23.04.2007
     * @author        sirocic
     */	
    private void setPanelFont() {
 	   Font labelFont;
 	   
 	   labelFont = (Font)learningModule.getConfiguration().getSettingByKey("FontLabel");
 	   
 	   setFont(labelFont);
 	   	  
    }
    
    /*
     * Methode:		setDialogTexte
     * 
     * Description:	Setz alle Text-Bausteine des Dialoges neu. Wichtig fr Sprachwechsel.
     *    
     * Parameters:   
     * 
     * Date: 		 23.04.2007
     * @author        sirocic
     */	
    private void setPanelTexte() {
 	  String file;
 	  
 	  if (loadedPage != null) {
		   file =  loadedPage.substring(loadedPage.lastIndexOf("/")+1);
		   this.setNewContent(file);
	   }
 	   
    }
    
    /*
     * Methode:		setDialogColors
     * 
     * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
     * 				Werte. 
     *    
     * Parameters:   Aktuelle Farbwerte fr Hintergrund und Vordergrund.
     * 
     * Date: 		 23.04.2007
     * @author        sirocic
     */	
    private void setPanelColors() {
    	Color backgroundColor, foregroundColor;
        
        if (learningModule.getConfiguration().getSettingByKey("ColorComponentBackground")!=null) {
	        backgroundColor = (Color)learningModule.getConfiguration().getSettingByKey("ColorComponentBackground");
	        foregroundColor = (Color)learningModule.getConfiguration().getSettingByKey("ColorComponentForeground");
	    }
	    else {
	    	backgroundColor = (Color)learningModule.getConfiguration().getSettingByKey("ColorModuleBackground");
	        foregroundColor = (Color)learningModule.getConfiguration().getSettingByKey("ColorModuleForeground");
	    }
        
 	   // 	 Hintergrund und Zeichenfarbe auf aktuelle Werte setzen
       setBackground(backgroundColor);
 	   setForeground(foregroundColor);
      
    }
    
    /*
     * Methode:		updateLanguageColorFonts
     *
     * Description:	Ld die aktuelle Seite nochmal bzw. dieselbe Seite in einer anderen Sprache.
     *
     * Date: 		07.03.2007
     * 
     * @author sirocic
     */
    public void updateLanguageColorFonts()
    {

 	   // 	 FARBWECHSEL ...
  	   setPanelColors();

	   // FONTWECHSEL
	   setPanelFont();
	  
	   // SPRACHWECHSEL ...
	   setPanelTexte();
 
    }
    

}
