/* leviObject.java
 * 
 * Created on 05.08.2005
 * @author sirocic
 */
package learningVisualizations.scenario;

import java.util.ArrayList;

import learningVisualizations.LeviModule;


/**
 * @author sirocic
 *
 * Class: leviObject
 * 
 * Description: The class "leviObject" is a general object which can be simulated and visualized.
 * 
 * Date:	05.08.2005
 */
public abstract class LeviObject {
	
	final static public int DEFAULTWIDTH=80;
	final static public int DEFAULTHEIGHT=100;
	
	private LeviModule learningModule;
	 
	private String name;  // Name for the object item. Will be displayed in the GUI
	private static int nextId=1; // Distinct next Id for all new objects.
	private int id;    // Internal identity for the object (distinct)
	private int topY;   // Y coordinate of the upper left corner of the communication figur
	private int leftX;  // X coordinate of the upper left corner of the communication figur
	private int width;   // width of the buffer
	private int height;  // height of the buffer
	
	// HISTORY enthlt die Objektzustnde (Rckwrts Visualisierung)
	private ArrayList history;
	

  /**
   * Methode:	  Contructors
   * Description:	 
   * Parameters:  name = scenario name, id = object identity
   * Date: 		  05.08.2005
   * @author      sirocic
   */
	public LeviObject() {
		this.name = "";
    	setId();
    	this.topY=0;
   	    this.leftX=0;
   	    this.width = DEFAULTWIDTH;
   	    this.height = DEFAULTHEIGHT;
   	    learningModule = null;
   	    // History initialisieren
     	history = new ArrayList();
     	  
	}	
	public LeviObject(LeviModule learningModule, String name, int topY, int leftX) {
		this.learningModule = learningModule;
		this.name = name;
		setId();
		this.topY=topY;
		this.leftX=leftX;
		this.width = DEFAULTWIDTH;
   	    this.height = DEFAULTHEIGHT;
   	    // History initialisieren
     	history = new ArrayList();
     	  
	}
	public LeviObject(LeviModule learningModule, String name, int id, int topY, int leftX) {
		this.learningModule = learningModule;
		this.name = name;
		setId(id);
		this.topY=topY;
		this.leftX=leftX;
		this.width = DEFAULTWIDTH;
   	    this.height = DEFAULTHEIGHT;
   	    history = new ArrayList();
	}
    public LeviObject(LeviModule learningModule, String name, int id, int topY, int leftX, int width, int height) {
    	this.learningModule = learningModule;
    	this.name = name;
    	setId(id);
    	this.topY=topY;
   	    this.leftX=leftX;
   	    this.width = width;
   	    this.height = height;
   	    history = new ArrayList();
   }
    public LeviObject(LeviModule learningModule, String name, int topY, int leftX, int width, int height) {
    	this.learningModule = learningModule;
    	this.name = name;
    	setId();
    	this.topY=topY;
   	    this.leftX=leftX;
   	    this.width = width;
   	    this.height = height;
   	    history = new ArrayList();
   }
   
  /*
   * Methode:		simulate 
   * 
   * Description:	Will be executed during a simulation step. 
   * 				It represents the behavior of the Object. Should be overwritten in the sub classes.
   * 
   * @param int time Aktuelle Zeitpunkt der Simuliert wird.
   * @return boolean - True, wenn die Simulation korrekt durchgefhrt werden konnte.
   * Date: 		    09.08.2005
   * @author        sirocic
   */
   public abstract boolean simulate(int time);
   
   /*
    * Methode:		init
    * 
    * Description:	Initialisiert das Objekt.
    * 
    * Date: 		    30.01.2008
    * @author        sirocic
    *
    */
    public abstract void init() ;
    
    /*
     * Methode:		simulateBack
     * 
     * Description:	Will be executed during a simulation step back. It sets the process back to
     *               a previous state. Within every simulation step the state of the process will
     *               be saved in a history. For simulating one step back the relevant process state
     *               will be set to the current state. 
     * 
     * Date: 		27.06.2006
     * @author        sirocic
     *
     */
     public abstract void goStepBack(int time);
     
   // ACCESSOR METHODES
   public LeviModule getLearningModule()
   {
       return learningModule;
   }
   public String getName() {
   	   return name;
   }
   public int getId() {
   	   return id;
   }
   public int getTopY() {
	   return topY;
   }
   public int getLeftX() {
	   return leftX;
   }
   public int getWidth() {
	   return width;
   }
   public int getHeight() {
	   return height;
   }
   public boolean equals(LeviObject object){
	   return ( (this.id == object.getId()) && (this.getClass().equals(object.getClass())));
   }
   
   // MUTATOR METHODES
   public void setName(String newName) {
   	   name=newName;
   }
   public void setId() {
   	   id=nextId;
   	   nextId++;
   }
   public void setId(int newId) {
   	   id=newId;
   	   
   	   // Wenn ein ID-Wert aus dem Szenariofile gelesen wurde und das Objekt
   	   // diese ID erhlt, muss der Wert fr die nchste gltige neue ID (nextId)
   	   // auf die nchst grere ganze Zahl der gelesenen ID gesetzt werden,
   	   // wenn sie grer oder gleich der aktuellen nextID ist.
   	   if (newId >= nextId)
   		   nextId = newId + 1;
   	   
   }
   public static void setNextId(int newNextId) {
   		   nextId = newNextId;
   	   
   }
   public void setTopY(int topY) {
   		this.topY=topY;
   }
   public void setLeftX(int leftX) {
   	   this.leftX=leftX;
   }
   public void setWidth(int width) {
	   this.width=width;
   }
   public void setHeight(int height) {
	   this.height=height;
   }
	public ArrayList getHistory() {
		return history;
	}
	public void setHistory(ArrayList history) {
		this.history = history;
	}
	public static int getNextId() {
		return nextId;
	}
	public void setLearningModule(LeviModule learningModule) {
		this.learningModule = learningModule;
	}

}
