/* leviScenario.java
 *
 * Created on 05.07.2005
 * @author sirocic
 */
package learningVisualizations.scenario;

import java.util.Iterator;

import javax.xml.parsers.*;

import learningVisualizations.LeviModule;

import org.w3c.dom.*;

/**
 * @author sirocic
 *
 * Class: leviScenario
 *
 * Description: The class "leviScenario" is a general scenario of the simulated and visualized content.
 *
 * Date:	05.07.2005
 */
public abstract class LeviScenario implements LeviScenarioInterface {

    // Konstantendefinitionen
	public static final String XMLTAGMODEL = "Model";
	public static final String XMLTAGSCENARIO = "Scenario";
    public final static int MAXNAMELEN=20;

    private LeviModule learningModule;

    private String scenarioname; // Scenario name
    private boolean open;
    private boolean changed;
    private boolean scenarioOk;

      /*
       * @author sirocic
       *
       * Methode:		Contructors
       *
       * Description:
       *
       * Parameters: name = Scenario name
       *
       * Date: 		05.07.2005
       */
       public LeviScenario() {
              scenarioname = "XXX";
              open = false;
              scenarioOk = false;
              changed=false;
           learningModule = null;
       }
       public LeviScenario(LeviModule learningModule) {
              this();
           // setting the module
           this.learningModule = learningModule;
           this.scenarioname = learningModule.getRessources().getResourceByKey("default.newscenario");
       }

       // ACCESSOR METHODES ...
       /*
        * Methode:		isOk
        *
        * Description:	Returns the result value of the scenario check.
        *
        * Parameters:   return = true if the current scenario is a correct simulatable scenario
        *
        * Date: 		 28.09.2005
        * @author        sirocic
        */
       public boolean isOk() {
    	      scenarioOk = checkScenario();
    	      
              return scenarioOk;
       }

       /*
        * Methode:		checkScenario
        *
        * Description:	Checks if the scenario is a correct scenario or not.
        * 				HAVE TO BE SPECIFIED IN THE SUBCLASS !!!
        *
        * Parameters:   return = true if the current scenario is a correct simulatable scenario
        *
        * Date: 		 28.09.2005
        * @author        sirocic
        */
       public abstract boolean checkScenario() ;

       /*
        * Methode:		createObjectsFromXML
        * 
        * Description:	Erzeugt zu den im XML-file spezifizierten Objekten die entsprechenden Szenario-objekte.
        * 
        * Parameters:   xmlObjects = xml element of the objects
        * 
        * Date: 		30.01.2008
        * @author       sirocic
        */
        public abstract void createObjectsFromXML(Element xmlObjects);
        
       /**
        * Methode:		buildDOMElements
        *
        * Description:	Creates the DOM structure of the current scenario.
        *
        * Parameters:   return = The DOM element scenario
        *
        * Date: 		 21.04.2006
        * @param scenario
        * @param xmlDoc
        * @return The scenario as DOM Element tree.
        * @author        sirocic
        */
       public abstract Element buildDOMElements(Element scenario, Document xmlDoc) throws ParserConfigurationException ;

       public LeviModule getLearningModule()
       {
           return learningModule;
       }
       public String getName() {
              return scenarioname;
       }
       public boolean isOpen() {
              return open;
       }
       public boolean hasChanged() {
              return changed;
       }
       public void setOpen(boolean value) {
              open=value;
       }


       //	 MUTATOR METHODES ...
       /*
        * Methode:		init
        * 
        * Description:	Initialisiert das Szenario d.h. alle Objekte des Szenario, die einen Zustand
        * 				whrend der Simulation annehmen. Sollte vorm Start vom Simulator aufgerufen werden.
        *    			Wird falls notwendig vom Szenario der jeweiligen Komponente implementiert.
        *    
        * Date: 		 07.02.2007
        * @author        sirocic
        */	
       public abstract void init() ;
       
       public void setLearningModule(LeviModule c)
       {
          learningModule=c;
       }
       public void setScenarioOk(boolean value) {
              scenarioOk=value;
       }
       public void setChanged() {
              changed=true;
       }
       public void setUnchanged() {
              changed=false;
       }
       public void setName(String name) {
              scenarioname=name;
       }

}
