/* leviSimulator.java
 *
 * Created on 23.06.2005
 * @author sirocic
 */
package learningVisualizations.simulator;

import java.awt.EventQueue;

import learningVisualizations.LeviModule;




/**
 * @author sirocic
 *
 * Class: leviSimulator
 *
 * Description: The class "leviSimulator" is the general simulator.
 *
 * Date:	23.06.2005
 */
public abstract class LeviSimulator extends Thread  implements LeviSimulatorInterface {

    //	 CONSTANTS
    public static final int STEPWIDTH = 1;
    public static       int SLEEPUNIT=200;
    public static final int MAXSLEEPFACTOR=30;
    public static       int MINSLEEPFACTOR=1;

    private LeviModule learningModule;
    private LeviTimer timer;
    private boolean stopRequested;
    private LeviSuspendRequestor suspender;
    private int nextStopTime;
    protected int sleepFactor;

    private boolean backward, foreward;
    private boolean play, pause, stop;

  /*
   * Methode:		Contructors
   * Description:
   * Date: 		23.06.2005
   * @author sirocic
   */
   public LeviSimulator() {
          // building the parts of the simulator
          this.timer = new LeviTimer();
          this.timer.setStep(STEPWIDTH);
          this.suspender = new LeviSuspendRequestor();

          // setting the attributs
          this.stopRequested = false; // simulation does not run now!
          this.nextStopTime = 0;
          this.sleepFactor = 15;
          this.backward = false;
          this.foreward = false;
          this.play = false;
          this.pause = false;
          this.stop = true;
          this.learningModule=null;
   }
   public LeviSimulator(LeviModule learningModule) {
      this();
      // setting the module
      this.learningModule = learningModule;
      // Check if we actually have this value and set it if we do.
      Integer sf = (Integer) learningModule.getConfiguration().getSettingByKey("SleepFactor");
      if( sf != null ) {
          this.sleepFactor = sf.intValue();
      }
   }


   // EXECUTES A STEP OF THE SZENARIOSIMULATION
   public abstract boolean executeStep() ;
   // SET A SIMULATION STEP BACK
   public abstract void executeStepBack();

   public void init() {

        // set time to start value 0
        timer.reset();

        // reset the stop time for a stepwise simulation
        nextStopTime = 0;

    }

   public void startSimulatorThread() {
	   start();
   }
   /*
    * Methode:		run
    *
    * Description:	Runs the simulation of the current Scenario.
    *
    * Parameters:
    *
    * Date: 		 28.09.2005
    * @author        sirocic
    * @see java.lang.Thread#run()
    */
   public void run() {

       // TRACE OUTPUT
       if (LeviModule.isTraceActive()) System.out.println("CALL leviSimulator.run:");    

       while ( !isStopRequested() ) {
           try
           {
               // if the simulator thread was started it have to wait until the simulation will be startet too.
               suspender.waitForResume();
               
               // suspend if the next step time is archieved
               if ((timer.getTime() == getNextStopTime())) {
                   suspender.set(true);
               }

               suspender.waitForResume();
               
               if (isForeward()) {
                   // simulate one time step...
                   executeStep();

                   // Set next time value
                   timer.nextTime();
               }
               // DF 2006-06-25: Explicitly ask for backward, because
               // both forward and backward may be false!
               else if (isBackward()) {
                   // simulate one time step...
                   executeStepBack();

                   // Set next time value
                   timer.previousTime();
               }
        
               // refresh Visualizer
               EventQueue.invokeLater( new Runnable() {
                   public void run() {
                       getLearningModule().getVisualizer().visualize();
                   }
               });
                   
               sleep(SLEEPUNIT*sleepFactor);
            
               // Wenn alle Animationen abgeschlossen sind darf's erst weitergehen.
               while (getLearningModule().getVisualizer().isBusy()) 
            	   sleep(SLEEPUNIT*sleepFactor);
               
           } // End while

           catch (InterruptedException exception)
           {
               // Interrupt during a sleep
               // just finish the method

               // DF 2006-06-07: semantics changed. Ignore the exception
               // (continue loop)
           }
       }


       // TRACE OUTPUT
       if (LeviModule.isTraceActive()) System.out.println("END leviSimulator.run !!!!");
   }

   // METHODES FOR THE SIMULATOR CONTROL ...
   // Diese Methoden gehren spter auf jeden Fall ins SimulatorInterface!!!!!
   /*
    * Methode:		playSimulation
    *
    * Description:	Starten den Endlosablauf der Simulation. Der Simulator-Thread luft
    *               bis stopsimulation oder pausSimulation aufgerufen wird.
    *
    * Parameters:
    *
    * Date: 		 20.01.2006
    * @author        sirocic
    */
   public void playSimulation()
   {

       if (timer.getTime()==0) {
          // Start simulation from the beginning
         // init simulator ...
         init();
       }
       // BS 15.12.05 Anforderung von Alex
       setNextStopTime(-1);
       // END BS 15.12.05 Anforderung von Alex

       // start simulation ...
       foreward = true;
       backward = false;
       play = true;
       pause = false;
       stop = false;

       //resume
       suspender.set(false);

   }

   /*
    * Methode:		stopSimulation
    *
    * Description:	Hlt die Simulation an und setzt den Simulator zurck auf den
    *               Simulationsstartpunkt.
    *
    * Parameters:
    *
    * Date: 		 20.01.2006
    * @author        sirocic
    */
   public void stopSimulation()
   {

        // suspending the simulation
        suspender.set(true);

        // reset simulator to starting time.
       init();

       // set the status
       foreward = false;
       backward = false;
       play = false;
       pause = false;
       stop = true;
       this.interrupt();

   }

   /*
    * Methode:		pauseSimulation
    *
    * Description:	Hlt die Simulation an.
    *
    * Parameters:
    *
    * Date: 		 20.01.2006
    * @author        sirocic
    */
   public void pauseSimulation()
   {
        // suspend (halt) the simulator
        suspender.set(true);

        //set the status
        play = false;
        pause = true;
        stop = false;
        this.interrupt();
   }

   /*
    * Methode:		stepForwardSimulation
    *
    * Description:	Veranlat den Simulator einen Schritt weiter zu simulieren und dann wieder
    *               anzuhalten.
    *
    * Parameters:
    *
    * Date: 		 20.01.2006
    * @author        sirocic
    */
   public void stepForwardSimulation()
   {
       int nextTime;

       if (timer.getTime()==0) {
          // init simulator ...
          init();
       }

       // start simulation step ...
       nextTime = timer.getTime() + timer.getStep();
       setNextStopTime(nextTime);

       suspender.set(false);

       //set the status
       foreward = true;
       backward = false;
       play = false;
       pause = true;
       stop = false;
       this.interrupt();
   }

   /*
    * Methode:		stepBackwardSimulation
    *
    * Description:	Veranlat den Simulator einen Schritt zurck zu simulieren und dann wieder
    *               anzuhalten.
    *
    * Parameters:
    *
    * Date: 		 20.01.2006
    * @author        sirocic
    */
   public void stepBackwardSimulation()
   {
       int backTime;

       if (timer.getTime()>0) {

           // start simulation ONE step BACK ...
           backTime = timer.getTime() - timer.getStep();
           setNextStopTime(backTime);
           //set the status
           backward = true;
           foreward = false;
           play = false;
           pause = true;
           stop = false;

           // resume
           suspender.set(false);
           this.interrupt();
       }

   }
   /*
    * Methode:		speedupSimulation
    *
    * Description:	Beschleunigt die Simulation. Inkrementieren eines Factors fr die Lnge
    *               des Waits im Simulationszyklus.
    *
    * Parameters:
    *
    * Date: 		 16.05.2006
    * @author        sirocic
    */
   public void speedupSimulation()
   {
       if (sleepFactor > MINSLEEPFACTOR)
           sleepFactor--;
   }
   /*
    * Methode:		slowdownSimulation
    *
    * Description:	Verlangsamt die Simulation. Dekrementieren eines Factors fr die Lnge
    *               des Waits im Simulationszyklus.
    *
    * Parameters:
    *
    * Date: 		 16.05.2006
    * @author        sirocic
    */
   public void slowdownSimulation()
   {
       if (sleepFactor <= MAXSLEEPFACTOR )
       sleepFactor++;
   }

   // METHODES FOR THE SIMULATOR THREAD CONTROL ...
   // DF 20060530: changed visibility from protected to public, so there's a
   // chance for the module to stop this thread and exit cleanly without
   // simply destroying the VM
   public void requestStop()
   {
       stopRequested=true;
       interrupt();

   }
   protected void requestSuspend()
   {
       suspender.set(true);
   }
   protected void requestResume()
   {
       suspender.set(false);
   }

   public void setLearningModule(LeviModule c)
   {
      learningModule=c;
   }
   public void setNextStopTime(int time)
   {
       nextStopTime = time;
   }
   public void setBackward(boolean b)
   {
        backward=b;
   }
   public void setForeward(boolean b)
   {
        foreward=b;
   }
   public void setPlay(boolean b)
   {
        play=b;
   }
   public void setPause(boolean b)
   {
        pause=b;
   }
   public void setStop(boolean b)
   {
        stop=b;
   }
   // ACCESSOR METHODES...
   public LeviModule getLearningModule()
   {
       return learningModule;
   }
   public boolean isStopRequested()
   {
          return stopRequested;
   }
   public LeviTimer getTimer()
   {
       return timer;
   }
   public LeviSuspendRequestor getSuspender()
   {
       return suspender;
   }
   public int getNextStopTime()
   {
       return nextStopTime;
   }
   public boolean isBackward()
   {
       return backward;
   }
   public boolean isForeward()
   {
       return foreward;
   }
   public boolean isPlay()
   {
       return play;
   }
   public boolean isPause()
   {
       return pause;
   }
   public boolean isStop()
   {
       return stop;
   }
   public int getSleepFactor()
   {
       return sleepFactor;
   }
   public int getSleepUnit()
   {
       return SLEEPUNIT;
   }

}
