/* leviScenarioVisualizerMouseListener.java
 * 
 * Created on 19.10.2006
 * @author sirocic
 */
package learningVisualizations.visualizer;
import java.awt.*;
import java.awt.event.*;

import learningVisualizations.LeviModule;
import learningVisualizations.editor.LeviScenarioEditorVisualizer;
import learningVisualizations.gui.LeviPopup;
import learningVisualizations.gui.LeviScenarioPanel;





/**
 * @author sirocic
 *
 * Class: leviScenarioVisualizerMouseListener
 * 
 * Description: Handles the mouse events for the scenario scenarioPanel or visualizer, respectivly.
 * 
 * Date:	17.10.2006
 */
public class LeviScenarioVisualizerMouseListener extends MouseAdapter {
	
	protected LeviModule learningModule;
	private LeviPopup popup;
	protected LeviScenarioPanel scenarioPanel;
	protected LeviShapeMaker selected;
	
	/* 
    * Methode:	Constructors
    * 
    * Date: 18.03.2005
    * @author sirocic
    */
	public LeviScenarioVisualizerMouseListener()
	{
		super();
		
		this.scenarioPanel = null;
		this.selected = null;
		this.popup = null;
	}
	public LeviScenarioVisualizerMouseListener(LeviModule learningModule, LeviScenarioPanel scenarioPanel)
	{
		this();
		this.learningModule = learningModule;
		this.scenarioPanel = scenarioPanel;
		this.popup = learningModule.getFrame().getPopupMenu();
	}
	public LeviScenarioVisualizerMouseListener(LeviModule learningModule)
	{
		this();
		this.learningModule = learningModule;
		this.popup = learningModule.getFrame().getPopupMenu();
	}
	public LeviScenarioVisualizerMouseListener(LeviVisualizer visualizerPanel)
	{
		this();
		this.learningModule = visualizerPanel.getLearningModule();
		this.popup = visualizerPanel.getLearningModule().getFrame().getPopupMenu();
	}
	
	//	 MUTATOR METHODES ...
	
	/*
	 * Methode:		mousePressed
	 * 
	 * Description:	This method will be called if the user press a mouse key.
	 *              
	 * 
	 * Date: 		17.10.2006
	 * @author      sirocic
	 */
	public void mousePressed(MouseEvent event) {

		// TRACE OUTPUT
	  	if (LeviModule.isTraceActive()) System.out.println("CALL leviScenarioVisualizerMouseListener.mousePressed:");
		  
	  	pressedAction(event);
	     
	  	// TRACE OUTPUT
	  	if (LeviModule.isTraceActive()) System.out.println("END leviScenarioVisualizerMouseListener.mousePressed");
  	
	}
	
	/*
	 * Methode:		mouseReleased
	 * 
	 * Description:	Diese Methode wird aufgerufen, wenn der Benutzer die Maustaste wieder loslt.
	 *              Beim Verschieben eines Objektes mu nichts weiter gemacht werden an der Stelle.
	 *              
	 * Date: 		17.10.2006
	 * @author      sirocic
	 */
	public void mouseReleased(MouseEvent event) {
	
		// TRACE OUTPUT
	  	if (LeviModule.isTraceActive()) System.out.println("CALL leviScenarioVisualizerMouseListener.mouseReleased:");
		  
	  	releasedAction(event);
	        	
	  	// TRACE OUTPUT
	  	if (LeviModule.isTraceActive()) System.out.println("END leviScenarioVisualizerMouseListener.mouseReleased");
	
	}
	
  /*
   * Methode:		mouseClicked
   * 
   * Description:	This method contains the actions which will take place if the user has
   *                clicked the mouse. If a buffer or process is founded at the mouse positionl,
   *                this will be the currently selected object in the scenarioPanel.
   * 
   * Date: 		    18.03.2005
   * @author        sirocic
   */
	public void mouseClicked(MouseEvent event) {
		
	  // TRACE OUTPUT
	  if (LeviModule.isTraceActive()) System.out.println("CALL leviScenarioVisualizerMouseListener.mouseClicked:");
	 
	  clickedAction(event);
		
  	  // TRACE OUTPUT
  	  if (LeviModule.isTraceActive()) System.out.println("END leviScenarioVisualizerMouseListener.mouseClicked");

	}
		

		
	/*
	 * Methode:		pressedAction
	 * 
	 * Description:	Diese Methode ist fr die allgemeine Eventbehandlung beim Drcken einer Maustaste
	 *              verantwortlich, die fr alle Lernmodule identisch ist. Das ist z.b wenn die rechte
	 *              Maustaste bettigt wurde fr PopupMenu, wird diese angezeigt. Wurde nicht die
	 *              linke Maustaste bettigt, wird zunchst geprft, ob die Maus ber einem Objekt ist
	 *              oder nicht. Wurde also ein Objekt selectiert, wird dieses in der Variable <selected>
	 *              abgelegt, andernfalls ist <selected>=null.
	 *              
	 *              Diese Methode dient beim Entwickeln von Subklassen dazu doppelten Code zu vermeiden
	 * 
	 * Date: 		17.10.2006
	 * @author      sirocic
	 */
	public void pressedAction(MouseEvent event) {
		
		// TRACE OUTPUT
	  	if (LeviModule.isTraceActive()) System.out.println("CALL leviScenarioVisualizerMouseListener.pressedAction:");
	
	  	// Sceanrio Editor gets the focus ...
	  	if (scenarioPanel!= null)
	  		scenarioPanel.getScenarioParent().getParentPanel().requestFocus();
	  	
        if (event.isPopupTrigger()) {
        	// Popup-Kontext zeigen ... 
            popup.show(event.getComponent(),
            		event.getX(), event.getY());
        }
        else {
        	// Anderen Mause-Events behandeln

		  	// befindet sich die Maus ber einem Objekt?
        	if (scenarioPanel!= null)
        		selected = scenarioPanel.find(event.getPoint());

        }
	  
	  	// TRACE OUTPUT
	  	if (LeviModule.isTraceActive()) System.out.println("END leviScenarioVisualizerMouseListener.pressedAction");
  	
	}
	
	/*
	 * Methode:		releasedAction
	 * 
	 * Description:	Diese Methode ist fr die allgemeine Eventbehandlung beim Loslassen einer Maustaste
	 *              verantwortlich, die fr alle Lernmodule identisch ist. Im Moment bestehen diese nur
	 *              aus der Behandlung des Popupmenu-Events.
	 *              
	 *              Diese Methode dient beim Entwickeln von Subklassen dazu doppelten Code zu vermeiden
	 * 
	 * Date: 		19.10.2006
	 * @author      sirocic
	 */
	public void releasedAction(MouseEvent event) {
		
		// TRACE OUTPUT
	  	if (LeviModule.isTraceActive()) System.out.println("CALL leviScenarioVisualizerMouseListener.releasedAction:");
		
        if (event.isPopupTrigger()) {
        	// Popup-Kontext zeigen ... 
            popup.show(event.getComponent(),
            		event.getX(), event.getY());
        }
	  
	  	// TRACE OUTPUT
	  	if (LeviModule.isTraceActive()) System.out.println("END leviScenarioVisualizerMouseListener.releasedAction");
  	
	}
	
	/*
	 * Methode:		clickedAction
	 * 
	 * Description:	Diese Methode ist fr die allgemeine Eventbehandlung beim Klicken mit der Maustaste
	 *              verantwortlich, die fr alle Lernmodule identisch ist. Das sind z.B. der Scenarioeditor
	 *              bekommt den Fokus und das eventuell selektierte Objekt finden. Wenn das bereits
	 *              zuvor ausgewhlte Objekt selektiert wurde, wird der Editor in den MOVE Modus gesetzt,
	 *              da so ein Verschieben des Ojekts eingeleitet wird.  
	 *              
	 *              Diese Methode dient beim Entwickeln von Subklassen dazu doppelten Code zu vermeiden
	 * 
	 * Date: 		17.10.2006
	 * @author      sirocic
	 */
	public void clickedAction(MouseEvent event) {
		
		// TRACE OUTPUT
	  	if (LeviModule.isTraceActive()) System.out.println("CALL leviScenarioVisualizerMouseListener.clickedAction:");

	  	if (selected != null) {
	  		// ES WURDE EIN OBJEKT ANGEKLICKT !!!
		  
	   }		

	   // TRACE OUTPUT
	   if (LeviModule.isTraceActive()) System.out.println("END leviScenarioVisualizerMouseListener.clickedAction");
  			 
	}
	
   public void setScenarioPanel(LeviScenarioPanel parentScenarioPanel)
   {
   		this.scenarioPanel = parentScenarioPanel;
   }
   public void setPopup(LeviPopup popup)
   {
   		this.popup = popup;
   }
   
   // ACCE
   //METHODES ...
   public LeviScenarioPanel getScenarioPanel()
   {
   		return scenarioPanel;
   }
	
}

