/* leviShapeMaker.java
 * 
 * Created on 21.09.2005
 * @author sirocic
 */
package learningVisualizations.visualizer;

import java.awt.*;
import java.awt.geom.Point2D;
import javax.swing.*;

import learningVisualizations.LeviModule;
import learningVisualizations.scenario.LeviObject;



/**
 * @author sirocic
 *
 * Class: leviShapeMaker
 * 
 * Description: General class for the classes which draws the object shapes.
 * 
 * Date:	21.09.2005
 */
public abstract class LeviShapeMaker extends JPanel {
	
	// Konstanten
	final static public int SELECTBOXSIDE=5; // Hlfte der Seitenlnge eines Rahmenkstchens zur
	                                           // Markierung der Selektion
	final static public int OFFSET=10; // Offset fr die Anpassung der Panelgre (Breite/Hhe) 
									   // an die aktuellen Inhalte.
	
	private LeviObject object;
	private boolean selected;
	
	/**
    * Methode:	Constructors
    * 
    * Date: 21.09.2005
    * @author sirocic
    */
	public LeviShapeMaker()
	{
		this.object = null;
		this.selected = false;
		this.setOpaque(true);
	}
	public LeviShapeMaker(LeviObject object)
	{
		this.object = object;
		this.selected = false;
		this.setOpaque(true);
	}
	
	//ACCESSOR METHODES ...
	
	// METHODEN DIE IMPLEMENTIERTE WERDEN SOLLTEN, WENN DIE DARSTELLUNG NICHT ALS PANEL SONDERN ALS SHAPE
	// ERFOLGT.
	// zeichnet das jeweilige Object so wie es im Editor dargestellt wird.
	public void paintOnEditor(Graphics2D g2, Color drawColor, Color fillColor, Font labelFont, Font smallLabelFont ) {}
	
	public void fitPanel() {}
	// zeichnet das jeweilige Object so wie es im Visualizer dargestellt wird.
	public void paintOnVisualizer(Graphics2D g2, Color drawColor, Color fillColor, Color highlightColor, Font labelFont, Font smallLabelFont){}
    // Gibt den Pfad der zurck.
	public Shape makeShape() { return null; }
	
	public LeviObject getLeviObject()
	{
       return object;
	}
	public boolean isSelected()
	{
       return selected;
	}
   /*
    * Methode:		contains
    *
    * Description:	Bestimmt, ob der bergebene Punkt im Panel liegt oder nicht. berschreibt die Mehtode
    * 				contains von JComponent, das diese irgendwie nicht funktioniert. Wird das Objekt als
    * 				Shape visualisiert, mu makeShape() aufgerufen werden und dann mit der contains-Methode
    * 				des erzeugten Shape-Objekts gearbeitet werden.
    *
    * Parameters:    p = point , true = punkt liegt innerhalb des Panels.
    *
    * Date: 		 11.12.2006
    * @author        sirocic
    * 
    */
	public boolean contains(Point p) {
		boolean found= false;
		Shape objektShape;
		
      // TRACE OUTPUT
      if (LeviModule.isTraceActive()) System.out.println("CALL leviShapeMaker.contains: point=("+p.getX()+","+p.getY()+")");
      
      objektShape =makeShape();
      
      if ( objektShape != null)
    	  // Das Objekt wird als Shape visualisiert!!!
    	  // berprfen, ob der Punkt innerhalb des shapes liegt.
    	  found = objektShape.contains(p);
      else {
    	  // Das Objekt wird als JPanel visualisiert!!!
    	  // Die Methode contains von JPanel funktioniert irgendwie nicht.
    	  // Daher hier selber gestrickt. 
    	  if ( (p.getX() > getX()) && (p.getY() > getY())) {
	    	  if ((p.getX() < getX()+getWidth()) && (p.getY() < getY()+getHeight()))
	    		  found =true;
    	  }
      }
      
      // TRACE OUTPUT
      if (LeviModule.isTraceActive()) System.out.println("END leviShapeMaker.contains: ");
    
	  return found;
	  
	}
   /*
    * Methode:		updateLanguageColorFonts
    *
    * Description:	In diese Oberklasse gibt es nicht farb- oder sprachabhnges. Methode ist leer und
    * 				wird von der Subklass berschrieben bei Bedarf.
    *
    * Date: 		24.04.2007
    * 
    * @author sirocic
    */
   public void updateLanguageColorFonts() {}
	   
	//	MUTATOR METHODES ...
	public void setLeviObject(LeviObject obj)
	{
       object = obj;
	}
	public void setSelected(boolean value)
	{
		selected = value;
	}
	
}
