/* leviVisualizer.java
 *
 * Created on 23.06.2005
 * @author sirocic
 */
package learningVisualizations.visualizer;

import java.awt.*;

import javax.swing.*;

import learningVisualizations.LeviModule;
import learningVisualizations.editor.LeviScenarioEditorVisualizer;
import learningVisualizations.gui.LeviToolbar;

/**
 * @author sirocic
 *
 * Class: leviVisualizer
 *
 * Description: The class "leviVisualizer" is the general visualizer.
 *
 * Date:	23.06.2005
 */
public abstract class LeviVisualizer extends JPanel
					implements LeviVisualizerInterface {

    // CONSTANTS
    static final long serialVersionUID=1;

    // ATTRIBUTES
    // general attributes of the visualizer
    private LeviModule learningModule; // the parent module for the data transfer between the module parts
    protected boolean busy;
    
    // 	GUI ELEMENTS
    private LeviToolbar toolbar;
    private MenuContainer visualizerContentPanel;
    protected LeviScenarioVisualizerInterface scenarioVisualizer; // Panel frs Szenario
    /**
     * This constructor must not be used! leviVisualizer does not work without
     * learningModule.
     * Making this private gives an appropriate compiler error as a hint.
     *
     * TODO: Delete the constructor's contents after approval.
     */
   private LeviVisualizer() {
        super();
        this.toolbar = new LeviToolbar(null);
        this.visualizerContentPanel = null;
        this.learningModule = null;
        busy = false;
   }

   /**
    * Construct a leviVisualizer for the learningModule
    * @param learningModule
    */
   public LeviVisualizer(LeviModule learningModule) {

          super();

          // TRACE OUTPUT
          if (LeviModule.isTraceActive()) System.out.println("CALL leviVisualizer.Contructors(module)");

          // setting the module
          this.learningModule = learningModule;
          busy = false;

          // init the visualizer panel
          this.toolbar = new LeviToolbar(learningModule);
          // this assumes that there is no changelistener yet.
          learningModule.getConfiguration().addPropertyChangeListener(toolbar);
          this.setLayout(new BorderLayout());
          this.visualizerContentPanel = null;

          // FARBWECHSEL ...
     	  setPanelColors();

    	  // FONTWECHSEL
    	  setPanelFont();
    	  
          // TRACE OUTPUT
          if (LeviModule.isTraceActive()) System.out.println("END leviVisualizer.Contructors(module)");

   }
   
   // Diese Methode wird im Thread vom Simulator (leviSimulator) aufgerufen, um die Simulations-
   // ergebnis zu visualisieren. Ob die nur ein repaint() des Visualizers ist oder mehr, hngt
   // von der jeweiligen Komponente ab und sollte dort implementierte werden.
   public abstract void visualize() ;
 
   public abstract boolean isBusy();
   
   /*
    * Methode:		paintComponent
    *
    * Description:	Calls the paintComponent methode of the JPanel.
    *
    * Parameters:
    *
    * Date: 		 13.09.2005
    * @author        sirocic
    */
   public void paintComponent(Graphics g) {

      // TRACE OUTPUT
      if (LeviModule.isTraceActive()) System.out.println("CALL leviVisualizer.paintComponent:");

      // here calling the paint methode of the JPanel
      super.paintComponent(g);

      // TRACE OUTPUT
      if (LeviModule.isTraceActive()) System.out.println("END leviVisualizer.paintComponent");

   }
   /*
    * Methode:		setDialogFont
    * 
    * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
    * 				Werte. 
    *    
    * Parameters:   Aktuelle Font fr Labels und Textfelder.
    * 
    * Date: 		 23.04.2007
    * @author        sirocic
    */	
   private void setPanelFont() {
	   Font labelFont;
	   
	   labelFont = (Font)learningModule.getConfiguration().getSettingByKey("FontLabel");
	   
	   setFont(labelFont);
	   	  
   }
   
   
   /*
    * Methode:		setDialogTexte
    * 
    * Description:	Setz alle Text-Bausteine des Dialoges neu. Wichtig fr Sprachwechsel.
    *    
    * Parameters:   
    * 
    * Date: 		 23.04.2007
    * @author        sirocic
    */	
   protected void setPanelTexte() { }
   

   /*
    * Methode:		setDialogColors
    * 
    * Description:	Setz alle Hintergrund- und Vordergrundfarben der GUI-Element auf die aktuellen
    * 				Werte. 
    *    
    * Parameters:   Aktuelle Farbwerte fr Hintergrund und Vordergrund.
    * 
    * Date: 		 23.04.2007
    * @author        sirocic
    */	
   private void setPanelColors() {
	   Color backgroundColor, foregroundColor;
       
       if (learningModule.getConfiguration().getSettingByKey("ColorComponentBackground")!=null) {
	        backgroundColor = (Color)learningModule.getConfiguration().getSettingByKey("ColorComponentBackground");
	        foregroundColor = (Color)learningModule.getConfiguration().getSettingByKey("ColorComponentForeground");
	    }
	    else {
	    	backgroundColor = (Color)learningModule.getConfiguration().getSettingByKey("ColorModuleBackground");
	        foregroundColor = (Color)learningModule.getConfiguration().getSettingByKey("ColorModuleForeground");
	    }
       
	   // 	 Hintergrund und Zeichenfarbe auf aktuelle Werte setzen
       setBackground(backgroundColor);
	   setForeground(foregroundColor);
     
   }
   
   /*
    * Methode:		updateLanguageColorFonts
    *
    * Description:	Ldt sprachabhngige Teile neu. In der Toolbar knnen z.B. sprachabhngige
    * 				Buttons sein 
    *
    * Date: 		12.03.2007
    * 
    * @author sirocic
    */
   public void updateLanguageColorFonts()
   {
	   
	   //		 SUBKOMPONENTE UPDATEN...
	   this.getToolbar().updateLanguageColorFonts();
	   if (scenarioVisualizer != null) {
		   scenarioVisualizer.updateLanguageColorFonts();
	   }

	   // 	 FARBWECHSEL ...
	   setPanelColors();

	   // FONTWECHSEL
	   setPanelFont();
	  
	   // SPRACHWECHSEL ...
	   setPanelTexte();
	   
   }
   public LeviScenarioVisualizerInterface getScenarioVisualizer()
   {
       return scenarioVisualizer;
   }
   
   public void cleanMe() {
	   busy = false;
	   if (getScenarioVisualizer()!=null)
		   getScenarioVisualizer().cleanMe();
   }
   public void repaintMe() {
	   this.repaint();
   }

   public void setLearningModule(LeviModule c)
   {
      learningModule=c;
   }
   public void setVisualizerContentPanel(MenuContainer panel)
   {
       visualizerContentPanel=panel;
   }
   public void setScenarioVisualizer(LeviScenarioEditorVisualizer scenarioPanel)
   {
       this.scenarioVisualizer=scenarioPanel;
   }
   // ACCESSOR METHODES...
   public LeviModule getLearningModule()
   {
       return learningModule;
   }
   public MenuContainer getVisualizerContentPanel()
   {
       return visualizerContentPanel;
   }
   public LeviToolbar getToolbar()
   {
       return toolbar;
   }
   public void setToolbar(LeviToolbar t)
   {
       if( toolbar != null ) {
           if(learningModule.getConfiguration() != null) {
               learningModule.getConfiguration().removePropertyChangeListener(toolbar);
           }
       }
       toolbar=t;
       if(learningModule.getConfiguration() != null) {
           learningModule.getConfiguration().addPropertyChangeListener(toolbar);
       }
   }

}
